@@ -46,8 +46,15 @@ browse_refresh_stop( Browse *browse )
 	printf( "browse_refresh_stop: idle = %d\n", browse->idle_id );
 #endif /*DEBUG*/
 
-	IM_FREEF( gtk_idle_remove, browse->idle_id );
-	IM_FREEF( slist_free_all, browse->files );
+	if( browse->files )
+		set_glabel( browse->label, "Search incomplete!" );
+	else
+		set_glabel( browse->label, 
+			"Image files found in: \"%s\"", browse->dirname );
+
+	FREEFI( gtk_idle_remove, browse->idle_id );
+	FREEF( slist_free_all, browse->files );
+	mainw_countdown_end();
 }
 
 static void
@@ -64,7 +71,7 @@ browse_destroy( GtkObject *object )
 	 */
 	browse_refresh_stop( browse );
 	browse->filesel->browse = NULL;
-	IM_FREE( browse->dirname );
+	FREE( browse->dirname );
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -80,9 +87,9 @@ browse_build( GtkWidget *widget )
 	printf( "browse_build: %s\n", IWINDOW( browse )->title );
 #endif /*DEBUG*/
 
-	iwindow_set_title( IWINDOW( browse ), _( "Thumbnails" ) );
+	iwindow_set_title( IWINDOW( browse ), "Thumbnails" );
 	idialog_set_callbacks( idlg,
-		iwindow_true_cb, NULL, NULL, NULL );
+		iwindow_true_cb, NULL, NULL, NULL, NULL );
 
 	/* Call all builds in superclasses.
 	 */
@@ -101,26 +108,26 @@ browse_build( GtkWidget *widget )
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( browse->swin ), 
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
         gtk_box_pack_start( GTK_BOX( work ), browse->swin, TRUE, TRUE, 2 );
-        gtk_window_set_default_size( GTK_WINDOW( browse ), 
-		80 + 70 * NUM_COLUMNS, 400 );
+        gtk_widget_set_usize( GTK_WIDGET( browse->swin ), 
+		20 + 70 * NUM_COLUMNS, 200 );
 	gtk_widget_show( browse->swin );
 
 	gtk_widget_show_all( idlg->work );
 }
 
 static void
-browse_class_init( BrowseClass *class )
+browse_class_init( BrowseClass *klass )
 {
 	GtkObjectClass *object_class;
 	iWindowClass *iwindow_class;
 
-	object_class = (GtkObjectClass *) class;
-	iwindow_class = (iWindowClass *) class;
+	object_class = (GtkObjectClass *) klass;
+	iwindow_class = (iWindowClass *) klass;
 
 	object_class->destroy = browse_destroy;
 	iwindow_class->build = browse_build;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_IDIALOG );
 }
 
 static void
@@ -146,10 +153,10 @@ browse_init( Browse *browse )
 GtkType
 browse_get_type( void )
 {
-	static GtkType type = 0;
+	static GtkType browse_type = 0;
 
-	if( !type)  {
-		static const GtkTypeInfo info = {
+	if (!browse_type) {
+		static const GtkTypeInfo browse_info = {
 			"Browse",
 			sizeof( Browse ),
 			sizeof( BrowseClass ),
@@ -160,10 +167,10 @@ browse_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		type = gtk_type_unique( TYPE_IDIALOG, &info );
+		browse_type = gtk_type_unique( TYPE_IDIALOG, &browse_info );
 	}
 
-	return( type );
+	return( browse_type );
 }
 
 GtkWidget *
@@ -178,7 +185,7 @@ void
 browse_set_filesel( Browse *browse, Filesel *filesel )
 {
 	browse->filesel = filesel;
-	iwindow_set_parent( IWINDOW( browse ), GTK_WIDGET( filesel ) );
+	idialog_set_parent( IDIALOG( browse ), GTK_WIDGET( filesel ) );
 }
 
 /* Single click on a button.
@@ -189,8 +196,7 @@ button_single_cb( GtkWidget *wid, Browse *browse )
 {
 	Imagedisplay *id = IMAGEDISPLAY( GTK_BIN( wid )->child );
 
-	filesel_set_filename( browse->filesel, 
-		IOBJECT( id->conv->ii )->name );
+	filesel_set_filename( browse->filesel, id->conv->ii->name );
 }
 
 /* Double click on a button.
@@ -201,8 +207,7 @@ button_double_cb( GtkWidget *wid, Browse *browse )
 {
 	Imagedisplay *id = IMAGEDISPLAY( GTK_BIN( wid )->child );
 
-	filesel_set_filename( browse->filesel, 
-		IOBJECT( id->conv->ii )->name );
+	filesel_set_filename( browse->filesel, id->conv->ii->name );
 	idialog_done_trigger( IDIALOG( browse->filesel ), 0 ); 
 }
 
@@ -232,30 +237,28 @@ browse_add_widget( Browse *browse, GtkWidget *but )
 static void
 browse_add_image( Browse *browse, Imageinfo *ii )
 {
-	Conversion *conv;
 	Imagedisplay *id;
 
 	GtkWidget *but;
-	VipsBuf caption;
+	BufInfo caption;
         char buf[MAX_LINELENGTH];
 
-	conv = conversion_new( ii );
-	conv->tile_size = 16;
-	id = imagedisplay_new( conv );
-	gtk_widget_set_size_request( GTK_WIDGET( id ), 64, 64 );
-	imagedisplay_set_shrink_to_fit( id, TRUE );
+	id = imagedisplay_new( conversion_new( ii ), NULL, NULL );
+        id->sheight = 1;
+	GTK_WIDGET_UNSET_FLAGS( GTK_WIDGET( id ), GTK_CAN_FOCUS );
+	imagedisplay_set_size( id, 64, 64 );
+	imagedisplay_set_mag( id, 0 );
 
 	but = gtk_button_new();
 	gtk_container_add( GTK_CONTAINER( but ), GTK_WIDGET( id ) );
 	gtk_widget_show_all( but );
 
 	doubleclick_add( but, FALSE,
-		DOUBLECLICK_FUNC( button_single_cb ), browse, 
-		DOUBLECLICK_FUNC( button_double_cb ), browse );
+		button_single_cb, browse, button_double_cb, browse );
 
-	vips_buf_init_static( &caption, buf, MAX_LINELENGTH );
-	get_image_info( &caption, IOBJECT( ii )->name );
-	set_tooltip( but, "%s", vips_buf_all( &caption ) );
+	buf_init_static( &caption, buf, MAX_LINELENGTH );
+	get_image_info( &caption, ii->name );
+	set_tooltip( but, "%s", buf_all( &caption ) );
 
 	browse_add_widget( browse, but );
 }
@@ -267,27 +270,27 @@ browse_add_error( Browse *browse, char *name )
 {
 	GtkWidget *but;
 	char txt[MAX_STRSIZE];
-	VipsBuf buf;
+	BufInfo buf;
 
-	but = gtk_button_new_with_label( im_skip_dir( name ) );
+	but = gtk_button_new_with_label( im__skip_dir( name ) );
 	gtk_widget_show_all( but );
 
-	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	buf_init_static( &buf, txt, MAX_STRSIZE );
 	get_image_info( &buf, name );
-	vips_buf_appends( &buf, "\n" );
-	vips_buf_appends( &buf, error_get_top() );
-	set_tooltip( but, "%s", vips_buf_all( &buf ) );
+	buf_appends( &buf, "\n" );
+	buf_appends( &buf, error_string );
+	set_tooltip( but, "%s", buf_all( &buf ) );
 
 	browse_add_widget( browse, but );
 
-	error_clear();
+	ierrors( "" );
 }
 
 static void *
 browse_add_file( char *name, Browse *browse )
 {
-	if( strcmp( im_skip_dir( name ), "." ) == 0 || 
-		strcmp( im_skip_dir( name ), ".." ) == 0 )
+	if( strcmp( im__skip_dir( name ), "." ) == 0 || 
+		strcmp( im__skip_dir( name ), ".." ) == 0 )
 		return( NULL );
 
 	browse->files = g_slist_prepend( browse->files, im_strdupn( name ) );
@@ -295,18 +298,6 @@ browse_add_file( char *name, Browse *browse )
 	return( NULL );
 }
 
-static void
-browse_update_status( Browse *browse )
-{
-	if( !browse->files )
-		set_glabel( browse->label, 
-			_( "Image files found in: \"%s\"" ), browse->dirname );
-	else if( browse->idle_id )
-		set_glabel( browse->label, _( "Searching ..." ) );
-	else
-		set_glabel( browse->label, _( "Search incomplete!" ) );
-}
-
 static gint
 browse_idle( Browse *browse )
 {
@@ -319,7 +310,6 @@ browse_idle( Browse *browse )
 
 	if( !browse->files ) {
 		browse_refresh_stop( browse );
-		browse_update_status( browse );
 		return( FALSE );
 	}
 
@@ -330,24 +320,27 @@ browse_idle( Browse *browse )
 	 */
 	gtk_object_ref( GTK_OBJECT( browse ) );
 
-	ii = imageinfo_new_input( main_imageinfogroup, 
-		GTK_WIDGET( browse ), NULL, name );
+	ii = imageinfo_new_input( NULL, name );
 
-	/* We might have had our window closed during that load.
+	/* Our cancel button can be pushed during open.
 	 */
-	if( browse->idle_id ) {
+	if( !GTK_OBJECT_DESTROYED( GTK_OBJECT( browse ) ) ) {
 		if( ii )
 			browse_add_image( browse, ii );
 		else
 			browse_add_error( browse, name );
 	}
 
+	if( mainw_countdown_animate( 99 ) ) 
+		browse_refresh_stop( browse );
+
 	gtk_object_unref( GTK_OBJECT( browse ) );
 
 	/* If we're using it, it will have been reffed by the imagedisplay in
 	 * the button.
 	 */
-	MANAGED_UNREF( ii );
+	if( ii )
+		imageinfo_destroy_nonheap( ii );
 
 	im_free( name );
 
@@ -360,10 +353,10 @@ void
 browse_refresh( Browse *browse, const gchar *dirname )
 {
 	Filesel *filesel = browse->filesel;
-	int type = filesel_get_filetype( filesel );
+ 	GtkFileSelection2 *fs = GTK_FILE_SELECTION2( filesel );
 
-	char buf[FILENAME_MAX];
-	VipsBuf patt;
+        const char *suffix = gtk_file_selection2_get_file_type( fs );
+	char buf[PATHLEN];
 
 #ifdef DEBUG
 	printf( "browse_refresh: %s\n", dirname );
@@ -371,8 +364,8 @@ browse_refresh( Browse *browse, const gchar *dirname )
 
 	browse_refresh_stop( browse );
 
-	IM_SETSTR( browse->dirname, dirname );
-	IM_FREEF( gtk_widget_destroy, browse->table );
+	SETSTR( browse->dirname, dirname );
+	FREEF( gtk_widget_destroy, browse->table );
         browse->row = 0;
 
         browse->table = gtk_table_new( 1, NUM_COLUMNS, FALSE );
@@ -381,13 +374,10 @@ browse_refresh( Browse *browse, const gchar *dirname )
 	gtk_scrolled_window_add_with_viewport( 
 		GTK_SCROLLED_WINDOW( browse->swin ), browse->table );
         gtk_widget_show( browse->table );
+	set_glabel( browse->label, "Searching ..." );
 
-	vips_buf_init_static( &patt, buf, FILENAME_MAX );
-	filesel_make_patt( filesel->type[type], &patt );
-
-	(void) path_map_dir( dirname, vips_buf_all( &patt ), 
+	im_snprintf( buf, PATHLEN, "*%s", suffix );
+	(void) path_map_dir( dirname, buf, 
 		(path_map_fn) browse_add_file, browse );
 	browse->idle_id = gtk_idle_add( (GtkFunction) browse_idle, browse );
-
-	browse_update_status( browse );
 }
