@@ -16,9 +16,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -38,17 +38,16 @@
  */
 
 static gboolean
-class_is_class( PElement *instance )
+class_isclass( PElement *instance )
 {
 	if( !PEISCLASS( instance ) ) {
+		BufInfo buf;
 		char txt[50];
-		VipsBuf buf = VIPS_BUF_STATIC( txt );
 
-		if( !itext_value( reduce_context, &buf, instance ) )
+		buf_init_static( &buf, txt, 50 );
+		if( !itext_decompile( &buf, instance ) )
 			return( FALSE );
-		error_top( _( "Bad argument." ) );
-		error_sub( _( "Object %s is not a class." ), 
-			vips_buf_all( &buf ) );
+		ierrors( "object: %s\nis not a class", buf_all( &buf ) );
 
 		return( FALSE );
 	}
@@ -56,15 +55,6 @@ class_is_class( PElement *instance )
 	return( TRUE );
 }
 
-Compile *
-class_get_compile( PElement *instance )
-{
-	if( !class_is_class( instance ) )
-		return( NULL );
-
-	return( PEGETCLASSCOMPILE( instance ) );
-}
-
 /* Look up "super" in a class ... try to do it quickly.
  */
 gboolean
@@ -72,55 +62,20 @@ class_get_super( PElement *instance, PElement *out )
 {
 	Compile *compile;
 
-	if( !(compile = class_get_compile( instance )) )
+	if( !class_isclass( instance ) )
 		return( FALSE );
-	g_assert( compile->super );
+	compile = PEGETCLASSCOMPILE( instance );
+	assert( compile->super );
 
 	return( class_get_symbol( instance, compile->super, out ) );
 }
 
-void *
-class_map( PElement *instance, class_map_fn fn, void *a, void *b )
-{
-	PElement member;
-	HeapNode *p;
-
-	if( !PEISCLASS( instance ) ) 
-		return( NULL );
-
-	/* Loop over the instance member list.
-	 */
-	PEGETCLASSMEMBER( &member, instance );
-	if( !PEISELIST( &member ) )
-		for( p = PEGETVAL( &member ); p; p = GETRIGHT( p ) ) {
-			HeapNode *hn;
-			PElement s, v;
-			Symbol *sym;
-			void *result;
-
-			/* Get the sym/value pair, get the sym.
-			 */
-			hn = GETLEFT( p );
-			PEPOINTLEFT( hn, &s );
-			PEPOINTRIGHT( hn, &v );
-			sym = PEGETSYMREF( &s );
-
-			if( (result = fn( sym, &v, a, b )) )
-				return( result );
-
-		}
-
-	return( NULL );
-}
-
 /* Look up a member in a class instance by name. If lookup fails in this 
- * instance, try the superclass. Don't search secrets. Point sym and value 
- * at the symbol we found and its value. sym can be NULL for no result
- * required.
+ * instance, try the superclass. Don't search secrets. Point "out" at the 
+ * value.
  */
 gboolean
-class_get_member( PElement *instance, const char *name, 
-	Symbol **sym_out, PElement *out )
+class_get_member( PElement *instance, const char *name, PElement *out )
 {
 	PElement member;
 	PElement super;
@@ -131,7 +86,7 @@ class_get_member( PElement *instance, const char *name,
 	pgraph( instance );
 #endif /*DEBUG_MEMBER*/
 
-	if( !class_is_class( instance ) )
+	if( !class_isclass( instance ) )
 		return( FALSE );
 
 	/* Search this instance member list.
@@ -139,8 +94,8 @@ class_get_member( PElement *instance, const char *name,
 	PEGETCLASSMEMBER( &member, instance );
 	if( !PEISELIST( &member ) )
 		for( p = PEGETVAL( &member ); p; p = GETRIGHT( p ) ) {
-			HeapNode *hn;
 			PElement s;
+			HeapNode *hn;
 			Symbol *sym;
 
 			/* Get the sym/value pair, get the sym.
@@ -151,12 +106,10 @@ class_get_member( PElement *instance, const char *name,
 			/* Match? 
 			 */
 			sym = PEGETSYMREF( &s );
-			if( strcmp( IOBJECT( sym )->name, name ) == 0 ) {
+			if( strcmp( MODEL( sym )->name, name ) == 0 ) {
 				/* Found!
 				 */
 				PEPOINTRIGHT( hn, out );
-				if( sym_out )
-					*sym_out = sym;
 
 #ifdef DEBUG_MEMBER
 				printf( "class_get_member: found: " );
@@ -178,24 +131,22 @@ class_get_member( PElement *instance, const char *name,
 			      ... guess how long that took to find
 			      ... put this back at some point
 
-		return( class_get_member( &super, name, sym_out, value ) ); 
+		return( class_get_member( &super, name, out ) ); 
 		 */
-		gboolean result = class_get_member( &super, name, 
-			sym_out, out ); 
+		gboolean result = class_get_member( &super, name, out ); 
 
 		return( result );
 	}
 
-	error_top( _( "Member not found." ) );
-	error_sub( _( "Member \"%s\" not found in class \"%s\"." ),
-		name, IOBJECT( PEGETCLASSCOMPILE( instance )->sym )->name );
+	ierrors( "member \"%s\" not found in class \"%s\"",
+		name, MODEL( PEGETCLASSCOMPILE( instance )->sym )->name );
 
 	return( FALSE );
 }
 
-/* Look up a symbol in a class. Write to out, or FALSE for not found. Look up 
- * by symbol pointer. Search secrets as well. Try the superclass if lookup 
- * fails.
+/* Look up a symbol in a class. Write to out, or FALSE for not
+ * found. Look up by symbol pointer. Search secrets as
+ * well. Try the superclass if lookup fails.
  */
 gboolean
 class_get_symbol( PElement *instance, Symbol *sym, PElement *out )
@@ -207,11 +158,11 @@ class_get_symbol( PElement *instance, Symbol *sym, PElement *out )
 #ifdef DEBUG_MEMBER
 	printf( "class_get_symbol: looking up " );
 	symbol_name_print( sym );
-	printf( "in class " );
+	printf( " in class " );
 	pgraph( instance );
 #endif /*DEBUG_MEMBER*/
 
-	if( !class_is_class( instance ) )
+	if( !class_isclass( instance ) )
 		return( FALSE );
 
 	PEGETCLASSSECRET( &secret, instance );
@@ -271,8 +222,8 @@ class_get_exact( PElement *instance, const char *name, PElement *out )
 	PElement pe;
 
 	pe = *instance;
-	while( !reduce_is_instanceof_exact( reduce_context, name, &pe ) ) {
-		if( !class_get_super( &pe, &pe ) || PEISELIST( &pe ) ) 
+	while( !reduce_isinstanceof_exact( reduce_context, name, &pe ) ) {
+		if( !class_get_super( instance, &pe ) || PEISELIST( &pe ) ) 
 			return( FALSE );
 	}
 
@@ -284,7 +235,7 @@ class_get_exact( PElement *instance, const char *name, PElement *out )
 /* Stuff we need for class build.
  */
 typedef struct {
-	Heap *heap;		/* Heap to build on */
+	Heap *hi;		/* Heap to build on */
 	Symbol *sym;		/* Sym we are local to */
 	PElement *arg;		/* Args to constructor */
 	PElement *this;		/* Base of instance we are building */
@@ -299,8 +250,13 @@ class_member_secret( ClassBuildInfo *pbi,
 	Symbol *sym, GSList *secret, PElement *out )
 {
 	Symbol *ssym;
-	Heap *heap = pbi->heap;
+	Heap *hi = pbi->hi;
 	HeapNode *apl;
+#ifdef DEBUG_VERBOSE
+	PElement p1;
+	BufInfo buf;
+	char txt[1024];
+#endif /*DEBUG_VERBOSE*/
 
 	if( !secret )
 		return( TRUE );
@@ -308,7 +264,7 @@ class_member_secret( ClassBuildInfo *pbi,
 
 	/* Make function application for this member.
 	 */
-	if( NEWNODE( heap, apl ) )
+	if( NEWNODE( hi, apl ) )
 		return( FALSE );
 	apl->type = TAG_APPL;
 	PEPUTLEFT( apl, out );
@@ -319,44 +275,25 @@ class_member_secret( ClassBuildInfo *pbi,
 		PEPUTRIGHT( apl, pbi->this );
 	}
 	else {
-		/* Look up ssym in pbi->sym's secrets ... should be there 
+		/* Look up ssym in sym's secrets ... should be there 
 		 * somewhere. Use it's index to find the pbi->arg[] we need.
 		 */
-		int pos = g_slist_index( 
-			pbi->sym->expr->compile->secret, ssym );
+		int pos = g_slist_index( sym->expr->compile->secret, ssym );
 
-		/* FIXME ... may not be if we've regenerated one of these
-		 * stupid things :-( change this so we always go through 
-		 * 'this'.
-		 */
-		if( pos < 0 || pos >= pbi->sym->expr->compile->nsecret ) {
-			error_top( _( "No such secret." ) );
-			error_sub( _( "Editing local classes which reference "
-				"non-local objects is a bit broken at the "
-				"moment :-(" ) );
-			return( FALSE );
-		}
+		assert( pos >= 0 && pos < sym->expr->compile->nsecret );
 
-		PEPUTRIGHT( apl, 
-			&pbi->arg[pbi->sym->expr->compile->nsecret - pos - 1] );
+		PEPUTRIGHT( apl, &pbi->arg[pos] );
 	}
 
 	PEPUTP( out, ELEMENT_NODE, apl );
 
 #ifdef DEBUG_VERBOSE
-{
-	PElement p1;
-	char txt[1024];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
-
+	buf_init_static( &buf, txt, 1024 );
 	PEPOINTRIGHT( apl, &p1 );
-	graph_pelement( pbi->heap, &buf, &p1, TRUE );
-	printf( "class_member_secret: secret arg " );
-	symbol_name_print( ssym );
-	printf( "to member " );
-	symbol_name_print( sym );
-	printf( "= %s\n", vips_buf_all( &buf ) );
-}
+	graph_pelement( hi, &buf, &p1, TRUE );
+	printf( "class_member_secret: secret arg \"%s\" "
+		"to member \"%s\" = %s\n", 
+		MODEL( ssym )->name, MODEL( sym )->name, buf_all( &buf ) );
 #endif /*DEBUG_VERBOSE*/
 
 	return( class_member_secret( pbi, sym, secret->next, out ) );
@@ -367,9 +304,13 @@ class_member_secret( ClassBuildInfo *pbi,
 static void *
 add_class_member( Symbol *sym, ClassBuildInfo *pbi, PElement *out )
 {
-	Heap *heap = pbi->heap;
+	Heap *hi = pbi->hi;
 	HeapNode *base, *sv;
 	PElement v;
+#ifdef DEBUG_VERBOSE
+	BufInfo buf;
+	char txt[1024];
+#endif /*DEBUG_VERBOSE*/
 
 	/* Is this something that should be part of a class.
 	 */
@@ -378,7 +319,7 @@ add_class_member( Symbol *sym, ClassBuildInfo *pbi, PElement *out )
 
 	/* Make new class-local-list element for this local.
 	 */
-	if( NEWNODE( heap, base ) )
+	if( NEWNODE( hi, base ) )
 		return( sym );
 	base->type = TAG_CONS;
 	PPUTLEFT( base, ELEMENT_ELIST, NULL );
@@ -387,7 +328,7 @@ add_class_member( Symbol *sym, ClassBuildInfo *pbi, PElement *out )
 
 	/* Make sym/value pair for this local.
 	 */
-	if( NEWNODE( heap, sv ) )
+	if( NEWNODE( hi, sv ) )
 		return( sym );
 	sv->type = TAG_CONS;
 	PPUT( sv, ELEMENT_SYMREF, sym, ELEMENT_SYMBOL, sym ); 
@@ -400,15 +341,10 @@ add_class_member( Symbol *sym, ClassBuildInfo *pbi, PElement *out )
 		return( sym );
 
 #ifdef DEBUG_VERBOSE
-{
-	char txt[1024];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-	graph_pelement( heap, &buf, &v, TRUE );
+	buf_init_static( &buf, txt, 1024 );
+	graph_pelement( hi, &buf, &v, TRUE );
 	printf( "add_class_member: member \"%s\" of class \"%s\" = %s\n",
-		IOBJECT( sym )->name, IOBJECT( pbi->sym )->name, 
-		vips_buf_all( &buf ) );
-}
+		MODEL( sym )->name, MODEL( pbi->sym )->name, buf_all( &buf ) );
 #endif /*DEBUG_VERBOSE*/
 
 	return( NULL );
@@ -420,25 +356,24 @@ static gboolean
 add_class_svpair( ClassBuildInfo *pbi, 
 	Symbol *sym, PElement *val, PElement *out )
 {
-	Heap *heap = pbi->heap;
+	Heap *hi = pbi->hi;
 	HeapNode *base, *sv;
+#ifdef DEBUG_VERBOSE
+	BufInfo buf;
+	char txt[1024];
+#endif /*DEBUG_VERBOSE*/
 
 #ifdef DEBUG_VERBOSE
-{
-	char txt[1024];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-	graph_pelement( heap, &buf, val, TRUE );
+	buf_init_static( &buf, txt, 1024 );
+	graph_pelement( hi, &buf, val, TRUE );
 	printf( "add_class_svpair: adding parameter \"%s\" to class "
-		"\"%s\" = %s\n", 
-		IOBJECT( sym )->name, IOBJECT( pbi->sym )->name,
-		vips_buf_all( &buf ) );
-}
+		"\"%s\" = %s\n", MODEL( sym )->name, MODEL( pbi->sym )->name,
+		buf_all( &buf ) );
 #endif /*DEBUG_VERBOSE*/
 
 	/* Make new class-local-list element for this parameter.
 	 */
-	if( NEWNODE( heap, base ) )
+	if( NEWNODE( hi, base ) )
 		return( FALSE );
 	base->type = TAG_CONS;
 	PPUTLEFT( base, ELEMENT_ELIST, NULL );
@@ -447,7 +382,7 @@ add_class_svpair( ClassBuildInfo *pbi,
 
 	/* Make sym/value pair for this parameter.
 	 */
-	if( NEWNODE( heap, sv ) )
+	if( NEWNODE( hi, sv ) )
 		return( FALSE );
 	sv->type = TAG_CONS;
 	PPUTLEFT( sv, ELEMENT_SYMREF, sym )
@@ -477,33 +412,34 @@ add_class_parameter( Symbol *sym, ClassBuildInfo *pbi, PElement *out )
 /* Add the name member ... build the name string carefully.
  */
 static void *
-class_new_single_name( Heap *heap, PElement *pe,
+class_new_single_name( Heap *hi, PElement *pe,
 	ClassBuildInfo *pbi, PElement *instance )
 {
-	Symbol *snm = compile_lookup( pbi->compile, MEMBER_NAME );
-	char txt[256];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
+	Symbol *snm = stable_find( pbi->compile->locals, MEMBER_NAME );
+	BufInfo buf;
+	char str[256];
 
 	/* Make class name string.
 	 */
+	buf_init_static( &buf, str, 256 );
 	symbol_qualified_name( pbi->sym, &buf );
 	PEPUTP( pe, ELEMENT_ELIST, NULL );
-	if( !heap_managedstring_new( heap, vips_buf_all( &buf ), pe ) ) 
-		return( heap );
+	if( !heap_string_new( hi, buf_all( &buf ), pe ) ) 
+		return( hi );
 
 	/* Add as a member.
 	 */
 	if( !add_class_svpair( pbi, snm, pe, instance ) ) 
-		return( heap );
+		return( hi );
 
 	return( NULL );
 }
 
 /* Make a single level class instance ... fn below then loops over a class
- * hierarchy with this.
+ * heierarchy with this.
  */
 static gboolean
-class_new_single( Heap *heap, 
+class_new_single( Heap *hi, 
 	Compile *compile, PElement *arg, PElement *this, PElement *out )
 {
 	Symbol *sym = compile->sym;
@@ -513,36 +449,9 @@ class_new_single( Heap *heap,
 	PElement p1;
 	ClassBuildInfo pbi;
 
-#ifdef DEBUG
-{
-	int i;
-
-	printf( "class_new_single: starting for " );
-	symbol_name_print( sym );
-	printf( "%d secrets, %d params\n", 
-		compile->nsecret, compile->nparam );
-
-	for( i = 0; i < compile->nsecret; i++ ) {
-		char txt[256];
-		VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-		graph_pelement( heap, &buf, &arg[i], TRUE );
-		printf( "\tsecret %2d = %s\n", i, vips_buf_all( &buf ) );
-	}
-
-	for( i = 0; i < compile->nparam; i++ ) {
-		char txt[256];
-		VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-		graph_pelement( heap, &buf, &arg[i + compile->nsecret], TRUE );
-		printf( "\targ %2d = %s\n", i, vips_buf_all( &buf ) );
-	}
-}
-#endif /*DEBUG*/
-
 	/* Make class base.
 	 */
-	if( NEWNODE( heap, base ) )
+	if( NEWNODE( hi, base ) )
 		return( FALSE );
 	base->type = TAG_CLASS;
 	PPUT( base, ELEMENT_COMPILEREF, compile, ELEMENT_ELIST, NULL ); 
@@ -550,7 +459,7 @@ class_new_single( Heap *heap,
 
 	/* Make node for holding secrets and members.
 	 */
-	if( NEWNODE( heap, sm ) )
+	if( NEWNODE( hi, sm ) )
 		return( FALSE );
 	sm->type = TAG_CONS;
 	PPUT( sm, ELEMENT_ELIST, NULL, ELEMENT_ELIST, NULL ); 
@@ -558,19 +467,19 @@ class_new_single( Heap *heap,
 
 	/* Build list of members.
 	 */
-	pbi.heap = heap;
+	pbi.hi = hi;
 	pbi.sym = sym;
 	pbi.arg = arg;
 	pbi.this = this;
 	pbi.compile = compile;
 	PEPOINTRIGHT( sm, &p1 );
-	if( icontainer_map_rev( ICONTAINER( compile ), 
-		(icontainer_map_fn) add_class_member, &pbi, &p1 ) ) 
+	if( stable_map_rev( compile->locals, 
+		(symbol_map_fn) add_class_member, &pbi, &p1, NULL ) ) 
 		return( FALSE );
 
 	/* Add name member.
 	 */
-	if( heap_safe_pointer( heap,
+	if( heap_safe_pointer( hi,
 		(heap_safe_pointer_fn) class_new_single_name, 
 		&pbi, &p1, NULL, NULL ) )
 		return( FALSE );
@@ -600,78 +509,21 @@ class_new_single( Heap *heap,
 
 #ifdef DEBUG
 {
-	char txt[256];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
+	BufInfo buf;
+	char str[256];
 
-	graph_pelement( heap, &buf, out, TRUE );
-	printf( "class_new_single: built instance of " );
-	symbol_name_print( sym );
-	printf( ":\n%s\n", vips_buf_all( &buf ) );
+	buf_init_static( &buf, str, 256 );
+	graph_pelement( hi, &buf, out, TRUE );
+	printf( "class_new_single: built instance of \"%s\": %s\n", 
+		MODEL( sym )->name, buf_all( &buf ) );
 }
 #endif /*DEBUG*/
 
 	return( TRUE );
 }
 
-/* Look at a scrap of graph and try to find a constructor it might be using.
- * This will only work for really basic functions :-( but it's enough to allow
- * us to pass extra secrets through the superclass. Used by (eg.) Colour when
- * it overrides Value and adds the colourspace arg.
- */
-static Compile *
-class_guess_constructor( PElement *fn )
-{
-	if( PEISCONSTRUCTOR( fn ) ) 
-		return( PEGETCOMPILE( fn ) );
-	else if( PEISNODE( fn ) ) {
-		HeapNode *hn = PEGETVAL( fn );
-
-		if( hn->type == TAG_APPL ) {
-			PElement left;
-
-			PEPOINTLEFT( hn, &left );
-
-			return( class_guess_constructor( &left ) );
-		}
-	}
-
-	return( NULL );
-}
-
-/* Look at arg0 and try to extract the arguments (all the RHS of the @ nodes).
- * Return the number of args we found, or -1 if we find crazy stuff.
- */
-static int
-class_guess_args( PElement arg[], PElement *fn )
-{
-	if( PEISCONSTRUCTOR( fn ) )
-		return( 0 );
-	else if( PEISNODE( fn ) ) {
-		PElement left;
-		int i;
-
-		PEPOINTLEFT( PEGETVAL( fn ), &left );
-		if( (i = class_guess_args( arg, &left )) == -1 )
-			return( -1 );
-		if( i >= MAX_SYSTEM ) {
-			error_top( _( "Too many arguments." ) );
-			error_sub( _( "You can't have more than %d "
-				"arguments to a superclass constructor." ),
-				MAX_SYSTEM );
-
-			return( -1 );
-		}
-
-		PEPOINTRIGHT( PEGETVAL( fn ), &arg[i] );
-
-		return( i + 1 );
-	}
-	else
-		return( -1 );
-}
-
 static void *
-class_new_super_sub( Heap *heap, PElement *p1,
+class_new_super_sub( Heap *hi, PElement *p1,
 	Compile *compile, PElement *arg, PElement *this, PElement *super )
 {
 	/* Build the superclass ... we overwrite the super 
@@ -680,65 +532,8 @@ class_new_super_sub( Heap *heap, PElement *p1,
 	 */
 	PEPUTPE( p1, super );
 
-	if( !class_new_single( heap, compile, arg, this, super ) ) 
-		return( heap );
-
-	return( NULL );
-}
-
-/* Clone a class instance. Copy pointers to the the args, secrets and super; 
- * rebuild with the specified "this". Instance and out can be equal.
- */
-static gboolean
-class_clone_super( Heap *heap, Compile *compile,
-	PElement *instance, PElement *this, PElement *out )
-{
-	PElement arg[MAX_SYSTEM];
-	const int nargs = compile->nsecret + compile->nparam;
-	PElement secret;
-	int i;
-
-	g_assert( nargs <= MAX_SYSTEM );
-
-#ifdef DEBUG_VERBOSE
-{
-	char txt[MAX_STRSIZE];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-	graph_pelement( heap, &buf, instance, TRUE );
-	printf( "class_new_clone: about to clone \"%s\": %s\n", 
-		IOBJECT( compile->sym )->name, vips_buf_all( &buf ) );
-}
-#endif /*DEBUG_VERBOSE*/
-
-	/* Pull out values of secrets and class args into arg[].
-	 */
-	PEGETCLASSSECRET( &secret, instance );
-	for( i = 0; i < nargs; i++ ) {
-		HeapNode *hn = PEGETVAL( &secret );
-		HeapNode *sv = GETLEFT( hn );
-		int index = nargs - i - 1;
-
-		PEPOINTRIGHT( sv, &arg[index] );
-		PEPOINTRIGHT( hn, &secret );
-	}
-
-	/* Build class again.
-	 */
-	return( class_new_single( heap, compile, arg, this, out ) );
-}
-
-static void *
-class_clone_super_sub( Heap *heap, PElement *p1,
-	Compile *compile, PElement *instance, PElement *this, PElement *out )
-{
-	/* instance and out can point to the same node, so save a pointer to  
-	 * instance to stop it being GCed.
-	 */
-	PEPUTPE( p1, instance );
-
-	if( !class_clone_super( heap, compile, instance, this, out ) ) 
-		return( heap );
+	if( !class_new_single( hi, compile, arg, this, super ) ) 
+		return( hi );
 
 	return( NULL );
 }
@@ -746,129 +541,105 @@ class_clone_super_sub( Heap *heap, PElement *p1,
 /* Does this class have a "super"? Build it and recurse.
  */
 gboolean
-class_new_super( Heap *heap, 
+class_new_super( Heap *hi, 
 	Compile *compile, PElement *this, PElement *instance )
 {
 	PElement super;
 
 	if( compile->has_super && class_get_super( instance, &super ) ) {
 		Compile *super_compile;
-		int len, fn_len;
+		int len;
 		PElement arg0;
 
 		/* It must be a list whose first element is the superclass
-		 * constructor, or a partially parameterised constructor, or 
-		 * the superclass itself (if it has already
-		 * been constructed, or has no args). Other elements in the
-		 * list are the remaining args.
-		 *
-		 * We keep the list form, since we want to not build the
-		 * superclass until now if we can help it ... otherwise we
-		 * have to construct once, then construct again when we clone.
+		 * constructor, or the superclass itself (if it has no args).
 		 */
 		if( (len = heap_list_length( &super )) < 1 ||
 			!heap_list_index( &super, 0, &arg0 ) ||
 			!heap_reduce_strict( &arg0 ) )
 			return( FALSE );
 
-		if( (super_compile = class_guess_constructor( &arg0 )) ) {
-			PElement fn_arg[MAX_SYSTEM];
-			PElement arg[MAX_SYSTEM];
+		if( PEISCONSTRUCTOR( &arg0 ) ) {
 			int i;
+			PElement arg[MAX_SYSTEM];
 
-			/* How many function args are there? 
+			super_compile = PEGETCOMPILE( &arg0 );
+
+			/* It's a constructor ... we need to give it some
+			 * args.
 			 */
-			if( (fn_len = class_guess_args( fn_arg, &arg0 )) < 0 )
-				return( FALSE );
-
-			/* Check total arg count.
-			 */
-			if( super_compile->nsecret != 0 ) {
-				char txt[1024];
-				VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-				slist_map2( super_compile->secret, 
-					(SListMap2Fn) symbol_name_error, 
-					&buf, NULL );
-
-				error_top( _( "Bad superclass." ) );
-				error_sub( _( "Superclass constructor \"%s\" "
-					"refers to non-local symbols %s" ),
-					symbol_name( super_compile->sym ),
-					vips_buf_all( &buf ) );
-
+			if( len >= MAX_SYSTEM ) {
+				ierrors( "too many args to superclass "
+					"constructor %s",
+					symbol_name( super_compile->sym ) );
 				return( FALSE );
 			}
-			if( len - 1 + fn_len != super_compile->nparam ) {
-				error_top( _( "Wrong number of arguments." ) );
-				error_sub( _( "Superclass constructor \"%s\" "
-					"expects %d arguments, not %d." ),
+
+			if( super_compile->nparam != len - 1 ) {
+				ierrors( "superclass constructor %s "
+					"expects %d arguments, not %d",
 					symbol_name( super_compile->sym ),
 					super_compile->nparam,
-					len - 1 + fn_len );
+					len - 1 );
+
+				return( FALSE );
+			}
+			if( super_compile->nsecret != 0 ) {
+				ierrors( "superclass constructor %s "
+					"should have no secrets",
+					symbol_name( super_compile->sym ) );
 
 				return( FALSE );
 			}
 
-			/* Grab the explicit args from the super list.
+			/* Build the arg array.
 			 */
 			for( i = 0; i < len - 1; i++ ) {
-				if( !heap_list_index( &super, len - 1 - i, 
-					&arg[i] ) ) 
+				if( !heap_list_index( &super, len - i - 1, 
+					&arg[i] ) )
 					return( FALSE );
 			}
 
-			/* Append the function args, but reverse them as we
-			 * go so we get most-nested arg last.
-			 */
-			for( i = 0; i < fn_len; i++ )
-				arg[i + len - 1] = fn_arg[fn_len - 1 - i];
-
 			/* Build the superclass ... we overwrite the super 
 			 * list with the constructed class, so make a copy of 
 			 * the pointer to stop it being GCed.
 			 */
-			if( heap_safe_pointer( heap, 
+			if( heap_safe_pointer( hi, 
 				(heap_safe_pointer_fn) class_new_super_sub,
 				super_compile, arg, this, &super ) )
 				return( FALSE );
-
 		}
 		else if( PEISCLASS( &arg0 ) ) {
-			/* Super is a constructed class ... clone it, but with
-			 * our "this" in there. Slow, but useful.
-			 */
 			super_compile = PEGETCLASSCOMPILE( &arg0 );
 
-			if( heap_safe_pointer( heap, 
-				(heap_safe_pointer_fn) class_clone_super_sub,
-				super_compile, &arg0, this, &super ) )
+			/* It's ready built ... there should not be any args.
+			 */
+			if( super_compile->nparam != len - 1 ) {
+				ierrors( "superclass constructor %s "
+					"expects %d arguments, not %d",
+					symbol_name( super_compile->sym ),
+					super_compile->nparam,
+					len - 1 );
+				return( FALSE );
+			}
+
+			/* The instance we have does not have the right "this"
+			 * wired into it ... we must construct a new one.
+			 */
+			if( !class_new_single( hi, 
+				super_compile, NULL, this, &super ) ) 
 				return( FALSE );
 		}
 		else {
-			char txt1[300];
-			VipsBuf buf1 = VIPS_BUF_STATIC( txt1 );
-			char txt2[300];
-			VipsBuf buf2 = VIPS_BUF_STATIC( txt2 );
-
-			error_top( _( "Bad superclass." ) );
-
-			itext_value( reduce_context, &buf1, &arg0 );
-			vips_buf_appendf( &buf2,
-				_( "First element in superclass of \"%s\" "
-				"must be class or constructor." ),
+			ierrors( "first element in superclass of %s must be "
+				"class or constructor",
 				symbol_name( compile->sym ) );
-			vips_buf_appendf( &buf2, "\n" );
-			vips_buf_appendf( &buf2, _( "You passed:" ) );
-			error_sub( "%s\n  %s", 
-				vips_buf_all( &buf2 ), vips_buf_all( &buf1 ) );
-
 			return( FALSE );
 		}
 
 		/* And recursively build any superclasses.
 		 */
-		if( !class_new_super( heap, super_compile, this, &super ) )
+		if( !class_new_super( hi, super_compile, this, &super ) )
 			return( FALSE );
 	}
 
@@ -878,7 +649,7 @@ class_new_super( Heap *heap,
 /* Make a class instance. 
  */
 gboolean
-class_new( Heap *heap, Compile *compile, HeapNode **arg, PElement *out )
+class_new( Heap *hi, Compile *compile, HeapNode **arg, PElement *out )
 {
 	int i;
 	PElement pe_arg[MAX_SYSTEM];
@@ -886,10 +657,7 @@ class_new( Heap *heap, Compile *compile, HeapNode **arg, PElement *out )
 	/* Make a set of arg pointers.
 	 */
 	if( compile->nparam + compile->nsecret >= MAX_SYSTEM ) {
-		error_top( _( "Too many arguments." ) );
-		error_sub( _( "Too many arguments to class constructor \"%s\". "
-			"No more than %d arguments are supported." ),
-			symbol_name( compile->sym ), MAX_SYSTEM );
+		ierrors( "too many args to class" );
 		return( FALSE );
 	}
 	for( i = 0; i < compile->nparam + compile->nsecret; i++ ) {
@@ -898,22 +666,23 @@ class_new( Heap *heap, Compile *compile, HeapNode **arg, PElement *out )
 
 	/* Build the base instance.
 	 */
-	if( !class_new_single( heap, compile, pe_arg, out, out ) )
+	if( !class_new_single( hi, compile, pe_arg, out, out ) )
 		return( FALSE );
 
 	/* And recursively build any superclasses.
 	 */
-	if( !class_new_super( heap, compile, out, out ) )
+	if( !class_new_super( hi, compile, out, out ) )
 		return( FALSE );
 
 #ifdef DEBUG_BUILD
 {
-	char txt[MAX_STRSIZE];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
+	BufInfo buf;
+	char str[MAX_STRSIZE];
 
-	graph_pelement( heap, &buf, out, TRUE );
+	buf_init_static( &buf, str, MAX_STRSIZE );
+	graph_pelement( hi, &buf, out, TRUE );
 	printf( "class_new: built instance of \"%s\": %s\n", 
-		IOBJECT( compile->sym )->name, vips_buf_all( &buf ) );
+		MODEL( compile->sym )->name, buf_all( &buf ) );
 }
 #endif /*DEBUG_BUILD*/
 
@@ -924,25 +693,25 @@ class_new( Heap *heap, Compile *compile, HeapNode **arg, PElement *out )
  * regenerate all the members. instance and out can be equal.
  */
 gboolean
-class_clone_args( Heap *heap, PElement *instance, PElement *out )
+class_clone_args( Heap *hi, PElement *instance, PElement *out )
 {
 	HeapNode *arg[MAX_SYSTEM];
 	Compile *compile = PEGETCLASSCOMPILE( instance );
 	const int nargs = compile->nsecret + compile->nparam;
 	PElement secret;
 	int i;
+#ifdef DEBUG_VERBOSE
+	BufInfo buf;
+	char str[NAMELEN];
+#endif /*DEBUG_VERBOSE*/
 
-	g_assert( nargs <= MAX_SYSTEM );
+	assert( nargs <= MAX_SYSTEM );
 
 #ifdef DEBUG_VERBOSE
-{
-	char txt[MAX_STRSIZE];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-	graph_pelement( heap, &buf, instance, TRUE );
+	buf_init_static( &buf, str, NAMELEN );
+	graph_pelement( hi, &buf, instance, TRUE );
 	printf( "class_clone_args: about to clone \"%s\": %s\n", 
-		IOBJECT( compile->sym )->name, vips_buf_all( &buf ) );
-}
+		MODEL( compile->sym )->name, buf_all( &buf ) );
 #endif /*DEBUG_VERBOSE*/
 
 	/* Pull out values of secrets and class args into RHS of arg[].
@@ -959,34 +728,30 @@ class_clone_args( Heap *heap, PElement *instance, PElement *out )
 
 	/* Build class again.
 	 */
-	return( class_new( heap, compile, &arg[0], out ) );
+	return( class_new( hi, compile, &arg[0], out ) );
 }
 
 /* Build a class instance picking parameters from C args ... handy for
  * making a new toggle instance on a click, for example.
  */
 gboolean
-class_newv( Heap *heap, const char *name, PElement *out, ... )
+class_newv( Heap *hi, const char *name, PElement *out, ... )
 {
 	va_list ap;
 	Symbol *sym;
 	Compile *compile;
-	HeapNode args[MAX_SYSTEM];
-	HeapNode *pargs[MAX_SYSTEM];
+	HeapNode args[ MAX_SYSTEM ];
+	HeapNode *pargs[ MAX_SYSTEM ];
 	int i;
 
-	if( !(sym = compile_lookup( symbol_root->expr->compile, name )) ||
+	if( !(sym = stable_find( symbol_root->expr->compile->locals, name )) ||
 		!is_value( sym ) || !is_class( sym->expr->compile ) ) {
-		error_top( _( "Class not found." ) );
-		error_sub( _( "Class \"%s\" not found." ), name );
+		ierrors( "class \"%s\" not found", name );
 		return( FALSE );
 	}
 	compile = sym->expr->compile;
 	if( compile->nparam >= MAX_SYSTEM ) {
-		error_top( _( "Too many arguments." ) );
-		error_sub( _( "Too many arguments to class constructor \"%s\". "
-			"No more than %d arguments are supported." ),
-			symbol_name( compile->sym ), MAX_SYSTEM );
+		ierrors( "too many arguments to class" );
 		return( FALSE );
 	}
 
@@ -1001,28 +766,28 @@ class_newv( Heap *heap, const char *name, PElement *out, ... )
 	}
         va_end( ap );
 
-	return( class_new( heap, compile, &pargs[0], out ) );
+	return( class_new( hi, compile, &pargs[0], out ) );
 }
 
 static void
 class_typecheck_error( PElement *instance, const char *name, const char *type )
 {
-	char txt[1024];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
 	PElement val;
+	gboolean res;
+	BufInfo buf;
+	char txt[1024];
 
-	vips_buf_appendf( &buf, _( "Member \"%s\" of class \"%s\" "
-		"should be of type \"%s\", instead it's:" ), 
-		name, 
-		IOBJECT( PEGETCLASSCOMPILE( instance )->sym )->name, 
-		type );
-	vips_buf_appends( &buf, "\n   " );
-	if( class_get_member( instance, name, NULL, &val ) &&
-		!itext_value( reduce_context, &buf, &val ) )
+	res = class_get_member( instance, name, &val );
+	assert( res );
+
+	buf_init_static( &buf, txt, 1024 );
+	buf_appendf( &buf, "member \"%s\" of class \"%s\" should be of type "
+		"\"%s\", instead it's:\n   ",
+		name, MODEL( PEGETCLASSCOMPILE( instance )->sym )->name, type );
+	if( !itext_decompile( &buf, &val ) )
 		return;
 
-	error_top( _( "Bad argument." ) );
-	error_sub( "%s", vips_buf_all( &buf ) );
+	ierrors( "%s", buf_all( &buf ) );
 }
 
 /* A function that gets a type from a class.
@@ -1035,7 +800,7 @@ class_get_member_check( PElement *instance, const char *name, const char *type,
 {
 	PElement val;
 
-	if( !class_get_member( instance, name, NULL, &val ) )
+	if( !class_get_member( instance, name, &val ) )
 		return( FALSE );
 
 	if( !fn( &val, a ) ) {
@@ -1083,7 +848,7 @@ class_get_member_class( PElement *instance, const char *name,
 		(ClassGetFn) heap_get_class, out ) )
 		return( FALSE );
 
-	if( !heap_is_instanceof( type, out, &result ) ) 
+	if( !heap_isinstanceof( type, out, &result ) ) 
 		return( FALSE );
 	if( !result ) {
 		class_typecheck_error( instance, name, type );
@@ -1114,7 +879,7 @@ class_get_member_string( PElement *instance, const char *name,
 {
 	PElement val;
 
-	if( !class_get_member( instance, name, NULL, &val ) )
+	if( !class_get_member( instance, name, &val ) )
 		return( FALSE );
 
 	if( !heap_get_string( &val, buf, sz ) ) {
@@ -1131,8 +896,8 @@ class_get_member_instance( PElement *instance,
 {
 	gboolean result;
 
-	return( class_get_member( instance, name, NULL, out ) &&
-		heap_is_instanceof( klass, out, &result ) &&
+	return( class_get_member( instance, name, out ) &&
+		heap_isinstanceof( klass, out, &result ) &&
 		result );
 }
 
@@ -1142,7 +907,7 @@ class_get_member_matrix_size( PElement *instance, const char *name,
 {
 	PElement val;
 
-	if( !class_get_member( instance, name, NULL, &val ) )
+	if( !class_get_member( instance, name, &val ) )
 		return( FALSE );
 
 	if( !heap_get_matrix_size( &val, xsize, ysize ) ) {
@@ -1160,7 +925,7 @@ class_get_member_matrix( PElement *instance, const char *name,
 {
 	PElement val;
 
-	if( !class_get_member( instance, name, NULL, &val ) )
+	if( !class_get_member( instance, name, &val ) )
 		return( FALSE );
 
 	if( !heap_get_matrix( &val, buf, n, xsize, ysize ) ) {
@@ -1179,7 +944,7 @@ class_get_member_realvec( PElement *instance, const char *name,
 	PElement val;
 	int l;
 
-	if( !class_get_member( instance, name, NULL, &val ) )
+	if( !class_get_member( instance, name, &val ) )
 		return( FALSE );
 
 	if( (l = heap_get_realvec( &val, buf, n )) < 0 ) {
