@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -31,9 +31,6 @@
  */
 #define MEMBER_BANDS "bands"
 #define MEMBER_CAPTION "caption"
-#define MEMBER_XCAPTION "xcaption"
-#define MEMBER_YCAPTION "ycaption"
-#define MEMBER_SERIES_CAPTIONS "series_captions"
 #define MEMBER_DISPLAY "display"
 #define MEMBER_CHECK "check"
 #define MEMBER_FILENAME "filename"
@@ -56,24 +53,15 @@
 #define MEMBER_OO_BINARY2 "oo_binary'"
 #define MEMBER_OO_UNARY "oo_unary"
 #define MEMBER_COLOUR_SPACE "colour_space"
-#define MEMBER_EXPR "expr"
-#define MEMBER_INTERVAL "interval"
-#define MEMBER_OPTIONS "options"
-
 #define MEMBER_VISLEVEL "_vislevel"
-#define MEMBER_ACTION "action"
-#define MEMBER_LABEL "label"
-#define MEMBER_ICON "icon"
-#define MEMBER_TOOLTIP "tooltip"
 
 /* The class names we know about.
  */
-#define CLASS_SLIDER "Scale"
+#define CLASS_SLIDER "Slider"
 #define CLASS_TOGGLE "Toggle"
 #define CLASS_IMAGE "Image"
 #define CLASS_COLOUR "Colour"
-#define CLASS_NUMBER "Number"
-#define CLASS_STRING "String"
+#define CLASS_REAL "Real"
 #define CLASS_OPTION "Option"
 #define CLASS_MATRIX "Matrix_vips"
 #define CLASS_ARROW "Arrow"
@@ -81,43 +69,23 @@
 #define CLASS_AREA "Area"
 #define CLASS_HGUIDE "HGuide"
 #define CLASS_VGUIDE "VGuide"
-#define CLASS_MARK "Mark"
 #define CLASS_POINT "Point"
-#define CLASS_PATHNAME "Pathname"
-#define CLASS_FONTNAME "Fontname"
+#define CLASS_FILENAME "Filename"
 #define CLASS_SEPARATOR "Separator"
-#define CLASS_GROUP "Group"
-#define CLASS_LIST "List"
-#define CLASS_MENU "Menu"
-#define CLASS_MENUITEM "Menuitem"
-#define CLASS_MENUACTION "Menuaction"
-#define CLASS_MENUPULLRIGHT "Menupullright"
-#define CLASS_MENUSEPARATOR "Menuseparator"
-#define CLASS_EXPRESSION "Expression"
-#define CLASS_CLOCK "Clock"
-#define CLASS_REAL "Real"
-#define CLASS_VECTOR "Vector"
-#define CLASS_PLOT "Plot"
 
-/* What we loop over a class instance with.
- */
-typedef void *(*class_map_fn)( Symbol *, PElement *, void *, void * );
-
-Compile *class_get_compile( PElement *instance );
 gboolean class_get_super( PElement *instance, PElement *out );
-void *class_map( PElement *instance, class_map_fn fn, void *a, void *b );
-gboolean class_get_member( PElement *instance, const char *name, 
-	Symbol **sym_out, PElement *value );
+gboolean class_get_member( PElement *class, const char *name, PElement *out );
 gboolean class_get_symbol( PElement *class, Symbol *sym, PElement *out );
 gboolean class_get_exact( PElement *instance, const char *name, PElement *out );
 
-gboolean class_new_super( Heap *heap, 
+gboolean
+class_new_super( Heap *hi, 
 	Compile *compile, PElement *this, PElement *instance );
-gboolean class_new( Heap *heap, 
+gboolean class_new( Heap *hi, 
 	Compile *compile, HeapNode **args, PElement *out );
-gboolean class_clone( Heap *heap, PElement *class, PElement *out );
-gboolean class_clone_args( Heap *heap, PElement *klass, PElement *out );
-gboolean class_newv( Heap *heap, const char *name, PElement *out, ... );
+gboolean class_clone( Heap *hi, PElement *class, PElement *out );
+gboolean class_clone_args( Heap *hi, PElement *klass, PElement *out );
+gboolean class_newv( Heap *hi, const char *name, PElement *out, ... );
 
 gboolean class_get_member_real( PElement *klass, const char *name, 
 	double *out );
