@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -28,16 +28,12 @@
  */
 
 #define TYPE_COLOUR (colour_get_type())
-#define COLOUR( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_COLOUR, Colour ))
+#define COLOUR( obj ) (GTK_CHECK_CAST( (obj), TYPE_COLOUR, Colour ))
 #define COLOUR_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_COLOUR, ColourClass))
-#define IS_COLOUR( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_COLOUR ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_COLOUR, ColourClass ))
+#define IS_COLOUR( obj ) (GTK_CHECK_TYPE( (obj), TYPE_COLOUR ))
 #define IS_COLOUR_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_COLOUR ))
-#define COLOUR_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_COLOUR, ColourClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_COLOUR ))
 
 struct _Colour {
 	Classmodel parent_class;
@@ -46,10 +42,15 @@ struct _Colour {
 	 */
 	double value[3];
 	char *colour_space;
+	int type;			/* colour_space as a VIPS Type */
+
+	Imageinfo *ii;			/* Colour as a 1 pixel image */
+	guint ii_area_changed_sid;	/* Listen for edits */
+	guint ii_destroy_sid;	
 
 	/* Build view caption here.
 	 */
-	VipsBuf caption;
+	BufInfo caption;
 };
 
 typedef struct _ColourClass {
@@ -59,7 +60,6 @@ typedef struct _ColourClass {
 	 */
 } ColourClass;
 
-Imageinfo *colour_ii_new( Colour *colour );
-void colour_set_rgb( Colour *colour, double rgb[3] );
+GtkType colour_get_type( void );
 
-GType colour_get_type( void );
+Classmodel *colour_new( Rhs *rhs );
