@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,18 +29,15 @@
 
 #define TYPE_COLUMN (column_get_type())
 #define COLUMN( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_COLUMN, Column ))
+	(GTK_CHECK_CAST( (obj), TYPE_COLUMN, Column ))
 #define COLUMN_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_COLUMN, ColumnClass))
-#define IS_COLUMN( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_COLUMN ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_COLUMN, ColumnClass ))
+#define IS_COLUMN( obj ) (GTK_CHECK_TYPE( (obj), TYPE_COLUMN ))
 #define IS_COLUMN_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_COLUMN ))
-#define COLUMN_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_COLUMN, ColumnClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_COLUMN ))
 
 struct _Column {
-	Filemodel parent_object;
+	Filemodel parent_class;
 
 	/* Our context.
 	 */
@@ -49,19 +46,15 @@ struct _Column {
 
         /* Appearance state info.
          */
-        int x, y;		/* Position */
-        gboolean open;		/* Currently popped down */
-        gboolean selected;
+        int x, y;			/* Position */
+        gboolean open;                  /* Currently popped down */
+        gboolean selected;              /* Currently selected */
+        gboolean sform;                 /* In show-formula mode */
 
 	/* Other state.
 	 */
-        int next;		/* Index of next symbol we make */
-        Row *last_select;	/* Last row clicked ... for x sel */
-
-	/* A pending scrollto.
-	 */
-	guint scrollto_timeout;
-	ModelScrollPosition pending_position;
+        int next;                       /* Index of next symbol we make */
+        Row *last_select;               /* Last row clicked ... for x sel */
 };
 
 typedef struct _ColumnClass {
@@ -80,18 +73,10 @@ GtkType column_get_type( void );
 
 Column *column_new( Workspace *ws, const char *name );
 
-Column *column_get_last_new( void );
-void column_clear_last_new( void );
-
 Row *column_get_bottom( Column *col );
-gboolean column_add_n_names( Column *col, 
-	const char *name, VipsBuf *buf, int nparam );
-gboolean column_is_empty( Column *col );
+gboolean column_add_n_names( Column *col, BufInfo *buf, int nparam );
+gboolean column_isempty( Column *col );
 
 void column_set_offset( int x_off, int y_off );
 
 char *column_name_new( Column *col );
-
-void column_set_open( Column *col, gboolean open );
-
-void column_scrollto( Column *col, ModelScrollPosition position );
