@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -38,7 +38,10 @@
 struct _Conversionview {
 	GtkFrame parent_class;
 
-	Imagemodel *imagemodel;
+	Conversion *conv;
+
+	guint changed_sid;
+	guint destroy_sid;
 
 	Tslider *scale;
 	Tslider *offset;
@@ -55,4 +58,4 @@ typedef struct _ConversionviewClass {
 } ConversionviewClass;
 
 GtkType conversionview_get_type( void );
-Conversionview *conversionview_new( Imagemodel *imagemodel );
+Conversionview *conversionview_new( Conversion *conv );
