@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,11 +29,6 @@
 
 #include "ip.h"
 
-/* A lot of this file is just for debugging. Uncomment to enable all the
- * debugging code.
-#define DEBUG
- */
-
 /* Dump a binary operator.
  */
 char *
@@ -64,13 +59,11 @@ decode_BinOp( BinOp op )
 	case BI_PNOTEQ:		return( "!==" );
 	case BI_LESS:		return( "<" );
 	case BI_LESSEQ:		return( "<=" );
-	case BI_MORE:		return( ">" );
-	case BI_MOREEQ:		return( ">=" );
 	case BI_IF:		return( "if_then_else" );
 	case BI_CONS:		return( ":" );
 
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -101,7 +94,7 @@ decode_UnOp( UnOp op )
 	case UN_PLUS:		return( "+" );
 
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -122,12 +115,12 @@ decode_NodeType( NodeType tag )
 	case TAG_COMPLEX:	return( "TAG_COMPLEX" );
 	case TAG_CLASS:		return( "TAG_CLASS" );
 	case TAG_GEN:		return( "TAG_GEN" );
-	case TAG_FILE:		return( "TAG_FILE" );
 	case TAG_SHARED:	return( "TAG_SHARED" );
 	case TAG_REFERENCE:	return( "TAG_REFERENCE" );
+	case TAG_DOT:		return( "TAG_DOT" );
 
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -148,7 +141,7 @@ decode_CombinatorType( CombinatorType comb )
 	case COMB_K:		return( "K" );
 	case COMB_GEN:		return( "G" );
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -159,65 +152,86 @@ decode_CombinatorType( CombinatorType comb )
 /* Decode a symbol type.
  */
 char *
-decode_SymbolType( SymbolType t )
+decode_SymType( SymType t )
 {	
 	switch( t ) {
 	case SYM_VALUE:		return( "SYM_VALUE" );
 	case SYM_PARAM:		return( "SYM_PARAM" );
 	case SYM_ZOMBIE:	return( "SYM_ZOMBIE" );
 	case SYM_WORKSPACE:	return( "SYM_WORKSPACE" );
-	case SYM_WORKSPACEROOT:	return( "SYM_WORKSPACEROOT" );
+	case SYM_WORKSPACEGROUP:return( "SYM_WORKSPACEGROUP" );
 	case SYM_ROOT:		return( "SYM_ROOT" );
 	case SYM_EXTERNAL:	return( "SYM_EXTERNAL" );
 	case SYM_BUILTIN:	return( "SYM_BUILTIN" );
 
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 		return( NULL );
 	}
 }
 
-/* Decode a symbol type into something users might like to see.
- */
-char *
-decode_SymbolType_user( SymbolType t )
-{	
-	switch( t ) {
-	case SYM_VALUE:		return( _( "value" ) );
-	case SYM_PARAM:		return( _( "parameter" ) );
-	case SYM_ZOMBIE:	return( _( "zombie" ) );
-	case SYM_WORKSPACE:	return( _( "workspace" ) );
-	case SYM_WORKSPACEROOT:	return( _( "workspace root" ) );
-	case SYM_ROOT:		return( _( "root symbol" ) );
-	case SYM_EXTERNAL:	return( _( "external symbol" ) );
-	case SYM_BUILTIN:	return( _( "built-in symbol" ) );
+void *
+dump_stable( SymTable *tab )
+{
+	stable_name_print( tab );
+	printf( "\n" );
 
-	default:
-		g_assert( FALSE );
-		return( NULL );
-	}
+	printf( "%d on traverse list\n", g_slist_length( tab->traverse ) );
+	printf( "tab->size = %d\n", tab->size );
+	printf( "tab->sofar = %d\n", tab->sofar );
+
+	printf( "symbols: " );
+	(void) stable_map( tab,
+		(symbol_map_fn) dump_tiny, NULL, NULL, NULL );
+	printf( "\n" );
+
+	return( NULL );
 }
 
-/* From here on, just used for debugging.
- */
-#ifdef DEBUG
-
-#warning "DEBUG on in dump.c"
-
 /* Do a tiny dump of a symbol. Just a few characters.
  */
 void *
 dump_tiny( Symbol *sym )
 {
-	char txt[100];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
+	BufInfo buf;
+	char str[ 100 ];
 
-	printf( "(%p) ", sym );
+	buf_init_static( &buf, str, 100 );
 	symbol_qualified_name( sym, &buf );
 	if( sym->dirty )
 		printf( "*" );
-	printf( "%s %s; ", 
-		decode_SymbolType( sym->type ), vips_buf_all( &buf ) );
+	printf( "%s %s; ", decode_SymType( sym->type ), buf_all( &buf ) );
+
+	return( NULL );
+}
+
+/* Dump a heapinfo.
+ */
+void *
+dump_heapinfo( Heap *heap )
+{
+	printf( "heap = 0x%x\n", (unsigned int) heap );
+	printf( "heap->name = %s\n", NN( heap->name ) );
+	printf( "heap->expr->sym = " );
+	if( heap->compile )
+		if( heap->compile->sym )
+			printf( "(0x%x) (sym)\n", 
+				(unsigned int) heap->compile->sym ); 
+		else
+			printf( "(compile, but no sym)\n" );
+	else
+		printf( "(no associated symbol)\n" );
+	printf( "heap->mxb (max blocks) = %d\n", heap->mxb );
+	printf( "heap->rsz (nodes per block) = %d\n", heap->rsz );
+	printf( "heap->nb (number of blocks) = %d\n", heap->nb );
+	printf( "heap->emark = %d pointers\n", 
+		g_hash_table_size( heap->emark ) );
+	printf( "heap->rmark = %d pointers\n", 
+		g_slist_length( heap->rmark ) );
+	printf( "heap->ncells (cells allocated) = %d\n", heap->ncells );
+	printf( "heap->nfree (cells free at last GC) = %d\n", heap->nfree );
+	printf( "heap->itable (Imageinfo blocks) = %d pointers\n", 
+		g_slist_length( heap->itable ) );
 
 	return( NULL );
 }
@@ -241,32 +255,22 @@ dump_expr( Expr *expr )
 {
 	Symbol *sym = expr->sym;
 
-	printf( "expr (%p)->sym->name = \"%s\"\n", 
-		expr, IOBJECT( sym )->name );
-	if( expr->row )
-		printf( "%s->row = (set)\n", IOBJECT( sym )->name ); 
+	printf( "expr (0x%x)->sym->name = \"%s\"\n", 
+		(unsigned int) expr, MODEL( sym )->name );
+	printf( "%s->row = %s\n", MODEL( sym )->name, 
+		expr->row?"(set)":"(not set)" );
 
-	if( expr->compile ) {
-		printf( "%s->compile:\n", IOBJECT( sym )->name );
-		dump_compile( expr->compile );
-	}
-
-	if( sym->dirty ) 
-		printf( "<symbol is dirty ... can't print root>\n" );
-	else if( !PEISNOVAL( &expr->root ) ) {
-		printf( "%s->expr->root = ", IOBJECT( sym )->name );
+	printf( "%s->expr->root = ", MODEL( sym )->name );
+	if( !sym->dirty )
 		pgraph( &expr->root );
-	}
+	else
+		printf( "<symbol is dirty ... can't print root>" );
+	printf( "\n" );
 
-	if( expr->err )
-		printf( "%s->expr->err = %s\n", 
-			IOBJECT( sym )->name, bool_to_char( expr->err ) );
-	if( expr->error_top )
-		printf( "%s->expr->error_top = \"%s\"\n", 
-			IOBJECT( sym )->name, NN( expr->error_top ) );
-	if( expr->error_sub )
-		printf( "%s->expr->error_sub = \"%s\"\n", 
-			IOBJECT( sym )->name, NN( expr->error_sub ) );
+	printf( "%s->expr->err = %s\n", 
+		MODEL( sym )->name, bool_to_char( expr->err ) );
+	printf( "%s->expr->errstr = \"%s\"\n", 
+		MODEL( sym )->name, NN( expr->errstr ) );
 }
 
 /* Dump a compile, tiny.
@@ -287,78 +291,68 @@ void
 dump_compile( Compile *compile )
 {
 	Symbol *sym = compile->sym;
+	BufInfo buf;
+	char str[ 100 ];
 
-	printf( "compile (%p)->sym->name = \"%s\"\n", 
-		compile, IOBJECT( sym )->name );
+	printf( "compile (0x%x)->sym->name = \"%s\"\n", 
+		(unsigned int) compile, MODEL( sym )->name );
 
-#ifdef VERBOSE
 	printf( "%s->class = %s\n", 
-		IOBJECT( sym )->name, bool_to_char( compile->is_klass ) );
+		MODEL( sym )->name, bool_to_char( compile->is_klass ) );
 	printf( "%s->super = %s\n", 
-		IOBJECT( sym )->name, bool_to_char( compile->has_super ) );
+		MODEL( sym )->name, bool_to_char( compile->has_super ) );
 
 	printf( "%s->compile->text = \"%s\"\n", 
-		IOBJECT( sym )->name, NN( compile->text ) );
+		MODEL( sym )->name, NN( compile->text ) );
 	printf( "%s->compile->prhstext = \"%s\"\n", 
-		IOBJECT( sym )->name, NN( compile->prhstext ) );
+		MODEL( sym )->name, NN( compile->prhstext ) );
 	printf( "%s->compile->rhstext = \"%s\"\n", 
-		IOBJECT( sym )->name, NN( compile->rhstext ) );
-#endif /*VERBOSE*/
+		MODEL( sym )->name, NN( compile->rhstext ) );
 
-	if( compile->tree ) {
-		printf( "%s->compile->tree = \n", IOBJECT( sym )->name ); 
+	printf( "%s->compile->tree = \n", MODEL( sym )->name ); 
+	if( compile->tree ) 
 		(void) dump_tree( compile->tree ); 
-	}
-#ifdef VERBOSE
-	printf( "%s->compile->treefrag = %d pointers\n", IOBJECT( sym )->name,
+	else
+		printf( "   (no compile tree)\n" );
+	printf( "%s->compile->treefrag = %d pointers\n", MODEL( sym )->name,
 		g_slist_length( compile->treefrag ) );
-#endif /*VERBOSE*/
-
-	if( icontainer_get_n_children( ICONTAINER( compile ) ) > 0 ) {
-		printf( "%s->compile->children =\n", 
-			IOBJECT( sym )->name );
-		(void) icontainer_map( ICONTAINER( compile ), 
-			(icontainer_map_fn) dump_symbol, NULL, NULL );
-	}
-
-#ifdef VERBOSE
-{
-	char txt[100];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
 
+	printf( "%s->compile->locals = 0x%x:\n", 
+		MODEL( sym )->name, (unsigned int) compile->locals );
+	(void) stable_map( compile->locals, 
+		(symbol_map_fn) dump_symbol, NULL, NULL, NULL );
 	printf( "%s->compile->nparam = %d\n", 
-		IOBJECT( sym )->name, compile->nparam );
-	printf( "%s->compile->param = ", IOBJECT( sym )->name );
+		MODEL( sym )->name, compile->nparam );
+	printf( "%s->compile->param = ", MODEL( sym )->name );
 	(void) slist_map( compile->param, (SListMapFn) dump_tiny, NULL );
 	printf( "\n" );
 	printf( "%s->compile->nsecret = %d\n", 
-		IOBJECT( sym )->name, compile->nsecret );
-	printf( "%s->compile->secret = ", IOBJECT( sym )->name );
+		MODEL( sym )->name, compile->nsecret );
+	printf( "%s->compile->secret = ", MODEL( sym )->name );
 	(void) slist_map( compile->secret, (SListMapFn) dump_tiny, NULL );
 	printf( "\n" );
-	printf( "%s->compile->this = ", IOBJECT( sym )->name );
+	printf( "%s->compile->this = ", MODEL( sym )->name );
 	if( compile->this )
 		dump_tiny( compile->this );
 	else
 		printf( "(null)" );
 	printf( "\n" );
-	printf( "%s->compile->super = ", IOBJECT( sym )->name );
+	printf( "%s->compile->super = ", MODEL( sym )->name );
 	if( compile->super )
 		dump_tiny( compile->super );
 	else
 		printf( "(null)" );
 	printf( "\n" );
-	printf( "%s->compile->children = ", IOBJECT( sym )->name );
+	printf( "%s->compile->children = ", MODEL( sym )->name );
 	(void) slist_map( compile->children, (SListMapFn) dump_tiny, NULL );
 	printf( "\n" );
 
+	buf_init_static( &buf, str, 80 );
 	graph_element( compile->heap, &buf, &compile->base, FALSE );
 	printf( "%s->compile->base = %s\n", 
-		IOBJECT( sym )->name, vips_buf_all( &buf ) );
+		MODEL( sym )->name, buf_all( &buf ) );
 	if( compile->heap )
-		iobject_dump( IOBJECT( compile->heap ) );
-}
-#endif /*VERBOSE*/
+		dump_heapinfo( compile->heap );
 }
 
 /* Print a full symbol and all it's children. 
@@ -369,17 +363,21 @@ dump_symbol( Symbol *sym )
 	printf( "\n\nsym->name = " );
 	(void) dump_tiny( sym );
 	printf( "\n" );
+	printf( "sym->tab = table " );
+	stable_name_print( sym->tab );
+	printf( "\n" );
 
-#ifdef VERBOSE
-	printf( "%s->patch = %d pointers\n", IOBJECT( sym )->name,
+	printf( "%s->dest = %d\n", MODEL( sym )->name, sym->dest );
+
+	printf( "%s->patch = %d pointers\n", MODEL( sym )->name,
 		g_slist_length( sym->patch ) );
-#endif /*VERBOSE*/
 
 	if( sym->expr ) 
 		dump_expr( sym->expr );
+	else
+		printf( "%s->expr = <no expr info>\n", MODEL( sym )->name );
 
-#ifdef VERBOSE
-	printf( "%s->base = ", IOBJECT( sym )->name );
+	printf( "%s->base = ", MODEL( sym )->name );
 	if( !sym->dirty ) {
 		PElement root;
 
@@ -391,8 +389,8 @@ dump_symbol( Symbol *sym )
 	printf( "\n" );
 
 	printf( "%s->dirty = %s\n", 
-		IOBJECT( sym )->name, bool_to_char( sym->dirty ) );
-	printf( "%s->parents = ", IOBJECT( sym )->name );
+		MODEL( sym )->name, bool_to_char( sym->dirty ) );
+	printf( "%s->parents = ", MODEL( sym )->name );
 	(void) slist_map( sym->parents, (SListMapFn) dump_compile_tiny, NULL );
 	printf( "\n" );
 
@@ -400,16 +398,15 @@ dump_symbol( Symbol *sym )
 	 */
 	dump_links( sym );
 	printf( "%s->ndirtychildren = %d\n", 
-		IOBJECT( sym )->name, sym->ndirtychildren );
+		MODEL( sym )->name, sym->ndirtychildren );
 	printf( "%s->leaf = %s\n", 
-		IOBJECT( sym )->name, bool_to_char( sym->leaf ) );
+		MODEL( sym )->name, bool_to_char( sym->leaf ) );
 
-	printf( "%s->tool = kit ", IOBJECT( sym )->name );
+	printf( "%s->tool = kit ", MODEL( sym )->name );
 	if( sym->tool )
 		dump_kit( sym->tool->kit );
 	else
 		printf( "<NULL>\n" );
-#endif /*VERBOSE*/
 
 	return( NULL );
 }
@@ -419,8 +416,8 @@ dump_symbol( Symbol *sym )
 void
 dump_symbol_table( void )
 {	
-	(void) icontainer_map( ICONTAINER( symbol_root->expr->compile ), 
-		(icontainer_map_fn) dump_symbol, NULL, NULL );
+	(void) stable_map( symbol_root->expr->compile->locals, 
+		(symbol_map_fn) dump_symbol, NULL, NULL, NULL );
 }
 
 /* Tiny dump a tool.
@@ -442,7 +439,7 @@ dump_tiny_tool( Tool *tool )
 		break;
 
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 
 	return( NULL );
@@ -453,10 +450,10 @@ dump_tiny_tool( Tool *tool )
 void *
 dump_kit( Toolkit *kit )
 {
-	printf( "kit->name = %s; ", IOBJECT( kit )->name );
-	printf( "%s->tools = ", IOBJECT( kit )->name );
-	icontainer_map( ICONTAINER( kit ), 
-		(icontainer_map_fn) dump_tiny_tool, NULL, NULL );
+	printf( "kit->name = %s; ", MODEL( kit )->name );
+	printf( "%s->tools = ", MODEL( kit )->name );
+	model_map( MODEL( kit ), 
+		(model_map_fn) dump_tiny_tool, NULL, NULL );
 	printf( "\n" );
 
 	return( NULL );
@@ -467,7 +464,7 @@ dump_kit( Toolkit *kit )
 Symbol *
 sym( char *name )
 {	
-	return( compile_lookup( symbol_root->expr->compile, name ) );
+	return( stable_find( symbol_root->expr->compile->locals, name ) );
 }
 
 /* Print from a name.
@@ -488,11 +485,12 @@ psym( char *name )
 void 
 pgraph( PElement *graph )
 {	
+	BufInfo buf;
 	char txt[10240];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
 
-	graph_pelement( reduce_context->heap, &buf, graph, TRUE );
-	printf( "%s\n", vips_buf_all( &buf ) );
+	buf_init_static( &buf, txt, 10240 );
+	graph_pelement( reduce_context->hi, &buf, graph, TRUE );
+	printf( "%s\n", buf_all( &buf ) );
 }
 
 /* Print symbol value from name.
@@ -500,15 +498,15 @@ pgraph( PElement *graph )
 void 
 psymv( char *name )
 {	
-	Symbol *s;
+	BufInfo buf;
+	char txt[1024];
+	Symbol *s = sym( name );
 
-	if( (s = sym( name )) ) {
-		char txt[1024];
-		VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-		graph_pelement( reduce_context->heap, 
+	if( s ) {
+		buf_init_static( &buf, txt, 1024 );
+		graph_pelement( reduce_context->hi, 
 			&buf, &s->expr->root, TRUE );
-		printf( "%s = %s\n", name, vips_buf_all( &buf ) );
+		printf( "%s = %s\n", name, buf_all( &buf ) );
 	}
 }
 
@@ -517,9 +515,11 @@ psymv( char *name )
 static void
 print_element( int nsp, EType type, void *arg )
 {
+	Imageinfo *ii;
+
 	switch( type ) {
 	case ELEMENT_NOVAL:
-		printf( "no value (%d)\n", GPOINTER_TO_INT( arg ) );
+		printf( "no value\n" );
 		break;
 
 	case ELEMENT_NODE:
@@ -528,11 +528,11 @@ print_element( int nsp, EType type, void *arg )
 		break;
 
 	case ELEMENT_SYMBOL:
-		printf( "symbol \"%s\"", IOBJECT( arg )->name );
+		printf( "symbol \"%s\"", MODEL( arg )->name );
 		break;
 
 	case ELEMENT_SYMREF:
-		printf( "symref \"%s\"", IOBJECT( arg )->name );
+		printf( "symref \"%s\"", MODEL( arg )->name );
 		break;
 
 	case ELEMENT_COMPILEREF:
@@ -541,16 +541,15 @@ print_element( int nsp, EType type, void *arg )
 		break;
 
 	case ELEMENT_CONSTRUCTOR:
-		printf( "constructor \"%s\"", IOBJECT( arg )->name );
+		printf( "constructor \"%s\"", MODEL( arg )->name );
 		break;
 
 	case ELEMENT_CHAR:
-		printf( "char \"%c\"", GPOINTER_TO_UINT( arg ) );
+		printf( "char \"%c\"", (int)arg );
 		break;
 
 	case ELEMENT_BOOL:
-		printf( "bool \"%s\"", bool_to_char(
-			(gboolean) GPOINTER_TO_UINT( arg ) ) );
+		printf( "bool \"%s\"", bool_to_char( (gboolean)arg ) );
 		break;
 
 	case ELEMENT_BINOP:
@@ -570,16 +569,21 @@ print_element( int nsp, EType type, void *arg )
 		printf( "tag \"%s\"", (char*)arg ); 
 		break;
 
-	case ELEMENT_MANAGED:
-		printf( "Managed* %p", arg ); 
-		break;
-
 	case ELEMENT_ELIST:
 		printf( "empty-list []" ); 
 		break;
 
+	case ELEMENT_IMAGE:
+		ii = (Imageinfo*)arg;
+
+		if( ii && ii->im )
+			printf( "image \"%s\"",	ii->im->filename ); 
+		else
+			printf( "no-image" ); 
+		break;
+
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 }
 
@@ -616,6 +620,7 @@ graph_heap( int nsp, HeapNode *hn )
 	switch( hn->type ) {
 	case TAG_APPL:
 	case TAG_CONS:
+	case TAG_DOT:
 		printf( "\n" );
 		printf( "%s", spc( nsp ) );
 		printf( "left:  " );
@@ -636,7 +641,7 @@ graph_heap( int nsp, HeapNode *hn )
 
 	case TAG_CLASS:
 		printf( "instance-of-class \"%s\"\n",
-			IOBJECT( hn->body.ptrs.left )->name );
+			MODEL( hn->body.ptrs.left )->name );
 		printf( " secrets=(" );
 		print_element( nsp, 
 			GETRIGHT( hn )->ltype, 
@@ -660,17 +665,17 @@ graph_heap( int nsp, HeapNode *hn )
 			GETRIGHT( GETRIGHT( hn ) )->body.num );
 		break;
 
-	case TAG_FILE:
-		printf( "list generator file=%s\n", 
-			MANAGEDFILE( GETLEFT( hn ) )->file->fname );
-		break;
-
 	case TAG_FREE:
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 }
 
+void 
+graph_test( Heap *hi )
+{
+}
+
 /* Pretty-print a const.
  */
 static void
@@ -681,9 +686,6 @@ dump_parseconst( ParseConst *pc )
 		printf( "%G", pc->val.num );
 		break;
 
-	case PARSE_CONST_COMPLEX:
-		printf( "%Gj", pc->val.num );
-
 	case PARSE_CONST_STR:
 		printf( "\"%s\"", pc->val.str );
 		break;
@@ -701,7 +703,7 @@ dump_parseconst( ParseConst *pc )
 		break;
 
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 }
 
@@ -709,7 +711,7 @@ dump_parseconst( ParseConst *pc )
  */
 void *
 dump_tree( ParseNode *n )
-{
+{	
 	switch( n->type ) {
 	case NODE_NONE:
 		printf( "node->type == NODE_NONE\n" );
@@ -730,7 +732,7 @@ dump_tree( ParseNode *n )
 		break;
 
 	case NODE_LEAF:
-		printf( "Leaf symbol (%p): ", n->leaf );
+		printf( "Leaf symbol: " );
 		(void) dump_tiny( n->leaf );
 		printf( "\n" );
 		break;
@@ -740,7 +742,8 @@ dump_tree( ParseNode *n )
 		break;
 
 	case NODE_BINOP:
-		printf( "Binary operator %s\n", decode_BinOp( n->biop ) );
+		printf( "Binary operator %s\n", 
+			decode_BinOp( n->biop ) );
 		printf( "Left expression:\n" );
 		(void) dump_tree( n->arg1 );
 		printf( "Right expression:\n" );
@@ -748,7 +751,8 @@ dump_tree( ParseNode *n )
 		break;
 
 	case NODE_UOP:
-		printf( "Unary operator %s\n", decode_UnOp( n->uop ) );
+		printf( "Unary operator %s\n", 
+			decode_UnOp( n->uop ) );
 		printf( "Arg expression:\n" );
 		(void) dump_tree( n->arg1 );
 		break;
@@ -794,7 +798,7 @@ dump_tree( ParseNode *n )
 		break;
 
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 
 	return( NULL );
@@ -814,14 +818,10 @@ dump_link( Link *link )
 {
 	printf( "link->parent = " );
 	symbol_name_print( link->parent );
-	if( link->parent->dirty )
-		printf( "(dirty)" );
 	printf( "\n" );
 
 	printf( "link->child = " );
 	symbol_name_print( link->child );
-	if( link->child->dirty )
-		printf( "(dirty)" );
 	printf( "\n" );
 
 	printf( "link->serial = %d\n", link->serial );
@@ -853,10 +853,8 @@ dump_symbol_heap( Symbol *sym )
 {
 	printf( "symbol " );
 	symbol_name_print( sym );
-	printf( "has graph:\n" );
+	printf( " has graph:\n" );
 	if( sym->expr ) 
 		pgraph( &sym->expr->root );
 	printf( "\n" );
 }
-
-#endif /*DEBUG*/
