@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -31,13 +31,14 @@ char *decode_BinOp( BinOp op );
 char *decode_UnOp( UnOp op );
 char *decode_NodeType( NodeType tag );
 char *decode_CombinatorType( CombinatorType comb );
-char *decode_SymbolType( SymbolType t );
-char *decode_SymbolType_user( SymbolType t );
+char *decode_SymType( SymType t );
 
+void *dump_stable( SymTable *tab );
 void *dump_tiny( Symbol *sym );
 void *dump_symbol( Symbol *sym );
 void dump_expr( Expr *expr );
 void dump_compile( Compile *compile );
+void *dump_heapinfo( Heap *heap );
 void dump_symbol_table( void );
 void *dump_kit( Toolkit *kit );
 Symbol *sym( char *name );
@@ -46,7 +47,7 @@ void psymv( char *name );
 void pgraph( PElement *graph );
 
 void graph_heap( int nsp, HeapNode *hn );
-void graph_test( Heap *heap );
+void graph_test( Heap *hi );
 
 void *dump_tree( ParseNode *n );
 
