@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,6 +27,8 @@
 
  */
 
+#include "ip.h"
+
 /* Trace error_set()/_clear().
 #define DEBUG_ERROR
  */
@@ -39,50 +41,9 @@
 #define DEBUG
  */
 
-#include "ip.h"
-
-/* Our signals. 
- */
-enum {
-	SIG_NEW_VALUE,		/* new value for root */
-	SIG_LAST
-};
-
-static iContainerClass *parent_class = NULL;
-
-static guint expr_signals[SIG_LAST] = { 0 };
-
 /* Set of expressions containing errors.
  */
-GSList *expr_error_all = NULL;
-
-void *
-expr_error_print( Expr *expr, VipsBuf *buf )
-{
-	g_assert( expr->err );
-
-	vips_buf_appendf( buf, _( "error in \"%s\"" ), 
-		IOBJECT( expr->sym )->name );
-	if( expr->sym->tool ) 
-		tool_error( expr->sym->tool, buf );
-	else if( expr->row ) {
-		Workspace *ws = expr->row->ws;
-		Workspacegroup *wsg = workspace_get_workspacegroup( ws );
-
-		vips_buf_appendf( buf, " (" );
-		row_qualified_name( expr->row, buf );
-		if( FILEMODEL( wsg )->filename )
-			vips_buf_appendf( buf, " - %s", 
-				FILEMODEL( wsg )->filename );
-		vips_buf_appendf( buf, ")" );
-	}
-
-	/* Don't show error_top, it's just a summary of error_sub.
-	 */
-	vips_buf_appendf( buf, ": %s\n", expr->error_sub );
-
-	return( NULL );
-}
+GSList *error_list = NULL;
 
 static Expr *
 expr_map_all_sub( Symbol *sym, map_expr_fn fn, void *a )
@@ -107,10 +68,9 @@ expr_map_all( Expr *expr, map_expr_fn fn, void *a )
 
 	/* And over any locals.
 	 */
-	if( expr->compile && (res = (Expr *) 
-		icontainer_map( ICONTAINER( expr->compile ), 
-			(icontainer_map_fn) expr_map_all_sub, 
-			(void *) fn, a )) )
+	if( expr->compile &&
+		(res = (Expr *) stable_map( expr->compile->locals, 
+		(symbol_map_fn) expr_map_all_sub, (void *) fn, a, NULL )) )
 		return( res );
 
 	return( NULL );
@@ -119,8 +79,9 @@ expr_map_all( Expr *expr, map_expr_fn fn, void *a )
 void *
 expr_name_print( Expr *expr )
 {
-	printf( "expr(%p) ", expr );
+	printf( "expr(0x%x) ", (unsigned int) expr );
 	symbol_name_print( expr->sym );
+	printf( " " );
 
 	if( expr->row ) {
 		printf( "(row " );
@@ -131,15 +92,6 @@ expr_name_print( Expr *expr )
 	return( NULL );
 }
 
-void
-expr_name( Expr *expr, VipsBuf *buf )
-{
-	if( expr->row ) 
-		row_qualified_name( expr->row, buf );
-	else
-		symbol_qualified_name( expr->sym, buf );
-}
-
 Expr *
 expr_get_parent( Expr *expr )
 {
@@ -169,7 +121,7 @@ expr_get_parent_dynamic( Expr *expr )
 		 */
 		Workspace *ws = expr->row->top_col->ws;
 
-		return( ws->sym->expr );
+		return( SYMBOL( ws )->expr );
 	}
 }
 
@@ -194,63 +146,12 @@ expr_get_root_dynamic( Expr *expr )
 
 	if( is_top( expr->sym ) )
 		return( expr );
-	else if( expr->row && expr->row->top_row && expr->row->top_row->expr )
-		return( expr->row->top_row->expr );
 	else if( (parent = expr_get_parent_dynamic( expr )) )
 		return( expr_get_root_dynamic( parent ) );
 	else
 		return( NULL ); 
 }
 
-/* Is an expr part of a row, including enclosing exprs. 
- *
- * For example, row A1 could be "[x::x<-A2]", that would be expanded to 
- * something like 
- * "$lcomp0 {$lcomp0 = foldr $f0 [] A2 {$f0 x $sofar = x : $sofar}}"
- * Now, row A1 depends on A2, but expr A1 will not ... it's $lcomp0, the local
- * expr of A1, that will get called for expr_dirty.
- *
- * Return NULL for expr is not a row and has no enclosing rows.
- */
-static Row *
-expr_get_row( Expr *expr )
-{
-	if( expr->row )
-		return( expr->row );
-	else if( is_top( expr->sym ) )
-		return( NULL );
-	else
-		return( expr_get_row( expr_get_parent( expr ) ) );
-}
-
-void 
-expr_new_value( Expr *expr )
-{
-#ifdef DEBUG
-{
-	PElement *root = &expr->root;
-
-	printf( "expr_new_value: " );
-	symbol_name_print( expr->sym );
-	printf( ": " );
-	graph_pointer( root );
-}
-#endif /*DEBUG*/
-
-	g_signal_emit( G_OBJECT( expr ), expr_signals[SIG_NEW_VALUE], 0 );
-}
-
-/* An expr has lost a value.
- */
-void
-expr_value_destroy( Expr *expr )
-{
-	/* Break ImageInfo link (if any).
-	 */
-	if( expr->imageinfo )
-		imageinfo_expr_remove( expr, expr->imageinfo );
-}
-
 /* Clean up an expr, ready to have a new def parsed into it.
  */
 void *
@@ -266,13 +167,12 @@ expr_strip( Expr *expr )
 	if( slist_map( expr->dynamic_links, 
 		(SListMapFn) link_expr_destroy, NULL ) )
 		return( expr );
-	g_assert( !expr->static_links );
-	g_assert( !expr->dynamic_links );
+	assert( !expr->static_links );
+	assert( !expr->dynamic_links );
 
 	/* Junk error stuff. 
 	 */
-	IM_FREE( expr->error_top );
-	IM_FREE( expr->error_sub );
+	FREE( expr->errstr );
 
 	/* Unref the compile.
 	 */
@@ -282,14 +182,13 @@ expr_strip( Expr *expr )
 	return( NULL );
 }
 
-static void 
-expr_dispose( GObject *gobject )
+void *
+expr_destroy( Expr *expr )
 {
-	Expr *expr = EXPR( gobject );
 	Symbol *sym = expr->sym;
 
 #ifdef DEBUG
-	printf( "expr_dispose: " );
+	printf( "expr_destroy: " );
 	expr_name_print( expr );
 	printf( "\n" );
 #endif /*DEBUG*/
@@ -300,24 +199,35 @@ expr_dispose( GObject *gobject )
 	 */
 	expr_value_destroy( expr );
 
-	/* Unlink from symbol.
+	/* If it's a secondary expr, need to remove from sym.
 	 */
-	if( sym->expr == expr )
-		sym->expr = NULL;
+	if( g_slist_find( sym->ext_expr, expr ) )
+		 sym->ext_expr = g_slist_remove( sym->ext_expr, expr );
 
 	if( expr->row ) {
 		Row *row = expr->row;
 
 		/* If this is the sym for a top row, kill the row too.
 		 * Otherwise just break the link and wait for the next row
-		 * refresh to do the kill for us.
+		 * refresh to do the kkill for us.
 		 */
 		if( row == row->top_row ) {
-			IDESTROY( row );
+			/* Nasty: if row has us as an expr_private, we don't 
+			 * want row_destroy() to try to kill us in turn. 
+			 * Break the link now.
+			 */
+			if( row->expr_private == expr ) {
+				row->expr_private = NULL;
+				expr->row = NULL;
+			}
+
+			gtk_object_destroy( GTK_OBJECT( row ) );
 		}
 		else {
 			row->expr = NULL;
+			row->expr_private = NULL;
 			row->sym = NULL;
+			row->sym_private = NULL;
 
 			expr->row = NULL;
 
@@ -335,107 +245,9 @@ expr_dispose( GObject *gobject )
 		}
 	}
 
-	G_OBJECT_CLASS( parent_class )->dispose( gobject );
-}
+	FREE( expr );
 
-static void
-expr_info( iObject *iobject, VipsBuf *buf )
-{
-	Expr *expr = EXPR( iobject );
-
-	if( expr->err ) {
-		vips_buf_appends( buf, _( "Error" ) );
-		vips_buf_appendf( buf, ": %s\n%s\n", 
-			expr->error_top, expr->error_sub );
-	}
-}
-
-static void
-expr_real_new_value( Expr *expr )
-{
-	PElement *root = &expr->root;
-
-	expr_value_destroy( expr );
-	if( PEISIMAGE( root ) && PEGETII( root ) ) 
-		imageinfo_expr_add( PEGETII( root ), expr );
-
-	/* If this is the main expr for this symbol, signal new value there
-	 * too.
-	 */
-	if( expr->sym->expr == expr )
-		symbol_new_value( expr->sym );
-}
-
-static void
-expr_class_init( ExprClass *class )
-{
-	GObjectClass *gobject_class = (GObjectClass *) class;
-	iObjectClass *iobject_class = (iObjectClass *) class;
-
-	parent_class = g_type_class_peek_parent( class );
-
-	/* Create signals.
-	 */
-	expr_signals[SIG_NEW_VALUE] = g_signal_new( "new_value",
-		G_OBJECT_CLASS_TYPE( gobject_class ),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET( ExprClass, new_value ),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE, 0 );
-
-	/* Init methods.
-	 */
-	gobject_class->dispose = expr_dispose;
-
-	iobject_class->info = expr_info;
-
-	class->new_value = expr_real_new_value;
-
-	/* Static init.
-	 */
-}
-
-static void
-expr_init( Expr *expr )
-{
-	expr->sym = NULL;
-	expr->row = NULL;
-	expr->compile = NULL;
-
-	expr->static_links = NULL;
-	expr->dynamic_links = NULL;
-
-	expr->imageinfo = NULL;
-
-	expr->err = FALSE;
-	expr->error_top = NULL;
-	expr->error_sub = NULL;
-}
-
-GType
-expr_get_type( void )
-{
-	static GType type = 0;
-
-	if( !type ) {
-		static const GTypeInfo info = {
-			sizeof( ExprClass ),
-			NULL,           /* base_init */
-			NULL,           /* base_finalize */
-			(GClassInitFunc) expr_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
-			sizeof( Expr ),
-			32,             /* n_preallocs */
-			(GInstanceInitFunc) expr_init,
-		};
-
-		type = g_type_register_static( TYPE_ICONTAINER, 
-			"Expr", &info, 0 );
-	}
-
-	return( type );
+	return( NULL );
 }
 
 Expr *
@@ -443,11 +255,12 @@ expr_new( Symbol *sym )
 {
 	Expr *expr;
 
-	expr = EXPR( g_object_new( TYPE_EXPR, NULL ) );
+	if( !(expr = IM_NEW( NULL, Expr )) )
+		return( NULL );
 
 	expr->sym = sym;
-	PEPOINTE( &expr->root, &sym->base );
-	icontainer_child_add( ICONTAINER( sym ), ICONTAINER( expr ), -1 );
+	expr->row = NULL;
+	expr->compile = NULL;
 
 #ifdef DEBUG
 	printf( "expr_new: " );
@@ -455,10 +268,19 @@ expr_new( Symbol *sym )
 	printf( "\n" );
 #endif /*DEBUG*/
 
+	expr->static_links = NULL;
+	expr->dynamic_links = NULL;
+
+	PEPOINTE( &expr->root, &sym->base );
+
+	expr->imageinfo = NULL;
+
+	expr->err = FALSE;
+	expr->errstr = NULL;
+
 	return( expr );
 }
 
-
 /* Clone an existing expr. 
  */
 Expr *
@@ -477,34 +299,33 @@ expr_clone( Symbol *sym )
 		 * row, at the same scope level as sym.
 		 */
                 expr = expr_new( sym );
+		/*
+                (void) compile_new_local( expr );
+		 */
 	}
 
 	return( expr );
 }
 
-/* Mark an expression as containing an error, save the error buffers.
+/* Mark an expression as containing an error, save error_string.
  */
 void *
 expr_error_set( Expr *expr )
 {
+#ifdef DEBUG_ERROR
+	printf( "expr_error_set: error in " );
+	symbol_name_print( expr->sym );
+	printf( ": %s\n", error_string );
+#endif /*DEBUG_ERROR*/
+
+	/* Always update the error message.
+	 */
+	SETSTR( expr->errstr, error_string );
+
 	/* Was not in error? Add to error set.
 	 */
 	if( !expr->err ) {
-#ifdef DEBUG_ERROR
-		printf( "expr_error_set: error in " );
-		symbol_name_print( expr->sym );
-		printf( ": %s %s\n", error_get_top(), error_get_sub() );
-#endif /*DEBUG_ERROR*/
-
-		IM_SETSTR( expr->error_top, error_get_top() );
-		IM_SETSTR( expr->error_sub, error_get_sub() );
-
-		/* Zap the value of the expr ... it may contain pointers to
-		 * dead stuff.
-		 */
-		PEPUTP( &expr->root, ELEMENT_NOVAL, (void *) 99 );
-
-		expr_error_all = g_slist_prepend( expr_error_all, expr );
+		error_list = g_slist_prepend( error_list, expr );
 		expr->err = TRUE;
 		if( expr->row )
 			row_error_set( expr->row );
@@ -525,13 +346,10 @@ void
 expr_error_get( Expr *expr )
 {
 	if( !expr->err )
-		error_clear();
+		ierrors( "" );
 	else {
-		g_assert( expr->error_top );
-		g_assert( expr->error_sub );
-
-		error_top( "%s", expr->error_top );
-		error_sub( "%s", expr->error_sub );
+		assert( expr->errstr );
+		ierrors( expr->errstr );
 	}
 }
 
@@ -548,7 +366,7 @@ expr_error_clear( Expr *expr )
 #endif /*DEBUG_ERROR*/
 
 		expr->err = FALSE;
-		expr_error_all = g_slist_remove( expr_error_all, expr );
+		error_list = g_slist_remove( error_list, expr );
 		if( expr->row )
 			row_error_clear( expr->row );
 
@@ -559,7 +377,7 @@ expr_error_clear( Expr *expr )
 
 /* Mark an expr dirty.
  *
- * Two cases: if expr has a row, this is part of a display. Use the row
+ * Two cases: if expr->row, this is part of a display. Use the row
  * stuff to mark this expr dirty. Then use symbol_dirty() to mark on from the
  * root of this row.
  *
@@ -572,19 +390,10 @@ expr_error_clear( Expr *expr )
 void *
 expr_dirty( Expr *expr, int serial )
 {
-	Row *row;
+	if( expr->row ) {
+		Symbol *top_sym = expr->row->top_row->sym;
 
-#ifdef DEBUG
-	printf( "expr_dirty: " );
-	symbol_name_print( expr->sym );
-	printf( "\n" );
-#endif /*DEBUG*/
-
-	if( (row = expr_get_row( expr )) &&
-		row->top_row->sym ) {
-		Symbol *top_sym = row->top_row->sym;
-
-		row_dirty( row, TRUE );
+		row_dirty( expr->row );
 		symbol_dirty( top_sym, serial );
 	}
 	else
@@ -596,9 +405,8 @@ expr_dirty( Expr *expr, int serial )
 void *
 expr_dirty_intrans( Expr *expr, int serial )
 {
-	if( expr->row &&
-		expr->row->top_row->sym ) {
-		row_dirty_intrans( expr->row, TRUE );
+	if( expr->row ) {
+		row_dirty_intrans( expr->row );
 		symbol_dirty( expr->row->top_row->sym, serial );
 	}
 	else
@@ -608,55 +416,39 @@ expr_dirty_intrans( Expr *expr, int serial )
 }
 
 void
-expr_tip_sub( Expr *expr, VipsBuf *buf )
+expr_tip_sub( Expr *expr, BufInfo *buf )
 {
 	Compile *compile = expr->compile;
 
-	if( is_top( expr->sym ) ) {
-		vips_buf_appends( buf, _( "top level" ) );
-		vips_buf_appends( buf, " " );
-	}
+	if( is_top( expr->sym ) ) 
+		buf_appends( buf, "top level " );
 
-	if( compile && 
-		is_class( compile ) ) {
-		vips_buf_appends( buf, _( "class" ) );
-		vips_buf_appends( buf, " " );
-		if( compile->nparam == 0 ) {
-			vips_buf_appends( buf, _( "instance" ) );
-			vips_buf_appends( buf, " " );
-		}
-		else {
-			vips_buf_appends( buf, _( "definition" ) );
-			vips_buf_appends( buf, " " );
-		}
+	if( compile && is_class( compile ) ) {
+		buf_appends( buf, "class " );
+		if( compile->nparam == 0 )
+			buf_appends( buf, "instance " );
+		else
+			buf_appends( buf, "definition " );
 
-		vips_buf_appendf( buf, "\"%s\"", IOBJECT( expr->sym )->name );
+		buf_appendf( buf, "\"%s\"", MODEL( expr->sym )->name );
 	}
 	else if( expr->sym->type == SYM_PARAM )
-		vips_buf_appendf( buf, _( "parameter \"%s\"" ), 
-			IOBJECT( expr->sym )->name );
-	else if( compile ) {
-		if( is_member( expr->sym ) ) {
-			vips_buf_appends( buf, _( "member" ) );
-			vips_buf_appends( buf, " " );
-		}
+		buf_appendf( buf, "parameter \"%s\"", 
+			MODEL( expr->sym )->name );
+	else {
+		if( is_member( expr->sym ) ) 
+			buf_appends( buf, "member " );
 
-		if( compile->nparam == 0 ) {
-			vips_buf_appends( buf, _( "value" ) );
-			vips_buf_appends( buf, " " );
-		}
-		else {
-			vips_buf_appends( buf, _( "function" ) );
-			vips_buf_appends( buf, " " );
-		}
+		if( compile->nparam == 0 )
+			buf_appends( buf, "value " );
+		else
+			buf_appends( buf, "function " );
 
-		vips_buf_appendf( buf, "\"%s\"", IOBJECT( expr->sym )->name );
+		buf_appendf( buf, "\"%s\"", MODEL( expr->sym )->name );
 	}
 
 	if( !is_top( expr->sym ) ) {
-		vips_buf_appends( buf, " " );
-		vips_buf_appends( buf, _( "of" ) );
-		vips_buf_appends( buf, " " );
+		buf_appends( buf, " of " );
 		expr_tip_sub( expr_get_parent( expr ), buf );
 	}
 }
@@ -664,13 +456,47 @@ expr_tip_sub( Expr *expr, VipsBuf *buf )
 /* Look at an expr, make a tooltip.
  */
 void
-expr_tip( Expr *expr, VipsBuf *buf )
+expr_tip( Expr *expr, BufInfo *buf )
 {
-	expr_name( expr, buf ); 
-	vips_buf_appends( buf, ": " );
+	if( expr->row )
+		row_qualified_name( expr->row, buf );
+	else
+		symbol_qualified_name( expr->sym, buf );
+	buf_appends( buf, ": " );
+
 	expr_tip_sub( expr, buf );
 }
 
+/* An expr has lost a value.
+ */
+void
+expr_value_destroy( Expr *expr )
+{
+	/* Break ImageInfo link (if any).
+	 */
+	if( expr->imageinfo )
+		imageinfo_expr_remove( expr, expr->imageinfo );
+}
+
+/* An expr has a new value.
+ */
+void
+expr_value_new( Expr *expr )
+{
+	PElement *root = &expr->root;
+
+#ifdef DEBUG
+	printf( "expr_value_new: " );
+	symbol_name_print( expr->sym );
+	printf( ": " );
+	pgraph( root );
+#endif /*DEBUG*/
+
+	expr_value_destroy( expr );
+	if( PEISIMAGE( root ) && PEGETII( root ) ) 
+		imageinfo_expr_add( PEGETII( root ), expr );
+}
+
 /* Bind unresolved refs in an expr. Bind for every enclosing dynamic scope.
  */
 void
@@ -689,5 +515,6 @@ expr_resolve( Expr *expr )
 		/* May try to resolve out through a parameter.
 		 */
 		if( i->compile )
-			compile_resolve_dynamic( expr->compile, i->compile );
+			stable_resolve_dynamic( expr->compile->locals, 
+				i->compile->locals );
 }
