@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,27 +27,14 @@
 
  */
 
-#define TYPE_EXPR (expr_get_type())
-#define EXPR( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_EXPR, Expr ))
-#define EXPR_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_EXPR, ExprClass))
-#define IS_EXPR( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_EXPR ))
-#define IS_EXPR_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_EXPR ))
-#define EXPR_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_EXPR, ExprClass ))
+/* List of expr containing errors.
+ */
+extern GSList *error_list;
 
 /* What we track to parse and compile some text. Can have several 
  * of these for a symbol, all with different values.
  */
 struct _Expr {
-	/* We don't contain anything, but we are contained by Symbol, so we
-	 * need to be an iContainer subclass.
-	 */
-	iContainer parent_object;
-
 	Symbol *sym;		/* We are an expr for this symbol, scopewise */
 	Row *row;		/* (optional) we have this display */
 
@@ -64,39 +51,21 @@ struct _Expr {
 	Imageinfo *imageinfo;
 
 	gboolean err;		/* TRUE if there is an error in this expr */
-	char *error_top;
-	char *error_sub;
+	char *errstr;		/* Saved error string for this expr */
 };
 
-typedef struct _ExprClass {
-	iContainerClass parent_class;
-
-	/* 
-
-		new_value	expr has been recalced and root points to a
-				new piece of graph
-
-	 */
-
-	void (*new_value)( Expr *expr );
-} ExprClass;
-
-extern GSList *expr_error_all;
-
-void *expr_error_print( Expr *expr, VipsBuf *buf );
-
 typedef void *(*map_expr_fn)( Expr *, void *, void * );
 Expr *expr_map_all( Expr *expr, map_expr_fn fn, void *a );
 
 void *expr_name_print( Expr *expr );
-void expr_name( Expr *expr, VipsBuf *buf );
 
 Expr *expr_get_parent( Expr *expr );
 Expr *expr_get_root( Expr *expr );
 Expr *expr_get_root_dynamic( Expr *expr );
 
-GType expr_get_type( void );
+void *expr_destroy( Expr *expr );
 void *expr_strip( Expr *expr );
+
 Expr *expr_new( Symbol *sym );
 Expr *expr_clone( Symbol *sym );
 
@@ -111,8 +80,9 @@ void *expr_link_break( Expr *expr, Symbol *child );
 void *expr_dirty( Expr *expr, int serial );
 void *expr_dirty_intrans( Expr *expr, int serial );
 
-void expr_tip( Expr *expr, VipsBuf *buf );
+void expr_tip( Expr *expr, BufInfo *buf );
 
-void expr_new_value( Expr *expr );
+void expr_value_new( Expr *expr );
+void expr_value_destroy( Expr *expr );
 
 void expr_resolve( Expr *expr );
