@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -30,16 +30,12 @@
 #define FILEMODEL_LOAD_STATE( obj ) ((FilemodelLoadState *) obj)
 
 #define TYPE_FILEMODEL (filemodel_get_type())
-#define FILEMODEL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_FILEMODEL, Filemodel ))
+#define FILEMODEL( obj ) (GTK_CHECK_CAST( (obj), TYPE_FILEMODEL, Filemodel ))
 #define FILEMODEL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_FILEMODEL, FilemodelClass))
-#define IS_FILEMODEL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_FILEMODEL ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_FILEMODEL, FilemodelClass ))
+#define IS_FILEMODEL( obj ) (GTK_CHECK_TYPE( (obj), TYPE_FILEMODEL ))
 #define IS_FILEMODEL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_FILEMODEL ))
-#define FILEMODEL_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_FILEMODEL, FilemodelClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_FILEMODEL ))
 
 struct _Filemodel {
 	Model model;
@@ -50,16 +46,6 @@ struct _Filemodel {
 	gboolean auto_load;	/* TRUE if loaded from path_start */
 
 	int x_off, y_off;	/* Save offset for things below this */
-
-	/* When we loaded this filemodel, the version numbers we saw in the
-	 * XML file.
-	 */
-	gboolean versioned;	/* Set means from a versioned file */
-	int major;
-	int minor;
-	int micro;
-
-	iWindow *window_hint;	/* Our views set this as a hint */
 };
 
 typedef struct _FilemodelClass {
@@ -70,49 +56,34 @@ typedef struct _FilemodelClass {
 		top_load	top level load function ... controls how the
 				rest of the load happens ... eg. merge,
 				rename, etc.
-
-		set_modified	set/clear the modified state
-
-		top_save	top level save ... intercept this to override
-
+	
 	 */
 
 	gboolean (*top_load)( Filemodel *filemodel, 
 		ModelLoadState *state, Model *parent, xmlNode *xnode );
-	void (*set_modified)( Filemodel *filemodel, gboolean modified );
-	gboolean (*top_save)( Filemodel *filemodel, const char *filename );
 
-	FileselFileType **filetype;
-	const char *filetype_pref;
+	GtkFileSelection2FileType **filetype;
+	int default_filetype;
 } FilemodelClass;
 
 void filemodel_register( Filemodel *filemodel );
-void filemodel_unregister( Filemodel *filemodel );
 
 void *filemodel_top_load( Filemodel *filemodel, 
 	ModelLoadState *state, Model *parent, xmlNode *xnode );
 
 void filemodel_set_filename( Filemodel *filemodel, const char *filename );
 void filemodel_set_modified( Filemodel *filemodel, gboolean state );
-void filemodel_set_window_hint( Filemodel *filemodel, iWindow *iwnd );
-iWindow *filemodel_get_window_hint( Filemodel *filemodel );
 
-GType filemodel_get_type( void );
+GtkType filemodel_get_type( void );
 
 void filemodel_set_offset( Filemodel *filemodel, int x_off, int y_off );
-gboolean filemodel_top_save( Filemodel *filemodel, const char *filename );
-gboolean filemodel_load_all( Filemodel *filemodel, Model *parent, 
-	const char *filename, const char *filename_user );
-gboolean filemodel_load_all_openfile( Filemodel *filemodel, 
-	Model *parent, iOpenFile *of );
+gboolean filemodel_save_all( Filemodel *filemodel, const char *filename );
+gboolean filemodel_load_all( Filemodel *filemodel, 
+	Model *parent, const char *filename );
 
 void filemodel_inter_saveas( iWindow *parent, Filemodel *filemodel );
 void filemodel_inter_save( iWindow *parent, Filemodel *filemodel );
-void filemodel_inter_savenempty_cb( iWindow *iwnd, void *client, 
-	iWindowNotifyFn nfn, void *sys );
 void filemodel_inter_savenempty( iWindow *parent, Filemodel *filemodel );
-void filemodel_inter_savenclose_cb( iWindow *iwnd, void *client, 
-	iWindowNotifyFn nfn, void *sys );
 void filemodel_inter_savenclose( iWindow *parent, Filemodel *filemodel );
 void filemodel_inter_loadas( iWindow *parent, Filemodel *filemodel );
 void filemodel_inter_replace( iWindow *parent, Filemodel *filemodel );
