@@ -35,22 +35,17 @@
 
 /* Fwd ref.
  */
-static void lisp_pelement( VipsBuf *buf, PElement *base, 
+static void lisp_pelement( BufInfo *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent );
 
 /* Print a sym-value list.
  */
 static void
-lisp_symval( VipsBuf *buf, PElement *base, 
-	GSList **back, gboolean fn, int indent, PElement *stop )
+lisp_symval( BufInfo *buf, PElement *base, 
+	GSList **back, gboolean fn, int indent )
 {
 	gboolean error = FALSE;
 
-	/* Reached the "stop" element?
-	 */
-	if( stop && *base->type == *stop->type && *base->ele == *stop->ele )
-		return;
-
 	if( PEISNODE( base ) ) {
 		HeapNode *hn = PEGETVAL( base );
 		PElement pe;
@@ -67,17 +62,17 @@ lisp_symval( VipsBuf *buf, PElement *base,
 
 			PEPOINTLEFT( hn2, &pe );
 			if( !error && PEISSYMREF( &pe ) ) {
-				vips_buf_appendf( buf, "\n%s", spc( indent ) );
+				buf_appendf( buf, "\n%s", spc( indent ) );
 				symbol_qualified_name( 
 					PEGETSYMREF( &pe ), buf );
-				vips_buf_appendf( buf, " = " );
+				buf_appendf( buf, " = " );
 
 				PEPOINTRIGHT( hn2, &pe );
 				lisp_pelement( buf, &pe, 
 					back, fn, indent + TAB );
 
 				PEPOINTRIGHT( hn, &pe );
-				lisp_symval( buf, &pe, back, fn, indent, stop );
+				lisp_symval( buf, &pe, back, fn, indent );
 			}
 			else
 				error = TRUE;
@@ -89,7 +84,7 @@ lisp_symval( VipsBuf *buf, PElement *base,
 		error = TRUE;
 
 	if( error )
-		vips_buf_appendf( buf, "\n%s<*** malformed symval list>", 
+		buf_appendf( buf, "\n%s<*** malformed symval list>", 
 			spc( indent ) );
 }
 
@@ -97,14 +92,14 @@ lisp_symval( VipsBuf *buf, PElement *base,
  * element, so we print a ", " followed by us.
  */
 static void
-lisp_list( VipsBuf *buf, PElement *base, 
+lisp_list( BufInfo *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
 	if( PEISNODE( base ) ) {
 		HeapNode *hn = PEGETVAL( base );
 		PElement pe;
 
-		vips_buf_appends( buf, ", " );
+		buf_appends( buf, ", " );
 
 		if( hn->type == TAG_CONS ) {
 			PEPOINTLEFT( hn, &pe );
@@ -116,20 +111,15 @@ lisp_list( VipsBuf *buf, PElement *base,
 		else
 			lisp_pelement( buf, base, back, fn, indent );
 	}
-	else if( PEISMANAGEDSTRING( base ) ) {
-		vips_buf_appends( buf, ", Managedstring <" );
-		vips_buf_appends( buf, PEGETMANAGEDSTRING( base )->string );
-		vips_buf_appends( buf, ">" );
-	}
 	else if( !PEISELIST( base ) ) 
 		lisp_pelement( buf, base, back, fn, indent );
 }
 
 /* Print a [char] ... fall back to lisp_list() if we hit a non-char
- * element. base is the RHS of a cons, so it can be a managedstring too.
+ * element.
  */
 static void
-lisp_string( VipsBuf *buf, PElement *base, 
+lisp_string( BufInfo *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
 	gboolean error = FALSE;
@@ -144,18 +134,18 @@ lisp_string( VipsBuf *buf, PElement *base,
 		PEPOINTLEFT( hn, &pe );
 		if( !error ) {
 			if( PEISCHAR( &pe ) ) {
-				vips_buf_appendf( buf, "%c", PEGETCHAR( &pe ) );
+				buf_appendf( buf, "%c", PEGETCHAR( &pe ) );
 
 				PEPOINTRIGHT( hn, &pe );
 				lisp_string( buf, &pe, back, fn, indent );
 			}
 			else {
-				vips_buf_appends( buf, "\":[" );
+				buf_appends( buf, "\":[" );
 				lisp_pelement( buf, &pe, back, fn, indent );
 
 				PEPOINTRIGHT( hn, &pe );
 				lisp_list( buf, &pe, back, fn, indent );
-				vips_buf_appends( buf, "]" );
+				buf_appends( buf, "]" );
 
 				error = TRUE;
 			}
@@ -163,8 +153,6 @@ lisp_string( VipsBuf *buf, PElement *base,
 		else
 			error = TRUE;
 	}
-	else if( PEISMANAGEDSTRING( base ) ) 
-		vips_buf_appends( buf, PEGETMANAGEDSTRING( base )->string );
 	else if( !PEISELIST( base ) ) 
 		error = TRUE;
 }
@@ -172,7 +160,7 @@ lisp_string( VipsBuf *buf, PElement *base,
 /* Print a graph LISP-style.
  */
 static void
-lisp_node( VipsBuf *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
+lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 {
 	int i;
 	PElement p1, p2;
@@ -182,76 +170,65 @@ lisp_node( VipsBuf *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 	if( hn->flgs & FLAG_PRINT ) {
 		if( (i = g_slist_index( *back, hn )) == -1 ) {
 			*back = g_slist_prepend( *back, hn );
-			vips_buf_appendf( buf, "<" ); 
-			vips_buf_appendf( buf, _( "circular" ) ); 
-			vips_buf_appendf( buf, " (%p)>", hn );
-		}
-		else {
-			vips_buf_appendf( buf, "<" );
-			vips_buf_appendf( buf, _( "circular to label %d" ), i );
-			vips_buf_appendf( buf, ">" );
+			buf_appendf( buf, "<circular (0x%x)>", 
+				(unsigned int) hn );
 		}
+		else 
+			buf_appendf( buf, "<circular to label %d>", i );
 
 		return;
 	}
 	hn->flgs |= FLAG_PRINT;
 
-	if( (i = g_slist_index( *back, hn )) != -1 ) {
-		vips_buf_appendf( buf, "*" );
-		vips_buf_appendf( buf, _( "label %d" ), i );
-		vips_buf_appendf( buf, ": " );
-	}
+	if( (i = g_slist_index( *back, hn )) != -1 ) 
+		buf_appendf( buf, "*label %d: ", i );
 
 	switch( hn->type ) {
 	case TAG_APPL:
 		if( fn ) {
 			PEPOINTLEFT( hn, &p1 );
 			PEPOINTRIGHT( hn, &p2 );
-			vips_buf_appends( buf, "(" );
+			buf_appends( buf, "(" );
 			lisp_pelement( buf, &p1, back, fn, indent );
-			vips_buf_appends( buf, " " );
+			buf_appends( buf, " " );
 			lisp_pelement( buf, &p2, back, fn, indent );
-			vips_buf_appends( buf, ")" );
-		}
-		else {
-			vips_buf_appends( buf, "<" );
-			vips_buf_appends( buf, _( "unevaluated" ) );
-			vips_buf_appends( buf, ">" );
+			buf_appends( buf, ")" );
 		}
+		else
+			buf_appends( buf, "<unevaluated>" );
 
 		break;
 
 	case TAG_CONS:
 		PEPOINTLEFT( hn, &p1 );
 		if( PEISCHAR( &p1 ) ) {
-			vips_buf_appendf( buf, "\"%c", PEGETCHAR( &p1 ) );
+			buf_appendf( buf, "\"%c", PEGETCHAR( &p1 ) );
 			PEPOINTRIGHT( hn, &p2 );
 			lisp_string( buf, &p2, back, fn, indent );
-			vips_buf_appends( buf, "\"" );
+			buf_appends( buf, "\"" );
 		}
 		else {
-			vips_buf_appends( buf, "[" );
+			buf_appends( buf, "[" );
 			lisp_pelement( buf, &p1, back, fn, indent );
 			PEPOINTRIGHT( hn, &p2 );
 			lisp_list( buf, &p2, back, fn, indent );
-			vips_buf_appends( buf, "]" );
+			buf_appends( buf, "]" );
 		}
 		break;
 
 	case TAG_DOUBLE:
-		vips_buf_appendf( buf, "%g", hn->body.num );
+		buf_appendf( buf, "%g", hn->body.num );
 		break;
 
 	case TAG_COMPLEX:
-		vips_buf_appendf( buf, "(%g,%g)", 
+		buf_appendf( buf, "(%g,%g)", 
 			GETLEFT( hn )->body.num, GETRIGHT( hn )->body.num );
 		break;
 
 	case TAG_CLASS:
 		if( fn ) {
-			vips_buf_appendf( buf, "\n%s", spc( indent ) );
-			vips_buf_appendf( buf, _( "class (%p)" ), hn );
-			vips_buf_appendf( buf, " " );
+			buf_appendf( buf, "\n%sclass (0x%x) ", 
+				spc( indent ), (unsigned int) hn );
 		}
 
 		PEPOINTLEFT( hn, &p1 );
@@ -260,157 +237,149 @@ lisp_node( VipsBuf *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 		if( fn ) {
 			hn = GETRIGHT( hn );
 
-			vips_buf_appendf( buf, "\n%s", spc( indent + TAB ) ); 
-			vips_buf_appendf( buf, _( "members" ) );
-			vips_buf_appendf( buf, " = { " );
+			buf_appendf( buf, 
+				"\n%smembers = { ", spc( indent + TAB ) ); 
 			PEPOINTRIGHT( hn, &p1 );
-			lisp_symval( buf, &p1, 
-				back, fn, indent + TAB * 2, NULL );
-			vips_buf_appendf( buf, "\n%s}", spc( indent + TAB ) ); 
+			lisp_symval( buf, &p1, back, fn, indent + TAB * 2 );
+			buf_appendf( buf, "\n%s}", spc( indent + TAB ) ); 
 
 			PEPOINTLEFT( hn, &p2 );
 			if( *p1.type != *p2.type || *p1.ele != *p2.ele ) { 
-				vips_buf_appendf( buf, "\n%s", 
-					spc( indent + TAB ) );
-				vips_buf_appendf( buf, _( "secret" ) );
-				vips_buf_appendf( buf, " = { " );
-				lisp_symval( buf, &p2, 
-					back, fn, indent + TAB * 2, &p1 );
-				vips_buf_appendf( buf, 
+				buf_appendf( buf, "\n%ssecret = { ", 
+					spc( indent + TAB ) ); 
+				lisp_symval( buf, 
+					&p2, back, fn, indent + TAB * 2 );
+				buf_appendf( buf, 
 					"\n%s} ", spc( indent + TAB ) ); 
 			}
 		}
 
 		break;
 
+	case TAG_DOT:
+		PEPOINTLEFT( hn, &p1 );
+		PEPOINTRIGHT( hn, &p2 );
+		buf_appends( buf, ".(" );
+		lisp_pelement( buf, &p1, back, fn, indent );
+		buf_appends( buf, " from " );
+		lisp_pelement( buf, &p2, back, fn, indent );
+		buf_appends( buf, ")" );
+		break;
+
 	case TAG_GEN:
-		vips_buf_appendf( buf, "[%g,%g...", 
+		buf_appendf( buf, "[%g,%g...", 
 			GETLEFT( hn )->body.num, 
 			GETLEFT( GETRIGHT( hn ) )->body.num ); 
 		if( GETRT( GETRIGHT( hn ) ) == ELEMENT_ELIST )
-			vips_buf_appends( buf, "[ ]]" );
+			buf_appends( buf, "[ ]]" );
 		else
-			vips_buf_appendf( buf, "%g]",
+			buf_appendf( buf, "%g]",
 				GETRIGHT( GETRIGHT( hn ) )->body.num ); 
 		break;
 
 	case TAG_SHARED:
 		PEPOINTLEFT( hn, &p1 );
-		i = GPOINTER_TO_INT( GETRIGHT( hn ) );
-		vips_buf_appendf( buf, "SHARE%d[", i );
+		i = (int) GETRIGHT( hn );
+		buf_appendf( buf, "SHARE%d[", i );
 		lisp_pelement( buf, &p1, back, fn, indent );
-		vips_buf_appends( buf, "]" );
+		buf_appends( buf, "]" );
 		break;
 
 	case TAG_REFERENCE:
-		i = GPOINTER_TO_INT( GETRIGHT( GETLEFT( hn ) ) );
-		vips_buf_appendf( buf, "REF%d", i );
+		i = (int) GETRIGHT( GETLEFT( hn ) );
+		buf_appendf( buf, "REF%d", i );
 		break;
 
 	case TAG_FREE:
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 }
 
 /* Print a pelement LISP-style.
  */
 static void
-lisp_pelement( VipsBuf *buf, PElement *base, 
+lisp_pelement( BufInfo *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
+	Imageinfo *ii;
 	HeapNode *hn;
 	EType type = PEGETTYPE( base );
 
 	switch( type ) {
 	case ELEMENT_NOVAL:
-		vips_buf_appends( buf, "<" );
-		vips_buf_appendf( buf, _( "no value (type %d)" ), 
-			GPOINTER_TO_INT( PEGETVAL( base ) ) );
-		vips_buf_appends( buf, ">" );
+		buf_appends( buf, "<no-value>" );
 		break;
 
 	case ELEMENT_NODE:
-		if( !(hn = PEGETVAL( base )) ) {
-			vips_buf_appends( buf, "<" );
-			vips_buf_appends( buf, _( "NULL pointer" ) );
-			vips_buf_appends( buf, ">" );
-		}
+		if( !(hn = PEGETVAL( base )) )
+			buf_appends( buf, "<NULL pointer>" );
 		else
 			lisp_node( buf, hn, back, fn, indent );
 		break;
 
 	case ELEMENT_SYMBOL:
-		vips_buf_appends( buf, "<" ); 
-		vips_buf_appends( buf, _( "symbol" ) ); 
-		vips_buf_appends( buf, " \"" ); 
+		buf_appends( buf, "<symbol \"" ); 
 		symbol_qualified_name( PEGETSYMBOL( base ), buf );
-		vips_buf_appends( buf, "\">" ); 
+		buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_CONSTRUCTOR:
-		vips_buf_appends( buf, "<" ); 
-		vips_buf_appends( buf, _( "constructor" ) ); 
-		vips_buf_appends( buf, " \"" ); 
+		buf_appends( buf, "<constructor \"" ); 
 		symbol_qualified_name( PEGETCOMPILE( base )->sym, buf );
-		vips_buf_appends( buf, "\">" ); 
+		buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_SYMREF:
-		vips_buf_appends( buf, "<" ); 
-		vips_buf_appends( buf, _( "symref" ) ); 
-		vips_buf_appends( buf, " \"" ); 
+		buf_appends( buf, "<symref \"" ); 
 		symbol_qualified_name( PEGETSYMBOL( base ), buf );
-		vips_buf_appends( buf, "\">" ); 
+		buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_COMPILEREF:
-		vips_buf_appends( buf, "<" ); 
-		vips_buf_appends( buf, _( "compileref" ) ); 
-		vips_buf_appends( buf, " \"" ); 
-		symbol_qualified_name( PEGETCOMPILE( base )->sym, buf );
-		vips_buf_appends( buf, "\">" ); 
+		buf_appends( buf, "<compileref \"" ); 
+		symbol_qualified_name( PEGETCOMPILE( base )->sym , buf );
+		buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_CHAR:
-		vips_buf_appendf( buf, "'%c'", (int) PEGETCHAR( base ) );
+		buf_appendf( buf, "'%c'", (int)PEGETCHAR( base ) );
 		break;
 
 	case ELEMENT_BOOL:
-		vips_buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
+		buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
 		break;
 
 	case ELEMENT_BINOP:
-		vips_buf_appends( buf, decode_BinOp( PEGETBINOP( base ) ) );
+		buf_appends( buf, decode_BinOp( PEGETBINOP( base ) ) );
 		break;
 
 	case ELEMENT_UNOP:
-		vips_buf_appends( buf, decode_UnOp( PEGETUNOP( base ) ) );
+		buf_appends( buf, decode_UnOp( PEGETUNOP( base ) ) );
+		break;
+
+	case ELEMENT_IMAGE:
+		if( (ii = PEGETII( base )) ) 
+			buf_appendf( buf, "<image \"%s\">", ii->im->filename ); 
+		else
+			buf_appends( buf, "<no-image>" ); 
+
 		break;
 
 	case ELEMENT_ELIST:
-		vips_buf_appends( buf, "[ ]" ); 
+		buf_appends( buf, "[ ]" ); 
 		break;
 
 	case ELEMENT_TAG:
-		vips_buf_appendf( buf, "<" );
-		vips_buf_appendf( buf, _( "tag \"%s\"" ), PEGETTAG( base ) );
-		vips_buf_appendf( buf, ">" );
-		break;
-
-	case ELEMENT_MANAGED:
-		vips_buf_appendf( buf, "<Managed* %p>", PEGETVAL( base ) );
+		buf_appendf( buf, "<tag \"%s\">", PEGETTAG( base ) );
 		break;
 
 	case ELEMENT_COMB:
-		vips_buf_appends( buf, 
-			decode_CombinatorType( PEGETCOMB( base ) ) );
+		buf_appends( buf, decode_CombinatorType( PEGETCOMB( base ) ) );
 		break;
 
 	default:
-		vips_buf_appendf( buf, "<" );
-		vips_buf_appendf( buf, _( "unknown element tag %d" ), type );
-		vips_buf_appendf( buf, ">" );
+		buf_appendf( buf, "<unknown element tag %d>", type );
 		break;
 	}
 }
@@ -418,82 +387,74 @@ lisp_pelement( VipsBuf *buf, PElement *base,
 /* Print a node to a buffer. If fn is true, trace into functions.
  */
 void
-graph_node( Heap *heap, VipsBuf *buf, HeapNode *root, gboolean fn )
+graph_node( Heap *hi, BufInfo *buf, HeapNode *root, gboolean fn )
 {
 	GSList *back;
+	BufInfo buf2;
 	char txt[4];
-	VipsBuf buf2 = VIPS_BUF_STATIC( txt );
 
 	/* May be called before heap is built.
 	 */
-	if( !heap )
+	if( !hi )
 		return;
 
 	back = NULL;
-	heap_clear( heap, FLAG_PRINT );
+	buf_init_static( &buf2, txt, 4 );
+	heap_clear( hi, FLAG_PRINT );
 	lisp_node( &buf2, root, &back, fn, 0 );
-	heap_clear( heap, FLAG_PRINT );
+	heap_clear( hi, FLAG_PRINT );
 	lisp_node( buf, root, &back, fn, 0 );
-	IM_FREEF( g_slist_free, back );
+	FREEF( g_slist_free, back );
 }
 
 /* As above, but start from a pelement.
  */
 void
-graph_pelement( Heap *heap, VipsBuf *buf, PElement *root, gboolean fn )
+graph_pelement( Heap *hi, BufInfo *buf, PElement *root, gboolean fn )
 {
 	GSList *back;
+	BufInfo buf2;
 	char txt[4];
-	VipsBuf buf2 = VIPS_BUF_STATIC( txt );
 
 	/* May be called before heap is built.
 	 */
-	if( !heap )
+	if( !hi )
 		return;
 
 	/* We print twice ... the first time through we build the list of back
 	 * pointers so we can label the graph correctly.
 	 */
+	buf_init_static( &buf2, txt, 4 );
 	back = NULL;
 
-	heap_clear( heap, FLAG_PRINT );
+	heap_clear( hi, FLAG_PRINT );
 	lisp_pelement( &buf2, root, &back, fn, 0 );
 
-	heap_clear( heap, FLAG_PRINT );
+	heap_clear( hi, FLAG_PRINT );
 	lisp_pelement( buf, root, &back, fn, 0 );
 
-	IM_FREEF( g_slist_free, back );
+	FREEF( g_slist_free, back );
 }
 
 /* As above, but start from an element.
  */
 void
-graph_element( Heap *heap, VipsBuf *buf, Element *root, gboolean fn )
+graph_element( Heap *hi, BufInfo *buf, Element *root, gboolean fn )
 {
 	PElement base;
 
 	PEPOINTE( &base, root );
-	graph_pelement( heap, buf, &base, fn );
-}
-
-void
-graph_pointer( PElement *root )
-{
-	char txt[1000];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-	graph_pelement( reduce_context->heap, &buf, root, TRUE );
-	printf( "%s\n", vips_buf_all( &buf ) );
+	graph_pelement( hi, buf, &base, fn );
 }
 
 /* Fwd ref.
  */
-static void shell_pelement( PElement *base );
+static void csh_pelement( PElement *base );
 
-/* Print a graph shell-style.
+/* Print a graph csh-style.
  */
 static void
-shell_node( HeapNode *hn )
+csh_node( HeapNode *hn )
 {
 	PElement p1, p2;
 
@@ -511,39 +472,22 @@ shell_node( HeapNode *hn )
 	case TAG_REFERENCE:
 	case TAG_SHARED:
 	case TAG_GEN:
+	case TAG_DOT:
 		break;
 		
 	case TAG_CONS:
-{
-		gboolean string_mode;
-
-		PEPOINTLEFT( hn, &p1 );
-		string_mode = PEISCHAR( &p1 );
-
 		for(;;) {
-			if( string_mode )
-				printf( "%c", PEGETCHAR( &p1 ) );
-			else
-				shell_pelement( &p1 );
+			PEPOINTLEFT( hn, &p1 );
+			csh_pelement( &p1 );
 
 			PEPOINTRIGHT( hn, &p2 );
-			if( PEISMANAGEDSTRING( &p2 ) ) {
-				printf( "%s\n", 
-					PEGETMANAGEDSTRING( &p2 )->string );
+			if( PEISELIST( &p2 ) ) 
 				break;
-
-			}
-			else if( PEISELIST( &p2 ) ) 
-				break;
-
-			if( !string_mode )
+			else {
 				printf( "\n" );
-			hn = PEGETVAL( &p2 );
-			PEPOINTLEFT( hn, &p1 );
-			if( string_mode && !PEISCHAR( &p1 ) )
-				string_mode = FALSE;
+				hn = PEGETVAL( &p2 );
+			}
 		}
-}
 		break;
 
 	case TAG_DOUBLE:
@@ -557,15 +501,17 @@ shell_node( HeapNode *hn )
 
 	case TAG_FREE:
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 }
 
-/* Print a pelement shell-style.
+/* Print a pelement csh-style.
  */
 static void
-shell_pelement( PElement *base )
+csh_pelement( PElement *base )
 {
+	Imageinfo *ii;
+
 	switch( PEGETTYPE( base ) ) {
 	/* Only allow concrete base types.
 	 */
@@ -582,7 +528,7 @@ shell_pelement( PElement *base )
 		break;
 
 	case ELEMENT_NODE:
-		shell_node( PEGETVAL( base ) );
+		csh_node( PEGETVAL( base ) );
 		break;
 
 	case ELEMENT_CHAR:
@@ -593,23 +539,24 @@ shell_pelement( PElement *base )
 		printf( "%s", bool_to_char( PEGETBOOL( base ) ) );
 		break;
 
+	case ELEMENT_IMAGE:
+		if( (ii = PEGETII( base )) ) 
+			printf( "%s", imageinfo_get( FALSE, ii )->filename );
+		else
+			printf( "no-image" ); 
+
+		break;
+
 	case ELEMENT_ELIST:
 		printf( "[ ]" ); 
 		break;
 
-	case ELEMENT_MANAGED:
-		if( PEISIMAGE( base ) )
-			printf( "%s", PEGETIMAGE( base )->filename );
-		else if( PEISMANAGEDSTRING( base ) ) 
-			printf( "%s", PEGETMANAGEDSTRING( base )->string );
-		break;
-
 	default:
-		g_assert( FALSE );
+		assert( FALSE );
 	}
 }
 
-/* Print a pelement shell-style.
+/* Print a pelement csh-style.
  */
 void
 graph_value( PElement *root )
@@ -617,11 +564,87 @@ graph_value( PElement *root )
 	Reduce *rc = reduce_context;
 
 	if( !reduce_pelement( rc, reduce_spine_strict, root ) ) {
-		iwindow_alert( NULL, GTK_MESSAGE_ERROR );
+		box_alert( NULL );
 		return;
 	}
 
-	heap_clear( reduce_context->heap, FLAG_PRINT );
-	shell_pelement( root );
+	heap_clear( reduce_context->hi, FLAG_PRINT );
+	csh_pelement( root );
 	printf( "\n" );
 }
+
+/* Sub. fn of below.
+ */
+static void
+output_node( HeapNode *hn )
+{
+	PElement p1, p2;
+
+	switch( hn->type ) {
+	case TAG_CLASS:
+	case TAG_APPL:
+	case TAG_REFERENCE:
+	case TAG_SHARED:
+	case TAG_GEN:
+	case TAG_DOT:
+	case TAG_DOUBLE:
+	case TAG_COMPLEX:
+		break;
+		
+	case TAG_CONS:
+		PEPOINTLEFT( hn, &p1 );
+		set_output( &p1 );
+
+		PEPOINTRIGHT( hn, &p2 );
+		if( !PEISELIST( &p2 ) ) {
+			hn = PEGETVAL( &p2 );
+			output_node( hn );
+		}
+		break;
+
+	case TAG_FREE:
+	default:
+		assert( FALSE );
+	}
+}
+
+/* Force all images to files, set no-delete.
+ */
+void
+set_output( PElement *base )
+{
+	Imageinfo *ii;
+
+	switch( PEGETTYPE( base ) ) {
+	/* Only allow concrete base types.
+	 */
+	case ELEMENT_SYMREF:
+	case ELEMENT_COMPILEREF:
+	case ELEMENT_CONSTRUCTOR:
+	case ELEMENT_BINOP:
+	case ELEMENT_UNOP:
+	case ELEMENT_COMB:
+	case ELEMENT_TAG:
+	case ELEMENT_SYMBOL:
+	case ELEMENT_NOVAL:
+	case ELEMENT_CHAR:
+	case ELEMENT_BOOL:
+	case ELEMENT_ELIST:
+		break;
+
+	case ELEMENT_NODE:
+		output_node( PEGETVAL( base ) );
+		break;
+
+	case ELEMENT_IMAGE:
+		if( (ii = PEGETII( base )) ) {
+			if( imageinfo_file( ii ) ) 
+				error_exit( error_string );
+			ii->dfile = FALSE;
+		}
+		break;
+
+	default:
+		assert( FALSE );
+	}
+}
