@@ -27,10 +27,9 @@
 
 */
 
-void graph_node( Heap *heap, VipsBuf *buf, HeapNode *root, gboolean fn );
-void graph_pelement( Heap *heap, VipsBuf *buf, PElement *root, gboolean fn );
-void graph_element( Heap *heap, VipsBuf *buf, Element *root, gboolean fn );
-void graph_pointer( PElement *root );
+void graph_node( Heap *hi, BufInfo *buf, HeapNode *root, gboolean fn );
+void graph_pelement( Heap *hi, BufInfo *buf, PElement *root, gboolean fn );
+void graph_element( Heap *hi, BufInfo *buf, Element *root, gboolean fn );
 
 /* Reduce and print, csh-style output.
  */
@@ -38,4 +37,4 @@ void graph_value( PElement *root );
 
 /* Make sure all images are written to disc, and won't be deleted on exit.
  */
-void *save_objects( PElement *base, char *filename );
+void set_output( PElement *base );
