@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -41,7 +41,7 @@ typedef unsigned char NodeType;
 #define TAG_CLASS (8)		/* Class object */
 #define TAG_SHARED (9)		/* Root of a common sub-expression */
 #define TAG_REFERENCE (10)	/* Reference to a common sub-expression */
-#define TAG_FILE (12)		/* Generate list from file */
+#define TAG_DOT (11)		/* 'dot' member reference */
 
 /* Element types. Generally represent operators.
  */
@@ -57,7 +57,7 @@ typedef unsigned char EType;
 #define ELEMENT_UNOP (8)	/* Unary operator */
 #define ELEMENT_COMB (9)	/* Combinator */
 #define ELEMENT_TAG (10)	/* RHS of '.' operator */
-#define ELEMENT_MANAGED (11)	/* A managed object */
+#define ELEMENT_IMAGE (11)	/* A pointer into the image list */
 #define ELEMENT_CONSTRUCTOR (12)/* Class constructor */
 #define ELEMENT_ELIST (13)	/* Empty list */
 
@@ -92,14 +92,14 @@ typedef enum combinator_type {
 /* An element ... a tag plus a pointer. Use one of these to hold a pointer
  * into a heap.
  */
-typedef struct _Element {
+typedef struct element {
 	EType type;
 	void *ele;
 } Element;
 
 /* A node on the heap. Should fit in 12 bytes on most machines.
  */
-typedef struct _HeapNode {
+typedef struct heap_node {
 	/* Elements: either a pair of pointers, or a double. Sensible on most
 	 * 32-bit systems, not so great on 64 bitters.
 	 */
@@ -184,7 +184,7 @@ typedef struct pelement {
 }
 #define PEPUTP(PE,T,V) {\
 	EType t99 = (T);\
-	void *v99 = GUINT_TO_POINTER(V);\
+	void *v99 = (void*)((int)(V));\
 	\
 	*((PE)->type) = t99;\
 	*((PE)->ele) = v99;\
@@ -215,21 +215,13 @@ typedef struct pelement {
 #define PEISCHAR(P) (PEGETTYPE(P) == ELEMENT_CHAR)
 #define PEISCLASS(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_CLASS)
 #define PEISCONSTRUCTOR(P) (PEGETTYPE(P) == ELEMENT_CONSTRUCTOR)
+#define PEISDOT(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_DOT)
 #define PEISCOMB(P) (PEGETTYPE(P) == ELEMENT_COMB)
 #define PEISCOMPLEX(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_COMPLEX)
-#define PEISTAG(P) (PEGETTYPE(P) == ELEMENT_TAG)
-#define PEISMANAGED(P) (PEGETTYPE(P) == ELEMENT_MANAGED)
-#define PEISMANAGEDGOBJECT(P) (PEISMANAGED(P) && \
-	IS_MANAGEDGOBJECT( PEGETVAL(P) ))
-#define PEISMANAGEDSTRING(P) (PEISMANAGED(P) && \
-	IS_MANAGEDSTRING(PEGETVAL(P)))
-#define PEISIMAGE(P) (PEISMANAGED(P) && IS_IMAGEINFO( PEGETVAL(P) ))
-#define PEISVIPSOBJECT(P) \
-	(PEISMANAGEDGOBJECT(P) && VIPS_IS_OBJECT( PEGETMANAGEDGOBJECT(P) )) 
-#define PEISFILE(P) (PEISMANAGED(P) && IS_MANAGEDFILE(PEGETVAL(P)))
 #define PEISELIST(P) (PEGETTYPE(P) == ELEMENT_ELIST)
-#define PEISFLIST(P) ((PEISNODE(P) && PEGETVAL(P)->type == TAG_CONS) || \
-	PEISMANAGEDSTRING(P))
+#define PEISFLIST(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_CONS)
+#define PEISTAG(P) (PEGETTYPE(P) == ELEMENT_TAG)
+#define PEISIMAGE(P) (PEGETTYPE(P) == ELEMENT_IMAGE)
 #define PEISLIST(P) (PEISELIST(P) || PEISFLIST(P))
 #define PEISNOVAL(P) (PEGETTYPE(P) == ELEMENT_NOVAL)
 #define PEISNUM(P) (PEISREAL(P) || PEISCOMPLEX(P))
@@ -244,22 +236,16 @@ typedef struct pelement {
  */
 #define PEGETSYMBOL(P) ((Symbol*)PEGETVAL(P))
 #define PEGETSYMREF(P) ((Symbol*)PEGETVAL(P))
-#define PEGETCOMPILE(P) ((Compile*)(PEGETVAL(P)))
+#define PEGETCOMPILE(P) (COMPILE(PEGETVAL(P)))
 #define PEGETBINOP(P) ((BinOp)PEGETVAL(P))
 #define PEGETUNOP(P) ((UnOp)PEGETVAL(P))
 #define PEGETCOMB(P) ((CombinatorType)PEGETVAL(P))
 #define PEGETTAG(P) ((char*)PEGETVAL(P))
 #define PEGETREAL(P) (PEGETVAL(P)->body.num)
-#define PEGETBOOL(P) ((gboolean)GPOINTER_TO_UINT(PEGETVAL(P)))
-#define PEGETCHAR(P) ((unsigned char)(GPOINTER_TO_UINT(PEGETVAL(P))))
+#define PEGETBOOL(P) ((gboolean)PEGETVAL(P))
+#define PEGETCHAR(P) ((unsigned char)((int)PEGETVAL(P)))
 #define PEGETIMAGE(P) (((Imageinfo*)PEGETVAL(P))->im)
 #define PEGETII(P) ((Imageinfo*)PEGETVAL(P))
-#define PEGETFILE(P) ((Managedfile*)PEGETVAL(P))
-#define PEGETMANAGED(P) ((Managed*)PEGETVAL(P))
-#define PEGETMANAGEDSTRING(P) ((Managedstring*)PEGETVAL(P))
-#define PEGETMANAGEDGOBJECT(P) (((Managedgobject*)PEGETVAL(P))->object)
-#define PEGETVIPSOBJECT(P) \
-		((VipsObject*)(((Managedgobject*)PEGETVAL(P))->object))
 
 #define PEGETHD(P1,P2) PEPOINTLEFT(PEGETVAL(P2), P1)
 #define PEGETTL(P1,P2) PEPOINTRIGHT(PEGETVAL(P2), P1)
@@ -267,6 +253,9 @@ typedef struct pelement {
 #define PEGETREALPART(P) (GETLEFT(PEGETVAL(P))->body.num)
 #define PEGETIMAGPART(P) (GETRIGHT(PEGETVAL(P))->body.num)
 
+#define PEGETDOTCOMPILE(P) (COMPILE(GETLEFT(PEGETVAL(P))))
+#define PEGETDOTTAG(P) ((char*)GETRIGHT(PEGETVAL(P)))
+
 #define PEGETCLASSCOMPILE(P) (COMPILE(GETLEFT(PEGETVAL(P))))
 #define PEGETCLASSSECRET(P1,P2) PEPOINTLEFT(GETRIGHT(PEGETVAL(P2)),P1)
 #define PEGETCLASSMEMBER(P1,P2) PEPOINTRIGHT(GETRIGHT(PEGETVAL(P2)),P1)
@@ -274,7 +263,7 @@ typedef struct pelement {
 /* A block on the heap.
  */
 struct _HeapBlock {
-	Heap *heap;		/* Heap we are part of */
+	Heap *hi;		/* Heap we are part of */
 	HeapBlock *next;	/* Next block in chain */
 	HeapNode *node;		/* Nodes on this block */
 	int sz;			/* Number of nodes in this block */
@@ -284,129 +273,99 @@ struct _HeapBlock {
  */
 typedef int (*heap_max_fn)( Heap * );
 
-#define TYPE_HEAP (heap_get_type())
-#define HEAP( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_HEAP, Heap ))
-#define HEAP_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_HEAP, HeapClass))
-#define IS_HEAP( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_HEAP ))
-#define IS_HEAP_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_HEAP ))
-#define HEAP_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_HEAP, HeapClass ))
-
+/* Base heap descriptor.
+ */
 struct _Heap {
-	iObject parent_object;
-
-	Compile *compile;	/* If non-null, assoc. compile */
-
+	Compile *compile;	/* Debugging ... if non-null, assos. compile */
 	heap_max_fn max_fn;	/* Max nodes in this heap */
 	int mxb;		/* Max blocks until next check */
 	int rsz;		/* Nodes to allocate in each extra block */
 	int nb;			/* Number of blocks attached */
 	HeapBlock *hb;		/* List of current blocks */
 	HeapNode *free;		/* Start of free-node chain (sweep to here) */
-
+	GHashTable *emark;	/* List of elements to mark on GC */
+	GSList *rmark;		/* List of Reduce to mark on GC */
 	int ncells;		/* Cells allocated */
 	int nfree;		/* Cells free */
+	GSList *itable;		/* Imageinfos associated with this heap */
 	int serial;		/* Last serial number we used */
 	gboolean filled;	/* Set on heap full */
 
-	GHashTable *emark;	/* Set of elements to mark on GC */
-	GHashTable *rmark;	/* Set of Reduce to mark on GC */
-	GHashTable *mtable;	/* Managed associated with this heap */
+	const char *name;	/* Handy for debug */
 
         guint gc_tid;		/* id of gc delay timer */
-
-	/* Set this to force unreffed objects out immediately. Handy for leak 
-	 * testing.
-	 */
-	gboolean flush;
 };
 
-typedef struct _HeapClass {
-	iObjectClass parent_class;
-
-	/* My methods.
-	 */
-} HeapClass;
-
 /* Get a node from the free-list. No check for free-list exhausted! Set sym
  * pointer in node to heap sym pointer.
  */
 #ifdef DEBUG_HEAP
-#define EXTRACTNODE( H, A ) \
-	(heap_sanity( H ), (A) = (H)->free, (H)->free = GETLEFT( A ), 0)
-#else /*!DEBUG_HEAP*/
-#define EXTRACTNODE( H, A ) \
-	((A) = (H)->free, (H)->free = GETLEFT( A ), 0)
+#define EXTRACTNODE(H,A) \
+	(heap_sanity(H),(A)=(H)->free,(H)->free=GETLEFT(A),0)
+#else /*DEBUG_HEAP*/
+#define EXTRACTNODE(H,A) \
+	((A)=(H)->free,(H)->free=GETLEFT(A),0)
 #endif /*DEBUG_HEAP*/
 
 /* Allocate a new node from heap H, pop the pointer into A, return non-zero if
  * alloc failed. Node is uninitialised!
  */
-#define NEWNODE( H, A ) ( \
-	(H)->free ? \
-		EXTRACTNODE( H, A ): \
-		(((A) = heap_getmem( H )) ? 0 : -1) \
+#define NEWNODE(H,A) ( \
+	(H)->free ?\
+		EXTRACTNODE(H,A):\
+		(((A)=heap_getmem(H)) ? 0 : -1)\
 	)
 
-typedef void *(*heap_safe_pointer_fn)( Heap *heap, PElement *, 
+typedef void *(*heap_safe_pointer_fn)( Heap *hi, PElement *, 
 	void *, void *, void *, void * );
-void *heap_safe_pointer( Heap *heap, heap_safe_pointer_fn fn, 
+void *heap_safe_pointer( Heap *hi, heap_safe_pointer_fn fn, 
 	void *a, void *b, void *c, void *d );
 
 typedef void *(*heap_map_fn)( HeapNode *, void *, void *);
 void *heap_map( HeapNode *hn, heap_map_fn fn, void *a, void *b );
 
-int heap_sanity( Heap *heap );
+int heap_sanity( Heap *hi );
 
 void heap_check_all_destroyed( void );
-void heap_destroy( Heap *heap );
-GType heap_get_type( void );
+void heap_destroy( Heap *hi );
 Heap *heap_new( Compile *compile, heap_max_fn max_fn, int stsz, int rsz );
-HeapNode *heap_getmem( Heap *heap );
-gboolean heap_gc( Heap *heap );
-void heap_gc_request( Heap *heap );
-void heap_register_element( Heap *heap, Element *root );
-void heap_unregister_element( Heap *heap, Element *root );
-void heap_register_reduce( Heap *heap, Reduce *rc );
-void heap_unregister_reduce( Heap *heap, Reduce *rc );
-void heap_set( Heap *heap, NodeFlags setmask );
-void heap_clear( Heap *heap, NodeFlags clearmask );
-int heap_serial_new( Heap *heap );
+HeapNode *heap_getmem( Heap *hi );
+gboolean heap_gc( Heap *hi );
+void heap_gc_request( Heap *hi );
+void heap_register_element( Heap *hi, Element *root );
+void heap_unregister_element( Heap *hi, Element *root );
+void heap_register_reduce( Heap *hi, Reduce *rc );
+void heap_unregister_reduce( Heap *hi, Reduce *rc );
+void heap_set( Heap *hi, NodeFlags setmask );
+void heap_clear( Heap *hi, NodeFlags clearmask );
+int heap_serial_new( Heap *hi );
 
-gboolean heap_bool_new( Heap *heap, gboolean val, PElement *out );
-gboolean heap_real_new( Heap *heap, double in, PElement *out );
-gboolean heap_element_new( Heap *heap, Element *e, PElement *out );
-gboolean heap_complex_element_new( Heap *heap, 
+gboolean heap_bool_new( Heap *hi, gboolean val, PElement *out );
+gboolean heap_real_new( Heap *hi, double in, PElement *out );
+gboolean heap_image_new( Heap *hi, Imageinfo *ii, PElement *out );
+gboolean heap_element_new( Heap *hi, Element *e, PElement *out );
+gboolean heap_complex_element_new( Heap *hi, 
 	PElement *rp, PElement *ip, PElement *out );
-gboolean heap_complex_new( Heap *heap, double rp, double ip, PElement *out );
-gboolean heap_realvec_new( Heap *heap, int n, double *vec, PElement *out );
-gboolean heap_intvec_new( Heap *heap, int n, int *vec, PElement *out );
+gboolean heap_complex_new( Heap *hi, double rp, double ip, PElement *out );
+gboolean heap_dot_tag_new( Heap *hi, 
+	const char *tag, Compile *compile, PElement *out );
+gboolean heap_dot_sym_new( Heap *hi, 
+	Symbol *sym, Compile *compile, PElement *out );
+gboolean heap_realvec_new( Heap *hi, int n, double *vec, PElement *out );
 void heap_list_init( PElement *list );
-gboolean heap_list_add( Heap *heap, PElement *list, PElement *data );
+gboolean heap_list_add( Heap *hi, PElement *list, PElement *data );
 gboolean heap_list_next( PElement *list );
 gboolean heap_list_cat( Reduce *rc, PElement *a, PElement *b, PElement *out );
 void heap_appl_init( PElement *base, PElement *func );
-gboolean heap_appl_add( Heap *heap, PElement *base, PElement *parm );
-gboolean heap_matrix_new( Heap *heap, 
+gboolean heap_appl_add( Heap *hi, PElement *base, PElement *parm );
+gboolean heap_matrix_new( Heap *hi, 
 	int xsize, int ysize, double *vec, PElement *out );
-gboolean heap_string_new( Heap *heap, const char *str, PElement *out );
-gboolean heap_managedstring_new( Heap *heap, const char *str, PElement *out );
-gboolean heap_lstring_new( Heap *heap, GSList *labels, PElement *out );
-gboolean heap_file_new( Heap *heap, const char *filename, PElement *out );
+gboolean heap_string_new( Heap *hi, const char *str, PElement *out );
+gboolean heap_lstring_new( Heap *hi, GSList *labels, PElement *out );
 
-gboolean heap_error_typecheck( PElement *e, 
-	const char *name, const char *type );
 typedef void *(*heap_map_list_fn)( PElement *, void *, void * );
 void *heap_map_list( PElement *base, heap_map_list_fn fn, void *a, void *b );
-typedef void *(*heap_map_dict_fn)( const char *, PElement *, void *a, void *b );
-void *heap_map_dict( PElement *base, heap_map_dict_fn fn, void *a, void *b );
 
-gboolean heap_get_list( PElement *list );
-gboolean heap_get_list_next( PElement *list, PElement *data );
 gboolean heap_get_string( PElement *base, char *buf, int n );
 gboolean heap_get_lstring( PElement *base, GSList **labels );
 gboolean heap_get_bool( PElement *base, gboolean *out );
@@ -414,37 +373,21 @@ gboolean heap_get_real( PElement *base, double *out );
 gboolean heap_get_class( PElement *base, PElement *out );
 gboolean heap_get_image( PElement *base, Imageinfo **out );
 int heap_get_realvec( PElement *base, double *buf, int n );
-int heap_get_imagevec( PElement *base, Imageinfo **buf, int n );
 gboolean heap_get_matrix_size( PElement *base, int *xsize, int *ysize );
 gboolean heap_get_matrix( PElement *base, 
 	double *buf, int n, int *xsize, int *ysize );
 
-gboolean heap_is_elist( PElement *base, gboolean *out );
-gboolean heap_is_list( PElement *base, gboolean *out );
-gboolean heap_is_string( PElement *base, gboolean *out );
-gboolean heap_is_realvec( PElement *base, gboolean *out );
-gboolean heap_is_imagevec( PElement *base, gboolean *out );
-gboolean heap_is_matrix( PElement *base, gboolean *out );
-gboolean heap_is_class( PElement *base, gboolean *out );
-gboolean heap_is_instanceof_exact( const char *name, PElement *klass, 
+gboolean heap_iselist( PElement *base, gboolean *out );
+gboolean heap_isstring( PElement *base, gboolean *out );
+gboolean heap_isrealvec( PElement *base, gboolean *out );
+gboolean heap_ismatrix( PElement *base, gboolean *out );
+gboolean heap_isclass( PElement *base, gboolean *out );
+gboolean heap_isinstanceof_exact( const char *name, PElement *klass, 
 	gboolean *out);
-gboolean heap_is_instanceof( const char *name, PElement *klass, gboolean *out );
+gboolean heap_isinstanceof( const char *name, PElement *klass, gboolean *out );
 
 int heap_list_length( PElement *base );
-int heap_list_length_max( PElement *base, int max_length );
 gboolean heap_list_index( PElement *base, int n, PElement *out );
 gboolean heap_reduce_strict( PElement *base );
 
-gboolean heap_copy( Heap *heap, Compile *compile, PElement *out );
-
-gboolean heap_ip_to_gvalue( PElement *in, GValue *out );
-gboolean heap_gvalue_to_ip( GValue *in, PElement *out );
-
-void graph_node( Heap *heap, VipsBuf *buf, HeapNode *root, gboolean fn );
-void graph_pelement( Heap *heap, VipsBuf *buf, PElement *root, gboolean fn );
-void graph_element( Heap *heap, VipsBuf *buf, Element *root, gboolean fn );
-void graph_pointer( PElement *root );
-
-/* Reduce and print, csh-style output.
- */
-void graph_value( PElement *root );
+gboolean heap_copy( Heap *hi, Element *root, PElement *out );
