@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -35,15 +35,22 @@
 
 static ModelClass *parent_class = NULL;
 
+/* Trigger the new_heap method for a heapmodel.
+ */
 void *
 heapmodel_new_heap( Heapmodel *heapmodel, PElement *root )
 {
-	HeapmodelClass *heapmodel_class = HEAPMODEL_GET_CLASS( heapmodel );
+	HeapmodelClass *klass;
 
-	if( heapmodel_class->new_heap ) {
+	if( !heapmodel )
+		return( NULL );
+
+	klass = HEAPMODEL_CLASS( GTK_OBJECT( heapmodel )->klass );
+
+	if( klass->new_heap ) {
 		void *res;
 		
-		res = heapmodel_class->new_heap( heapmodel, root );
+		res = klass->new_heap( heapmodel, root );
 
 		return( res );
 	}
@@ -51,22 +58,29 @@ heapmodel_new_heap( Heapmodel *heapmodel, PElement *root )
 	return( NULL );
 }
 
+/* Trigger the update_model method for a heapmodel.
+ */
 void *
 heapmodel_update_model( Heapmodel *heapmodel )
 {
-	HeapmodelClass *heapmodel_class = HEAPMODEL_GET_CLASS( heapmodel );
+	HeapmodelClass *klass;
+
+	if( !heapmodel )
+		return( NULL );
+
+	klass = HEAPMODEL_CLASS( GTK_OBJECT( heapmodel )->klass );
 
 #ifdef DEBUG
 	printf( "heapmodel_update_model: %s ",
-		G_OBJECT_TYPE_NAME( heapmodel ) );
+		OBJECT_CLASS_NAME( heapmodel ) );
 	row_name_print( heapmodel->row );
 	printf( " modified = %d\n", heapmodel->modified );
 #endif /*DEBUG*/
 
-	if( heapmodel_class->update_model && !heapmodel->modified ) {
+	if( klass->update_model && !heapmodel->modified ) {
 		void *res;
 
-		res = heapmodel_class->update_model( heapmodel );
+		res = klass->update_model( heapmodel );
 
 		return( res );
 	}
@@ -74,15 +88,22 @@ heapmodel_update_model( Heapmodel *heapmodel )
 	return( NULL );
 }
 
+/* Trigger the update_heap method for a model.
+ */
 void *
 heapmodel_update_heap( Heapmodel *heapmodel )
 {
-	HeapmodelClass *heapmodel_class = HEAPMODEL_GET_CLASS( heapmodel );
+	HeapmodelClass *klass;
 
-	if( heapmodel_class->update_heap && heapmodel->modified ) {
+	if( !heapmodel )
+		return( NULL );
+
+	klass = HEAPMODEL_CLASS( GTK_OBJECT( heapmodel )->klass );
+
+	if( klass->update_heap && heapmodel->modified ) {
 		void *res;
 
-		res = heapmodel_class->update_heap( heapmodel );
+		res = klass->update_heap( heapmodel );
 
 		return( res );
 	}
@@ -90,26 +111,46 @@ heapmodel_update_heap( Heapmodel *heapmodel )
 	return( NULL );
 }
 
+/* Trigger the clear_edited method for a heapmodel.
+ */
 void *
 heapmodel_clear_edited( Heapmodel *heapmodel )
 {
-	HeapmodelClass *heapmodel_class = HEAPMODEL_GET_CLASS( heapmodel );
+	HeapmodelClass *klass;
 
-	if( heapmodel_class->clear_edited )
-		return( heapmodel_class->clear_edited( heapmodel ) );
+	if( !heapmodel )
+		return( NULL );
+
+	klass = HEAPMODEL_CLASS( GTK_OBJECT( heapmodel )->klass );
+
+	if( klass->clear_edited )
+		return( klass->clear_edited( heapmodel ) );
 
 	return( NULL );
 }
 
+static void
+heapmodel_destroy( GtkObject *object )
+{
+	Heapmodel *heapmodel;
+
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_HEAPMODEL( object ) );
+
+	heapmodel = HEAPMODEL( object );
+
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
+}
+
 static Rhs *
 heapmodel_get_rhs( Heapmodel *heapmodel )
 {
-	iContainer *p;
+	Model *p;
 
 	/* Search for the enclosing RHS ... may not be one if (eg.) this is a
 	 * top-level row.
 	 */
-	for( p = ICONTAINER( heapmodel )->parent; p; p = p->parent )
+	for( p = MODEL( heapmodel )->parent; p; p = p->parent )
 		if( IS_RHS( p ) )
 			return( RHS( p ) );
 
@@ -122,7 +163,7 @@ heapmodel_get_row( Heapmodel *heapmodel )
 	Rhs *rhs;
 
 	if( IS_RHS( heapmodel ) )
-		return( ROW( ICONTAINER( heapmodel )->parent ) );
+		return( ROW( MODEL( heapmodel )->parent ) );
 	else if( (rhs = heapmodel_get_rhs( heapmodel )) )
 		return( HEAPMODEL( rhs )->row );
 	else
@@ -130,14 +171,13 @@ heapmodel_get_row( Heapmodel *heapmodel )
 }
 
 static void
-heapmodel_parent_add( iContainer *child )
+heapmodel_parent_add( Model *child, Model *parent )
 {
 	Heapmodel *heapmodel = HEAPMODEL( child );
 
-	g_assert( IS_HEAPMODEL( child->parent ) || 
-		IS_FILEMODEL( child->parent ) ); 
+	assert( IS_HEAPMODEL( parent ) || IS_FILEMODEL( parent ) ); 
 
-	ICONTAINER_CLASS( parent_class )->parent_add( child );
+	MODEL_CLASS( parent_class )->parent_add( child, parent );
 
 	/* Update our context.
 	 */
@@ -148,7 +188,7 @@ heapmodel_parent_add( iContainer *child )
 static void *
 heapmodel_real_new_heap( Heapmodel *heapmodel, PElement *root )
 {
-	iobject_changed( IOBJECT( heapmodel ) );
+	model_changed( MODEL( heapmodel ) );
 
 	return( NULL );
 }
@@ -156,7 +196,7 @@ heapmodel_real_new_heap( Heapmodel *heapmodel, PElement *root )
 static void *
 heapmodel_real_update_model( Heapmodel *heapmodel )
 {
-	iobject_changed( IOBJECT( heapmodel ) );
+	model_changed( MODEL( heapmodel ) );
 
 	return( NULL );
 }
@@ -164,7 +204,7 @@ heapmodel_real_update_model( Heapmodel *heapmodel )
 static void *
 heapmodel_real_update_heap( Heapmodel *heapmodel )
 {
-	g_assert( heapmodel->modified );
+	assert( heapmodel->modified );
 
 	heapmodel_set_modified( heapmodel, FALSE );
 
@@ -178,16 +218,19 @@ heapmodel_real_clear_edited( Heapmodel *heapmodel )
 }
 
 static void
-heapmodel_class_init( HeapmodelClass *class )
+heapmodel_class_init( HeapmodelClass *klass )
 {
-	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
-	iContainerClass *icontainer_class = (iContainerClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	HeapmodelClass *heapmodel_class = (HeapmodelClass *) klass;
+	ModelClass *model_class = (ModelClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_MODEL );
+
+	object_class->destroy = heapmodel_destroy;
 
 	/* Init methods.
 	 */
-	icontainer_class->parent_add = heapmodel_parent_add;
+	model_class->parent_add = heapmodel_parent_add;
 
 	heapmodel_class->new_heap = heapmodel_real_new_heap;
 	heapmodel_class->update_heap = heapmodel_real_update_heap;
@@ -204,26 +247,24 @@ heapmodel_init( Heapmodel *heapmodel )
 	heapmodel->modified = FALSE;
 }
 
-GType
+GtkType
 heapmodel_get_type( void )
 {
-	static GType heapmodel_type = 0;
+	static GtkType heapmodel_type = 0;
 
 	if( !heapmodel_type ) {
-		static const GTypeInfo info = {
-			sizeof( HeapmodelClass ),
-			NULL,           /* base_init */
-			NULL,           /* base_finalize */
-			(GClassInitFunc) heapmodel_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
+		static const GtkTypeInfo info = {
+			"Heapmodel",
 			sizeof( Heapmodel ),
-			32,             /* n_preallocs */
-			(GInstanceInitFunc) heapmodel_init,
+			sizeof( HeapmodelClass ),
+			(GtkClassInitFunc) heapmodel_class_init,
+			(GtkObjectInitFunc) heapmodel_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
 		};
 
-		heapmodel_type = g_type_register_static( TYPE_MODEL, 
-			"Heapmodel", &info, 0 );
+		heapmodel_type = gtk_type_unique( TYPE_MODEL, &info );
 	}
 
 	return( heapmodel_type );
@@ -234,64 +275,12 @@ heapmodel_set_modified( Heapmodel *heapmodel, gboolean modified )
 {
 	if( heapmodel->modified != modified ) {
 #ifdef DEBUG
-{
-		HeapmodelClass *heapmodel_class = 
-			HEAPMODEL_GET_CLASS( heapmodel );
-
-		printf( "heapmodel_set_modified: %s::", 
-			G_OBJECT_CLASS_NAME( heapmodel_class ) );
+		printf( "heapmodel_set_modified: " );
 		row_name_print( heapmodel->row );
 		printf( " %s\n", bool_to_char( modified ) );
-}
 #endif /*DEBUG*/
 
 		heapmodel->modified = modified;
-		iobject_changed( IOBJECT( heapmodel ) );
+		model_changed( MODEL( heapmodel ) );
 	}
 }
-
-/* Generate a descriptive name for a heapmodel. Used for captions.
- */
-gboolean
-heapmodel_name( Heapmodel *heapmodel, VipsBuf *buf )
-{
-	Row *row = heapmodel->row;
-	Expr *expr;
-	Symbol *sym;
-	Toolitem *toolitem;
-
-	if( !row || !(expr = row->expr) || !PEISCLASS( &expr->root ) ) 
-		return( FALSE );
-	sym = PEGETCLASSCOMPILE( &expr->root )->sym;
-
-	/* If this is an action member we should be able to look up
-	 * it's sym and get a descriptive label.
-	 */
-	if( (toolitem = toolitem_lookup( row->ws->kitg, sym )) )
-		vips_buf_appends( buf, toolitem->name );
-	else 
-		symbol_qualified_name_relative( row->ws->sym, sym, buf );
-
-	return( TRUE );
-}
-
-/* Print the value member to a buf.
- */
-gboolean
-heapmodel_value( Heapmodel *heapmodel, VipsBuf *buf )
-{
-	Expr *expr;
-	PElement value;
-
-	if( !heapmodel->row ||
-		!(expr = heapmodel->row->expr) ||
-		expr->err || 
-		expr->sym->dirty ||
-		!class_get_member( &expr->root, MEMBER_VALUE, NULL, &value ) ) 
-		return( FALSE );
-
-	itext_value( reduce_context, buf, &value );
-
-	return( TRUE );
-}
-
