@@ -16,9 +16,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,16 +29,12 @@
  */
 
 #define TYPE_HEAPMODEL (heapmodel_get_type())
-#define HEAPMODEL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_HEAPMODEL, Heapmodel ))
+#define HEAPMODEL( obj ) (GTK_CHECK_CAST( (obj), TYPE_HEAPMODEL, Heapmodel ))
 #define HEAPMODEL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_HEAPMODEL, HeapmodelClass))
-#define IS_HEAPMODEL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_HEAPMODEL ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_HEAPMODEL, HeapmodelClass ))
+#define IS_HEAPMODEL( obj ) (GTK_CHECK_TYPE( (obj), TYPE_HEAPMODEL ))
 #define IS_HEAPMODEL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_HEAPMODEL ))
-#define HEAPMODEL_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_HEAPMODEL, HeapmodelClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_HEAPMODEL ))
 
 struct _Heapmodel {
 	Model parent_class;
@@ -85,8 +81,6 @@ void *heapmodel_update_model( Heapmodel *heapmodel );
 void *heapmodel_update_heap( Heapmodel *heapmodel );
 void *heapmodel_clear_edited( Heapmodel *heapmodel );
 
-GType heapmodel_get_type( void );
+GtkType heapmodel_get_type( void );
 
 void heapmodel_set_modified( Heapmodel *heapmodel, gboolean modified );
-gboolean heapmodel_name( Heapmodel *heapmodel, VipsBuf *buf );
-gboolean heapmodel_value( Heapmodel *heapmodel, VipsBuf *buf );
