@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -28,34 +28,31 @@
  */
 
 #define TYPE_IARROW (iarrow_get_type())
-#define IARROW( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_IARROW, iArrow ))
+#define IARROW( obj ) (GTK_CHECK_CAST( (obj), TYPE_IARROW, Iarrow ))
 #define IARROW_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_IARROW, iArrowClass))
-#define IS_IARROW( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_IARROW ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_IARROW, IarrowClass ))
+#define IS_IARROW( obj ) (GTK_CHECK_TYPE( (obj), TYPE_IARROW ))
 #define IS_IARROW_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_IARROW ))
-#define IARROW_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_IARROW, iArrowClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_IARROW ))
 
-struct _iArrow {
+struct _Iarrow {
 	Classmodel parent_class;
 
 	/* Class fields.
 	 */
-	iRegionInstance instance;
+	IregionInstance instance;
 
-	/* Private ... build iobject caption here.
+	/* Build view caption here.
 	 */
-	VipsBuf caption_buffer;
+	BufInfo caption;
 };
 
-typedef struct _iArrowClass {
+typedef struct _IarrowClass {
 	ClassmodelClass parent_class;
 
 	/* My methods.
 	 */
-} iArrowClass;
+} IarrowClass;
 
-GType iarrow_get_type( void );
+GtkType iarrow_get_type( void );
+Classmodel *iarrow_new( Rhs *rhs );
