@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -35,41 +35,12 @@
 
 static iWindowClass *parent_class = NULL;
 
-/* An OK button: label (can be a stock) plus a callback.
- */
-typedef struct {
-	char *label;
-	iWindowFn done_cb;
-} OKButton;
-
-static void *
-okbutton_free( OKButton *ok )
-{
-	IM_FREEF( g_free, ok->label );
-	ok->done_cb = NULL;
-	IM_FREEF( g_free, ok );
-
-	return( NULL );
-}
-
-static OKButton *
-okbutton_new( char *label, iWindowFn done_cb )
-{
-	OKButton *ok;
-
-	ok = g_new( OKButton, 1 );
-	ok->label = g_strdup( label );
-	ok->done_cb = done_cb;
-
-	return( ok );
-}
-
 /* Handy destroy callback ... just free client.
  */
 void
 idialog_free_client( iDialog *idlg, void *client )
 {
-	IM_FREE( client );
+	FREE( client );
 }
 
 /* Notify our parent.
@@ -98,8 +69,8 @@ idialog_set_sensitive( GtkWidget *w, gboolean state )
 void
 idialog_set_ok_button_state( iDialog *idlg, gboolean state )
 {
-	slist_map( idlg->ok_but_l, 
-		(SListMapFn) idialog_set_sensitive, GINT_TO_POINTER( state ) );
+	slist_map( idlg->ok_l, 
+		(SListMapFn) idialog_set_sensitive, (void *) state );
 }
 
 /* Set all the button sensitivities.
@@ -122,10 +93,10 @@ idialog_popdown_notify( void *sys, iWindowResult result )
         iWindowSusp *susp = IWINDOW_SUSP( sys );
 	iDialog *idlg = IDIALOG( susp->client );
 
-	if( result == IWINDOW_YES ) 
+	if( result == IWINDOW_TRUE ) 
 		/* If our caller hasn't been notified yet, post off a FALSE.
 		 */
-		idialog_notify_parent( idlg, IWINDOW_NO );
+		idialog_notify_parent( idlg, IWINDOW_FALSE );
 
 	/* Pass result on to our suspension (ie. back to iwindow).
 	 */
@@ -170,7 +141,7 @@ idialog_done_notify( void *sys, iWindowResult result )
 
 	/* If all ok, popdown and tell our parent.
 	 */
-	if( result == IWINDOW_YES ) {
+	if( result == IWINDOW_TRUE ) {
 		/* Unless we're pinned up, that is.
 		 */
 		if( !(idlg->tog_pin && 
@@ -184,7 +155,7 @@ idialog_done_notify( void *sys, iWindowResult result )
 	/* Alert on failure.
 	 */
 	if( result == IWINDOW_ERROR )
-		iwindow_alert( GTK_WIDGET( idlg ), GTK_MESSAGE_ERROR );
+		box_alert( GTK_WIDGET( idlg ) );
 
 	/* Clean up.
 	 */
@@ -197,7 +168,7 @@ idialog_done_notify( void *sys, iWindowResult result )
 void
 idialog_done_trigger( iDialog *idlg, int pos )
 {
-	OKButton *ok = (OKButton *) g_slist_nth_data( idlg->ok_disp_l, pos );
+	iWindowFn fn = (iWindowFn) g_slist_nth_data( idlg->ok_cb_l, pos );
 
 #ifdef DEBUG
 	printf( "idialog_done_trigger: %s, %d\n", 
@@ -206,11 +177,10 @@ idialog_done_trigger( iDialog *idlg, int pos )
 
 	/* Trigger user done callback.
 	 */
-	g_assert( pos >= 0 );
-	g_assert( ok->done_cb );
+	assert( fn );
 	idialog_set_button_state( idlg, FALSE );
 	iwindow_notify_send( IWINDOW( idlg ), 
-		ok->done_cb, idlg->client, idialog_done_notify, idlg );
+		fn, idlg->client, idialog_done_notify, idlg );
 }
 
 /* Sub-fn of below.
@@ -228,15 +198,15 @@ idialog_cancel_notify( void *sys, iWindowResult result )
 
 	/* Send cancel message back to parent if our client cancel was OK.
 	 */
-	if( result == IWINDOW_YES ) {
-		idialog_notify_parent( idlg, IWINDOW_NO );
+	if( result == IWINDOW_TRUE ) {
+		idialog_notify_parent( idlg, IWINDOW_FALSE );
 		iwindow_kill( IWINDOW( idlg ) );
 	}
 
 	/* Alert on error.
 	 */
 	if( result == IWINDOW_ERROR )
-		iwindow_alert( GTK_WIDGET( idlg ), GTK_MESSAGE_ERROR );
+		box_alert( GTK_WIDGET( idlg ) );
 
 	/* Clean up.
 	 */
@@ -257,14 +227,51 @@ idialog_cancel_trigger( iDialog *idlg )
 		idlg->cancel_cb, idlg->client, idialog_cancel_notify, idlg );
 }
 
+/* Sub-fn of below.
+ */
+static void
+idialog_help_notify( void *sys, iWindowResult result )
+{
+	iDialog *idlg = IDIALOG( sys );
+
+#ifdef DEBUG
+	printf( "idialog_help_notify: %s\n", IWINDOW( idlg )->title );
+#endif /*DEBUG*/
+
+	idialog_set_button_state( idlg, TRUE );
+
+	/* Alert on error.
+	 */
+	if( result == IWINDOW_ERROR )
+		box_alert( GTK_WIDGET( idlg ) );
+
+	/* Clean up.
+	 */
+	iwindow_notify_return( IWINDOW( idlg ) );
+}
+
+static void
+idialog_help_trigger( iDialog *idlg )
+{
+#ifdef DEBUG
+        printf( "idialog_help_trigger: %s\n", IWINDOW( idlg )->title );
+#endif /*DEBUG*/
+
+        /* Trigger user help function.
+         */
+	idialog_set_button_state( idlg, FALSE );
+        iwindow_notify_send( IWINDOW( idlg ), 
+		idlg->help_cb, idlg->client, idialog_help_notify, idlg );
+}
+
 /* Button callbacks from gtk.
  */
 static void
 idialog_done_cb( GtkWidget *w, iDialog *idlg )
 {
-	int pos = g_slist_index( idlg->ok_but_l, w );
+	int pos = g_slist_index( idlg->ok_l, w );
 
-	g_assert( pos != -1 );
+	assert( pos != -1 );
 
 	idialog_done_trigger( idlg, pos );
 }
@@ -278,8 +285,7 @@ idialog_cancel_cb( GtkWidget *w, iDialog *idlg )
 static void
 idialog_help_cb( GtkWidget *w, iDialog *idlg )
 {
-	if( idlg->help_tag )
-		box_help( GTK_WIDGET( idlg ), idlg->help_tag );
+	idialog_help_trigger( idlg );
 }
 
 static void
@@ -308,11 +314,12 @@ idialog_destroy( GtkObject *object )
 		idlg->destroy_cb = NULL;
 	}
 
-	FREESID( idlg->destroy_sid, idlg->iobject );
-	slist_map( idlg->ok_l, (SListMapFn) okbutton_free, NULL );
-	IM_FREEF( g_slist_free, idlg->ok_l );
-	IM_FREEF( g_slist_free, idlg->ok_disp_l );
-	IM_FREEF( g_slist_free, idlg->ok_but_l );
+	FREESID( idlg->parent_sid, idlg->window_parent );
+	FREESID( idlg->object_sid, idlg->object );
+	slist_map( idlg->ok_txt_l, (SListMapFn) im_free, NULL );
+	FREEF( g_slist_free, idlg->ok_cb_l );
+	FREEF( g_slist_free, idlg->ok_txt_l );
+	FREEF( g_slist_free, idlg->ok_l );
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -326,73 +333,99 @@ idialog_realize( GtkWidget *widget )
 	printf( "idialog_realize: %s\n", IWINDOW( idlg )->title );
 #endif /*DEBUG*/
 
+	/* Centre over parent. We should be able to just set a hint and let
+	 * the WM do this, but it's very unreliable. Do it ourselves.
+	 */
+        if( idlg->window_parent ) {
+		GtkWindow *pwnd = GTK_WINDOW( 
+			gtk_widget_get_toplevel( idlg->window_parent ) );
+		GtkWidget *wid = GTK_WIDGET( idlg );
+		int x, y, w, h;
+		int dx, dy;
+		const int swidth = gdk_screen_width();
+		const int sheight = gdk_screen_height();
+
+		/* get_root_origin allows for WM decorations.
+		 */
+		gdk_window_get_size( GTK_WIDGET( pwnd )->window, &w, &h );
+		gdk_window_get_root_origin( GTK_WIDGET( pwnd )->window, 
+			&x, &y );
+		dx = x + w / 2 - wid->requisition.width / 2;
+		dy = y + h / 2 - wid->requisition.height / 2;
+
+		/* Clip against screen size. Allow a bit for desktop borders
+		 * added by the WM.
+		 */
+		dx = IM_CLIP( 50, dx, swidth - wid->requisition.width - 50 );
+		dy = IM_CLIP( 50, dy, sheight - wid->requisition.height - 50 );
+
+		gtk_widget_set_uposition( GTK_WIDGET( idlg ), dx, dy );
+
+#ifdef DEBUG
+		printf( "idialog_realize: centering over parent at %dx%d\n",
+			dx, dy );
+#endif /*DEBUG*/
+	}
+
 	GTK_WIDGET_CLASS( parent_class )->realize( widget );
 
 	if( idlg->entry )
 		gtk_widget_grab_focus( GTK_WIDGET( idlg->entry ) );
 }
 
+/* Our parent has been destroyed, kill us too.
+ */
+static void
+idialog_parent_destroy( GtkWidget *par, iDialog *idlg )
+{
+#ifdef DEBUG
+	printf( "idialog_parent_destroy: %s\n", IWINDOW( idlg )->title );
+#endif /*DEBUG*/
+
+	/* Here for dead parent ... if parent is dead, we won't need to remove
+	 * the dead-dad signal. 
+	 */
+	idlg->parent_sid = 0;
+
+	iwindow_kill( IWINDOW( idlg ) );
+}
+
 /* The object we represent has been destroyed, kill us too.
  */
 static void
-idialog_iobject_destroy( iObject *iobject, iDialog *idlg )
+idialog_object_destroy( GtkWidget *par, iDialog *idlg )
 {
 #ifdef DEBUG
-	printf( "idialog_iobject_destroy: %s\n", IWINDOW( idlg )->title );
+	printf( "idialog_object_destroy: %s\n", IWINDOW( idlg )->title );
 #endif /*DEBUG*/
 
-	/* This object has gone.
+	/* Here for dead parent ... if parent is dead, we won't need to remove
+	 * the dead-dad signal. 
 	 */
-	idlg->iobject = NULL;
+	idlg->object_sid = 0;
 
 	iwindow_kill( IWINDOW( idlg ) );
 }
 
 static void *
-idialog_build_ok( OKButton *ok, iDialog *idlg )
+idialog_build_ok( const char *txt, iDialog *idlg )
 {
 	GtkWidget *but;
 
-	but = build_button( ok->label, 
-		GTK_SIGNAL_FUNC( idialog_done_cb ), idlg );
-	idlg->ok_disp_l = g_slist_prepend( idlg->ok_disp_l, ok );
-	idlg->ok_but_l = g_slist_prepend( idlg->ok_but_l, but );
+	but = build_button( txt, idialog_done_cb, idlg );
+	idlg->ok_l = g_slist_prepend( idlg->ok_l, but );
 	gtk_box_pack_start( GTK_BOX( idlg->bb ), but, TRUE, TRUE, 0 );
 	gtk_widget_show( but );
 
 	return( NULL );
 }
 
-static void *
-idialog_build_cancel( iDialog *idlg )
-{
-	idlg->but_cancel = build_button( idlg->cancel_text,
-		GTK_SIGNAL_FUNC( idialog_cancel_cb ), idlg );
-	gtk_box_pack_start( GTK_BOX( idlg->bb ),
-		idlg->but_cancel, TRUE, TRUE, 0 );
-	gtk_widget_show( idlg->but_cancel );
-
-	return( NULL );
-}
-
-/* Set a button to be the dialog default. Turn off button_focus for complex
- * dialogs like file_chooser which have their on focus systems.
- */
-static void
-idialog_set_default( iDialog *idlg, GtkWidget *widget )
-{
-	if( idlg->button_focus ) 
-		gtk_widget_grab_focus( widget );
-
-	GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
-	gtk_window_set_default( GTK_WINDOW( idlg ), widget );
-}
-
 static void 
 idialog_build( GtkWidget *widget )
 {
 	iDialog *idlg = IDIALOG( widget );
 	iWindow *iwnd = IWINDOW( idlg );
+	GtkAccelGroup *accel_group;
 
 #ifdef DEBUG
 	printf( "idialog_build: %s\n", iwnd->title );
@@ -409,9 +442,9 @@ idialog_build( GtkWidget *widget )
 
         gtk_window_set_modal( GTK_WINDOW( idlg ), idlg->modal );
 
-        idlg->work = gtk_vbox_new( FALSE, 6 );
-        gtk_container_set_border_width( GTK_CONTAINER( idlg->work ), 12 );
-        gtk_box_pack_start( GTK_BOX( iwnd->work ), idlg->work, TRUE, TRUE, 0 );
+        idlg->work = gtk_vbox_new( FALSE, 2 );
+        gtk_container_set_border_width( GTK_CONTAINER( idlg->work ), 5 );
+        gtk_box_pack_start( GTK_BOX( iwnd->work ), idlg->work, TRUE, TRUE, 2 );
 
 	if( !idlg->nosep ) {
 		GtkWidget *sep;
@@ -422,124 +455,71 @@ idialog_build( GtkWidget *widget )
 		gtk_widget_show( sep );
 	}
 
-	idlg->hb = gtk_hbox_new( FALSE, 6 );
-        gtk_container_set_border_width( GTK_CONTAINER( idlg->hb ), 12 );
+	idlg->hb = gtk_hbox_new( FALSE, 0 );
         gtk_box_pack_start( GTK_BOX( iwnd->work ), idlg->hb, FALSE, FALSE, 0 );
         gtk_widget_show( idlg->hb );
 
+        idlg->bb = gtk_hbutton_box_new();
+        gtk_button_box_set_layout( GTK_BUTTON_BOX( idlg->bb ), 
+		GTK_BUTTONBOX_END );
+        gtk_button_box_set_spacing( GTK_BUTTON_BOX( idlg->bb ), 5 );
+        gtk_container_set_border_width( GTK_CONTAINER( idlg->bb ), 10 );
+        gtk_box_pack_end( GTK_BOX( idlg->hb ), idlg->bb, FALSE, FALSE, 0 );
+        gtk_widget_show( idlg->bb );
+
 	if( idlg->pinup ) {
-		idlg->tog_pin = gtk_check_button_new_with_label( 
-			_( "Pin up" ) );
-		set_tooltip( idlg->tog_pin, 
-			_( "Check this to pin the dialog up" ) );
+		idlg->tog_pin = gtk_check_button_new_with_label( "Pin up" );
+		set_tooltip( idlg->tog_pin, "Check this to pin the dialog up" );
                 gtk_box_pack_start( GTK_BOX( idlg->hb ),
                         idlg->tog_pin, FALSE, FALSE, 0 );
 		gtk_widget_show( idlg->tog_pin );
 	}
 
-        idlg->bb = gtk_hbutton_box_new();
-        gtk_button_box_set_layout( GTK_BUTTON_BOX( idlg->bb ), 
-		GTK_BUTTONBOX_END );
-	gtk_box_set_spacing( GTK_BOX( idlg->bb ), 6 );
-        gtk_box_pack_end( GTK_BOX( idlg->hb ), idlg->bb, FALSE, FALSE, 0 );
-        gtk_widget_show( idlg->bb );
+        if( idlg->help_cb ) {
+		GtkWidget *align;
 
-	/* Default button order: 
-	 *
-	 * Help        OK3 OK2 Cancel OK1
-	 *
-	 * win32 button order:
-	 *
-	 * OK1 OK2 OK3 Cancel Help
-	 */
+		align = gtk_alignment_new( 0.0, 0.5, 0.0, 0.0 );
+		gtk_container_set_border_width( GTK_CONTAINER( align ), 10 );
+                gtk_box_pack_start( GTK_BOX( idlg->hb ),
+                        align, FALSE, FALSE, 0 );
+		gtk_widget_show( align );
 
-#ifdef OS_WIN32
+                idlg->but_help = build_button( "Help", idialog_help_cb, idlg );
+                gtk_container_add( GTK_CONTAINER( align ), idlg->but_help );
+                gtk_widget_show( idlg->but_help );
+        }
 
-	/* OK buttons.
-	 */
-	slist_map( idlg->ok_l,
+        if( idlg->cancel_cb ) {
+                idlg->but_cancel = build_button( "Cancel", 
+			idialog_cancel_cb, idlg );
+                gtk_box_pack_start( GTK_BOX( idlg->bb ),
+                        idlg->but_cancel, TRUE, TRUE, 0 );
+                gtk_widget_show( idlg->but_cancel );
+                if( !idlg->nook && !idlg->ok_l )
+                        gtk_widget_grab_default( idlg->but_cancel );
+	}
+
+        slist_map_rev( idlg->ok_txt_l,
 		(SListMapFn) idialog_build_ok, idlg );
 
-        if( idlg->cancel_cb ) {
-		idialog_build_cancel( idlg );
-
-		/* Cancel grabs default if it's the only button. Set focus
-		 * too; user build can change this later.
-		 */
-                if( !idlg->ok_l ) 
-			idialog_set_default( idlg, idlg->but_cancel );
-	}
-
-        if( idlg->help_tag ) {
-                idlg->but_help = build_button( GTK_STOCK_HELP,
-			GTK_SIGNAL_FUNC( idialog_help_cb ), idlg );
-                gtk_widget_show( idlg->but_help );
-        }
-
-#else /*!OS_WIN32*/
-
-        if( idlg->help_tag ) {
-                idlg->but_help = build_button( GTK_STOCK_HELP,
-			GTK_SIGNAL_FUNC( idialog_help_cb ), idlg );
-                gtk_box_pack_end( GTK_BOX( idlg->bb ),
-                        idlg->but_help, TRUE, TRUE, 0 );
-		gtk_button_box_set_child_secondary( GTK_BUTTON_BOX( idlg->bb ),
-			idlg->but_help, TRUE );
-                gtk_widget_show( idlg->but_help );
-        }
-
-	/* Add OK2, 3, etc.
+	/* Escape triggers cancel, or OK if no cancel
 	 */
-	if( idlg->ok_l && idlg->ok_l->next )
-		slist_map_rev( idlg->ok_l->next,
-			(SListMapFn) idialog_build_ok, idlg );
-
-        if( idlg->cancel_cb ) {
-		idialog_build_cancel( idlg );
-
-		/* Cancel grabs default if it's the only button. Set focus
-		 * too; user build can change this later.
-		 */
-                if( !idlg->ok_l ) 
-			idialog_set_default( idlg, idlg->but_cancel );
-	}
-
-	/* Make OK1
-	 */
-	if( idlg->ok_l ) {
-		OKButton *ok1 = (OKButton *) idlg->ok_l->data;
-
-		idialog_build_ok( ok1, idlg );
-	}
-
-#endif /*lots*/
-
-	/* OK1 grabs the default.
-	 */
-	if( idlg->ok_but_l ) 
-		idialog_set_default( idlg, idlg->ok_but_l->data );
-
-	/* Escape triggers cancel, if there is a cancel.
-	 */
-	if( idlg->cancel_cb )
-		gtk_widget_add_accelerator( idlg->but_cancel,
-			"clicked", iwnd->accel_group, GDK_Escape, 0, 0 );
-	else {
-		/* If there's just 1 OK, that gets Esc too.
-		 */
-		if( idlg->ok_but_l &&
-			g_slist_length( idlg->ok_but_l ) == 1 )
-			gtk_widget_add_accelerator( 
-				GTK_WIDGET( idlg->ok_but_l->data ), "clicked", 
-				iwnd->accel_group, GDK_Escape, 0, 0 );
-	}
+	accel_group = gtk_accel_group_new();
+	gtk_accel_group_attach( accel_group, GTK_OBJECT( idlg ) );
+	gtk_widget_add_accelerator( 
+		idlg->cancel_cb ? 
+			idlg->but_cancel : GTK_WIDGET( idlg->ok_l->data ), 
+		"clicked", accel_group, GDK_Escape, 0, 0 );
 
 	/* F1 triggers help.
 	 */
-	if( idlg->but_help ) 
+	if( idlg->help_cb ) 
 		gtk_widget_add_accelerator( 
 			idlg->but_help,
-			"clicked", iwnd->accel_group, GDK_F1, 0, 0 );
+			"clicked", accel_group, GDK_F1, 0, 0 );
+
+	if( !idlg->nook && idlg->ok_l )
+		gtk_widget_grab_default( GTK_WIDGET( idlg->ok_l->data ) );
 
 	/* Build user dialog contents.
 	 */
@@ -547,28 +527,41 @@ idialog_build( GtkWidget *widget )
 		idlg->build( iwnd, idlg->work, 
 			idlg->build_a, idlg->build_b, idlg->build_c );
 
-	if( idlg->iobject )
-		idlg->destroy_sid = g_signal_connect( idlg->iobject, "destroy", 
-			G_CALLBACK( idialog_iobject_destroy ), idlg );
+	/* Link to parent.
+	 */
+        if( idlg->window_parent ) {
+                gtk_window_set_transient_for( GTK_WINDOW( idlg ),
+                        GTK_WINDOW( gtk_widget_get_toplevel( 
+				idlg->window_parent ) ) );
+		idlg->parent_sid = gtk_signal_connect( 
+			GTK_OBJECT( idlg->window_parent ), 
+			"destroy",
+			GTK_SIGNAL_FUNC( idialog_parent_destroy ), idlg );
+	}
+	if( idlg->object )
+		idlg->object_sid = gtk_signal_connect( idlg->object,
+			"destroy",
+			GTK_SIGNAL_FUNC( idialog_object_destroy ), idlg );
 
         gtk_widget_show( idlg->work );
 }
 
 static void
-idialog_class_init( iDialogClass *class )
+idialog_class_init( iDialogClass *klass )
 {
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
-	GtkWidgetClass *widget_class = (GtkWidgetClass *) class;
-	iWindowClass *iwindow_class = (iWindowClass *) class;
+	GtkObjectClass *object_class;
+	GtkWidgetClass *widget_class;
+	iWindowClass *iwindow_class;
 
-	parent_class = g_type_class_peek_parent( class );
+	object_class = (GtkObjectClass *) klass;
+	widget_class = (GtkWidgetClass *) klass;
+	iwindow_class = (iWindowClass *) klass;
+
+	parent_class = gtk_type_class( TYPE_IWINDOW );
 
 	object_class->destroy = idialog_destroy;
-
 	widget_class->realize = idialog_realize;
-
 	iwindow_class->build = idialog_build;
-	iwindow_class->transient = TRUE;
 
 	/* Create signals.
 	 */
@@ -586,31 +579,29 @@ idialog_init( iDialog *idlg )
 
 	/* Init our instance fields.
 	 */
-	idlg->iobject = NULL;
-        idlg->destroy_sid = 0;
+	idlg->window_parent = NULL;
+        idlg->parent_sid = 0;
+	idlg->object = NULL;
+        idlg->object_sid = 0;
 
 	idlg->work = NULL;
 
 	idlg->ok_l = NULL;
-	idlg->ok_disp_l = NULL;
-	idlg->ok_but_l = NULL;
-
+	idlg->ok_txt_l = NULL;
 	idlg->but_cancel = NULL;
 	idlg->but_help = NULL;
 	idlg->tog_pin = NULL;
 
 	idlg->entry = NULL;
 
+	idlg->nook = FALSE;
 	idlg->modal = FALSE;
 	idlg->pinup = FALSE;
-	idlg->nosep = FALSE;
-	idlg->button_focus = TRUE;
-
-	idlg->help_tag = NULL;
-
-	idlg->cancel_text = GTK_STOCK_CANCEL;
+	idlg->nosep = TRUE;
 
+	idlg->ok_cb_l = NULL;
 	idlg->cancel_cb = NULL;
+	idlg->help_cb = NULL;
 	idlg->popdown_cb = NULL;
 	idlg->destroy_cb = NULL;
 	idlg->client = NULL;
@@ -619,18 +610,15 @@ idialog_init( iDialog *idlg )
 
 	idlg->nfn = iwindow_notify_null;
 	idlg->sys = NULL;
-
-	gtk_window_set_position( GTK_WINDOW( idlg ),
-		GTK_WIN_POS_CENTER_ON_PARENT );
 }
 
 GtkType
 idialog_get_type( void )
 {
-	static GtkType type = 0;
+	static GtkType idialog_type = 0;
 
-	if( !type ) {
-		static const GtkTypeInfo info = {
+	if (!idialog_type) {
+		static const GtkTypeInfo idlg_info = {
 			"iDialog",
 			sizeof( iDialog ),
 			sizeof( iDialogClass ),
@@ -641,10 +629,10 @@ idialog_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		type = gtk_type_unique( TYPE_IWINDOW, &info );
+		idialog_type = gtk_type_unique( TYPE_IWINDOW, &idlg_info );
 	}
 
-	return( type );
+	return( idialog_type );
 }
 
 GtkWidget *
@@ -655,25 +643,33 @@ idialog_new()
 
 	/* Init gtk base class.
 	 */
-	gwnd->type = GTK_WINDOW_TOPLEVEL;
+	gwnd->type = GTK_WINDOW_DIALOG;
 
 	return( GTK_WIDGET( idlg ) );
 }
 
 void 
-idialog_set_iobject( iDialog *idlg, iObject *iobject )
+idialog_set_parent( iDialog *idlg, GtkWidget *window_parent )
 {
-	idlg->iobject = iobject;
+	idlg->window_parent = window_parent;
+}
+
+void 
+idialog_set_object( iDialog *idlg, GtkObject *object )
+{
+	idlg->object = object;
+}
+
+void 
+idialog_set_nook( iDialog *idlg, gboolean nook )
+{
+	idlg->nook = nook;
 }
 
 void 
 idialog_set_pinup( iDialog *idlg, gboolean pinup )
 {
 	idlg->pinup = pinup;
-
-	if( idlg->tog_pin ) 
-		gtk_toggle_button_set_active( 
-			GTK_TOGGLE_BUTTON( idlg->tog_pin ), TRUE );
 }
 
 void 
@@ -688,24 +684,13 @@ idialog_set_nosep( iDialog *idlg, gboolean nosep )
 	idlg->nosep = nosep;
 }
 
-void 
-idialog_set_button_focus( iDialog *idlg, gboolean button_focus )
-{
-	idlg->button_focus = button_focus;
-}
-
-void 
-idialog_set_help_tag( iDialog *idlg, const char *help_tag )
-{
-	IM_SETSTR( idlg->help_tag, help_tag );
-}
-
 void 
 idialog_set_callbacks( iDialog *idlg, 
-	iWindowFn cancel_cb, iWindowFn popdown_cb, 
+	iWindowFn cancel_cb, iWindowFn help_cb, iWindowFn popdown_cb, 
 	iDialogFreeFn destroy_cb, void *client )
 {
 	idlg->cancel_cb = cancel_cb;
+	idlg->help_cb = help_cb;
 	idlg->popdown_cb = popdown_cb;
 	idlg->destroy_cb = destroy_cb;
 	idlg->client = client;
@@ -715,17 +700,15 @@ void
 idialog_add_ok( iDialog *idlg, iWindowFn done_cb, const char *fmt, ... )
 {
 	va_list ap;
-	char buf[1024];
+	char buf[ 1024 ];
 
         va_start( ap, fmt );
         (void) im_vsnprintf( buf, 1024, fmt, ap );
         va_end( ap );
 
-	/* So the last OK button added is the default one (and at the head of
-	 * the list). [OK1, OK2, OK3, OK4]
-	 */
-	idlg->ok_l = g_slist_prepend( idlg->ok_l, 
-		okbutton_new( buf, done_cb ) );
+	idlg->ok_cb_l = g_slist_append( idlg->ok_cb_l, (void *) done_cb );
+	idlg->ok_txt_l = g_slist_append( idlg->ok_txt_l, 
+		im_strdup( NULL, buf ) );
 }
 
 void 
@@ -745,17 +728,20 @@ idialog_set_build( iDialog *idlg,
 	idlg->build_c = build_c;
 }
 
-void 
-idialog_set_cancel_text( iDialog *idlg, const char *cancel_text )
-{
-	idlg->cancel_text = cancel_text;
-}
-
 void 
 idialog_set_default_entry( iDialog *idlg, GtkEntry *entry )
 {
-	gtk_entry_set_activates_default( entry, TRUE );
-	idlg->entry = entry;
+	if( !idlg->nook && idlg->ok_l ) {
+		gtk_signal_connect_object( GTK_OBJECT( entry ), 
+			"focus_in_event", 
+			GTK_SIGNAL_FUNC( gtk_widget_grab_default ),
+			GTK_OBJECT( idlg->ok_l->data ) );
+		gtk_signal_connect_object( GTK_OBJECT( entry ), 
+			"activate", 
+			GTK_SIGNAL_FUNC( gtk_button_clicked ),
+			GTK_OBJECT( idlg->ok_l->data ) );
+		idlg->entry = entry;
+	}
 }
 
 /* Set up an entry inside a dialog ... set tooltip, set start
