@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -47,21 +47,21 @@ typedef struct _iDialog iDialog;
 typedef void (*iDialogFreeFn)( iDialog *, void * );
 
 struct _iDialog {
-	iWindow parent_object;
+	iWindow parent;
 
 	/* My instance vars.
 	 */
-	iObject *iobject;		/* Kill dialog if this obj goes */
-	guint destroy_sid;		/* Signal id for obj destroy */
+	GtkWidget *window_parent;	/* Our parent widget */
+	guint parent_sid;		/* Signal id for parent destroy */
+	GtkObject *object;		/* Kill dialog if this obj goes */
+	guint object_sid;		/* Signal id for parent destroy */
 
 	GtkWidget *work;		/* Our work area */
 	GtkWidget *hb;			
 	GtkWidget *bb;
 
-	GSList *ok_l;			/* List of OKbutton as set by user */
-	GSList *ok_disp_l;		/* List of OKbutton as displayed */
-	GSList *ok_but_l;		/* List of OK GtkButton as displayed */
-
+	GSList *ok_l;			/* List of OK buttons */
+	GSList *ok_txt_l;		/* List of text of OK buttons */
 	GtkWidget *but_cancel;
 	GtkWidget *but_help;
 	GtkWidget *tog_pin;		/* Optional pinup widget */
@@ -70,21 +70,10 @@ struct _iDialog {
 
 	/* Flags.
 	 */
+	gboolean nook;			/* No default-ok */
 	gboolean modal;			/* Modal/non-modal */
 	gboolean pinup;			/* Stay up on OK */
-	gboolean nosep;			/* Suppress hseparator */
-	gboolean button_focus;		/* TRUE to focus buttons */
-
-	/* Name of help tag ... if set, make a help button and link to display
-	 * of this.
-	 */
-	char *help_tag;
-
-	/* What we label the cancel button as (if any). Usually
-	 * GTK_STOCK_CANCEL, but instant-apply dialogs should change this to
-	 * GTK_STOCK_CLOSE.
-	 */
-	const char *cancel_text;
+	gboolean nosep;			/* Supress hseparator */
 
 	/* Per-instance build function.
 	 */
@@ -93,7 +82,9 @@ struct _iDialog {
 
 	/* Our callbacks.
 	 */
+	GSList *ok_cb_l;		/* List of OK callbacks */
 	iWindowFn cancel_cb;
+	iWindowFn help_cb;
 	iWindowFn popdown_cb;
 	iDialogFreeFn destroy_cb;	/* Called from _destroy() */
 	void *client;			/* Client data for callbacks */
@@ -120,24 +111,21 @@ void idialog_set_ok_button_state( iDialog *idlg, gboolean state );
 GtkType idialog_get_type( void );
 GtkWidget *idialog_new( void );
 
-void idialog_set_iobject( iDialog *idlg, iObject *iobject );
+void idialog_set_parent( iDialog *, GtkWidget *par );
+void idialog_set_object( iDialog *idlg, GtkObject *object );
+void idialog_set_nook( iDialog *, gboolean );
 void idialog_set_modal( iDialog *, gboolean );
 void idialog_set_pinup( iDialog *idlg, gboolean pinup );
 void idialog_set_nosep( iDialog *, gboolean );
-void idialog_set_button_focus( iDialog *idlg, gboolean button_focus );
-void idialog_set_help_tag( iDialog *, const char *help_tag );
 void idialog_set_callbacks( iDialog *, 
-	iWindowFn cancel_cb, iWindowFn popdown_cb, 
+	iWindowFn cancel_cb, iWindowFn help_cb, iWindowFn popdown_cb, 
 	iDialogFreeFn destroy_cb, void *client );
-void idialog_add_ok( iDialog *, iWindowFn done_cb, const char *fmt, ... )
-	__attribute__((format(printf, 3, 4)));
+void idialog_add_ok( iDialog *, iWindowFn done_cb, const char *fmt, ... );
 void idialog_set_notify( iDialog *, iWindowNotifyFn, void * );
 void idialog_set_build( iDialog *, iWindowBuildFn, void *, void *, void * );
-void idialog_set_cancel_text( iDialog *, const char *cancel_text );
 void idialog_set_default_entry( iDialog *idlg, GtkEntry *entry );
 void idialog_init_entry( iDialog *idlg, GtkWidget *entry, 
-	const char *tip, const char *fmt, ... )
-	__attribute__((format(printf, 4, 5)));
+	const char *tip, const char *fmt, ... );
 
 void idialog_done_trigger( iDialog *idlg, int pos );
 
