@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -28,45 +28,65 @@
  */
 
 #define TYPE_IMAGEPRESENT (imagepresent_get_type())
-#define IMAGEPRESENT( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_IMAGEPRESENT, Imagepresent ))
-#define IMAGEPRESENT_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), \
-		TYPE_IMAGEPRESENT, ImagepresentClass))
-#define IS_IMAGEPRESENT( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_IMAGEPRESENT ))
-#define IS_IMAGEPRESENT_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_IMAGEPRESENT ))
-#define IMAGEPRESENT_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), \
-		TYPE_IMAGEPRESENT, ImagepresentClass ))
+#define IMAGEPRESENT(obj) GTK_CHECK_CAST( obj, \
+	imagepresent_get_type(), Imagepresent )
+#define IMAGEPRESENT_CLASS(klass)  GTK_CHECK_CLASS_CAST( klass, \
+	imagepresent_get_type(), ImagepresentClass )
+#define IS_IMAGEPRESENT(obj) GTK_CHECK_TYPE( obj, imagepresent_get_type() )
+
+/* States we can be in ... modal stuff, scarey.
+ */
+typedef enum _ImagepresentState {
+	IMAGEPRESENT_STATE_SELECT = 0,	/* Pointer */
+	IMAGEPRESENT_STATE_PAN,		/* Hand panner */
+	IMAGEPRESENT_STATE_MAGIN,	/* Zoom in */
+	IMAGEPRESENT_STATE_MAGOUT,	/* Zoom out */
+	IMAGEPRESENT_STATE_PAINT,       /* Painting */
+	IMAGEPRESENT_STATE_LAST	
+} ImagepresentState;
 
 /* Track an image view canvas in one of these.
  */
 struct _Imagepresent {
-	GtkBin parent_object;
+	GtkVBox parent_class;	/* Main layout vbox */
 
 	/* Context.
 	 */
-	Imagemodel *imagemodel;		/* Keep model parts of widgets here */
+	Iimage *iimage;		/* Iimage we represent, if any */
 
 	/* Sub-widgets.
 	 */
-	Imagedisplay *id;		/* Image we display */
-	GtkScrolledWindow *swin;
-	GtkAdjustment *hadj;
-	GtkAdjustment *vadj;
-	GtkHRuler *hrule;		/* Rulers */
+	Imagedisplay *id;	/* Image we display */
+	GtkHScrollbar *hbar;	/* Scrollbars */
+	GtkVScrollbar *vbar;
+	GtkHRuler *hrule;	/* Rulers */
 	GtkVRuler *vrule;
-	GtkEventBox *heb;		/* EventBoxes holding rulers */
+	GtkEventBox *heb;	/* EventBoxes holding rulers */
 	GtkEventBox *veb;
+	GtkTooltips *tooltips;
+	GtkAccelGroup *group;	/* Popup menu stuff */
+	GtkWidget *menu;
+	GtkWidget *tool[IMAGEPRESENT_STATE_LAST];
+
+	/* Need to spot changes to our conversion.
+	 */
+	guint conv_changed_sid;
+	Imageinfo *last_paint_ii;
+
+	/* Input state, and a saved state during temp actions.
+	 */
+	ImagepresentState state;
+	ImagepresentState save_state; 
 	iWindowCursorContext *cntxt;
-	GtkWidget *ruler_menu;
 
 	/* Panning stuff.
 	 */
 	guint dx, dy;		/* Drag start position */
 
+	/* Display state of widgets.
+	 */
+	gboolean show_rulers;	/* Rulers shown/not */
+
 	/* Last known mouse position, mouse in window.
 	 */
 	int last_x, last_y;
@@ -74,7 +94,7 @@ struct _Imagepresent {
 
 	/* Background scroll stuff.
 	 */
-	guint scroll_tid;
+	gint scroll_timeout;
 	int u, v;
 
 	/* Rubberbanding.
@@ -90,11 +110,6 @@ struct _Imagepresent {
 	/* Regionviews drawing on us. Used for snap-to-guide stuff.
 	 */
 	GSList *regionviews;
-
-	/* The regionview that's currently grabbed ... maintained for us by
-	 * regionview.c ... see regionview_attach()/_detach()
-	 */
-	Regionview *grabbed;
 };
 
 /* Class structure.
@@ -102,21 +117,26 @@ struct _Imagepresent {
 typedef struct _ImagepresentClass {
 	/* Our parent.
 	 */
-	GtkBinClass parent_class;
+	GtkVBoxClass parent_class;
+
+	/* Signals.
+	 */
+	void (*changed)( Imagepresent * );
 } ImagepresentClass;
 
+void imagepresent_set_state( Imagepresent *ip, ImagepresentState state );
+
 gboolean imagepresent_snap_point( Imagepresent *ip, 
 	int x, int y, int *sx, int *sy );
 gboolean imagepresent_snap_rect( Imagepresent *ip, Rect *in, Rect *out );
 
 void imagepresent_paint_recalc( Imagepresent *ip );
 
-GType imagepresent_get_type( void );
-Imagepresent *imagepresent_new( Imagemodel *imagemodel );
+guint imagepresent_get_type( void );
+Imagepresent *imagepresent_new( Iimage *iimage, Conversion *conv );
 
 void imagepresent_set_position( Imagepresent *ip, int x, int w );
-void imagepresent_set_mag_pos( Imagepresent *ip, int mag, int ix, int iy );
-void imagepresent_zoom_to( Imagepresent *ip, int mag );
+void imagepresent_rulers( Imagepresent *ip, gboolean show );
 
 void imagepresent_scroll_start( Imagepresent *ip, int u, int v );
 void imagepresent_scroll_stop( Imagepresent *ip );
