@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -36,21 +36,25 @@
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_IMAGEVIEW ))
 
 typedef struct _Imageview {
-	Floatwindow parent_class;
-
-	/* Model stuff here.
-	 */
-	Imagemodel *imagemodel;
-	guint imagemodel_changed_sid;
+	iWindow parent_class;
 
+	Iimage *iimage;
 	Imagepresent *ip;
+	Conversion *conv;
+
+	guint changed_sid;
+	guint destroy_sid;
+	guint conv_changed_sid;
+	guint ws_changed_sid;
+
 	Conversionview *cv;
 	Statusview *sv;
 	Paintboxview *pbv;
+	GtkItemFactory *ifac;
 } Imageview;
 
 typedef struct _ImageviewClass {
-	FloatwindowClass parent_class;
+	iWindowClass parent_class;
 
 	/* My methods.
 	 */
@@ -58,5 +62,6 @@ typedef struct _ImageviewClass {
 
 GtkType imageview_get_type( void );
 void imageview_set_paint( Imageview *iv, gboolean paint );
-Imageview *imageview_new( iImage *iimage, GtkWidget *parent );
-Imageview *imageview_new_area( iImage *iimage, Rect *area, GtkWidget *parent );
+Imageview *imageview_new( Iimage *iimage );
+Imageview *imageview_new_area( Iimage *iimage, Rect *area );
+
