@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,19 +27,6 @@
 
  */
 
-/* We can get multiple includes sometimes, gah, thank you bison.
- */
-#ifndef IP_H
-#define IP_H
-
-/* DEBUG everywhere.
-#define DEBUG
- */
-
-/* Turn off VIPS's old and broken defines, we don't need them.
- */
-#define IM_NO_VIPS7_COMPAT
-
 /* Enable heap sanity checks on every alloc ... very slow ... also see heap.c
 #define DEBUG_HEAP
  */
@@ -48,26 +35,7 @@
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) gettext(String)
-#ifdef gettext_noop
-#define N_(String) gettext_noop(String)
-#else
-#define N_(String) (String)
-#endif
-#else /* NLS is disabled */
-#define _(String) (String)
-#define N_(String) (String)
-#define textdomain(String) (String)
-#define gettext(String) (String)
-#define dgettext(Domain,String) (String)
-#define dcgettext(Domain,String,Type) (String)
-#define bindtextdomain(Domain,Directory) (Domain) 
-#define bind_textdomain_codeset(Domain,Codeset) (Codeset) 
-#define ngettext(S, P, N) ((N) == 1 ? (S) : (P))
-#endif /* ENABLE_NLS */
-
+#include <assert.h>
 #include <time.h>
 #include <ctype.h>
 #include <errno.h>
@@ -100,9 +68,15 @@
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif /*HAVE_SYS_WAIT_H*/
+#ifdef HAVE_DIRENT_H
+#include <dirent.h>
+#endif /*HAVE_DIRENT_H*/
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /*HAVE_UNISTD_H*/
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+#endif /*HAVE_REGEX_H*/
 #ifdef HAVE_SYS_STATVFS_H
 #include <sys/statvfs.h>
 #endif /*HAVE_SYS_STATVFS_H*/
@@ -113,140 +87,101 @@ extern int statfs();
 #ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif /*HAVE_SYS_MOUNT_H*/
-#ifdef OS_WIN32
+#ifdef HAVE_WINDOWS_H
 #include <windows.h>
-#endif /*OS_WIN32*/
+#endif /*HAVE_WINDOWS_H*/
 #ifdef HAVE_FFTW
 #include <fftw.h>
 #endif /*HAVE_FFTW*/
-#ifdef HAVE_FFTW3
-#include <fftw3.h>
-#endif /*HAVE_FFTW3*/
-#include <fcntl.h>
 
 /* Have to include glib before dmalloc ... dmalloc may be included by vips.h
  */
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#ifdef HAVE_LIBGOFFICE
-#include <goffice/goffice.h>
-
-#include <goffice/app/go-plugin.h>
-#include <goffice/app/go-plugin-loader-module.h>
-
-#include <goffice/data/go-data-simple.h>
-
-#include <goffice/graph/gog-data-set.h>
-#include <goffice/graph/gog-label.h>
-#include <goffice/graph/gog-object.h>
-#include <goffice/graph/gog-plot.h>
-#include <goffice/graph/gog-series.h>
-#include <goffice/graph/gog-grid.h>
-#include <goffice/graph/gog-grid-line.h>
-#include <goffice/graph/gog-legend.h>
-#include <goffice/graph/gog-chart-map.h>
-
-
-#include <goffice/utils/go-color.h>
-#include <goffice/utils/go-marker.h>
-#endif /*HAVE_LIBGOFFICE*/
-
-#ifdef HAVE_LIBGVC
-#include <gvc.h>
-#endif /*HAVE_LIBGVC*/
-
 #include <vips/vips.h>
-#include <vips/vips7compat.h>
-#include <vips/debug.h>
+#include <vips/history.h>
+#include <vips/util.h>
+#include <vips/list.h>
+#include <vips/rect.h>
+#include <vips/colour.h>
+#include <vips/region.h>
+#include <vips/thread.h>
+#include <vips/threadgroup.h>
+#include <vips/dispatch.h>
+#include <vips/time.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
-/* If we're not using GNU C, elide __attribute__ 
- */
-#ifndef __GNUC__
-#  ifndef __attribute__
-#    define  __attribute__(x)  /*NOTHING*/
-#  endif
-#endif
-
 /* Our general widgets.
  */
-#include "formula.h"
+#include "gtkmenubar2.h"
+#include "orderitem.h"
+#include "orderlist.h"
+#include "led.h"
 #include "doubleclick.h"
 
-/* Generated marshallers.
+/* Most of our #defines.
  */
-#include "nipmarshal.h"
-
-/* XML namespace ... note, not nip2! We can't change this.
- */
-#define NAMESPACE "http://www.vips.ecs.soton.ac.uk/nip" 
-
+#define NAMELEN (256)		/* Length of a file name */
+#define PATHLEN PATH_MAX	/* Length of a path name */
 #define MAXFILES (4000)		/* Max. no of files in path */
 #define STACK_SIZE (1000)	/* Depth of eval stack */
 #define LEN_LABEL (512)		/* Label on windows */
+#define TABLE_SIZE (2133)	/* Size of large hash tables */
+#define SMALL_TABLE_SIZE (5)	/* Size of small hash tables */
 #define MAX_SYSTEM (50)		/* Max number of args we allow */
-#define MAX_BANDS (64)		/* Max number of bands in image */
 #define MAX_CSTACK (10)		/* Max number of cursors we stack */
-#define MAX_STRSIZE (100000)	/* Size of text for user defs */
+#define MAX_STRSIZE (16384)	/* Size of text for user defs */
 #define MAX_TRACE (1024)	/* Biggest thing we print in trace */
-#define MAX_SSTACK (40)		/* Scope stack for parser */
-#define VIPS_HOMEPAGE "https://github.com/jcupitt/nip2"
-#define IP_NAME PACKAGE "-" VERSION
-#define NIP_DOCPATH "$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S \
-        "doc" G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "html"
-#define VIPS_DOCPATH "$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S \
-	"doc" G_DIR_SEPARATOR_S "vips" G_DIR_SEPARATOR_S "html"
+#define MAX_SSTACK (20)		/* Scope stack for parser */
+#define _(A) (A)		/* Avoid i18l issues for now */
+#define VIPS_HOMEPAGE "http://www.vips.ecs.soton.ac.uk"
+#define IP_COPYRIGHT PACKAGE ": (c) 2003 The National Gallery"
+#define NIP_DOCPATH "$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR \
+	"doc" IM_DIR_SEP_STR PACKAGE IM_DIR_SEP_STR "html"
+#define VIPS_DOCPATH "$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR \
+	"doc" IM_DIR_SEP_STR "vips" IM_DIR_SEP_STR "html"
 #define IP_NAME PACKAGE "-" VERSION
+#define IP_SAVEDIR "$HOME" IM_DIR_SEP_STR "." IP_NAME
+#define NAMESPACE VIPS_HOMEPAGE "/" PACKAGE 
+				/* XML namespace */
+#define FULL_NAMESPACE NAMESPACE "/" VERSION
+				/* Namespace with version info */
 #define MAX_LINELENGTH (120)	/* Max chars we display of value */
-#define MAX_RECENT (10)		/* Number of recent items in file menu */
-#define NIP_COPYRIGHT "%s: &#0169;2023 libvips.org"
-
-/* Our stock_ids.
- */
-#define STOCK_NEXT_ERROR "nip-next-error"
-#define STOCK_DROPPER "nip-dropper"
-#define STOCK_DUPLICATE "nip-duplicate"
-#define STOCK_PAINTBRUSH "nip-paintbrush"
-#define STOCK_LINE "nip-linedraw"
-#define STOCK_TEXT "nip-text"
-#define STOCK_SMUDGE "nip-smudge"
-#define STOCK_FLOOD "nip-flood"
-#define STOCK_FLOOD_BLOB "nip-floodblob"
-#define STOCK_RECT "nip-rect"
-#define STOCK_MOVE "nip-move"
-#define STOCK_LOCK "nip-lock"
-#define STOCK_ALERT "nip-alert"
-#define STOCK_SELECT "nip-select"
-#define STOCK_LED_RED "nip-led-red"
-#define STOCK_LED_GREEN "nip-led-green"
-#define STOCK_LED_BLUE "nip-led-blue"
-#define STOCK_LED_YELLOW "nip-led-yellow"
-#define STOCK_LED_CYAN "nip-led-cyan"
-#define STOCK_LED_OFF "nip-led-off"
 
 /* How much we decompile for error messages.
  */
 #define MAX_ERROR_FRAG (100)
 
-/* win32 adds '_', sometimes. 
+/* Handle broken mkdirs()
  */
-#ifdef OS_WIN32
-#ifndef popen
+#if HAVE_MKDIR
+# if MKDIR_TAKES_ONE_ARG
+   /* Mingw32 */
+#  define mkdir(a,b) mkdir(a)
+# endif
+#else
+# ifdef HAVE__MKDIR
+   /* plain Win32 */
+#  include <direct.h>
+#  define mkdir(a,b) _mkdir(a)
+# else
+#  error "Don't know how to create a directory on this system."
+# endif
+#endif
+
+/* win32 adds '_'.
+ */
+#ifdef HAVE_WINDOWS_H
 #define popen(b,m) _popen(b,m)
-#endif /*popen*/
-#ifndef pclose
 #define pclose(f) _pclose(f)
-#endif /*pclose*/
 #define mktemp(f) _mktemp(f)
-#endif /*OS_WIN32*/
+#endif
 
 /* Fwd ref these.
  */
-typedef struct _Watch Watch;
-typedef struct _Toolitem Toolitem;
 typedef struct _BuiltinInfo BuiltinInfo;
 typedef struct _Classmodel Classmodel;
 typedef struct _Colour Colour;
@@ -260,30 +195,21 @@ typedef struct _Filemodel Filemodel;
 typedef struct _Heap Heap;
 typedef struct _HeapBlock HeapBlock;
 typedef struct _Heapmodel Heapmodel;
-typedef struct _iArrow iArrow;
-typedef struct _iImage iImage;
+typedef struct _Iarrow Iarrow;
+typedef struct _Iimage Iimage;
 typedef struct _Imagedisplay Imagedisplay;
-typedef struct _Managed Managed;
-typedef struct _Managedfile Managedfile;
-typedef struct _Managedgvalue Managedgvalue;
-typedef struct _Managedgobject Managedgobject;
-typedef struct _Managedstring Managedstring;
 typedef struct _Imageinfo Imageinfo;
 typedef struct _Imagepresent Imagepresent;
-typedef struct _Imagemodel Imagemodel;
-typedef struct _iRegion iRegion;
-typedef struct _iRegiongroup iRegiongroup;
+typedef struct _Iregion Iregion;
+typedef struct _Iregiongroup Iregiongroup;
 typedef struct _Link Link;
 typedef struct _LinkExpr LinkExpr;
 typedef struct _Model Model;
-typedef struct _iObject iObject;
-typedef struct _iContainer iContainer;
 typedef struct _Paintboxview Paintboxview;
 typedef struct _ParseConst ParseConst;
 typedef struct _ParseNode ParseNode;
 typedef struct _Program Program;
-typedef struct _String String;
-typedef struct _Number Number;
+typedef struct _Real Real;
 typedef struct _Reduce Reduce;
 typedef struct _Regionview Regionview;
 typedef struct _Rhs Rhs;
@@ -291,14 +217,9 @@ typedef struct _Rhsview Rhsview;
 typedef struct _Row Row;
 typedef struct _Rowview Rowview;
 typedef struct _Statusview Statusview;
-typedef struct _Plotstatus Plotstatus;
-typedef struct _Plot Plot;
-typedef struct _Plotwindow Plotwindow;
-typedef struct _Plotpresent Plotpresent;
-typedef struct _Plotmodel Plotmodel;
-typedef struct _Graphwindow Graphwindow;
 typedef struct _Subcolumn Subcolumn;
 typedef struct _Subcolumnview Subcolumnview;
+typedef struct _SymTable SymTable;
 typedef struct _Symbol Symbol;
 typedef struct _Tool Tool;
 typedef struct _Toolkit Toolkit;
@@ -307,71 +228,50 @@ typedef struct _Toolkitgroupview Toolkitgroupview;
 typedef struct _Toolkitview Toolkitview;
 typedef struct _Toolview Toolview;
 typedef struct _Trace Trace;
-typedef struct _Preview Preview;
-typedef struct _Infobar Infobar;
-typedef struct _iError iError;
-typedef struct _Log Log;
-typedef struct _vObject vObject;
 typedef struct _View View;
 typedef struct _Workspace Workspace;
-typedef struct _Workspaceview Workspaceview;
-typedef struct _Workspaceroot Workspaceroot;
 typedef struct _Workspacegroup Workspacegroup;
 typedef struct _Workspacegroupview Workspacegroupview;
-typedef struct _Prefworkspaceview Prefworkspaceview;
-typedef struct _Prefcolumnview Prefcolumnview;
+typedef struct _Workspaceview Workspaceview;
 typedef struct _iText iText;
-typedef struct _Expression Expression;
-typedef struct _Mainw Mainw;
-typedef struct _Toolviewitemgroup Toolviewitemgroup;
-typedef struct _Panechild Panechild;
-typedef struct _Toolkitbrowser Toolkitbrowser;
-typedef struct _Workspacedefs Workspacedefs;
 
 /* container map function typedefs.
  */
 typedef void *(*row_map_fn)( Row *, void *, void *, void * );
 typedef void *(*symbol_map_fn)( Symbol *, void *, void *, void * );
 typedef void *(*column_map_fn)( Column *, void * );
+typedef void *(*model_map_fn)( Model *, void *, void * );
+typedef void *(*model_map3_fn)( Model *, void *, void *, void * );
+typedef void *(*model_map4_fn)( Model *, void *, void *, void *, void * );
+typedef void *(*model_map5_fn)( Model *, void *, void *, void *, void *, 
+	void * );
 typedef void *(*view_map_fn)( View *, void *, void * );
 typedef void *(*rowview_map_fn)( Rowview *, void * );
 typedef void *(*workspace_map_fn)( Workspace *, void * );
 typedef void *(*toolkit_map_fn)( Toolkit *, void *, void * );
-typedef void *(*tool_map_fn)( Tool *, void *, void * );
 
 /* Util stuff.
  */
+#include "wild.h"
+#include "path.h"
 #include "util.h"
 #include "gtkutil.h"
-#include "path.h"
-#include "iobject.h"
-#include "icontainer.h"
 #include "iwindow.h"
 #include "idialog.h"
 #include "boxes.h"
-#include "popupbutton.h"
-#include "imageheader.h"
+#include "gtkfilesel2.h"	/* hacked to derive from idialog */
 #include "filesel.h"
-#include "managed.h"
-#include "managedfile.h"
-#include "managedgvalue.h"
-#include "managedgobject.h"
 #include "imageinfo.h"
 #include "imagedisplay.h"
 #include "colourdisplay.h"
-#include "imagemodel.h"
 #include "imagepresent.h"
-#include "floatwindow.h"
 #include "imageview.h"
 #include "tslider.h"
-#include "pane.h"
-#include "progress.h"
 
 /* Basic ip includes (order important).
  */
 #include "tree.h"
 #include "heap.h"
-#include "managedstring.h"
 #include "class.h"
 #include "link.h"
 #include "expr.h"
@@ -383,16 +283,12 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "filemodel.h"
 #include "symbol.h"
 #include "workspace.h"
-#include "workspaceroot.h"
 #include "workspacegroup.h"
 #include "toolkitgroup.h"
 #include "secret.h"
 #include "action.h"
 #include "reduce.h"
-#include "vobject.h"
-#include "vipsobject.h"
 #include "view.h"
-#include "graphicview.h"
 #include "spin.h"
 #include "row.h"
 #include "rowview.h"
@@ -410,81 +306,115 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "toolkitview.h"
 #include "toolview.h"
 #include "watch.h"
-#include "value.h"
-#include "panechild.h"
 
 /* Per module includes, any order
  */
-#include "workspacedefs.h"
-#include "toolkitbrowser.h"
-#include "defbrowser.h"
-#include "log.h"
-#include "error.h"
+#include "asynch.h"
 #include "trace.h"
 #include "program.h"
 #include "conversionview.h"
 #include "statusview.h"
-#include "plotstatus.h"
 #include "mainw.h"
-#include "preview.h"
+#include "browse.h"
 #include "builtin.h"
+#include "command.h"
 #include "compile.h"
 #include "dump.h"
+#include "graph.h"
 #include "main.h"
 #include "predicate.h"
 #include "slider.h"
-#include "clock.h"
-#include "pathname.h"
-#include "fontname.h"
-#include "group.h"
-#include "real.h"
-#include "vector.h"
+#include "filename.h"
 #include "colour.h"
-#include "number.h"
-#include "istring.h"
-#include "editview.h"
-#include "expression.h"
-#include "expressionview.h"
-#include "stringview.h"
-#include "numberview.h"
+#include "real.h"
+#include "realview.h"
 #include "matrix.h"
 #include "matrixview.h"
-#include "plot.h"
-#ifdef HAVE_LIBGOFFICE
-#include "plotview.h"
-#endif /*HAVE_LIBGOFFICE*/
-#include "plotmodel.h"
-#include "plotpresent.h"
-#include "plotwindow.h"
-#include "graphwindow.h"
 #include "option.h"
 #include "optionview.h"
 #include "iimage.h"
 #include "iregion.h"
 #include "iregiongroup.h"
 #include "iarrow.h"
-#include "valueview.h"
+#include "iarrowview.h"
 #include "sliderview.h"
-#include "pathnameview.h"
-#include "fontnameview.h"
+#include "filenameview.h"
 #include "colourview.h"
 #include "iimageview.h"
 #include "iregionview.h"
 #include "iregiongroupview.h"
-#include "prefs.h"
-#include "prefworkspaceview.h"
-#include "prefcolumnview.h"
 #include "regionview.h"
+#include "table.h"
 #include "itext.h"
 #include "itextview.h"
 #include "toggle.h"
 #include "toggleview.h"
-#include "call.h"
-#include "cache.h"
-#include "parser.h"
+#include "vips_call.h"
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
-#endif /*IP_H*/
+/* Global variables from parse.y.
+
+	FIXME move to its own header ... parser.h?
+
+ */
+
+/* Our input stream can be attached to either a string or a FILE. 
+ * Keep track of the state of play here.
+ */
+typedef struct {
+	iOpenFile *of;		/* Non-NULL if we read from a file */
+	char *str;		/* Non-NULL if we read from a string */
+	char *strpos;		/* Position in string */
+
+	char buf[MAX_STRSIZE];	/* Accumulate text of each definition here */
+	int bwp;		/* Write point in the above */
+	int bsp[MAX_SSTACK];	/* Start point stack */
+	int bspsp;		/* Stack pointer */
+
+	int lineno;		/* Current line number */
+	int charno;		/* Character in line */
+	int pcharno;		/* Characters in previous line */
+	int charpos;		/* Characters read by lex so far */
+
+	int oldchar;            /* unget buffer, -1 for no unget */
+} InputState;
+
+extern InputState input_state;
+
+/* Function declarations for parse.y.
+ */
+void yyerror( const char *fmt, ... );	/* Error during processing of expr */
+extern int yyleng;			/* lex stuff */
+
+/* Lex gathers tokens here for workspace.c
+ */
+extern BufInfo lex_text;
+
+/* Attach input for lex.
+ */
+void attach_input_file( iOpenFile *of );
+void attach_input_string( const char *str );
+int ip_input( void );
+void ip_unput( int ch );
+gboolean is_EOF( void );
+
+/* Parse stuff.
+ */
+
+/* Order and number important ... see table in parse_rhs()
+ */
+typedef enum {
+	PARSE_RHS = 0,		/* eg. "a + b" */
+	PARSE_PARAMS,		/* eg. "a b = a + b" */
+	PARSE_SUPER 		/* eg. "fred c d" */
+} ParseRhsSyntax;
+
+gboolean parse_toplevel( Toolkit *kit, int pos );
+gboolean parse_onedef( Toolkit *kit, int pos );
+gboolean parse_rhs( Expr *expr, ParseRhsSyntax syntax, gboolean optimise );
+
+extern Symbol *last_top_sym;
+
