@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -35,6 +35,26 @@
 
 static ClassmodelClass *parent_class = NULL;
 
+static void
+iregiongroup_destroy( GtkObject *object )
+{
+	Iregiongroup *iregiongroup;
+
+#ifdef DEBUG
+	printf( "iregiongroup_destroy\n" );
+#endif /*DEBUG*/
+
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_IREGIONGROUP( object ) );
+
+	iregiongroup = IREGIONGROUP( object );
+
+	/* My instance destroy stuff.
+	 */
+
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
+}
+
 static void *
 iregiongroup_update_model( Heapmodel *heapmodel )
 {
@@ -50,79 +70,71 @@ iregiongroup_update_model( Heapmodel *heapmodel )
 	/* Only display most-derived classes. Don't display "this".
 	 */
 	if( heapmodel->row->sym )
-		model_display( MODEL( heapmodel ),
+		model_set_display( MODEL( heapmodel ),
 			!is_super( heapmodel->row->sym ) &&
 			!is_this( heapmodel->row->sym ) );
 
 	return( NULL );
 }
 
-static View *
-iregiongroup_view_new( Model *model, View *parent )
-{
-	return( iregiongroupview_new() );
-}
-
 static void
-iregiongroup_class_init( iRegiongroupClass *class )
+iregiongroup_class_init( IregiongroupClass *klass )
 {
-	ModelClass *model_class = (ModelClass *) class;
-	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	ModelClass *model_class = (ModelClass *) klass;
+	HeapmodelClass *heapmodel_class = (HeapmodelClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_CLASSMODEL );
 
 	/* Create signals.
 	 */
 
 	/* Init methods.
 	 */
+	object_class->destroy = iregiongroup_destroy;
+
 	heapmodel_class->update_model = iregiongroup_update_model;
 
-	model_class->view_new = iregiongroup_view_new;
+	model_class->view_new = iregiongroupview_new;
 }
 
 static void
-iregiongroup_init( iRegiongroup *iregiongroup )
+iregiongroup_init( Iregiongroup *iregiongroup )
 {
 	/* Display turned on in _update_model() above.
 	 */
 	MODEL( iregiongroup )->display = FALSE;
 }
 
-GType
+GtkType
 iregiongroup_get_type( void )
 {
-	static GType type = 0;
+	static GtkType iregiongroup_type = 0;
 
-	if( !type ) {
-		static const GTypeInfo info = {
-			sizeof( iRegiongroupClass ),
-			NULL,           /* base_init */
-			NULL,           /* base_finalize */
-			(GClassInitFunc) iregiongroup_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
-			sizeof( iRegiongroup ),
-			32,             /* n_preallocs */
-			(GInstanceInitFunc) iregiongroup_init,
+	if( !iregiongroup_type ) {
+		static const GtkTypeInfo info = {
+			"Iregiongroup",
+			sizeof( Iregiongroup ),
+			sizeof( IregiongroupClass ),
+			(GtkClassInitFunc) iregiongroup_class_init,
+			(GtkObjectInitFunc) iregiongroup_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
 		};
 
-		type = g_type_register_static( TYPE_CLASSMODEL, 
-			"iRegiongroup", &info, 0 );
+		iregiongroup_type = gtk_type_unique( TYPE_CLASSMODEL, &info );
 	}
 
-	return( type );
+	return( iregiongroup_type );
 }
 
-iRegiongroup *
+Iregiongroup *
 iregiongroup_new( Classmodel *classmodel )
 {
-	iRegiongroup *iregiongroup;
+	Iregiongroup *iregiongroup = gtk_type_new( TYPE_IREGIONGROUP );
 
-	iregiongroup = IREGIONGROUP( g_object_new( TYPE_IREGIONGROUP, NULL ) );
-
-	icontainer_child_add( ICONTAINER( classmodel ), 
-		ICONTAINER( iregiongroup ), -1 );
+	model_child_add( MODEL( classmodel ), MODEL( iregiongroup ), -1 );
 
 #ifdef DEBUG
 	printf( "iregiongroup_new: " );
