@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,30 +29,26 @@
 
 #define TYPE_IREGIONGROUPVIEW (iregiongroupview_get_type())
 #define IREGIONGROUPVIEW( obj ) \
-	(GTK_CHECK_CAST( (obj), TYPE_IREGIONGROUPVIEW, iRegiongroupview ))
+	(GTK_CHECK_CAST( (obj), TYPE_IREGIONGROUPVIEW, Iregiongroupview ))
 #define IREGIONGROUPVIEW_CLASS( klass ) \
 	(GTK_CHECK_CLASS_CAST( (klass), TYPE_IREGIONGROUPVIEW, \
-		iRegiongroupviewClass ))
+		IregiongroupviewClass ))
 #define IS_IREGIONGROUPVIEW( obj ) \
 	(GTK_CHECK_TYPE( (obj), TYPE_IREGIONGROUPVIEW ))
 #define IS_IREGIONGROUPVIEW_CLASS( klass ) \
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_IREGIONGROUPVIEW ))
 
-typedef struct _iRegiongroupview {
+typedef struct _Iregiongroupview {
 	View parent_class;
 
-	/* Keep our classmodel here, we need it during destroy.
-	 */
-	Classmodel *classmodel;
+} Iregiongroupview;
 
-} iRegiongroupview;
-
-typedef struct _iRegiongroupviewClass {
+typedef struct _IregiongroupviewClass {
 	ViewClass parent_class;
 
 	/* My methods.
 	 */
-} iRegiongroupviewClass;
+} IregiongroupviewClass;
 
 GtkType iregiongroupview_get_type( void );
 View *iregiongroupview_new( void );
