@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,24 +29,24 @@
 
 #define TYPE_IREGIONVIEW (iregionview_get_type())
 #define IREGIONVIEW( obj ) \
-	(GTK_CHECK_CAST( (obj), TYPE_IREGIONVIEW, iRegionview ))
+	(GTK_CHECK_CAST( (obj), TYPE_IREGIONVIEW, Iregionview ))
 #define IREGIONVIEW_CLASS( klass ) \
-	(GTK_CHECK_CLASS_CAST( (klass), TYPE_IREGIONVIEW, iRegionviewClass ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_IREGIONVIEW, IregionviewClass ))
 #define IS_IREGIONVIEW( obj ) (GTK_CHECK_TYPE( (obj), TYPE_IREGIONVIEW ))
 #define IS_IREGIONVIEW_CLASS( klass ) \
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_IREGIONVIEW ))
 
-typedef struct _iRegionview {
-	iImageview parent_class;
+typedef struct _Iregionview {
+	Iimageview parent_class;
 
-} iRegionview;
+} Iregionview;
 
-typedef struct _iRegionviewClass {
-	iImageviewClass parent_class;
+typedef struct _IregionviewClass {
+	IimageviewClass parent_class;
 
 	/* My methods.
 	 */
-} iRegionviewClass;
+} IregionviewClass;
 
 GtkType iregionview_get_type( void );
 View *iregionview_new( void );
