@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -35,15 +35,21 @@
 
 static HeapmodelClass *parent_class = NULL;
 
+/* Max chars we print. Limit to MAX_LINELENGTH to stop out-of-control stuff.
+ */
+#define LINELENGTH ( \
+	IM_MIN( MAX_LINELENGTH, watch_int_get( "CALC_LINELENGTH", 80 ) ) \
+)
+
 static void
-itext_finalize( GObject *gobject )
+itext_destroy( GtkObject *object )
 {
 	iText *itext;
 
-	g_return_if_fail( gobject != NULL );
-	g_return_if_fail( IS_ITEXT( gobject ) );
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_ITEXT( object ) );
 
-	itext = ITEXT( gobject );
+	itext = ITEXT( object );
 
 #ifdef DEBUG
 	printf( "itext_destroy\n" );
@@ -51,46 +57,37 @@ itext_finalize( GObject *gobject )
 
 	/* My instance destroy stuff.
 	 */
-	IM_FREE( itext->formula );
-	IM_FREE( itext->formula_default );
-	vips_buf_destroy( &itext->value );
-	vips_buf_destroy( &itext->decompile );
+	FREE( itext->formula );
+	FREE( itext->formula_default );
+	buf_destroy( &itext->value );
+	buf_destroy( &itext->decompile );
 
-	G_OBJECT_CLASS( parent_class )->finalize( gobject );
-}
-
-static void
-itext_info( iObject *iobject, VipsBuf *buf )
-{
-	iText *itext = ITEXT( iobject );
-
-	vips_buf_appends( buf, _( "Formula" ) );
-	vips_buf_appendf( buf, ": %s\n", NN( itext->formula ) );
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
 
 /* Fwd ref this.
  */
-static gboolean itext_add_element( VipsBuf *buf, 
-	PElement *base, gboolean top, gboolean bracket );
+static gboolean itext_add_element( BufInfo *buf, 
+	PElement *base, gboolean top );
 
 /* Sub-fn of below, callback for list print. Eval and print the item into 
  * the buffer, separating with commas as required. 
  */
 static void *
-itext_add_list( PElement *base, VipsBuf *buf, gboolean *first )
+itext_add_list( PElement *base, BufInfo *buf, gboolean *first )
 {
 	Reduce *rc = reduce_context;
 
 	if( *first )
 		*first = FALSE;
 	else 
-		vips_buf_appends( buf, ", " );
+		buf_appends( buf, ", " );
 
 	/* Reduce the head, and print.
 	 */
 	if( !reduce_pelement( rc, reduce_spine, base ) ) 
 		return( base );
-	if( !itext_add_element( buf, base, FALSE, FALSE ) )
+	if( !itext_add_element( buf, base, FALSE ) )
 		return( base );
 
 	/* Buffer full? Abort list print.
@@ -105,7 +102,7 @@ itext_add_list( PElement *base, VipsBuf *buf, gboolean *first )
  * buffer.
  */
 static void *
-itext_add_string( PElement *base, VipsBuf *buf )
+itext_add_string( PElement *base, BufInfo *buf )
 {
 	Reduce *rc = reduce_context;
 
@@ -114,14 +111,12 @@ itext_add_string( PElement *base, VipsBuf *buf )
 	if( !reduce_pelement( rc, reduce_spine, base ) ) 
 		return( base );
 	if( PEISCHAR( base ) )
-		/* Don't escape chars in string mode.
-		 */
-		vips_buf_appendf( buf, "%c", PEGETCHAR( base ) );
+		buf_appendf( buf, "%c", PEGETCHAR( base ) );
 	else {
 		/* Help! Fall back to ordinary item print.
 		 */
-		vips_buf_appends( buf, ", " );
-		if( !itext_add_element( buf, base, FALSE, FALSE ) )
+		buf_appends( buf, ", " );
+		if( !itext_add_element( buf, base, FALSE ) )
 			return( base );
 	}
 
@@ -133,121 +128,62 @@ itext_add_string( PElement *base, VipsBuf *buf )
 	return( NULL );
 }
 
-/* Print a char ... we need to escape \n etc.
- */
-static void
-itext_add_char( int ch, VipsBuf *buf )
-{
-	char in[2];
-	char out[3];
-
-	in[0] = ch;
-	in[1] = '\0';
-	my_strecpy( out, in, FALSE );
-
-	vips_buf_appends( buf, out );
-}
-
-/* Print a complex. 
- */
-static void
-itext_add_complex( double rp, double ip, VipsBuf *buf )
-{
-	if( PRINT_CARTESIAN ) 
-		vips_buf_appendf( buf, "(%.12g, %.12g)", rp, ip );
-	else {
-		if( rp == 0 ) {
-			if( ip == 0 ) 
-				vips_buf_appendf( buf, "0" );
-			else 
-				vips_buf_appendf( buf, "%.12gj", ip );
-		}
-		else if( ip == 0 ) 
-			vips_buf_appendf( buf, "%.12g", rp );
-		else
-			vips_buf_appendf( buf, "%.12g + %.12gj", rp, ip );
-
-	}
-}
-
 /* Try to decompile.
  */
 static gboolean
-itext_decompile_element( VipsBuf *buf, PElement *base, gboolean top )
+itext_decompile_element( BufInfo *buf, PElement *base, gboolean top )
 {
-	Reduce *rc = reduce_context;
 	gboolean result;
 
 	/* Set the value label for a tally entry.
 	 */
 	if( PEISNOVAL( base ) ) 
-		vips_buf_appends( buf, _( "no value" ) );
+		buf_appends( buf, "No value" );
 	else if( PEISREAL( base ) ) 
-		vips_buf_appendf( buf, "%g", PEGETREAL( base ) );
+		buf_appendf( buf, "%g", PEGETREAL( base ) );
 	else if( PEISBOOL( base ) ) 
-		vips_buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
-	else if( PEISCHAR( base ) ) {
-		vips_buf_appends( buf, "'" );
-		itext_add_char( (int) PEGETCHAR( base ), buf );
-		vips_buf_appends( buf, "'" );
-	}
+		buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
+	else if( PEISCHAR( base ) ) 
+		buf_appendf( buf, "'%c'", (int) PEGETCHAR( base ) );
 	else if( PEISCOMPLEX( base ) ) 
-		itext_add_complex( PEGETREALPART( base ), PEGETIMAGPART( base ),
-			buf );
-	else if( PEISMANAGEDSTRING( base ) ) {
-		Managedstring *managedstring = PEGETMANAGEDSTRING( base );
-
-		vips_buf_appendf( buf, "\"%s\"", managedstring->string );
-	}
+		buf_appendf( buf, "(%g,%g)", 
+			PEGETREALPART( base ), PEGETIMAGPART( base ) );
 	else if( PEISELIST( base ) ) {
-		vips_buf_appends( buf, "[ ]" );
+		buf_appends( buf, "[ ]" );
 	}
-	else if( !heap_is_string( base, &result ) ) 
+	else if( !heap_isstring( base, &result ) ) 
 		/* Eval error.
 		 */
 		return( FALSE );
 	else if( result ) {
-		vips_buf_appends( buf, "\"" );
+		buf_appends( buf, "\"" );
 		if( heap_map_list( base,
 			(heap_map_list_fn) itext_add_string, buf, NULL ) )
 			return( FALSE );
-		vips_buf_appends( buf, "\"" );
+		buf_appends( buf, "\"" );
 	}
 	else if( PEISLIST( base ) ) {
 		gboolean first = TRUE;
 
-		vips_buf_appends( buf, "[" );
+		buf_appends( buf, "[" );
 		if( heap_map_list( base,
 			(heap_map_list_fn) itext_add_list, buf, &first ) )
 			return( FALSE );
-		vips_buf_appends( buf, "]" );
+		buf_appends( buf, "]" );
 	}
 	else if( PEISIMAGE( base ) ) {
 		Imageinfo *ii = PEGETII( base );
 
 		if( !top )
-			vips_buf_appends( buf, "(" );
+			buf_appends( buf, "(" );
 
-		if( ii && IOBJECT( ii )->name ) 
-			vips_buf_appendf( buf, "vips_image \"%s\"", 
-				IOBJECT( ii )->name );
+		if( ii && ii->name ) 
+			buf_appendf( buf, "vips_image \"%s\"", ii->name );
 		else 
-			vips_buf_appendf( buf, "vips_image <unknown>" );
+			buf_appendf( buf, "vips_image \"unknown\"" );
 
 		if( !top )
-			vips_buf_appends( buf, ")" );
-	}
-	else if( PEISMANAGED( base ) ) {
-		Managed *managed;
-
-		if( !(managed = PEGETMANAGED( base )) ) 
-			vips_buf_appendf( buf, "<NULL managed>" );  
-		else { 
-			vips_buf_appendf( buf, "<%s ", 
-				G_OBJECT_TYPE_NAME( managed ) );
-			iobject_info( IOBJECT( managed ), buf );
-			vips_buf_appends( buf, ">" );
-		}
+			buf_appends( buf, ")" );
 	}
 	else if( PEISCLASS( base ) ) {
 		Compile *compile = PEGETCLASSCOMPILE( base );
@@ -255,7 +191,7 @@ itext_decompile_element( VipsBuf *buf, PElement *base, gboolean top )
 		int i;
 
 		if( !top )
-			vips_buf_appends( buf, "(" );
+			buf_appends( buf, "(" );
 
 		symbol_qualified_name( compile->sym, buf );
 
@@ -274,7 +210,7 @@ itext_decompile_element( VipsBuf *buf, PElement *base, gboolean top )
 			PElement value;
 
 			PEPOINTRIGHT( sv, &value );
-			vips_buf_appends( buf, " " );
+			buf_appends( buf, " " );
 			if( !itext_decompile_element( buf, &value, FALSE ) )
 				return( FALSE );
 
@@ -282,15 +218,15 @@ itext_decompile_element( VipsBuf *buf, PElement *base, gboolean top )
 		}
 
 		if( !top )
-			vips_buf_appends( buf, ")" );
+			buf_appends( buf, ")" );
 
 	}
 	else if( PEISSYMREF( base ) ) 
-		vips_buf_appends( buf, IOBJECT( PEGETSYMREF( base ) )->name );
+		buf_appends( buf, MODEL( PEGETSYMREF( base ) )->name );
 	else if( PEISTAG( base ) ) 
-		vips_buf_appends( buf, PEGETTAG( base ) );
-	else 
-		graph_pelement( rc->heap, buf, base, TRACE_FUNCTIONS );
+		buf_appends( buf, PEGETTAG( base ) );
+	else
+		buf_appends( buf, "<function>" );
 
 	return( TRUE );
 }
@@ -298,12 +234,12 @@ itext_decompile_element( VipsBuf *buf, PElement *base, gboolean top )
 /* Little wrapper ... used for formatting error messages, etc. FALSE for eval
  * error.
  */
-static gboolean 
-itext_decompile( Reduce *rc, VipsBuf *buf, PElement *root )
+gboolean 
+itext_decompile( BufInfo *buf, PElement *root )
 {
 	/* Evaluate and print off values.
 	 */
-	if( !reduce_pelement( rc, reduce_spine, root ) ) 
+	if( !reduce_pelement( reduce_context, reduce_spine, root ) ) 
 		return( FALSE );
 
 	if( !itext_decompile_element( buf, root, TRUE ) && !buf->full ) 
@@ -315,45 +251,39 @@ itext_decompile( Reduce *rc, VipsBuf *buf, PElement *root )
 	return( TRUE );
 }
 
-/* Print function for computed values. top is TRUE only for the very top level
- * output. bracket means we should bracket compound expressions.
+/* Same, but everror on eval fail.
+ */
+void
+itext_decompile_ev( Reduce *rc, BufInfo *buf, PElement *root )
+{
+	if( !itext_decompile( buf, root ) )
+		everror( rc, "%s", error_string );
+}
+
+/* Print function for computed values. 
  */
 static gboolean
-itext_add_element( VipsBuf *buf, PElement *base, 
-	gboolean top, gboolean bracket )
+itext_add_element( BufInfo *buf, PElement *base, gboolean top )
 {
 	gboolean result;
 
 	/* Set the value label for a tally entry.
 	 */
 	if( PEISNOVAL( base ) ) 
-		vips_buf_appends( buf, _( "no value" ) );
+		buf_appends( buf, "No value" );
 	else if( PEISREAL( base ) ) 
-		vips_buf_appendf( buf, "%.7g", PEGETREAL( base ) );
+		buf_appendf( buf, "%.7g", PEGETREAL( base ) );
 	else if( PEISBOOL( base ) ) 
-		vips_buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
-	else if( PEISCHAR( base ) ) {
-		vips_buf_appends( buf, "'" );
-		itext_add_char( (int) PEGETCHAR( base ), buf );
-		vips_buf_appends( buf, "'" );
-	}
-	else if( PEISCOMPLEX( base ) ) {
-		itext_add_complex( PEGETREALPART( base ), PEGETIMAGPART( base ),
-			buf );
-	}
-	else if( PEISMANAGEDSTRING( base ) ) {
-		Managedstring *managedstring = PEGETMANAGEDSTRING( base );
-
-		if( !top ) 
-			vips_buf_appends( buf, "\"" );
-		vips_buf_appends( buf, managedstring->string );
-		if( !top ) 
-			vips_buf_appends( buf, "\"" );
-	}
+		buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
+	else if( PEISCHAR( base ) ) 
+		buf_appendf( buf, "'%c'", (int) PEGETCHAR( base ) );
+	else if( PEISCOMPLEX( base ) ) 
+		buf_appendf( buf, "(%.12g,%.12g)", 
+			PEGETREALPART( base ), PEGETIMAGPART( base ) );
 	else if( PEISELIST( base ) ) {
-		vips_buf_appends( buf, "[ ]" );
+		buf_appends( buf, "[ ]" );
 	}
-	else if( !heap_is_string( base, &result ) ) 
+	else if( !heap_isstring( base, &result ) ) 
 		/* Eval error.
 		 */
 		return( FALSE );
@@ -361,7 +291,7 @@ itext_add_element( VipsBuf *buf, PElement *base,
 		/* Only generate quotes for non-top-level string objects.
 		 */
 		if( !top ) 
-			vips_buf_appends( buf, "\"" );
+			buf_appends( buf, "\"" );
 
 		/* Print string contents.
 		 */
@@ -370,42 +300,49 @@ itext_add_element( VipsBuf *buf, PElement *base,
 			return( FALSE );
 
 		if( !top ) 
-			vips_buf_appends( buf, "\"" );
+			buf_appends( buf, "\"" );
 	}
 	else if( PEISLIST( base ) ) {
 		gboolean first = TRUE;
 
-		vips_buf_appends( buf, "[" );
+		buf_appends( buf, "[" );
 		if( heap_map_list( base,
 			(heap_map_list_fn) itext_add_list, buf, &first ) )
 			return( FALSE );
-		vips_buf_appends( buf, "]" );
+		buf_appends( buf, "]" );
 	}
 	else if( PEISIMAGE( base ) ) {
-		vips_buf_appendf( buf, "<" );
-		vips_buf_appendi( buf, imageinfo_get( FALSE, PEGETII( base ) ) );
-		vips_buf_appendf( buf, ">" );
-	}
-	else if( PEISMANAGED( base ) ) {
-		Managed *managed = PEGETMANAGED( base );
+		Imageinfo *ii = PEGETII( base );
+		IMAGE *im = imageinfo_get( FALSE, ii );
 
-		vips_buf_appends( buf, "<" );
-		iobject_info( IOBJECT( managed ), buf );
-		vips_buf_appends( buf, ">" );
+		if( !im ) {
+			buf_appends( buf, "No image value" );
+			return( TRUE );
+		}
+
+		/* Coded? Special warning.
+		 */
+		if( im->Coding != IM_CODING_NONE ) 
+			buf_appendf( buf, "%s, ", 
+				im_Coding2char( im->Coding ) );
+
+		/* Main stuff.
+		 */
+		buf_appendf( buf, "%dx%d %s pixels, %d band%s",
+			im->Xsize, im->Ysize,
+			decode_bandfmt( im->BandFmt ),
+			im->Bands, im->Bands==1?"":"s" );
 	}
 	else if( PEISCLASS( base ) ) {
 		Compile *compile = PEGETCLASSCOMPILE( base );
 		PElement params;
 		int i;
 
-		if( bracket && compile->nparam )
-			vips_buf_appends( buf, "(" );
-
 		/* Name.
 		 */
 		symbol_qualified_name( compile->sym, buf );
 
-		/* Skip over the secrets, then value-ize all the args.
+		/* Skip over the secrets, then decompile all the args.
 		 */
 		PEGETCLASSSECRET( &params, base );
 		for( i = 0; i < compile->nsecret; i++ ) {
@@ -420,53 +357,69 @@ itext_add_element( VipsBuf *buf, PElement *base,
 			PElement value;
 
 			PEPOINTRIGHT( sv, &value );
-			vips_buf_appends( buf, " " );
-			if( !itext_add_element( buf, &value, FALSE, TRUE ) )
+			buf_appends( buf, " " );
+			if( !itext_decompile_element( buf, &value, FALSE ) )
 				return( FALSE );
 
 			PEPOINTRIGHT( hn, &params );
 		}
-
-		if( bracket && compile->nparam )
-			vips_buf_appends( buf, ")" );
 	}
 	else if( PEISSYMREF( base ) ) {
 		Symbol *sym = PEGETSYMREF( base );
 
-		if( is_scope( sym ) ) {
-			vips_buf_appendf( buf, "<scope '" );
-			symbol_qualified_name( sym, buf );
-			vips_buf_appendf( buf, "'>" ); 
-		}
-		else {
-			vips_buf_appendf( buf, "<reference to symbol '" ), 
-			symbol_qualified_name( sym, buf );
-			vips_buf_appendf( buf, "'>" ); 
-		}
-	}
-	else if( PEISTAG( base ) ) 
-		vips_buf_appendf( buf, ".%s", PEGETTAG( base ) );
-	else {
-		vips_buf_appendf( buf, "<" ); 
-		vips_buf_appends( buf, _( "function" ) );
-		vips_buf_appendf( buf, ">" ); 
+		if( is_scope( sym ) )
+			buf_appendf( buf, "<scope \"%s\">", 
+				MODEL( sym )->name );
+		else
+			buf_appendf( buf, "<reference to symbol \"%s\">", 
+				MODEL( sym )->name );
 	}
+	else
+		buf_appends( buf, "<function>" );
 
 	return( TRUE );
 }
 
-/* Little wrapper ... used for formatting error messages, etc. FALSE for eval
- * error.
+/* Make a decompiled expr from a value.
  */
-gboolean 
-itext_value( Reduce *rc, VipsBuf *buf, PElement *root )
+static gboolean
+itext_make_decompiled_string( Expr *expr, BufInfo *buf )
 {
+	/* Old error on this expression?
+	 */
+	if( expr->err ) {
+		expr_error_get( expr );
+		return( FALSE );
+	}
+
 	/* Evaluate and print off values.
 	 */
-	if( !reduce_pelement( rc, reduce_spine, root ) ) 
+	if( !itext_decompile( buf, &expr->root ) )
 		return( FALSE );
 
-	if( !itext_add_element( buf, root, TRUE, FALSE ) && !buf->full ) 
+	return( TRUE );
+}
+
+/* Make a value string from an expression.
+ */
+static gboolean
+itext_make_value_string( Expr *expr, BufInfo *buf )
+{
+	Reduce *rc = reduce_context;
+
+	/* Old error on this expression?
+	 */
+	if( expr->err ) {
+		expr_error_get( expr );
+		return( FALSE );
+	}
+
+	/* Evaluate and print off values.
+	 */
+	if( !reduce_pelement( rc, reduce_spine, &expr->root ) ) 
+		return( FALSE );
+
+	if( !itext_add_element( buf, &expr->root, TRUE ) && !buf->full ) 
 		/* Tally eval failed, and buffer is not full ... must
 		 * have been an eval error.
 		 */
@@ -475,81 +428,6 @@ itext_value( Reduce *rc, VipsBuf *buf, PElement *root )
 	return( TRUE );
 }
 
-/* Same, but everror on eval fail.
- */
-void
-itext_value_ev( Reduce *rc, VipsBuf *buf, PElement *root )
-{
-	if( !itext_value( rc, buf, root ) )
-		reduce_throw( rc );
-}
-
-/* Decompile an Expr.
- */
-static gboolean
-itext_make_decompiled_string( Expr *expr, VipsBuf *buf )
-{
-	/* Old error on this expression?
-	 */
-	if( expr->err ) {
-		expr_error_get( expr );
-		return( FALSE );
-	}
-
-	/* Dirty? We can't print dirty values, since we might have pointers 
-	 * to deleted symbols in the heap (if we are dirty because one of our 
-	 * parents has been deleted).
-
-	 	FIXME ... this seem a bit restrictive :-( ... could just
-		block reads of symbol pointers instead?
-
-	 */
-	if( expr->sym->dirty ) {
-		vips_buf_appendf( buf, _( "Dirty value" ) );
-		return( TRUE );
-	}
-
-	/* Evaluate and print off values.
-	 */
-	if( !itext_decompile( reduce_context, buf, &expr->root ) )
-		return( FALSE );
-
-	return( TRUE );
-}
-
-/* Make a value string from an Expr.
- */
-gboolean
-itext_make_value_string( Expr *expr, VipsBuf *buf )
-{
-	/* Old error on this expression?
-	 */
-	if( expr->err ) {
-		expr_error_get( expr );
-		return( FALSE );
-	}
-
-	/* Dirty? We can't print dirty values, since we might have pointers 
-	 * to deleted symbols in the heap (if we are dirty because one of our 
-	 * parents has been deleted).
-
-	 	FIXME ... this seem a bit restrictive :-( ... could just
-		block reads of symbol pointers instead?
-
-	 */
-	if( expr->sym->dirty ) {
-		vips_buf_appendf( buf, _( "Dirty value" ) );
-		return( TRUE );
-	}
-
-	/* Evaluate and print off values.
-	 */
-	if( !itext_value( reduce_context, buf, &expr->root ) )
-		return( FALSE );
-
-	return( TRUE );
-}
-
 static void *
 itext_update_model( Heapmodel *heapmodel )
 {
@@ -560,25 +438,41 @@ itext_update_model( Heapmodel *heapmodel )
 #ifdef DEBUG
 	printf( "itext_update_model: " );
 	row_name_print( row );
-	if( row->sym && 
-		row->sym->dirty )
+	if( row->sym && row->sym->dirty )
 		printf( " (dirty)" );
 	printf( "\n" );
 #endif /*DEBUG*/
 
-	vips_buf_set_dynamic( &itext->value, LINELENGTH );
-	vips_buf_set_dynamic( &itext->decompile, LINELENGTH );
-	if( expr ) {
-		if( !itext_make_value_string( expr, &itext->value ) ||
-			!itext_make_decompiled_string( expr, 
-				&itext->decompile ) ) 
+	/* Update value display. We can't update the value display if this
+	 * sym is dirty, since we might have pointers to deleted symbols in
+	 * the heap (if we are dirty because one of our parents has been 
+	 * deleted).
+
+	 	FIXME ... this seem a bit restrictive :-( ... could just
+		block reads of symbol pointers in
+		itext_make_value_string() instead?
+
+	 */
+	buf_set_dynamic( &itext->value, LINELENGTH );
+	if( expr && !expr->err && !row->sym->dirty ) {
+		if( !itext_make_value_string( expr, &itext->value ) ) {
+			buf_appendline( &itext->value, error_string );
 			expr_error_set( expr );
+		}
+	}
+
+	buf_set_dynamic( &itext->decompile, LINELENGTH );
+	if( expr && !expr->err && !row->sym->dirty ) {
+		if( !itext_make_decompiled_string( expr, &itext->decompile ) ) {
+			buf_appendline( &itext->decompile, error_string );
+			expr_error_set( expr );
+		}
 	}
 
 #ifdef DEBUG
 	printf( "itext_update_model: " );
 	row_name_print( row );
-	printf( " has value: %s\n", vips_buf_all( &itext->value ) );
+	printf( " has value: %s\n", buf_all( &itext->value ) );
 #endif /*DEBUG*/
 
 	/* If this is a non-edited row, update the source.
@@ -589,14 +483,14 @@ itext_update_model( Heapmodel *heapmodel )
 		if( expr && expr->compile && expr->compile->rhstext ) 
 			new_formula = expr->compile->rhstext;
 		else 
-			new_formula = vips_buf_all( &itext->decompile );
+			new_formula = buf_all( &itext->decompile );
 
-		IM_SETSTR( itext->formula_default, new_formula );
+		SETSTR( itext->formula_default, new_formula );
 
 		/* Don't use itext_set_formula(), as we don't want to set
 		 * _modified or recomp.
 		 */
-		IM_SETSTR( itext->formula, itext->formula_default );
+		SETSTR( itext->formula, itext->formula_default );
 	}
 
 	return( HEAPMODEL_CLASS( parent_class )->update_model( heapmodel ) );
@@ -605,14 +499,14 @@ itext_update_model( Heapmodel *heapmodel )
 /* Build param lists.
  */
 static void *
-itext_update_heap_sub( Symbol *sym, VipsBuf *buf )
+itext_update_heap_sub( Symbol *sym, BufInfo *buf )
 {
-	vips_buf_appendf( buf, "%s ", IOBJECT( sym )->name );
+	buf_appendf( buf, "%s ", MODEL( sym )->name );
 
 	return( NULL );
 }
 
-/* heapmodel->modified is set ... parse, compile, and mark for recomp.
+/* model->modified is set ... parse, compile, and mark for recomp.
  */
 static void *
 itext_update_heap( Heapmodel *heapmodel )
@@ -621,50 +515,36 @@ itext_update_heap( Heapmodel *heapmodel )
         Row *row = heapmodel->row;
 	Expr *expr = row->expr;
 
-#ifdef DEBUG
-	printf( "itext_update_heap: " );
-	row_name_print( HEAPMODEL( itext )->row );
-	printf( "\n" );
-#endif /*DEBUG*/
+	char txt[MAX_STRSIZE];
+	BufInfo buf;
+	ParseRhsSyntax syntax;
 
-	/* We can have no modified text, but come here anyway. For example, we
-	 * could try eval, find an error due to an undefined symbol, and have
-	 * to retry later. Clearing the row error later will mark us modified, 
-	 * even though we have no text of our own.
-	 */
-	if( itext->formula ) {
-		char txt[MAX_STRSIZE];
-		VipsBuf buf = VIPS_BUF_STATIC( txt );
-		ParseRhsSyntax syntax;
-
-		if( row->sym &&
-			is_super( row->sym ) ) {
-			/* A super member ... special syntax.
-			 */
-			vips_buf_appendf( &buf, "%s", itext->formula );
-
-			syntax = PARSE_SUPER;
-		}
-		else {
-			/* Build a new params + '=' + rhs string.
-			 */
-			if( expr->compile ) 
-				(void) slist_map( expr->compile->param, 
-					(SListMapFn) itext_update_heap_sub, 
-					&buf );
-			vips_buf_appendf( &buf, "= %s;", itext->formula );
-
-			syntax = PARSE_PARAMS;
-		}
-
-		/* Parse and compile.
+	buf_init_static( &buf, txt, MAX_STRSIZE );
+	if( is_super( row->sym ) ) {
+		/* A super member ... special syntax.
 		 */
-		expr_error_clear( expr );
-		attach_input_string( vips_buf_all( &buf ) );
-		if( !parse_rhs( expr, syntax ) ) {
-			expr_error_set( expr );
-			return( heapmodel );
-		}
+		buf_appendf( &buf, "%s", itext->formula );
+
+		syntax = PARSE_SUPER;
+	}
+	else {
+		/* Build a new params + '=' + rhs string.
+		 */
+		if( expr->compile ) 
+			(void) slist_map( expr->compile->param, 
+				(SListMapFn) itext_update_heap_sub, &buf );
+		buf_appendf( &buf, "= %s;", itext->formula );
+
+		syntax = PARSE_PARAMS;
+	}
+
+	/* Parse and compile.
+	 */
+	expr_error_clear( expr );
+	attach_input_string( buf_all( &buf ) );
+	if( !parse_rhs( expr, syntax, FALSE ) ) {
+		expr_error_set( expr );
+		return( heapmodel );
 	}
 
 	/* Mark for recomp.
@@ -699,8 +579,7 @@ itext_clear_edited( Heapmodel *heapmodel )
 		else 
 			printf( "itext_clear_edited: FIXME!\n" );
 
-		if( heapmodel->row->expr )
-			expr_dirty( heapmodel->row->expr, link_serial_new() );
+		(void) expr_dirty( heapmodel->row->expr, link_serial_new() );
 
 		/* Don't clear HEAPMODEL( itext )->modified, we want to make
 		 * sure we re-parse and compile the default value to break any
@@ -712,14 +591,14 @@ itext_clear_edited( Heapmodel *heapmodel )
 }
 
 static void
-itext_parent_add( iContainer *child )
+itext_parent_add( Model *child, Model *parent )
 {
 	iText *itext = ITEXT( child );
 	Row *row;
 
-	g_assert( IS_RHS( child->parent ) );
+	assert( IS_RHS( parent ) );
 
-	ICONTAINER_CLASS( parent_class )->parent_add( child );
+	MODEL_CLASS( parent_class )->parent_add( child, parent );
 
 	row = HEAPMODEL( itext )->row;
 
@@ -744,7 +623,10 @@ itext_load( Model *model,
 	char formula[MAX_STRSIZE];
 	char formula2[MAX_STRSIZE];
 
-	g_assert( IS_RHS( parent ) );
+	if( !IS_RHS( parent ) ) {
+		ierrors( "itext_load: can only add a itext to a rhs" );
+		return( FALSE );
+	}
 
 	if( get_sprop( xnode, "formula", formula, MAX_STRSIZE ) ) {
 		model_loadstate_rewrite( state, formula, formula2 );
@@ -756,12 +638,6 @@ itext_load( Model *model,
 		state, parent, xnode ) );
 }
 
-static View *
-itext_view_new( Model *model, View *parent )
-{
-	return( itextview_new() );
-}
-
 static xmlNode *
 itext_save( Model *model, xmlNode *xnode )
 {
@@ -781,28 +657,23 @@ itext_save( Model *model, xmlNode *xnode )
 }
 
 static void
-itext_class_init( iTextClass *class )
+itext_class_init( iTextClass *klass )
 {
-	GObjectClass *gobject_class = (GObjectClass *) class;
-	iObjectClass *iobject_class = (iObjectClass *) class;
-	iContainerClass *icontainer_class = (iContainerClass *) class;
-	ModelClass *model_class = (ModelClass *) class;
-	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	ModelClass *model_class = (ModelClass *) klass;
+	HeapmodelClass *heapmodel_class = (HeapmodelClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_HEAPMODEL );
+
+	object_class->destroy = itext_destroy;
 
 	/* Create signals.
 	 */
 
 	/* Init methods.
 	 */
-	gobject_class->finalize = itext_finalize;
-
-	iobject_class->info = itext_info;
-
-	icontainer_class->parent_add = itext_parent_add;
-
-	model_class->view_new = itext_view_new;
+	model_class->view_new = itextview_new;
+	model_class->parent_add = itext_parent_add;
 	model_class->save = itext_save;
 	model_class->load = itext_load;
 
@@ -812,7 +683,7 @@ itext_class_init( iTextClass *class )
 
 	/* Static init.
 	 */
-	model_register_loadable( MODEL_CLASS( class ) );
+	model_register_loadable( MODEL_CLASS( klass ) );
 }
 
 static void
@@ -824,48 +695,43 @@ itext_init( iText *itext )
 
 	itext->formula = NULL;
 	itext->formula_default = NULL;
-	vips_buf_init( &itext->value );
-	vips_buf_init( &itext->decompile );
-	vips_buf_set_dynamic( &itext->value, LINELENGTH );
-	vips_buf_set_dynamic( &itext->decompile, LINELENGTH );
+	buf_init( &itext->value );
+	buf_init( &itext->decompile );
 	itext->edited = FALSE;
 
 	/* Some defaults changed in _parent_add() above.
 	 */
 }
 
-GType
+GtkType
 itext_get_type( void )
 {
-	static GType type = 0;
+	static GtkType itext_type = 0;
 
-	if( !type ) {
-		static const GTypeInfo info = {
-			sizeof( iTextClass ),
-			NULL,           /* base_init */
-			NULL,           /* base_finalize */
-			(GClassInitFunc) itext_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
+	if( !itext_type ) {
+		static const GtkTypeInfo itext_info = {
+			"iText",
 			sizeof( iText ),
-			32,             /* n_preallocs */
-			(GInstanceInitFunc) itext_init,
+			sizeof( iTextClass ),
+			(GtkClassInitFunc) itext_class_init,
+			(GtkObjectInitFunc) itext_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
 		};
 
-		type = g_type_register_static( TYPE_HEAPMODEL, 
-			"iText", &info, 0 );
+		itext_type = gtk_type_unique( TYPE_HEAPMODEL, &itext_info );
 	}
 
-	return( type );
+	return( itext_type );
 }
 
 iText *
 itext_new( Rhs *rhs )
 {
-	iText *itext;
+	iText *itext = gtk_type_new( TYPE_ITEXT );
 
-	itext = ITEXT( g_object_new( TYPE_ITEXT, NULL ) );
-	icontainer_child_add( ICONTAINER( rhs ), ICONTAINER( itext ), -1 );
+	model_child_add( MODEL( rhs ), MODEL( itext ), -1 );
 
 	return( itext );
 }
@@ -883,31 +749,21 @@ itext_set_edited( iText *itext, gboolean edited )
 #endif /*DEBUG*/
 
 		itext->edited = edited;
-		iobject_changed( IOBJECT( itext ) );
+		model_changed( MODEL( itext ) );
 	}
 
 	if( edited ) 
 		heapmodel_set_modified( heapmodel, TRUE );
 }
 
-gboolean
+void
 itext_set_formula( iText *itext, const char *formula )
 {
 	if( !itext->formula || strcmp( itext->formula, formula ) != 0 ) {
-#ifdef DEBUG
-		printf( "itext_set_formula: " );
-		row_name_print( HEAPMODEL( itext )->row );
-		printf( " \"%s\"\n", formula );
-#endif /*DEBUG*/
-
-		IM_SETSTR( itext->formula, formula );
+		SETSTR( itext->formula, formula )
 
 		heapmodel_set_modified( HEAPMODEL( itext ), TRUE );
 
-		iobject_changed( IOBJECT( itext ) );
-
-		return( TRUE );
+		model_changed( MODEL( itext ) );
 	}
-
-	return( FALSE );
 }
