@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -28,24 +28,20 @@
  */
 
 #define TYPE_ITEXT (itext_get_type())
-#define ITEXT( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_ITEXT, iText ))
+#define ITEXT( obj ) (GTK_CHECK_CAST( (obj), TYPE_ITEXT, iText ))
 #define ITEXT_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_ITEXT, iTextClass))
-#define IS_ITEXT( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_ITEXT ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_ITEXT, iTextClass ))
+#define IS_ITEXT( obj ) (GTK_CHECK_TYPE( (obj), TYPE_ITEXT ))
 #define IS_ITEXT_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_ITEXT ))
-#define ITEXT_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_ITEXT, iTextClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_ITEXT ))
 
 struct _iText {
 	Heapmodel parent_class;
 
-	VipsBuf value;			/* The value displayed as a [char] */
+	BufInfo value;			/* The value displayed as a [char] */
 	char *formula;			/* The formula we edit */
 	char *formula_default;		/* Formula we inherit */
-	VipsBuf decompile;		/* The value decompiled to a [char] */
+	BufInfo decompile;		/* The value decompiled to a [char] */
 
 	/* TRUE if the formula has been entered by the user and should be
 	 * saved.
@@ -63,12 +59,11 @@ typedef struct _iTextClass {
 	 */
 } iTextClass;
 
-GType itext_get_type( void );
+GtkType itext_get_type( void );
 iText *itext_new( Rhs *rhs );
 
-gboolean itext_value( Reduce *rc, VipsBuf *buf, PElement *root );
-void itext_value_ev( Reduce *rc, VipsBuf *buf, PElement *root );
-gboolean itext_make_value_string( Expr *expr, VipsBuf *buf );
+gboolean itext_decompile( BufInfo *buf, PElement *root );
+void itext_decompile_ev( Reduce *rc, BufInfo *buf, PElement *root );
 
 void itext_set_edited( iText *text, gboolean edited );
-gboolean itext_set_formula( iText *text, const char *formula );
+void itext_set_formula( iText *text, const char *formula );
