@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -40,7 +40,14 @@ typedef struct _iTextview {
 
 	/* Widgets.
 	 */
-        Formula *formula;
+        GtkWidget *wtable;		/* Lay out elements in this */
+        GtkWidget *wlframe;		/* Frame for label */
+        GtkWidget *wtext;		/* Edit def here */
+        GtkWidget *wlabel;		/* Info text */
+
+        gboolean edit;              	/* In edit mode */
+
+	guint reset_sid;		/* Listen for column reset here */
 } iTextview;
 
 typedef struct _iTextviewClass {
