@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -41,8 +41,6 @@ extern "C" {
 #define IS_IWINDOW( obj ) (GTK_CHECK_TYPE( (obj), TYPE_IWINDOW ))
 #define IS_IWINDOW_CLASS( klass ) \
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_IWINDOW ))
-#define IWINDOW_GET_CLASS( obj ) \
-	(GTK_CHECK_GET_CLASS( (obj), TYPE_IWINDOW, iWindowClass ))
 
 typedef struct _iWindow iWindow;
 
@@ -97,8 +95,6 @@ typedef enum _iWindowShape {
  */
 typedef struct {
 	iWindow *iwnd;
-	int priority;			/* Higher priority == more on top */
-	const char *name;		/* For debugging */
 
 	/* Shape currently requested by this user.
 	 */
@@ -108,9 +104,9 @@ typedef struct {
 /* The result from a window/dialog/whatever ... not just a bool.
  */
 typedef enum iwindow_result {
-	IWINDOW_ERROR = 0,		/* Tried but failed */
-	IWINDOW_YES,			/* User tried the action */
-	IWINDOW_NO			/* User cancelled */
+	IWINDOW_ERROR = 0,		/* Tried to YES, but failed */
+	IWINDOW_TRUE,			/* Success! */
+	IWINDOW_FALSE			/* Cancelled */
 } iWindowResult;
 
 /* Our callbacks don't return iWindowResult, instead
@@ -141,26 +137,13 @@ typedef struct {
 #define IWINDOW_SUSP( X ) ((iWindowSusp *) (X))
 
 struct _iWindow {
-	GtkWindow parent_object;
+	GtkWindow parent;
 
-	/* Parent window. Used for (eg.) image displays windows which we need
-	 * to float over the main workspace.
+	/* My instance vars.
 	 */
-	GtkWidget *parent;		/* Our parent widget */
-	iWindow *parent_window;		/* Our parent window */
-	guint parent_unmap_sid; 	/* Watch parent death here */
-
 	GtkWidget *work;
-	GtkAccelGroup *accel_group;
-	Infobar *infobar;
 
-	char *title;
-
-	/* Action stuff. We init this and add a few common actions to help out
-	 * subclasses.
-	 */
-	GtkActionGroup *action_group;
-	GtkUIManager *ui_manager;
+	const char *title;
 
 	/* Per instance build function.
 	 */
@@ -182,41 +165,27 @@ struct _iWindow {
 	 */
 	iWindowShape shape;		/* Global shape ... for hglass */
 	GSList *contexts;		/* Set of other requested shapes */
-	GdkWindow *work_window;		/* The window we actually set */
-
-	/* Size memorization.
-	 */
-	const char *width_pref;		/* Prefs we save width/height in */
-	const char *height_pref;
 };
 
 typedef struct _iWindowClass {
 	GtkWindowClass parent_class;
 
-	/* Per class build/popdown functions.
+	/* Per class build function.
 	 */
 	void (*build)( GtkWidget * );
-	void (*popdown)( GtkWidget * );
-
-	/* Whether windows of this class should be marked as transient for
-	 * their parents (eg. dialogs usually are).
-	 */
-	gboolean transient;
 } iWindowClass;
 
-int iwindow_number( void );
-iWindow *iwindow_pick_one( void );
 typedef void (*iWindowMapFn)( iWindow *, void * );
 void *iwindow_map_all( iWindowMapFn fn, void *a );
 
-iWindowCursorContext *iwindow_cursor_context_new( iWindow *, 
-	int, const char * );
+void set_hourglass( void );
+void set_pointer( void );
+iWindowCursorContext *iwindow_cursor_context_new( iWindow * );
 void iwindow_cursor_context_set_cursor( iWindowCursorContext *, iWindowShape );
 void iwindow_cursor_context_destroy( iWindowCursorContext *cntxt );
 
 iWindowSusp *iwindow_susp_new( iWindowFn, 
 	iWindow *, void *, iWindowNotifyFn, void * );
-void iwindow_susp_trigger( iWindowSusp * ); 
 void iwindow_susp_return( void *, iWindowResult );
 void iwindow_susp_comp( void *, iWindowResult );
 
@@ -231,21 +200,12 @@ void iwindow_notify_null( void *client, iWindowResult result );
 GtkType iwindow_get_type( void );
 
 GtkWidget *iwindow_new( GtkWindowType );
-void iwindow_set_title( iWindow *, const char *, ... )
-	__attribute__((format(printf, 2, 3)));
+void iwindow_set_title( iWindow *, const char *, ... );
 void iwindow_set_build( iWindow *, iWindowBuildFn, void *, void *, void * );
 void iwindow_set_popdown( iWindow *, iWindowFn, void * );
-void iwindow_set_size_prefs( iWindow *, const char *, const char * );
-void iwindow_set_work_window( iWindow *iwnd, GdkWindow *work_window );
-void iwindow_set_parent( iWindow *, GtkWidget *par );
 
 void iwindow_build( iWindow * );
 void *iwindow_kill( iWindow * );
-void iwindow_kill_action_cb( GtkAction *action, iWindow *iwnd );
-
-GtkWidget *iwindow_get_root( GtkWidget *widget );
-GtkWidget *iwindow_get_root_noparent( GtkWidget *widget );
-void iwindow_alert( GtkWidget *parent, GtkMessageType type );
 
 #ifdef __cplusplus
 }
