@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,7 +29,6 @@
 
 /*
 #define DEBUG
-#define DEBUG_DIRTY
  */
 
 #include "ip.h"
@@ -45,7 +44,7 @@ link_expr_destroy( LinkExpr *le )
 #ifdef DEBUG
 	printf( "link_expr_destroy: removing expr " );
 	symbol_name_print( le->expr->sym );
-	printf( "referencing link->child = " );
+	printf( " referencing link->child = " );
 	symbol_name_print( le->link->child );
 	printf( "\n" );
 #endif /*DEBUG*/
@@ -65,17 +64,17 @@ link_expr_new( Link *link, Expr *expr, gboolean dynamic )
 	GSList **elinks = dynamic ? &expr->dynamic_links : &expr->static_links;
         LinkExpr *le;
 
-	g_assert( expr_get_root_dynamic( expr )->sym == link->parent );
+	assert( expr_get_root_dynamic( expr )->sym == link->parent );
 
 #ifdef DEBUG
 	printf( "link_expr_new: expr " );
 	symbol_name_print( expr->sym );
-	printf( "references link->child = " );
+	printf( " references link->child = " );
 	symbol_name_print( link->child );
 	printf( "\n" );
 #endif /*DEBUG*/
 
-        if( !(le = INEW( NULL, LinkExpr )) )
+        if( !(le = IM_NEW( NULL, LinkExpr )) )
 		return( NULL );
 
         le->link = link; 
@@ -108,7 +107,7 @@ static void *symbol_dirty_set( Symbol *sym );
 static void *
 link_dirty_child( Link *link )
 {
-	g_assert( link->parent->ndirtychildren >= 0 );
+	assert( link->parent->ndirtychildren >= 0 );
 
 	link->parent->ndirtychildren += 1;
 
@@ -133,7 +132,7 @@ link_clean_child( Link *link )
 	/* One fewer dirty children!
 	 */
 	parent->ndirtychildren--;
-	g_assert( parent->ndirtychildren >= 0 );
+	assert( parent->ndirtychildren >= 0 );
 
 	/* Have we just cleaned the last dirty child of link->parent? If we
 	 * have and if link->parent has an error, clear the error so that
@@ -156,7 +155,7 @@ link_destroy( Link *link )
 #ifdef DEBUG
 	printf( "link_destroy: destroying link from " );
 	symbol_name_print( link->parent );
-	printf( "to " );
+	printf( " to " );
 	symbol_name_print( link->child );
 	printf( "\n" );
 #endif /*DEBUG*/
@@ -185,18 +184,18 @@ link_new( Symbol *child, Symbol *parent )
 {
         Link *link;
 
-	g_assert( is_top( parent ) && is_top( child ) );
-	g_assert( parent != child );
+	assert( is_top( parent ) && is_top( child ) );
+	assert( parent != child );
 
 #ifdef DEBUG
 	printf( "link_new: making link from " );
 	symbol_name_print( parent );
-	printf( "to " );
+	printf( " to " );
 	symbol_name_print( child );
 	printf( "\n" );
 #endif /*DEBUG*/
 
-        if( !(link = INEW( NULL, Link )) )
+        if( !(link = IM_NEW( NULL, Link )) )
 		return( NULL );
 
         link->parent = parent; 
@@ -259,25 +258,15 @@ link_expr_find_expr( Link *link, Expr *expr, gboolean dynamic )
 void *
 link_add( Symbol *child, Expr *expr, gboolean dynamic )
 {
-	Expr *parent = expr_get_root_dynamic( expr );
+	Symbol *parent = expr_get_root_dynamic( expr )->sym;
 	Link *link;
 	LinkExpr *le;
 
-#ifdef DEBUG
-	printf( "link_add: child = " );
-	symbol_name_print( child );
-	printf( "; expr = " );
-	expr_name_print( expr );
-	printf( "; dynamic = %s\n", bool_to_char( dynamic ) );
-#endif /*DEBUG*/
+	assert( is_top( child ) && is_top( parent ) );
+	assert( child != parent );
 
-	g_assert( parent );
-	g_assert( parent->sym );
-	g_assert( is_top( child ) && is_top( parent->sym ) );
-	g_assert( child != parent->sym );
-
-	if( !(link = link_find_child( child, parent->sym )) ) {
-		if( !(link = link_new( child, parent->sym )) )
+	if( !(link = link_find_child( child, parent )) ) {
+		if( !(link = link_new( child, parent )) )
 			return( child );
 	}
 
@@ -300,9 +289,9 @@ link_remove( Symbol *child, Expr *expr, gboolean dynamic )
 	Link *link = link_find_child( child, parent );
 	LinkExpr *le = link_expr_find_expr( link, expr, dynamic );
 
-	g_assert( is_top( parent ) && is_top( child ) );
-	g_assert( parent != child );
-	g_assert( link );
+	assert( is_top( parent ) && is_top( child ) );
+	assert( parent != child );
+	assert( link );
 
 	le->count--;
 	if( le->count == 0 ) {
@@ -357,8 +346,8 @@ link_children_expr( Expr *expr, Symbol *parent )
 
 		/* ... and recurse for sub-children.
 		 */
-		(void) icontainer_map( ICONTAINER( compile ),
-			(icontainer_map_fn) link_children, parent, NULL );
+		(void) stable_map( compile->locals,
+			(symbol_map_fn) link_children, parent, NULL, NULL );
 	}
 
         return( NULL );
@@ -420,15 +409,11 @@ symbol_ndirty( Symbol *sym )
 void *
 symbol_fix_counts( Symbol *sym )
 {
-#ifdef DEBUG
 	int old_count = sym->ndirtychildren;
-#endif /*DEBUG*/
 
 	sym->ndirtychildren = symbol_ndirty( sym );
 
-#ifdef DEBUG
-	g_assert( sym->ndirtychildren == old_count );
-#endif /*DEBUG*/
+	assert( sym->ndirtychildren == old_count );
 
 	symbol_state_change( sym );
 
@@ -448,16 +433,15 @@ symbol_link_destroy( Symbol *sym )
 void
 symbol_link_build( Symbol *sym )
 {
-	g_assert( is_top( sym ) );
+	assert( is_top( sym ) );
 
         /* Make static links for our expr and all subexprs. If this symbol 
 	 * is being edited, get stuff from the edited value.
          */
         if( sym->expr ) {
 		if( sym->expr->row )
-			(void) icontainer_map_all( 
-				ICONTAINER( sym->expr->row ), 
-				(icontainer_map_fn) link_row, sym );
+			(void) model_map_all( MODEL( sym->expr->row ), 
+				(model_map_fn) link_row, sym );
 		else
 			(void) link_children_expr( sym->expr, sym );
 	}
@@ -519,7 +503,7 @@ link_dirty_walk( Link *link, int serial )
 void *
 symbol_dirty_intrans( Symbol *sym, int serial )
 {
-	g_assert( is_top( sym ) );
+	assert( is_top( sym ) );
 
 	return( slist_map( sym->topparents, 
 		(SListMapFn) link_dirty_walk, GINT_TO_POINTER( serial ) ) );
@@ -528,7 +512,7 @@ symbol_dirty_intrans( Symbol *sym, int serial )
 static void *
 symbol_dirty_set( Symbol *sym )
 {
-	g_assert( is_top( sym ) );
+	assert( is_top( sym ) );
 
 	/* Clear error, to make sure we will recomp it.
 	 */
@@ -536,11 +520,11 @@ symbol_dirty_set( Symbol *sym )
 		expr_error_clear( sym->expr );
 
 	if( !sym->dirty ) {
-#ifdef DEBUG_DIRTY
+#ifdef DEBUG
 		printf( "symbol_dirty_set: " );
 		symbol_name_print( sym );
-		printf( "(%p)\n", sym );
-#endif /*DEBUG_DIRTY*/
+		printf( "\n" );
+#endif /*DEBUG*/
 
 		/* Change of state.
 		 */
@@ -564,7 +548,7 @@ symbol_dirty_set( Symbol *sym )
 void *
 symbol_dirty( Symbol *sym, int serial )
 {
-	g_assert( is_top( sym ) );
+	assert( is_top( sym ) );
 
 	symbol_dirty_set( sym );
 
@@ -579,10 +563,8 @@ link_dirty_total( Link *link, int serial )
 	/* Entering: note new recursion.
 	 */
 	if( recursion_depth++ > 1000 ) {
-		error_top( _( "Circular dependency." ) );
-		error_sub( _( "Circular dependency detected near "
-			"symbol \"%s\"." ),
-			IOBJECT( link->parent )->name );
+		ierrors( "circular dependency detected near symbol \"%s\"",
+			MODEL( link->parent )->name );
 		recursion_depth = 0;
 		return( link );
 	}
@@ -633,14 +615,14 @@ symbol_dirty_total( Symbol *sym, int serial )
 void *
 symbol_dirty_clear( Symbol *sym )
 {
-	g_assert( is_top( sym ) );
+	assert( is_top( sym ) );
 
 	if( sym->dirty ) {
-#ifdef DEBUG_DIRTY
+#ifdef DEBUG
 		printf( "symbol_dirty_clear: " );
 		symbol_name_print( sym );
-		printf( "(%p)\n", sym );
-#endif /*DEBUG_DIRTY*/
+		printf( "\n" );
+#endif /*DEBUG*/
 
 		/* Change of state.
 		 */
