@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -28,16 +28,12 @@
 */
 
 #define TYPE_MATRIX (matrix_get_type())
-#define MATRIX( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_MATRIX, Matrix ))
+#define MATRIX( obj ) (GTK_CHECK_CAST( (obj), TYPE_MATRIX, Matrix ))
 #define MATRIX_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_MATRIX, MatrixClass))
-#define IS_MATRIX( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_MATRIX ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_MATRIX, MatrixClass ))
+#define IS_MATRIX( obj ) (GTK_CHECK_TYPE( (obj), TYPE_MATRIX ))
 #define IS_MATRIX_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_MATRIX ))
-#define MATRIX_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_MATRIX, MatrixClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_MATRIX ))
 
 /* What kind of ui bits have we asked for for this matrix?
  */
@@ -54,19 +50,16 @@ typedef struct _Matrix {
 
 	/* Base class fields.
 	 */
-	MatrixValue value;
+	double *value;			/* Base coeffs */
+	int width;			/* Size of matrix */
+	int height;
 
 	/* Other class fields.
 	 */
 	MatrixDisplayType display;	/* Display as */
 	double scale;
 	double offset;
-
-	/* Is there a current selection on the matrixview? And if there is,
-	 * the cells it covers.
-	 */
-	gboolean selected;
-	Rect range;
+	char *filename;
 } Matrix;
 
 typedef struct _MatrixClass {
@@ -76,20 +69,14 @@ typedef struct _MatrixClass {
 	 */
 } MatrixClass;
 
-gboolean matrix_value_resize( MatrixValue *value, int width, int height );
+GtkType matrix_get_type( void );
+Classmodel *matrix_new( Rhs *rhs );
 
-GType matrix_get_type( void );
-
-/* Select rectangular areas of matricies.
- */
-void matrix_select( Matrix *matrix, int left, int top, int width, int height );
-void matrix_deselect( Matrix *matrix );
-
-void matrix_dmask_to_ip( DOUBLEMASK *dmask, VipsBuf *buf );
-gboolean matrix_dmask_to_heap( Heap *heap, DOUBLEMASK *dmask, PElement *out );
+void matrix_dmask_to_ip( DOUBLEMASK *dmask, BufInfo *buf );
+gboolean matrix_dmask_to_heap( Heap *hi, DOUBLEMASK *dmask, PElement *out );
 DOUBLEMASK *matrix_imask_to_dmask( INTMASK *imask );
 INTMASK *matrix_dmask_to_imask( DOUBLEMASK *dmask );
-gboolean matrix_imask_to_heap( Heap *heap, INTMASK *imask, PElement *out );
+gboolean matrix_imask_to_heap( Heap *hi, INTMASK *imask, PElement *out );
 DOUBLEMASK *matrix_ip_to_dmask( PElement *root );
 INTMASK *matrix_ip_to_imask( PElement *root );
 DOUBLEMASK *matrix_model_to_dmask( Matrix *matrix );
