@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,15 +27,6 @@
 
  */
 
-/* When scrolling, do we want the top or the bottom of the object visible.
- * Important for Columns, since we sometimes want to see the title bar and
- * sometimes the edit box at the bottom.
- */
-typedef enum {
-	MODEL_SCROLL_TOP,
-	MODEL_SCROLL_BOTTOM
-} ModelScrollPosition;
-
 /* How to rename symbols.
  */
 typedef struct _ModelRename {
@@ -47,7 +38,6 @@ typedef struct _ModelRename {
  */
 typedef struct _ModelLoadState {
 	char *filename;		/* Name we loaded from */
-	char *filename_user;	/* The filename to record in the model */
 	xmlDoc *xdoc;		/* Document we load from */
 
 	/* 
@@ -55,106 +45,101 @@ typedef struct _ModelLoadState {
 		FIXME ... a linked list? try a hash sometime
 		see model_loadstate_rewrite_name()
 
-		would probably only see a speedup for merging very large
+		would probably only see a speedup for merging large
 		workspaces, not something we do often
 
 	 */
 	GSList *renames;	/* Rename table for this load context */
 
-	/* The column renames we have planned. Don't rewrite exprs with these.
-	 */
-	GSList *column_renames;
-
-	/* Version info we read from this XML file.
-	 */
-	int major;
-	int minor;
-	int micro;
-
 	/* Log error messages here.
 	 */
 	char error_log_buffer[MAX_STRSIZE];
-	VipsBuf error_log;
-
-	/* Set this bool to rewrite string constants using the filename
-	 * rewrite system.
-	 */
-	gboolean rewrite_path;
-
-	/* The old_dir we added with path_rewrite_add() ... if non, NULL,
-	 * unset this rewrite rule on close.
-	 */
-	char *old_dir;
+	BufInfo error_log;
 } ModelLoadState;
 
 #define TYPE_MODEL (model_get_type())
-#define MODEL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_MODEL, Model ))
+#define MODEL( obj ) (GTK_CHECK_CAST( (obj), TYPE_MODEL, Model ))
 #define MODEL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_MODEL, ModelClass))
-#define IS_MODEL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_MODEL ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_MODEL, ModelClass ))
+#define IS_MODEL( obj ) (GTK_CHECK_TYPE( (obj), TYPE_MODEL ))
 #define IS_MODEL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_MODEL ))
-#define MODEL_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_MODEL, ModelClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_MODEL ))
+
+/* Test for is C a child of M.
+ */
+#define MODEL_IS_CHILD( M, C ) \
+	(g_slist_find( MODEL( M )->children, MODEL( C ) ) && \
+	MODEL( C )->parent == MODEL( M ))
 
 struct _Model {
-	iContainer parent_object;
+	GtkObject object;
 
 	/* My instance vars.
 	 */
-	gboolean display;	/* This model should have a view */
+	GSList *children;	/* Models which are sub-this one */
+	Model *parent;		/* Model we are inside */
+	int pos;		/* Position in parent */
 
-	/* For things that have a pop-up window (eg. iimage, plot), the
-	 * position and size of the window.
-	 */
-	int window_x, window_y;
-	int window_width, window_height;
+        char *name;             /* Model name */
+        char *caption;          /* Comment of some sort */
+
+	gboolean display;	/* This model should have a view */
 };
 
 typedef struct _ModelClass {
-	iContainerClass parent_class;
+	GtkObjectClass parent_class;
 
 	/* Build display methods.
 
 		view_new	make a view for this model ... make the top
 				view yourself, thereafter view will watch
-				child_add etc. and manage subviews
+				model_child_add etc. and manage subviews
 				automatically ... use model->display to create
 				and destroy views
 
 	 */
 
-	View *(*view_new)( Model *model, View *parent );
+	View *(*view_new)( void );
 
 	/* Change methods
 
-		edit		open an editor on the model
+		changed 	model has changed signal ... views listen to
+				this to decide when repaint
 
-		header		view model header 
+		edit		open an editor on the model
 
 		scrollto	try to make views visible
 
 		reset		signals views to reset ... eg. textview pops
-				back to whatever the ws says it should be
+				back to whatever the column says it should be
 				displaying (value or formula)
 
-		layout 		try to lay child view out
-
-		front		trigger view_child_front() for all views
-
-		display		create and destroy views
+		info		try to say some useful stuff 
 
 	 */
 
+	void (*changed)( Model * );
 	void (*edit)( GtkWidget *, Model * );
-	void (*header)( GtkWidget *, Model * );
-	void (*scrollto)( Model *, ModelScrollPosition );
+	void (*scrollto)( Model * );
 	void (*reset)( Model * );
-	void (*layout)( Model * );
-	void (*front)( Model * );
-	void (*display)( Model *, gboolean display );
+	void (*info)( Model *, BufInfo * );
+
+	/* Create/destroy methods.
+
+		child_add	a child has been added to us
+
+		child_remove	a child is about to be removed from us
+
+		parent_add	parent has been attached
+
+		parent_remove	parent is about to be removed
+
+	 */
+	 
+	void (*child_add)( Model *, Model *, int );
+	void (*child_remove)( Model *, Model * );
+	void (*parent_add)( Model *, Model * );
+	void (*parent_remove)( Model *, Model * );
 
 	/* Load and save methods.
 
@@ -181,18 +166,13 @@ typedef struct _ModelClass {
 	void (*empty)( Model * );
 } ModelClass;
 
+extern Model *model_base;
+
 extern ModelLoadState *model_loadstate;
 
-gboolean model_loadstate_rename_new( ModelLoadState *state, 
+ModelRename *model_loadstate_rename_new( ModelLoadState *state, 
 	const char *old_name, const char *new_name );
-gboolean model_loadstate_taken( ModelLoadState *state, const char *name );
-gboolean model_loadstate_column_rename_new( ModelLoadState *state, 
-	const char *old_name, const char *new_name );
-gboolean model_loadstate_column_taken( ModelLoadState *state, 
-	const char *name );
-ModelLoadState *model_loadstate_new( 
-	const char *filename, const char *filename_user );
-ModelLoadState *model_loadstate_new_openfile( iOpenFile *of );
+ModelLoadState *model_loadstate_new( const char *filename );
 void model_loadstate_destroy( ModelLoadState *state );
 char *model_loadstate_rewrite_name( char *name );
 void model_loadstate_rewrite( ModelLoadState *state, 
@@ -200,14 +180,12 @@ void model_loadstate_rewrite( ModelLoadState *state,
 
 void model_register_loadable( ModelClass *model_class );
 
-View *model_view_new( Model *model, View *parent );
-void model_scrollto( Model *model, ModelScrollPosition position );
-void model_layout( Model *model );
-void *model_reset( Model *model );
+View *model_view_new( Model *model );
+void *model_changed( Model *model );
+void model_scrollto( Model *model );
+void model_reset( Model *model );
 void *model_edit( GtkWidget *parent, Model *model );
-void *model_header( GtkWidget *parent, Model *model );
-void model_front( Model *model );
-void model_display( Model *model, gboolean display );
+void *model_info( Model *model, BufInfo *info );
 
 void *model_save( Model *model, xmlNode * );
 gboolean model_save_test( Model *model );
@@ -217,14 +195,47 @@ void *model_load( Model *model,
 void *model_load_text( Model *model, Model *parent, iOpenFile * );
 void *model_empty( Model *model );
 
+void *model_test_name( Model *model, const char *name );
+
+int model_get_n_children( Model *model );
+void *model_map( Model *model, model_map_fn fn, void *a, void *b );
+void *model_map3( Model *model, model_map3_fn fn, void *a, void *b, void *c );
+void *model_map4( Model *model, 
+	model_map4_fn fn, void *a, void *b, void *c, void *d );
+void *model_map5( Model *model, 
+	model_map5_fn fn, void *a, void *b, void *c, void *d, void *e ); 
+void *model_map_rev( Model *model, model_map_fn fn, void *a, void *b );
+void *model_map_all( Model *model, model_map_fn fn, void *a );
+void *model_map2_all( Model *model, model_map_fn fn, void *a, void *b );
+void *model_map3_all( Model *model, 
+	model_map3_fn fn, void *a, void *b, void *c );
+void *model_map4_all( Model *model, 
+	model_map4_fn fn, void *a, void *b, void *c, void *d );
+void *model_map_all_intrans( Model *model, model_map_fn fn, void *a );
+
+void model_pos_sort( Model *model );
+int model_pos_last( Model *model );
+void model_pos_renumber( Model *model );
+void model_name_sort( Model *model );
+
+void model_child_add( Model *model, Model *child, int pos );
+void model_child_add_before( Model *parent, Model *child, Model *before );
+void model_child_move( Model *child, int pos );
+void model_child_remove( Model *model, Model *child );
+void model_destroy_children( Model *parent );
+
+void model_set( Model *model, const char *name, const char *caption );
+
 gboolean model_new_xml( ModelLoadState *state, Model *parent, xmlNode *xnode );
 
-GType model_get_type( void );
+GtkType model_get_type( void );
 
 void model_base_init( void );
 
 View *model_build_display_all( Model *model, View *parent );
 
-void model_check_destroy( GtkWidget *parent, Model *model, iWindowFn done_cb );
+void model_check_destroy( GtkWidget *parent, Model *model );
+
+gboolean model_set_display( Model *model, gboolean display );
 
 void *model_clear_edited( Model *model );
