@@ -38,7 +38,6 @@
 #endif /*HAVE_UNISTD_H*/
 #include <string.h>
 
-#include <vips/vips.h>
 #include <vips/util.h>
 
 #include <gtk/gtk.h>
@@ -79,13 +78,13 @@ orderitem_destroy( GtkObject *object )
 }
 
 static void
-orderitem_class_init( OrderitemClass *class )
+orderitem_class_init( OrderitemClass *klass )
 {
 	GtkObjectClass *object_class;
 
-	object_class = (GtkObjectClass*) class;
+	object_class = (GtkObjectClass*) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( GTK_TYPE_LIST_ITEM );
 
 	object_class->destroy = orderitem_destroy;
 
@@ -185,8 +184,8 @@ orderitem_drag_data_get( GtkWidget *widget, GdkDragContext *context,
 	case TARGET_STRING:
 		lab = orderitem_get_label( widget );
 		gtk_selection_data_set( selection_data,
-			selection_data->target, 8, 
-			(guchar *) lab, strlen( lab ) );
+			selection_data->target,
+			8, lab, strlen( lab ) );
 		break;
 	}
 }
@@ -266,10 +265,10 @@ orderitem_init( Orderitem *oi )
 GtkType
 orderitem_get_type( void )
 {
-	static GtkType type = 0;
+	static GtkType orderitem_type = 0;
 
-	if( !type ) {
-		static const GtkTypeInfo info = {
+	if (!orderitem_type) {
+		static const GtkTypeInfo oi_info = {
 			"Orderitem",
 			sizeof( Orderitem ),
 			sizeof( OrderitemClass ),
@@ -280,10 +279,10 @@ orderitem_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		type = gtk_type_unique( GTK_TYPE_LIST_ITEM, &info );
+		orderitem_type = gtk_type_unique (GTK_TYPE_LIST_ITEM, &oi_info);
 	}
 
-	return( type );
+	return( orderitem_type );
 }
 
 GtkWidget *
