@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,27 +27,36 @@
 
  */
 
+/* Prefs we watch.
+ */
+#define PATH_SEARCH (watch_path_get( "CALC_PATH_SEARCH", \
+	path_search_default ))
+#define PATH_START (watch_path_get( "CALC_PATH_START", \
+	path_start_default ))
+#define PATH_TMP (watch_string_get( "CALC_PATH_TMP", \
+	"$HOME/.$PACKAGE-$VERSION/tmp" ))
+
+/* List of directories we have seen this session. 
+ */
+extern GSList *path_session;
+
 extern GSList *path_search_default;
 extern GSList *path_start_default;
-extern const char *path_tmp_default;
 
 /* Type of path_map functions.
  */
 typedef void *(*path_map_fn)( const char *, void *, void *, void * );
 
-void path_rewrite_free_all( void );
-void path_rewrite_add( const char *old, const char *new, gboolean lock );
-void path_rewrite( char *buf );
-void path_compact( char *path );
-void path_expand( char *path );
-char *path_rewrite_file( const char *patt );
-
 GSList *path_parse( const char *path );
 char *path_unparse( GSList *path );
-void path_free2( GSList *path );
+void path_free( GSList *path );
+void *path_map_exact( GSList *path, 
+	const char *patt, path_map_fn fn, void *a );
 void *path_map( GSList *path, const char *patt, path_map_fn fn, void *a );
 void *path_map_dir( const char *dir, const char *patt, 
 	path_map_fn fn, void *a );
-char *path_find_file( const char *patt );
+char *path_find_file( GSList *path, const char *patt );
+
+void path_add( const char *dir );
 
 void path_init( void );
