@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -38,44 +38,31 @@
 struct _Program {
 	iWindow parent_class;
 
-	/* The set of kits we manage.
-	 */
-	Toolkitgroup *kitg;
-
 	GtkWidget *text;
-	gboolean dirty;		/* Has the text changed since we set it */
-	guint text_hash;	/* Hash of the last text we set */
-	GtkWidget *tree;
-	Pane *lpane;
-	int pane_position;
-	Pane *rpane;
-	int rpane_position;
-	gboolean rpane_open;
-	Defbrowser *defbrowser;
-	guint refresh_timeout;	/* Timeout for UI refresh */
-	guint select_changed_sid;	
-	guint row_inserted_sid;	
-	guint row_deleted_sid;	
-
-	/* Track during drags.
-	 */
-	Toolkit *to_kit;
-	int to_pos;
-
-	/* Store for kit/tool view.
-	 */
-	GtkTreeStore *store;
+	GtkWidget *ctree;
+	GtkItemFactory *ifac;
 
 	/* Listen for all kit changes here.
 	 */
 	guint kitgroup_changed_sid;	
 	guint kitgroup_destroy_sid;		
 	
+	/* Track these during a tree update.
+	 */
+	GtkCTreeNode *sibling;	
+	GtkCTreeNode *parent;	
+
 	/* The current kit.
 	 */
 	Toolkit *kit;		
 	guint kit_destroy_sid;	
 
+	/* The symbol for the tool we are editing.
+	 */
+	Symbol *sym;	
+	gboolean dirty;		/* Has the text changed since we set it */
+	guint sym_changed_sid;	/* Listen for sym changes here */
+
 	/* The selected tool.
 	 */
 	Tool *tool;	
@@ -86,17 +73,17 @@ struct _Program {
 	 */
 	char *search;
 	gboolean csens;		/* Case sensitive */
-	gboolean fromtop;	/* Start search from beginning again */
-#ifdef HAVE_GREGEX
 	gboolean regexp;	/* Interpret as regexp */
-	GRegex *comp;		/* Compiled pattern */
-#endif /*HAVE_GREGEX*/
+	gboolean fromtop;	/* Start search from beginning again */
+#ifdef HAVE_REGEXEC
+	regex_t *comp;		/* Compiled pattern */
+#endif /*HAVE_REGEXEC*/
 
 	/* Current search position.
 	 */
 	Symbol *find_sym;	/* Tool containing search point */
-	size_t find_start;	/* Offset into tool text of found string */
-	size_t find_end;	
+	int find_start;		/* Offset into tool text of found string */
+	int find_end;	
 	guint find_sym_destroy_sid;/* Watch for find_sym death here */
 };
 
@@ -107,8 +94,10 @@ typedef struct _ProgramClass {
 	 */
 } ProgramClass;
 
+void program_freeze( void );
+void program_thaw( void );
+
 GtkType program_get_type( void );
-GtkWidget *program_text_new( void );
-Program *program_new( Toolkitgroup *kitg );
+Program *program_new( void );
 
 gboolean program_select( Program *program, Model *model );
