@@ -1,4 +1,4 @@
-/* a real in a workspace
+/* a colour real in a workspace
  */
 
 /*
@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -35,18 +35,25 @@
 #define IS_REAL_CLASS( klass ) \
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_REAL ))
 
-typedef struct _Real {
-	Value parent_object;
+struct _Real {
+	Classmodel parent_class;
 
-	/* Private ... build iobject caption here.
+	/* Class fields.
 	 */
-} Real;
+	double value;
+
+	/* Build view caption here.
+	 */
+	BufInfo caption;
+};
 
 typedef struct _RealClass {
-	ValueClass parent_class;
+	ClassmodelClass parent_class;
 
 	/* My methods.
 	 */
 } RealClass;
 
-GType real_get_type( void );
+GtkType real_get_type( void );
+
+Classmodel *real_new( Rhs *rhs );
