@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,10 +27,6 @@
 
  */
 
-/* Huge :-( this pushes sizeof(Reduce) up to 14MB. But we need to be able to
- * loop down very long lists, eg. for a 65k x 3 LUT held as a Matrix. Drop
- * this down when we represent matricies more sensibly.
- */
 #define SPINE_SIZE (80000)
 
 /* Reduction machine state. Not very opaque ... see mark_reduce()
@@ -58,7 +54,7 @@ struct _Reduce {
 
 	/* Heap we evaluate.
 	 */
-	Heap *heap;
+	Heap *hi;
 
 	/* Nested reductions ... need to be able to longjmp() out of stuff,
 	 * and restore the machine state.
@@ -70,12 +66,11 @@ struct _Reduce {
 	int tsp[SPINE_SIZE];
 };
 
+/* Push a HeapNode on the spine stack.
+ */
 #define RSPUSH(RC,N) { \
-	if( (RC)->sp == SPINE_SIZE ) { \
-		error_top( _( "Stack overflow." ) ); \
-		error_sub( _( "Spine stack overflow, runaway recursion?" ) ); \
-		reduce_throw( (RC) ); \
-	} \
+	if( (RC)->sp == SPINE_SIZE ) \
+		everror( (RC), "spine stack overflow\nrunaway recursion?" ); \
 	else \
 		(RC)->nstack[(RC)->sp++]=(N); \
 }
@@ -100,13 +95,11 @@ struct _Reduce {
  */
 #define RSGETWB(RC) ((RC)->wbstack[(RC)->fsp - 1])
 
+/* Push a new frame.
+ */
 #define RSPUSHFRAME(RC,OUT) { \
-	if( (RC)->fsp == SPINE_SIZE )  { \
-		error_top( _( "Stack overflow." ) ); \
-		error_sub( _( "Frame stack overflow, " \
-			"expression too complex." ) ); \
-		reduce_throw( (RC) ); \
-	} \
+	if( (RC)->fsp == SPINE_SIZE ) \
+		everror( (RC), "frame stack overflow" ); \
 	else { \
 		(RC)->wbstack[(RC)->fsp] = *out; \
 		(RC)->fstack[(RC)->fsp] = (RC)->sp; \
@@ -114,31 +107,28 @@ struct _Reduce {
 	} \
 }
 
+/* Pop a frame.
+ */
 #define RSPOPFRAME(RC) { \
-	if( (RC)->fsp == 0 ) { \
-		error_top( _( "Stack underflow." ) ); \
-		error_sub( _( "Frame stack underflow, you've found a bug!" ) ); \
-		reduce_throw( (RC) ); \
-	} \
+	if( (RC)->fsp == 0 ) \
+		everror( (RC), "frame stack underflow" ); \
 	else { \
 		(RC)->fsp--; \
 		(RC)->sp = (RC)->fstack[(RC)->fsp]; \
 	} \
 }
 
+/* Pop n items from spine stack.
+ */
 #define RSPOP(RC,N) { \
-	if( !RSCHECKARGS(RC,N) ) { \
-		error_top( _( "Stack underflow." ) ); \
-		error_sub( _( "Spine stack underflow, you've found a bug!" ) ); \
-		reduce_throw( (RC) ); \
-	} \
+	if( !RSCHECKARGS(RC,N) ) \
+		everror( (RC), "spine stack underflow" ); \
 	else \
 		(RC)->sp -= (N); \
 }
 
 /* Pop this code before any calls to reduce_*() to init stuff and catch 
- * errors. Arg is function return value. The missing running decrement is done
- * by throw().
+ * errors. Arg is function return value.
  */
 #define REDUCE_CATCH_START( R ) \
 { \
@@ -146,7 +136,6 @@ struct _Reduce {
 	rc->fsps[rc->running] = rc->fsp; \
 	rc->tsp[rc->running] = trace_get_mark(); \
 	if( setjmp( rc->error[rc->running++] ) ) { \
-		g_assert( rc->running >= 0 ); \
 		rc->sp = rc->sps[rc->running]; \
 		rc->fsp = rc->fsps[rc->running]; \
 		trace_pop_to( rc->tsp[rc->running] ); \
@@ -158,30 +147,23 @@ struct _Reduce {
  */
 #define REDUCE_CATCH_STOP \
 { \
-	rc->running -= 1; \
-	g_assert( rc->running >= 0 ); \
+	rc->running--; \
 }
 
 /* Util.
  */
-void reduce_throw( Reduce *rc ) __attribute__((noreturn));
+void everror( Reduce *rc, const char *fmt, ... );
+void veverror( Reduce *rc, const char *fmt, ... );
 
 typedef void *(*reduce_safe_pointer_fn)( Reduce *rc, PElement *, 
 	void *, void *, void *, void * );
 void *reduce_safe_pointer( Reduce *rc, reduce_safe_pointer_fn fn, 
 	void *a, void *b, void *c, void *d );
 
-void reduce_get_list( Reduce *rc, PElement *list );
-void reduce_error_typecheck( Reduce *rc, 
-	PElement *e, const char *name, const char *type );
 typedef void *(*reduce_map_list_fn)( Reduce *rc,
 	PElement *, void *, void * );
 void *reduce_map_list( Reduce *rc, 
 	PElement *base, reduce_map_list_fn fn, void *a, void *b );
-typedef void *(*reduce_map_dict_fn)( Reduce *, 
-	const char *, PElement *, void *a, void *b );
-void *reduce_map_dict( Reduce *rc, 
-	PElement *base, reduce_map_dict_fn fn, void *a, void *b );
 void reduce_clone_list( Reduce *rc, PElement *base, PElement *out );
 int reduce_get_string( Reduce *rc, PElement *base, char *buf, int n );
 int reduce_get_lstring( Reduce *rc, PElement *base, GSList **labels );
@@ -190,26 +172,21 @@ double reduce_get_real( Reduce *rc, PElement *base );
 void reduce_get_class( Reduce *rc, PElement *base );
 Imageinfo *reduce_get_image( Reduce *rc, PElement *base );
 int reduce_get_realvec( Reduce *rc, PElement *base, double *buf, int n );
-int reduce_get_imagevec( Reduce *rc, PElement *base, Imageinfo **buf, int n );
 int reduce_get_matrix( Reduce *rc, 
 	PElement *base, double *buf, int n, int *xsize, int *ysize );
 void reduce_get_matrix_size( Reduce *rc, 
 	PElement *base, int *xsize, int *ysize );
-gboolean reduce_is_elist( Reduce *rc, PElement *base );
-gboolean reduce_is_list( Reduce *rc, PElement *base );
-gboolean reduce_is_string( Reduce *rc, PElement *base );
-gboolean reduce_is_finitestring( Reduce *rc, PElement *base );
-gboolean reduce_is_realvec( Reduce *rc, PElement *base );
-gboolean reduce_is_imagevec( Reduce *rc, PElement *base );
-gboolean reduce_is_matrix( Reduce *rc, PElement *base );
-gboolean reduce_is_class( Reduce *rc, PElement *klass );
+gboolean reduce_iselist( Reduce *rc, PElement *base );
+gboolean reduce_isstring( Reduce *rc, PElement *base );
+gboolean reduce_isfinitestring( Reduce *rc, PElement *base );
+gboolean reduce_isrealvec( Reduce *rc, PElement *base );
+gboolean reduce_ismatrix( Reduce *rc, PElement *base );
+gboolean reduce_isclass( Reduce *rc, PElement *klass );
 int reduce_list_length( Reduce *rc, PElement *base );
-int reduce_list_length_max( Reduce *rc, PElement *base, int max_length );
 void reduce_list_index( Reduce *rc, PElement *base, int n, PElement *out );
-gboolean reduce_is_instanceof_exact( Reduce *rc, 
-	const char *name, PElement *instance );
-gboolean reduce_is_instanceof( Reduce *rc, 
-	const char *name, PElement *instance );
+gboolean reduce_isinstanceof_exact( Reduce *rc, 
+	const char *name, PElement *klass );
+gboolean reduce_isinstanceof( Reduce *rc, const char *name, PElement *klass );
 
 /* Main.
  */
