@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -50,7 +50,7 @@ typedef enum {
 typedef enum {
 	REGIONVIEW_REGION,	/* width & height > 0 */
 	REGIONVIEW_AREA,	/* width & height > 0 and locked */
-	REGIONVIEW_MARK,	/* width & height == 0 */
+	REGIONVIEW_POINT,	/* width & height == 0 */
 	REGIONVIEW_ARROW,	/* width & height unconstrained */
 	REGIONVIEW_HGUIDE,	/* width == image width, height == 0 */
 	REGIONVIEW_VGUIDE,	/* width == 0, height == image height */
@@ -58,23 +58,6 @@ typedef enum {
 	REGIONVIEW_BOX		/* floating dashed box for paintbox */
 } RegionviewType;
 
-/* Resize types.
- */
-typedef enum {
-	REGIONVIEW_RESIZE_NONE,
-	REGIONVIEW_RESIZE_MOVE,
-	REGIONVIEW_RESIZE_EDIT,
-	REGIONVIEW_RESIZE_TOPLEFT,
-	REGIONVIEW_RESIZE_TOP,
-	REGIONVIEW_RESIZE_TOPRIGHT,
-	REGIONVIEW_RESIZE_RIGHT,
-	REGIONVIEW_RESIZE_BOTTOMRIGHT,
-	REGIONVIEW_RESIZE_BOTTOM,
-	REGIONVIEW_RESIZE_BOTTOMLEFT,
-	REGIONVIEW_RESIZE_LEFT,
-	REGIONVIEW_RESIZE_LAST
-} RegionviewResize;
-
 struct _Regionview {
 	View view;
 
@@ -84,7 +67,7 @@ struct _Regionview {
 	/* State for resize/move etc.
 	 */
 	RegionviewState state;
-	RegionviewResize resize;/* Resize type */
+	iWindowShape shape;	/* Resize type */
 	int dx, dy;		/* Drag offset */
 	gboolean grabbed;	/* Currently tracking with mouse */
 
@@ -101,11 +84,13 @@ struct _Regionview {
 
 	/* The signals we've connected to.
 	 */
-	guint expose_sid;
+	guint expose_start_sid;
+	guint repaint_sid;
 	guint destroy_sid;
 	guint event_sid;
 	guint changed_sid;
 	guint conv_destroy_sid;
+	guint ws_changed_sid;
 	guint model_changed_sid;
 
 	/* Model info we read for display.
@@ -117,6 +102,7 @@ struct _Regionview {
 	gboolean unpainting;	/* We are unpainting */
 	Rect area;		/* Area of region ... image coordinates */
 	Rect label;		/* Area covered by label ... canvas cods */
+	guint click;		/* Timer for doubleclick determination */
 	int ascent;		/* Height of ascenders for text */
 	int dash_offset;
 	guint dash_crawl;	/* Timer for dash crawl animation */
@@ -127,7 +113,7 @@ struct _Regionview {
 
 	/* Text of label we display 
 	 */
-	VipsBuf caption;
+	BufInfo caption;
 };
 
 typedef struct _RegionviewClass {
@@ -137,7 +123,7 @@ typedef struct _RegionviewClass {
 	 */
 } RegionviewClass;
 
-void regionview_attach( Regionview *regionview, int x, int y );
+void regionview_attach( Regionview *regionview );
 
 GtkType regionview_get_type( void );
 Regionview *regionview_new( Classmodel *classmodel, 
