@@ -16,9 +16,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -36,56 +36,57 @@
 
 static HeapmodelClass *parent_class = NULL;
 
+static void
+rhs_destroy( GtkObject *object )
+{
+	Rhs *rhs;
+
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_RHS( object ) );
+
+	rhs = RHS( object );
+
+	/* My instance destroy stuff.
+	 */
+
+#ifdef DEBUG
+	printf( "rhs_destroy: " );
+	row_name_print( HEAPMODEL( rhs )->row );
+	printf( " (0x%x)\n", (unsigned int) rhs );
+#endif /*DEBUG*/
+
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
+}
+
 /* child is about to be added ... update our graphic/scol/text shortcuts.
  */
 static void
-rhs_child_add( iContainer *parent, iContainer *child, int pos )
+rhs_child_add( Model *parent, Model *child, int pos )
 {
 	Rhs *rhs = RHS( parent );
 
 	if( IS_SUBCOLUMN( child ) ) {
-		IDESTROY( rhs->scol );
-		rhs->scol = MODEL( child );
+		FREEO( rhs->scol );
+		rhs->scol = child;
 	}
 	else if( IS_ITEXT( child ) ) {
-		IDESTROY( rhs->itext );
-		rhs->itext = MODEL( child );
+		FREEO( rhs->itext );
+		rhs->itext = child;
 	}
 	else {
-		IDESTROY( rhs->graphic );
-		rhs->graphic = MODEL( child );
+		FREEO( rhs->graphic );
+		rhs->graphic = child;
 	}
 
-	ICONTAINER_CLASS( parent_class )->child_add( parent, child, pos );
+	MODEL_CLASS( parent_class )->child_add( parent, child, pos );
 }
 
 static void
-rhs_child_remove( iContainer *parent, iContainer *child )
+rhs_parent_add( Model *child, Model *parent )
 {
-	Rhs *rhs = RHS( parent );
+	assert( IS_ROW( parent ) );
 
-	if( (void *) child == (void *) rhs->graphic )
-		rhs->graphic = NULL;
-	else if( (void *) child == (void *) rhs->scol )
-		rhs->scol = NULL;
-	else if( (void *) child == (void *) rhs->itext )
-		rhs->itext = NULL;
-
-	ICONTAINER_CLASS( parent_class )->child_remove( parent, child );
-}
-
-static void
-rhs_parent_add( iContainer *child )
-{
-	g_assert( IS_ROW( child->parent ) );
-
-	ICONTAINER_CLASS( parent_class )->parent_add( child );
-}
-
-static View *
-rhs_view_new( Model *model, View *parent )
-{
-	return( rhsview_new() );
+	MODEL_CLASS( parent_class )->parent_add( child, parent );
 }
 
 static gboolean
@@ -94,11 +95,14 @@ rhs_load( Model *model,
 {
 	Rhs *rhs = RHS( model );
 
-	g_assert( IS_ROW( parent ) );
+	if( !IS_ROW( parent ) ) {
+		ierrors( "rhs_load: can only add a rhs to a row" );
+		return( FALSE );
+	}
 
 	/* Hmm. Is this guaranteed?
 	 */
-	g_assert( sizeof( RhsFlags ) == sizeof( int ) );
+	assert( sizeof( RhsFlags ) == sizeof( int ) );
 
 	if( !get_iprop( xnode, "vislevel", &rhs->vislevel ) ||
 		!get_iprop( xnode, "flags", (int *) &rhs->flags ) )
@@ -120,8 +124,8 @@ rhs_save( Model *model, xmlNode *xnode )
 	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
 		return( NULL );
 
-	if( !set_iprop( xthis, "vislevel", rhs->vislevel ) ||
-		!set_iprop( xthis, "flags", rhs->flags ) )
+	if( !set_prop( xthis, "vislevel", "%d", rhs->vislevel ) ||
+		!set_prop( xthis, "flags", "%d", rhs->flags ) )
 		return( NULL );
 
 	return( xthis );
@@ -131,77 +135,58 @@ rhs_save( Model *model, xmlNode *xnode )
  */
 typedef struct {
 	const char *name;
-	GType (*type)( void );
+	GtkType (*type)( void );
+	Classmodel *(*build)( Rhs * );
 } RhsGraphic;
 
-/* All our graphicdisplay widgets. Order is important! Most-derived classes
+/* All our graphicdisplay widgets. Order is important! Most derived classes
  * first.
  */
 static RhsGraphic rhs_graphic[] = {
-	{ CLASS_CLOCK, clock_get_type },
-	{ CLASS_EXPRESSION, expression_get_type },
-	{ CLASS_GROUP, group_get_type },
-	{ CLASS_LIST, group_get_type },
-	{ CLASS_PATHNAME, pathname_get_type },
-	{ CLASS_FONTNAME, fontname_get_type },
-	{ CLASS_TOGGLE, toggle_get_type },
-	{ CLASS_SLIDER, slider_get_type },
-	{ CLASS_COLOUR, colour_get_type },
-	{ CLASS_OPTION, option_get_type },
-	{ CLASS_MATRIX, matrix_get_type },
-	{ CLASS_ARROW, iarrow_get_type },
-	{ CLASS_REGION, iregion_get_type },
-	{ CLASS_PLOT, plot_get_type },
-	{ CLASS_IMAGE, iimage_get_type },
-	{ CLASS_NUMBER, number_get_type },
-	{ CLASS_REAL, real_get_type },
-	{ CLASS_VECTOR, vector_get_type },
-	{ CLASS_STRING, string_get_type }
+	{ CLASS_FILENAME, filename_get_type, filename_new },
+	{ CLASS_TOGGLE, toggle_get_type, toggle_new },
+	{ CLASS_SLIDER, slider_get_type, slider_new },
+	{ CLASS_COLOUR, colour_get_type, colour_new },
+	{ CLASS_OPTION, option_get_type, option_new },
+	{ CLASS_MATRIX, matrix_get_type, matrix_new },
+	{ CLASS_ARROW, iarrow_get_type, iarrow_new },
+	{ CLASS_REGION, iregion_get_type, iregion_new },
+	{ CLASS_IMAGE, iimage_get_type, iimage_new },
+	{ CLASS_REAL, real_get_type, real_new }
 };
 
 /* Create/destroy the graphic display.
  */
-static gboolean
+static void
 rhs_refresh_graphic( Rhs *rhs, PElement *root )
 {
 	gboolean result;
 	Row *row = HEAPMODEL( rhs )->row;
 	int i;
 
-	if( !heap_is_class( root, &result ) ) 
-		return( FALSE );
-
 	/* Only for non-parameter class objects.
 	 */
-	if( result && 
-		row->sym &&
-		row->sym->type != SYM_PARAM ) {
+	if( heap_isclass( root, &result ) && result && 
+		row->sym->type != SYM_PARAM )
 		for( i = 0; i < IM_NUMBER( rhs_graphic ); i++ ) {
 			const char *name = rhs_graphic[i].name;
 
-			if( !heap_is_instanceof( name, root, &result ) ) 
-				return( FALSE );
-			if( result ) 
-				break;
+			if( !heap_isinstanceof( name, root, &result ) ) 
+				return;
+			if( result ) {
+				GtkType type = rhs_graphic[i].type();
+
+				if( !rhs->graphic || 
+					!TYPE_EXACT( rhs->graphic, type )) 
+					rhs_graphic[i].build( rhs );
+
+				return;
+			}
 		}
 
-		if( i != IM_NUMBER( rhs_graphic ) ) {
-			GType type = rhs_graphic[i].type();
-
-			if( !rhs->graphic || !TYPE_EXACT( rhs->graphic, type ) )
-				classmodel_new_classmodel( type, rhs );
-		}
-		else
-			/* Not a class we know about.
-			 */
-			IDESTROY( rhs->graphic );
-	}
-	else
-		/* Should be no graphic display.
-		 */
-		IDESTROY( rhs->graphic );
-
-	return( TRUE );
+	/* Should be no graphic display.
+	 */
+	FREEO( rhs->graphic );
 }
 
 static void *
@@ -219,23 +204,21 @@ rhs_new_heap( Heapmodel *heapmodel, PElement *root )
 
 	/* Create/reuse/destroy the graphic display.
 	 */
-	if( !rhs_refresh_graphic( rhs, root ) )
-		return( rhs );
+	rhs_refresh_graphic( rhs, root );
 
 	/* Create/reuse/destroy class display. Only for non-param symbols.
 	 */
-	if( !heap_is_class( root, &result ) ) 
+	if( !heap_isclass( root, &result ) ) 
 		return( rhs );
-	if( result && 
-		row->sym &&
-		row->sym->type != SYM_PARAM ) {
+	if( result && row->sym->type != SYM_PARAM ) {
 		if( !rhs->scol || !IS_SUBCOLUMN( rhs->scol ) ) 
 			subcolumn_new( rhs, NULL );
 	}
-	else 
+	else {
 		/* Should be no klass display.
 		 */
-		IDESTROY( rhs->scol );
+		FREEO( rhs->scol );
+	}
 
 	/* Create/reuse/destroy text display.
 	 */
@@ -318,7 +301,7 @@ rhs_set_vislevel( Rhs *rhs, int vislevel )
 	printf( "... set to: %d\n", rhs->vislevel );
 #endif /*DEBUG*/
 
-	iobject_changed( IOBJECT( rhs ) );
+	model_changed( MODEL( rhs ) );
 }
 
 void
@@ -346,24 +329,24 @@ rhs_update_model( Heapmodel *heapmodel )
 }
 
 static void
-rhs_class_init( RhsClass *class )
+rhs_class_init( RhsClass *klass )
 {
-	iContainerClass *icontainer_class = (iContainerClass *) class;
-	ModelClass *model_class = (ModelClass *) class;
-	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	ModelClass *model_class = (ModelClass *) klass;
+	HeapmodelClass *heapmodel_class = (HeapmodelClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_HEAPMODEL );
 
 	/* Create signals.
 	 */
 
 	/* Init methods.
 	 */
-	icontainer_class->child_add = rhs_child_add;
-	icontainer_class->child_remove = rhs_child_remove;
-	icontainer_class->parent_add = rhs_parent_add;
+	object_class->destroy = rhs_destroy;
 
-	model_class->view_new = rhs_view_new;
+	model_class->view_new = rhsview_new;
+	model_class->child_add = rhs_child_add;
+	model_class->parent_add = rhs_parent_add;
 	model_class->load = rhs_load;
 	model_class->save = rhs_save;
 
@@ -372,7 +355,7 @@ rhs_class_init( RhsClass *class )
 
 	/* Static init.
 	 */
-	model_register_loadable( MODEL_CLASS( class ) );
+	model_register_loadable( MODEL_CLASS( klass ) );
 }
 
 static void
@@ -392,42 +375,39 @@ rhs_init( Rhs *rhs )
         rhs->itext = NULL;
 }
 
-GType
+GtkType
 rhs_get_type( void )
 {
-	static GType type = 0;
+	static GtkType rhs_type = 0;
 
-	if( !type ) {
-		static const GTypeInfo info = {
-			sizeof( RhsClass ),
-			NULL,           /* base_init */
-			NULL,           /* base_finalize */
-			(GClassInitFunc) rhs_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
+	if( !rhs_type ) {
+		static const GtkTypeInfo info = {
+			"Rhs",
 			sizeof( Rhs ),
-			32,             /* n_preallocs */
-			(GInstanceInitFunc) rhs_init,
+			sizeof( RhsClass ),
+			(GtkClassInitFunc) rhs_class_init,
+			(GtkObjectInitFunc) rhs_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
 		};
 
-		type = g_type_register_static( TYPE_HEAPMODEL, 
-			"Rhs", &info, 0 );
+		rhs_type = gtk_type_unique( TYPE_HEAPMODEL, &info );
 	}
 
-	return( type );
+	return( rhs_type );
 }
 
 Rhs *
 rhs_new( Row *row )
 {
-	Rhs *rhs = RHS( g_object_new( TYPE_RHS, NULL ) );
+	Rhs *rhs = gtk_type_new( TYPE_RHS );
 
-	icontainer_child_add( ICONTAINER( row ), ICONTAINER( rhs ), -1 );
+	model_child_add( MODEL( row ), MODEL( rhs ), -1 );
 
 #ifdef DEBUG
-	printf( "rhs_new: " );
-	row_name_print( HEAPMODEL( rhs )->row );
-	printf( " (%p)\n", rhs );
+	printf( "rhs_new: %s (0x%x)\n", 
+		MODEL( row )->name, (unsigned int) rhs );
 #endif /*DEBUG*/
 
 	return( rhs );
@@ -454,9 +434,9 @@ rhs_child_edited( Rhs *rhs )
 	else if( rhs->graphic && CLASSMODEL( rhs->graphic )->edited )
 		return( TRUE );
 	else if( rhs->scol ) 
-		return( icontainer_map( ICONTAINER( rhs->scol ),
-			(icontainer_map_fn) rhs_child_edited_sub, 
-			NULL, NULL ) != NULL );
+		return( model_map( rhs->scol,
+			(model_map_fn) rhs_child_edited_sub, NULL, NULL ) !=
+			NULL );
 	else
 		return( FALSE );
 }
