@@ -16,9 +16,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,16 +29,12 @@
  */
 
 #define TYPE_RHS (rhs_get_type())
-#define RHS( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_RHS, Rhs ))
+#define RHS( obj ) (GTK_CHECK_CAST( (obj), TYPE_RHS, Rhs ))
 #define RHS_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_RHS, RhsClass))
-#define IS_RHS( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_RHS ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_RHS, RhsClass ))
+#define IS_RHS( obj ) (GTK_CHECK_TYPE( (obj), TYPE_RHS ))
 #define IS_RHS_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_RHS ))
-#define RHS_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_RHS, RhsClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_RHS ))
 
 /* Which children are visible.
  */
@@ -66,7 +62,7 @@ typedef struct _RhsClass {
 	 */
 } RhsClass;
 
-GType rhs_get_type( void );
+GtkType rhs_get_type( void );
 Rhs *rhs_new( Row *row );
 
 void rhs_set_vislevel( Rhs *rhs, int vislevel );
