@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,15 +29,12 @@
 
 #define TYPE_ROW (row_get_type())
 #define ROW( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_ROW, Row ))
+	(GTK_CHECK_CAST( (obj), TYPE_ROW, Row ))
 #define ROW_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_ROW, RowClass))
-#define IS_ROW( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_ROW ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_ROW, RowClass ))
+#define IS_ROW( obj ) (GTK_CHECK_TYPE( (obj), TYPE_ROW ))
 #define IS_ROW_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_ROW ))
-#define ROW_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_ROW, RowClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_ROW ))
 
 /* For when we're flashing the showstate up.
  */
@@ -59,12 +56,14 @@ struct _Row {
 	Row *top_row;		/* Enclosing root row */
 
 	Symbol *sym;		/* Symbol we represent */
+	Symbol *sym_private;	/* The sym we free on destroy, if non-NULL */
 
 	Expr *expr;		/* The expr we edit */
+	Expr *expr_private;	/* The expr we free on destroy, if non-NULL */
 	gboolean err;		/* Set if this row is on the error list */
 
 	gboolean selected;	/* Selected or not */
-	gboolean is_class;	/* Display spin buttons */
+	gboolean isclass;	/* Display spin buttons */
 	gboolean popup;		/* Set to pop up view on 1st display */
 	gboolean to_save;	/* Should be saved (part of only-save-modded) */
 
@@ -87,35 +86,27 @@ typedef struct _RowClass {
 } RowClass;
 
 const char *row_name( Row *row );
-void row_qualified_name_relative( Symbol *context, Row *row, VipsBuf *buf );
-void row_qualified_name( Row *row, VipsBuf *buf );
+void row_qualified_name_relative( Symbol *context, Row *row, BufInfo *buf );
+void row_qualified_name( Row *row, BufInfo *buf );
 void *row_name_print( Row *row );
 
 void row_error_set( Row *row );
 void row_error_clear( Row *row );
 
-Workspace *row_get_workspace( Row *row );
-
-GType row_get_type( void );
+GtkType row_get_type( void );
 void row_link_symbol( Row *row, Symbol *sym, PElement *root );
 Row *row_new( Subcolumn *scol, Symbol *sym, PElement *root );
 
-void *row_dirty( Row *row, gboolean clear_dirty );
-void *row_dirty_intrans( Row *row, gboolean clear_dirty );
+void *row_dirty_set( Row *row );
+void *row_dirty( Row *row );
+void *row_dirty_intrans( Row *row );
 
 void row_recomp( Row *row );
 
-void *row_is_selected( Row *row );
 void *row_deselect( Row *row );
-void *row_select_ensure( Row *row );
 void *row_select( Row *row );
 void *row_select_extend( Row *row );
 void *row_select_toggle( Row *row );
 void row_select_modifier( Row *row, guint state );
 
-void row_show_dependents( Row *row );
-void row_hide_dependents( Row *row );
-void row_set_status( Row *row );
-
-Row *row_parse_name( Symbol *context, const char *path );
-
+void row_set_show( Row *row, RowShowState show );
