@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -33,6 +33,10 @@
 
 #include "ip.h"
 
+/* Draw LEDs rather than recolouring tally buttons.
+ */
+#define CALC_DISPLAY_LED (watch_bool_get( "CALC_DISPLAY_LED", FALSE ))
+
 static ModelClass *parent_class = NULL;
 
 enum {
@@ -60,18 +64,19 @@ rowview_destroy( GtkObject *object )
 
 #ifdef DEBUG
 	printf( "rowview_destroy: " );
-	row_name_print( ROW( VOBJECT( rview )->iobject ) );
+	row_name_print( ROW( VIEW( rview )->model ) );
 	printf( "\n" );
 #endif /*DEBUG*/
 
-	IM_FREE( rview->last_tooltip );
+	FREEFI( gtk_timeout_remove, rview->set_name_tid );
+	FREE( rview->last_tooltip );
 
 	/* Kill children ... must do this ourselves, since we are not a
 	 * self-contained widget.
 	 */
-	DESTROY_GTK( rview->but );
-	DESTROY_GTK( rview->spin );
-	DESTROY_GTK( rview->led );
+	FREEW( rview->but );
+	FREEW( rview->spin );
+	FREEW( rview->led );
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -93,18 +98,100 @@ rowview_attach( Rowview *rview, GtkWidget *child, int x,
 	gtk_widget_unref( child );
 }
 
-static void
-rowview_update_widgets( Rowview *rview )
+static gint
+rowview_set_name_cb( Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
-	int pos = ICONTAINER( row )->pos;
-	gboolean editable = row->ws->mode != WORKSPACE_MODE_NOEDIT;
+	rview->set_name_tid = 0;
+
+#ifdef DEBUG
+	printf( "rowview_set_name_cb: " );
+	row_name_print( ROW( VIEW( rview )->model ) );
+	printf( ".name = %s\n", rview->to_set_name );
+#endif /*DEBUG*/
+
+	set_name( rview->but, rview->to_set_name );
+	rview->set_name = rview->to_set_name;
+	rview->to_set_name = NULL;
+
+	return( FALSE );
+}
+
+static void
+rowview_set_name( Rowview *rview, const char *set_name )
+{
+	gboolean changed = FALSE;
+
+	/* If there's a timeout currently set, test against the name we're
+	 * planning to set. Pointer equality is mostly good enough .. we are 
+	 * setting with static strings.
+	 */
+	if( rview->set_name_tid && set_name != rview->to_set_name ) {
+		rview->to_set_name = set_name;
+		changed = TRUE;
+	}
+	/* No timeout? Test against the current name.
+	 */
+	else if( set_name != rview->set_name ) {
+		rview->to_set_name = set_name;
+		changed = TRUE;
+	}
+
+	if( changed ) {
+		/* Reset the timeout.
+		 */
+		FREEFI( gtk_timeout_remove, rview->set_name_tid );
+		rview->set_name_tid = gtk_timeout_add( 200, 
+			(GtkFunction) rowview_set_name_cb, rview );
+	}
+}
+
+static void *
+rowview_tip_sub( Link *link, BufInfo *buf )
+{
+	if( link->child->dirty ) {
+		symbol_qualified_name( link->child, buf );
+		buf_appends( buf, " " );
+	}
+
+	return( NULL );
+}
+
+static void
+rowview_tip( Rowview *rview, BufInfo *buf )
+{
+	Row *row = ROW( VIEW( rview )->model );
+
+	if( row == row->top_row && row->sym->dirty ) {
+		Symbol *sym = row->sym;
+
+		if( sym->ndirtychildren ) {
+			buf_appends( buf, ", blocked on " );
+			slist_map( sym->topchildren,
+				(SListMapFn) rowview_tip_sub, buf );
+		}
+	}
+}
+
+static void
+rowview_set_tooltip( Rowview *rview, const char *tip )
+{
+	if( !rview->last_tooltip || strcmp( rview->last_tooltip, tip ) != 0 ) {
+		set_tooltip( rview->but, "%s", tip );
+		SETSTR( rview->last_tooltip, tip );
+	}
+}
+
+static void 
+rowview_refresh( View *view )
+{
+	Rowview *rview = ROWVIEW( view );
+	Row *row = ROW( VIEW( rview )->model );
+	int pos = MODEL( row )->pos;
 
 #ifdef DEBUG
 	printf( "rowview_refresh: " );
 	row_name_print( row );
 	printf( "\n" );
-	printf( "\teditable == %d\n", editable ); 
 #endif /*DEBUG*/
 
 	/* Attach widgets to parent in new place.
@@ -129,25 +216,31 @@ rowview_update_widgets( Rowview *rview )
 				GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL );
 	}
 
+        set_glabel( rview->label, "%s", row_name( row ) );
+
+	/* Spin visible only if this is a class. 
+	 */
+	widget_visible( rview->spin, rview->visible && row->isclass );
+
         /* Set colours.
          */
 	if( CALC_DISPLAY_LED ) {
-		char *stock_id;
+		LedColour colour;
 
-		stock_id = STOCK_LED_OFF;
 		if( row->selected )
-			stock_id = STOCK_LED_GREEN;
+			colour = LED_GREEN;
 		else if( row->show == ROW_SHOW_PARENT )
-			stock_id = STOCK_LED_CYAN;
+			colour = LED_CYAN;
 		else if( row->show == ROW_SHOW_CHILD )
-			stock_id = STOCK_LED_BLUE;
+			colour = LED_BLUE;
 		else if( row->err )
-			stock_id = STOCK_LED_RED;
+			colour = LED_RED;
 		else if( row->dirty )
-			stock_id = STOCK_LED_YELLOW;
+			colour = LED_YELLOW;
+		else
+			colour = LED_OFF;
 
-		gtk_image_set_from_stock( GTK_IMAGE( rview->led ), 
-			stock_id, GTK_ICON_SIZE_MENU );
+		led_set_colour( LED( rview->led ), colour ); 
 	}
 	else {
 		gchar *name = "";
@@ -163,53 +256,41 @@ rowview_update_widgets( Rowview *rview )
 		else if( row->dirty )
 			name = "dirty_widget";
 
-		gtk_widget_set_name( rview->but, name );
+		rowview_set_name( rview, name );
 	}
-	widget_visible( rview->led, 
-		rview->visible && CALC_DISPLAY_LED && editable );
 
-	/* Update button.
-	 */
-        set_glabel( rview->label, "%s", row_name( row ) );
-	widget_visible( rview->but, rview->visible && editable );
+	if( row->expr && row->expr->err ) 
+		/* Display error message in tooltip.
+		 */
+		rowview_set_tooltip( rview, row->expr->errstr );
+	else if( row->expr ) {
+		BufInfo buf;
+		char txt[200];
 
-	/* Spin visible only if this is a class. 
-	 */
-	widget_visible( rview->spin, 
-		rview->visible && row->is_class && editable );
+		buf_init_static( &buf, txt, 200 );
+		expr_tip( row->expr, &buf );
+		rowview_tip( rview, &buf );
+		rowview_set_tooltip( rview, buf_all( &buf ) );
+	}
+	else 
+		rowview_set_tooltip( rview, 
+			"Left-click to select, "
+			"shift-left-click to extend select, "
+			"double-left-click to edit, "
+			"right-click for menu, "
+			"left-drag to move" );
 
-	if( rview->rhsview )
-		widget_visible( GTK_WIDGET( rview->rhsview ), rview->visible );
-}
-
-static void 
-rowview_reset( View *view )
-{
-	Rowview *rview = ROWVIEW( view );
-
-	rowview_update_widgets( rview );
-
-	VIEW_CLASS( parent_class )->reset( view );
-}
-
-static void 
-rowview_refresh( vObject *vobject )
-{
-	Rowview *rview = ROWVIEW( vobject );
-
-	rowview_update_widgets( rview );
-
-	VOBJECT_CLASS( parent_class )->refresh( vobject );
+	VIEW_CLASS( parent_class )->refresh( view );
 }
 
 /* Single click on button callback.
  */
 static void
-rowview_single_cb( GtkWidget *wid, GdkEvent *event, Rowview *rview )
+rowview_single_cb( GtkWidget *wid, Rowview *rview, guint state )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 
-	row_select_modifier( row, event->button.state );
+	row_select_modifier( row, state );
 }
 
 /* Edit our object.
@@ -217,11 +298,15 @@ rowview_single_cb( GtkWidget *wid, GdkEvent *event, Rowview *rview )
 static gboolean
 rowview_edit( Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Model *graphic = row->child_rhs->graphic;
 
-	if( graphic )
-		model_edit( GTK_WIDGET( rview->sview ), graphic );
+	if( !graphic || !MODEL_CLASS( GTK_OBJECT( graphic )->klass )->edit ) {
+		ierrors( "object not editable" );
+		return( FALSE );
+	}
+
+	model_edit( GTK_WIDGET( rview->sview ), graphic );
 
 	return( TRUE );
 }
@@ -229,10 +314,10 @@ rowview_edit( Rowview *rview )
 /* Double click on button callback.
  */
 static void
-rowview_double_cb( GtkWidget *button, GdkEvent *event, Rowview *rview )
+rowview_double_cb( GtkWidget *button, Rowview *rview )
 {
 	if( !rowview_edit( rview ) ) 
-		iwindow_alert( button, GTK_MESSAGE_ERROR );
+		box_alert( button );
 }
 
 /* Edit in menu.
@@ -241,30 +326,7 @@ static void
 rowview_edit_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
 	if( !rowview_edit( rview ) ) 
-		iwindow_alert( button, GTK_MESSAGE_ERROR );
-}
-
-/* Show info.
- */
-static gboolean
-rowview_header( Rowview *rview )
-{
-	Row *row = ROW( VOBJECT( rview )->iobject );
-	Model *graphic = row->child_rhs->graphic;
-
-	if( graphic )
-		model_header( GTK_WIDGET( rview->sview ), graphic );
-
-	return( TRUE );
-}
-
-/* Info in menu.
- */
-static void
-rowview_header_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
-{
-	if( !rowview_header( rview ) ) 
-		iwindow_alert( button, GTK_MESSAGE_ERROR );
+		box_alert( button );
 }
 
 /* Clone the current item.
@@ -272,23 +334,20 @@ rowview_header_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 static void
 rowview_clone_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Workspace *ws = row->top_col->ws;
 
 	/* Only allow clone of top level rows.
 	 */
 	if( row->top_row != row ) {
-		error_top( _( "Can't duplicate." ) );
-		error_sub( "%s", 
-			_( "You can only duplicate top level rows." ) );
-		iwindow_alert( button, GTK_MESSAGE_INFO );
+		box_info( button, "you can only clone top level rows" );
 		return;
 	}
 
         workspace_deselect_all( ws );
         row_select( row );
-        if( !workspace_selected_duplicate( ws ) )
-		iwindow_alert( button, GTK_MESSAGE_ERROR );
+        if( !workspace_clone_selected( ws ) )
+                box_alert( NULL );
         workspace_deselect_all( ws );
 
         symbol_recalculate_all();
@@ -299,12 +358,10 @@ rowview_clone_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 static void
 rowview_ungroup_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 
-        workspace_deselect_all( row->ws );
-        row_select( row );
-        if( !workspace_selected_ungroup( row->ws ) )
-		iwindow_alert( button, GTK_MESSAGE_ERROR );
+        if( mainw_ungroup( row ) )
+                box_alert( NULL );
         symbol_recalculate_all();
 }
 
@@ -314,12 +371,15 @@ static void
 rowview_save_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
 	iWindow *iwnd = IWINDOW( view_get_toplevel( VIEW( rview ) ) );
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Model *graphic = row->child_rhs->graphic;
 
-	if( graphic )
-		classmodel_graphic_save( CLASSMODEL( graphic ),
-			GTK_WIDGET( iwnd ) ); 
+	if( !graphic ) {
+		box_error( GTK_WIDGET( iwnd ), "object not saveable" );
+		return;
+	}
+
+	classmodel_graphic_save( GTK_WIDGET( iwnd ), CLASSMODEL( graphic ) );
 }
 
 /* Replace the current item.
@@ -328,12 +388,15 @@ static void
 rowview_replace_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
 	iWindow *iwnd = IWINDOW( view_get_toplevel( VIEW( rview ) ) );
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Model *graphic = row->child_rhs->graphic;
 
-	if( graphic )
-		classmodel_graphic_replace( CLASSMODEL( graphic ),
-			GTK_WIDGET( iwnd ) ); 
+	if( !graphic ) {
+		box_error( GTK_WIDGET( iwnd ), "object not replaceable" );
+		return;
+	}
+
+	classmodel_graphic_replace( GTK_WIDGET( iwnd ), CLASSMODEL( graphic ) );
 }
 
 /* Recalculate the current item.
@@ -341,23 +404,14 @@ rowview_replace_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 static void
 rowview_recalc_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Workspace *ws = row->top_col->ws;
 
-	/* Mark dirty from this sym on, and force a recalc even if recalc is
-	 * off.
-	 */
         workspace_deselect_all( ws );
         row_select( row );
-        if( !workspace_selected_recalc( ws ) )
-		iwindow_alert( button, GTK_MESSAGE_ERROR );
+        if( !mainw_selected_recalc() )
+                box_alert( NULL );
         workspace_deselect_all( ws );
-
-	/* Now ... pick up any errors.
-	 */
-	if( row->sym &&
-		!symbol_recalculate_check( row->sym ) )
-		iwindow_alert( button, GTK_MESSAGE_ERROR );
 }
 
 /* Reset the current item.
@@ -365,10 +419,10 @@ rowview_recalc_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 static void
 rowview_clear_edited_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 
-	(void) icontainer_map_all( ICONTAINER( row ),
-		(icontainer_map_fn) model_clear_edited, NULL );
+	(void) model_map_all( MODEL( row ),
+		(model_map_fn) model_clear_edited, NULL );
 	symbol_recalculate_all();
 }
 
@@ -377,12 +431,19 @@ rowview_clear_edited_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 static void
 rowview_remove_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Workspace *ws = row->top_col->ws;
 
+	/* Only allow remove of top level rows.
+	 */
+	if( row->top_row != row ) {
+		box_info( button, "you can only remove top level rows" );
+		return;
+	}
+
 	workspace_deselect_all( ws );
 	row_select( row );
-	workspace_selected_remove_yesno( ws, button );
+	mainw_selected_remove();
 }
 
 /* Callback for up/down spin button.
@@ -391,35 +452,45 @@ static void
 rowview_spin_up_cb( GtkWidget *widget, gpointer client )
 {
 	Rowview *rview = ROWVIEW( client );
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Rhs *rhs = row->child_rhs;
 
 	rhs_vislevel_down( rhs );
-	workspace_set_modified( row->ws, TRUE );
+	workspace_set_modified( row->sym );
 }
 
 static void
 rowview_spin_down_cb( GtkWidget *widget, gpointer client )
 {
 	Rowview *rview = ROWVIEW( client );
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
 	Rhs *rhs = row->child_rhs;
 
 	rhs_vislevel_up( rhs );
-	workspace_set_modified( row->ws, TRUE );
+	workspace_set_modified( row->sym );
 }
 
-/* Scroll to make tally entry visible. 
+/* Scroll to make tally entry visible. If it's in a closed column, pop the
+ * column open first.
  */
 static void
-rowview_scrollto( View *view, ModelScrollPosition position )
+rowview_scrollto( View *view )
 {
 	Rowview *rview = ROWVIEW( view );
+	Row *row = ROW( VIEW( rview )->model );
+	Column *col = row->top_col;
 	Columnview *cview = view_get_columnview( VIEW( rview ) );
 	Workspaceview *wview = cview->wview;
 
         int x, y, w, h;
 
+        /* If the column is closed, open it.
+         */
+        if( !col->open ) {
+                col->open  = TRUE;
+		model_changed( MODEL( col ) );
+        }
+
         /* Extract position of tally row in RC widget.
          */
         rowview_get_position( rview, &x, &y, &w, &h );
@@ -429,24 +500,21 @@ rowview_scrollto( View *view, ModelScrollPosition position )
 static void
 rowview_drag( Rowview *rview_from, Rowview *rview_to )
 {
-	Row *row_from = ROW( VOBJECT( rview_from )->iobject );
-	Row *row_to = ROW( VOBJECT( rview_to )->iobject );
+	Row *row_from = ROW( VIEW( rview_from )->model );
+	Row *row_to = ROW( VIEW( rview_to )->model );
 
 	if( row_from->top_col != row_to->top_col ) {
-		error_top( _( "Not implemented." ) );
-		error_sub( _( "Drag between columns not yet implemented." ) );
-		iwindow_alert( GTK_WIDGET( rview_from ), GTK_MESSAGE_ERROR );
+		box_info( NULL, "drag between columns not yet implemented" );
 		return;
 	}
 
-	icontainer_child_move( ICONTAINER( row_from ), 
-		ICONTAINER( row_to )->pos );
+	model_child_move( MODEL( row_from ), MODEL( row_to )->pos );
 
 	/* Refresh all the rows, to make sure we move all rows to their new
 	 * slots.
 	 */
-	icontainer_map( ICONTAINER( row_from->scol ),
-		(icontainer_map_fn) iobject_changed, NULL, NULL );
+	model_map( MODEL( row_from->scol ),
+		(model_map_fn) model_changed, NULL, NULL );
 
         workspace_deselect_all( row_from->ws );
 }
@@ -507,20 +575,20 @@ rowview_link( View *view, Model *model, View *parent )
 	 */
 	if( row->top_row == row ) {
 		gtk_drag_source_set( rview->but, GDK_BUTTON1_MASK, 
-			rowview_target_table, IM_NUMBER( rowview_target_table ),
+			rowview_target_table, IM_NUMBER( rowview_target_table ), 
 			GDK_ACTION_COPY );
 		gtk_signal_connect( GTK_OBJECT( rview->but ), "drag_data_get",
 			GTK_SIGNAL_FUNC( rowview_drag_data_get ), rview );
 
 		gtk_drag_dest_set( rview->but, GTK_DEST_DEFAULT_ALL,
-			rowview_target_table, IM_NUMBER( rowview_target_table ),
+			rowview_target_table, IM_NUMBER( rowview_target_table ), 
 			GDK_ACTION_COPY );
 		gtk_signal_connect( GTK_OBJECT( rview->but ), 
 			"drag_data_received",
 			GTK_SIGNAL_FUNC( rowview_drag_data_received ), rview );
 	}
 
-	rowview_menu_attach( rview, rview->but );
+	rview->popup_sid = rowview_menu_attach( rview, rview->but );
 }
 
 static void
@@ -528,8 +596,8 @@ rowview_child_add( View *parent, View *child )
 {
 	Rowview *rowview = ROWVIEW( parent );
 
-	g_assert( IS_RHSVIEW( child ) );
-	g_assert( !rowview->rhsview );
+	assert( IS_RHSVIEW( child ) );
+	assert( !rowview->rhsview );
 
 	rowview->rhsview = RHSVIEW( child );
 
@@ -541,8 +609,8 @@ rowview_child_remove( View *parent, View *child )
 {
 	Rowview *rowview = ROWVIEW( parent );
 
-	g_assert( IS_RHSVIEW( child ) );
-	g_assert( rowview->rhsview );
+	assert( IS_RHSVIEW( child ) );
+	assert( rowview->rhsview == RHSVIEW( child ) );
 
 	rowview->rhsview = NULL;
 
@@ -550,87 +618,129 @@ rowview_child_remove( View *parent, View *child )
 }
 
 static void
-rowview_class_init( RowviewClass *class )
+rowview_class_init( RowviewClass *klass )
 {
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
-	vObjectClass *vobject_class = (vObjectClass *) class;
-	ViewClass *view_class = (ViewClass *) class;
-
 	GtkWidget *pane;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	ViewClass *view_class = (ViewClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_VIEW );
+
+	object_class->destroy = rowview_destroy;
 
 	/* Create signals.
 	 */
 
 	/* Init methods.
 	 */
-	object_class->destroy = rowview_destroy;
-
-	vobject_class->refresh = rowview_refresh;
-
 	view_class->link = rowview_link;
 	view_class->child_add = rowview_child_add;
 	view_class->child_remove = rowview_child_remove;
-	view_class->reset = rowview_reset;
+	view_class->refresh = rowview_refresh;
 	view_class->scrollto = rowview_scrollto;
 
         /* Other init.
          */
-	pane = rowview_popup_menu = popup_build( _( "Row menu" ) );
-	popup_add_but( pane, _( "_Edit" ), 
-		POPUP_FUNC( rowview_edit_cb ) );
-	popup_add_but( pane, _( "_Header" ), 
-		POPUP_FUNC( rowview_header_cb ) );
-	popup_add_but( pane, STOCK_DUPLICATE,
-		POPUP_FUNC( rowview_clone_cb ) );
-	popup_add_but( pane, _( "U_ngroup" ), 
-		POPUP_FUNC( rowview_ungroup_cb ) );
-	popup_add_but( pane, GTK_STOCK_SAVE_AS,
-		POPUP_FUNC( rowview_save_cb ) );
-	popup_add_but( pane, _( "Replace From _File" ), 
-		POPUP_FUNC( rowview_replace_cb ) );
-	popup_add_but( pane, _( "_Recalculate" ), 
-		POPUP_FUNC( rowview_recalc_cb ) );
-	popup_add_but( pane, _( "Re_set" ), 
-		POPUP_FUNC( rowview_clear_edited_cb ) );
+	pane = rowview_popup_menu = popup_build( "Row menu" );
+	popup_add_but( pane, "Edit ...", rowview_edit_cb );
+	popup_add_but( pane, "Clone", rowview_clone_cb );
+	popup_add_but( pane, "Ungroup", rowview_ungroup_cb );
+	popup_add_but( pane, "Save ...", rowview_save_cb );
+	popup_add_but( pane, "Replace from file ...", rowview_replace_cb );
+	popup_add_but( pane, "Recalculate", rowview_recalc_cb );
+	popup_add_but( pane, "Reset", rowview_clear_edited_cb );
 	menu_add_sep( pane );
-	popup_add_but( pane, GTK_STOCK_DELETE,
-		POPUP_FUNC( rowview_remove_cb ) );
+	popup_add_but( pane, "Remove ...", rowview_remove_cb );
 }
 
 static void
-rowview_enter_cb( GtkWidget *widget, Rowview *rview )
+rowview_flash_help( GtkWidget *widget, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
+	Expr *expr = row->expr;
 
-	row_set_status( row );
-	row_show_dependents( row );
+	char str[MAX_LINELENGTH];
+	BufInfo buf;
+
+	/* No symbol? eg. on load error.
+	 */
+	if( !expr )
+		return;
+
+	buf_init_static( &buf, str, MAX_LINELENGTH );
+
+	row_qualified_name( row, &buf );
+
+	if( expr->err ) {
+		buf_appends( &buf, ": " );
+		buf_appendline( &buf, expr->errstr );
+	}
+	else if( row->child_rhs->itext ) {
+		iText *itext = ITEXT( row->child_rhs->itext );
+
+		buf_appends( &buf, " = " );
+
+		if( !row->top_col->sform )
+			buf_appendline( &buf, itext->formula );
+		else 
+			buf_appendline( &buf, buf_all( &itext->value ) );
+	}
+
+	set_glabel( mainw_message, "%s", buf_all( &buf ) );
+}
+
+static void *
+rowview_show_parent( Link *link, RowShowState show )
+{
+	if( link->parent->expr && link->parent->expr->row ) 
+		row_set_show( link->parent->expr->row, show );
+
+	return( NULL );
+}
+
+static void *
+rowview_show_child( Link *link, RowShowState show )
+{
+	if( link->child->expr && link->child->expr->row ) 
+		row_set_show( link->child->expr->row, show );
+
+	return( NULL );
 }
 
 static void
-rowview_leave_cb( GtkWidget *widget, Rowview *rview )
+rowview_show_dependants( GtkWidget *widget, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
+	Symbol *topsym = row->top_row->sym;
 
-	row_hide_dependents( row );
-}
+#ifdef DEBUG
+	printf( "rowview_show_dependants: " );
+	row_name_print( ROW( VIEW( rview )->model ) );
+	printf( "\n" );
+#endif /*DEBUG*/
 
-static gboolean
-rowview_focus_cb( GtkWidget *widget, GtkDirectionType dir, Rowview *rview )
-{
-        view_scrollto( VIEW( rview ), MODEL_SCROLL_TOP );
-
-        return( FALSE );
+	slist_map( topsym->topparents,
+		(SListMapFn) rowview_show_parent, (void *) ROW_SHOW_PARENT );
+	slist_map( topsym->topchildren,
+		(SListMapFn) rowview_show_child, (void *) ROW_SHOW_CHILD );
 }
 
 static void
-rowview_tooltip_generate( GtkWidget *widget, VipsBuf *buf, Rowview *rview )
+rowview_hide_dependants( GtkWidget *widget, Rowview *rview )
 {
-	Row *row = ROW( VOBJECT( rview )->iobject );
+	Row *row = ROW( VIEW( rview )->model );
+	Symbol *topsym = row->top_row->sym;
 
-	iobject_info( IOBJECT( row ), buf );
-	vips_buf_removec( buf, '\n' );
+#ifdef DEBUG
+	printf( "rowview_hide_dependants: " );
+	row_name_print( ROW( VIEW( rview )->model ) );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	slist_map( topsym->topparents,
+		(SListMapFn) rowview_show_parent, (void *) ROW_SHOW_NONE );
+	slist_map( topsym->topchildren,
+		(SListMapFn) rowview_show_child, (void *) ROW_SHOW_NONE );
 }
 
 static void
@@ -638,15 +748,22 @@ rowview_init( Rowview *rview )
 {
         rview->visible = TRUE; 
 	rview->rnum = -1;
+        rview->popup_sid = 0; 
+        rview->set_name_tid = 0; 
+        rview->set_name = NULL; 
+        rview->to_set_name = NULL; 
         rview->last_tooltip = NULL; 
 
 	/* Make leds.
 	 */
-	rview->led = gtk_image_new_from_stock( STOCK_LED_OFF, 
-		GTK_ICON_SIZE_MENU );
+	rview->led = led_new();
+	led_set_type( LED( rview->led ), 1 );
 	gtk_misc_set_alignment( GTK_MISC( rview->led ), 0.5, 0.0 );
 	gtk_misc_set_padding( GTK_MISC( rview->led ), 2, 2 );
 
+	if( CALC_DISPLAY_LED ) 
+		gtk_widget_show( rview->led );
+
         /* Make fold/unfold button.
          */
 	rview->spin = spin_new();
@@ -655,28 +772,25 @@ rowview_init( Rowview *rview )
         gtk_signal_connect( GTK_OBJECT( rview->spin ), "down_click",
                 GTK_SIGNAL_FUNC( rowview_spin_down_cb ), rview );
         gtk_widget_show( rview->spin );
-	set_tooltip( rview->spin, _( "Click to open or close class" ) );
+	set_tooltip( rview->spin, "Click to open or close class" );
 
         /* Make name button.
          */
         rview->but = gtk_button_new();
         gtk_widget_show( rview->but );
         doubleclick_add( rview->but, FALSE,
-                DOUBLECLICK_FUNC( rowview_single_cb ), rview, 
-		DOUBLECLICK_FUNC( rowview_double_cb ), rview );
+                rowview_single_cb, rview, rowview_double_cb, rview );
         rview->label = gtk_label_new( "" );
         gtk_misc_set_alignment( GTK_MISC( rview->label ), 1, 0 );
         gtk_misc_set_padding( GTK_MISC( rview->label ), 2, 0 );
         gtk_container_add( GTK_CONTAINER( rview->but ), rview->label );
         gtk_widget_show( rview->label );
         gtk_signal_connect( GTK_OBJECT( rview->but ), "enter",
-                GTK_SIGNAL_FUNC( rowview_enter_cb ), rview );
+                GTK_SIGNAL_FUNC( rowview_flash_help ), rview );
+        gtk_signal_connect( GTK_OBJECT( rview->but ), "enter",
+                GTK_SIGNAL_FUNC( rowview_show_dependants ), rview );
         gtk_signal_connect( GTK_OBJECT( rview->but ), "leave",
-                GTK_SIGNAL_FUNC( rowview_leave_cb ), rview );
-        gtk_signal_connect( GTK_OBJECT( rview->but ), "focus",
-                GTK_SIGNAL_FUNC( rowview_focus_cb ), rview );
-	set_tooltip_generate( rview->but, 
-		(TooltipGenerateFn) rowview_tooltip_generate, rview, NULL );
+                GTK_SIGNAL_FUNC( rowview_hide_dependants ), rview );
 }
 
 GtkType
@@ -710,59 +824,54 @@ rowview_new( void )
 	return( VIEW( rview ) );
 }
 
-/* Find the position and size of a row in the enclosing GtkFixed.
+/* Find the position and size of a row in the enclosing GtkLayout.
  */
 void
 rowview_get_position( Rowview *rview, int *x, int *y, int *w, int *h )
 {
-        Columnview *cview = view_get_columnview( VIEW( rview ) );
+	Columnview *cview = view_get_columnview( VIEW( rview ) );
+	Workspaceview *wview = cview->wview;
 
-        if( GTK_WIDGET_VISIBLE( rview->spin ) ) {
-                *x = rview->spin->allocation.x;
-                *y = rview->spin->allocation.y;
-                *w = rview->spin->allocation.width;
-                *h = rview->spin->allocation.height;
-        }
-        else {
-                *x = rview->but->allocation.x;
-                *y = rview->but->allocation.y;
-                *w = 0;
-                *h = 0;
-        }
+	if( CALC_DISPLAY_LED ) {
+		*x = rview->led->allocation.x;
+		*y = rview->led->allocation.y;
+
+		*w = rview->led->allocation.width;
+		*h = rview->led->allocation.height;
+	}
+	else {
+		*x = rview->but->allocation.x;
+		*y = rview->but->allocation.y;
+
+		*w = 0;
+		*h = 0;
+	}
 
         *w += rview->but->allocation.width;
-        *h = IM_MAX( rview->but->allocation.height, *h );
-
-        if( GTK_WIDGET_VISIBLE( rview->led ) ) {
-                *w += rview->led->allocation.width;
-                *h = IM_MAX( rview->led->allocation.height, *h );
-        }
+        *h += rview->but->allocation.height;
 
         *w += GTK_WIDGET( rview->rhsview )->allocation.width;
-        *h = IM_MAX( GTK_WIDGET( rview->rhsview )->allocation.height, *h );
+        *h += GTK_WIDGET( rview->rhsview )->allocation.height;
 
-        /* Title bar, plus separator.
-         */
-        *y += cview->title->allocation.height + 2;
-
-        *x += cview->main->allocation.x;
-        *y += cview->main->allocation.y;
-
-#ifdef DEBUG
-        printf( "rowview_get_position: " );
-        row_name_print( ROW( VOBJECT( rview )->iobject ) );
-        printf( ": x = %d, y = %d, w = %d, h = %d\n", *x, *y, *w, *h );
-#endif /*DEBUG*/
+        *x += cview->main->allocation.x + wview->vp.left;
+        *y += cview->main->allocation.y + wview->vp.top;
 }
 
-/* Hide/show a row. 
+/* Hide/show a row. We can't use MODEL( row )->display for this, since tables
+ * don't like it :-( just _show()/_hide() rather then _create()/_destroy()
  */
 void 
 rowview_set_visible( Rowview *rview, gboolean visible )
 {
+	Row *row = ROW( VIEW( rview )->model );
+
 	if( rview->visible != visible ) {
+		widget_visible( rview->but, visible );
+		widget_visible( rview->spin, visible && row->isclass );
+		widget_visible( rview->led, visible && CALC_DISPLAY_LED );
+		if( rview->rhsview )
+			widget_visible( GTK_WIDGET( rview->rhsview ), visible );
 		rview->visible = visible;
-		rowview_update_widgets( rview );
 	}
 }
 
