@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -28,19 +28,15 @@
  */
 
 #define TYPE_SLIDER (slider_get_type())
-#define SLIDER( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_SLIDER, Slider ))
+#define SLIDER( obj ) (GTK_CHECK_CAST( (obj), TYPE_SLIDER, Slider ))
 #define SLIDER_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_SLIDER, SliderClass))
-#define IS_SLIDER( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_SLIDER ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_SLIDER, SliderClass ))
+#define IS_SLIDER( obj ) (GTK_CHECK_TYPE( (obj), TYPE_SLIDER ))
 #define IS_SLIDER_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_SLIDER ))
-#define SLIDER_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_SLIDER, SliderClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_SLIDER ))
 
 typedef struct _Slider {
-	Classmodel parent_object;
+	Classmodel model;
 
 	double from, to, value;
 } Slider;
@@ -52,4 +48,5 @@ typedef struct _SliderClass {
 	 */
 } SliderClass;
 
-GType slider_get_type( void );
+GtkType slider_get_type( void );
+Classmodel *slider_new( Rhs *rhs );
