@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -33,34 +33,40 @@
 
 #include "ip.h"
 
-static GraphicviewClass *parent_class = NULL;
+#define CALC_RECOMP_SLIDER (watch_bool_get( "CALC_RECOMP_SLIDER", FALSE ))
+
+static ViewClass *parent_class = NULL;
+
+static void
+sliderview_destroy( GtkObject *object )
+{
+	Sliderview *sliderview;
+
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_SLIDERVIEW( object ) );
+
+	sliderview = SLIDERVIEW( object );
+
+	/* My instance destroy stuff.
+	 */
+
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
+}
 
 static void 
-sliderview_refresh( vObject *vobject )
+sliderview_refresh( View *view )
 {
-	Sliderview *sliderview = SLIDERVIEW( vobject );
-	Slider *slider = SLIDER( VOBJECT( sliderview )->iobject );
+	Sliderview *sliderview = SLIDERVIEW( view );
+	Slider *slider = SLIDER( view->model );
 	Tslider *tslider = sliderview->tslider;
 
         const double range = slider->to - slider->from;
         const double lrange = log10( range );
-	const char *caption = IOBJECT( slider )->caption;
 
 #ifdef DEBUG
 	printf( "sliderview_refresh\n" );
 #endif /*DEBUG*/
 
-	/* Compatibility ... we used to not have a caption. Don't display
-	 * anything if there's o caption.
-	 */
-	if( caption ) {
-		if( strcmp( caption, "" ) != 0 )
-			set_glabel( sliderview->label, _( "%s:" ), 
-				caption );
-		else
-			set_glabel( sliderview->label, "%s", "" );
-	}
-
 	tslider->from = slider->from;
 	tslider->to = slider->to;
 	tslider->svalue = slider->value;
@@ -82,14 +88,14 @@ sliderview_refresh( vObject *vobject )
 
 	tslider_changed( tslider );
 
-	VOBJECT_CLASS( parent_class )->refresh( vobject );
+	VIEW_CLASS( parent_class )->refresh( view );
 }
 
 static void *
 sliderview_scan( View *view )
 {
 	Sliderview *sliderview = SLIDERVIEW( view );
-	Slider *slider = SLIDER( VOBJECT( sliderview )->iobject );
+	Slider *slider = SLIDER( VIEW( sliderview )->model );
 	Classmodel *classmodel = CLASSMODEL( slider );
 	Expr *expr = HEAPMODEL( classmodel )->row->expr;
 
@@ -109,34 +115,22 @@ sliderview_scan( View *view )
 }
 
 static void
-sliderview_link( View *view, Model *model, View *parent )
+sliderview_class_init( SliderviewClass *klass )
 {
-	Sliderview *sliderview = SLIDERVIEW( view );
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	ViewClass *view_class = (ViewClass *) klass;
 
-	VIEW_CLASS( parent_class )->link( view, model, parent );
+	parent_class = gtk_type_class( TYPE_VIEW );
 
-	if( GRAPHICVIEW( view )->sview )
-		gtk_size_group_add_widget( GRAPHICVIEW( view )->sview->group,   
-			sliderview->label );
-}
-
-static void
-sliderview_class_init( SliderviewClass *class )
-{
-	vObjectClass *vobject_class = (vObjectClass *) class;
-	ViewClass *view_class = (ViewClass *) class;
-
-	parent_class = g_type_class_peek_parent( class );
+	object_class->destroy = sliderview_destroy;
 
 	/* Create signals.
 	 */
 
 	/* Init methods.
 	 */
-	vobject_class->refresh = sliderview_refresh;
-
+	view_class->refresh = sliderview_refresh;
 	view_class->scan = sliderview_scan;
-	view_class->link = sliderview_link;
 }
 
 /* Drag on slider.
@@ -144,7 +138,7 @@ sliderview_class_init( SliderviewClass *class )
 static void
 sliderview_change_cb( Tslider *tslider, Sliderview *sliderview )
 {
-	Slider *slider = SLIDER( VOBJECT( sliderview )->iobject );
+	Slider *slider = SLIDER( VIEW( sliderview )->model );
 
 #ifdef DEBUG
 	printf( "sliderview_change_cb\n" );
@@ -161,21 +155,10 @@ sliderview_change_cb( Tslider *tslider, Sliderview *sliderview )
 static void
 sliderview_init( Sliderview *sliderview )
 {
-	GtkWidget *hbox;
-
-	hbox = gtk_hbox_new( FALSE, 12 );
-        gtk_box_pack_start( GTK_BOX( sliderview ), hbox, TRUE, FALSE, 0 );
-
-        sliderview->label = gtk_label_new( "" );
-        gtk_misc_set_alignment( GTK_MISC( sliderview->label ), 0, 0.5 );
-        gtk_misc_set_padding( GTK_MISC( sliderview->label ), 2, 1 );
-	gtk_box_pack_start( GTK_BOX( hbox ), sliderview->label, 
-		FALSE, FALSE, 0 );
-
 	sliderview->tslider = tslider_new();
 	tslider_set_conversions( sliderview->tslider, NULL, NULL );
-        gtk_box_pack_start( GTK_BOX( hbox ), 
-		GTK_WIDGET( sliderview->tslider ), TRUE, TRUE, 6 );
+        gtk_box_pack_start( GTK_BOX( sliderview ), 
+		GTK_WIDGET( sliderview->tslider ), FALSE, FALSE, 0 );
 
         gtk_signal_connect_object( GTK_OBJECT( sliderview->tslider ), 
 		"text_changed",
@@ -209,7 +192,7 @@ sliderview_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		sliderview_type = gtk_type_unique( TYPE_GRAPHICVIEW, &sinfo );
+		sliderview_type = gtk_type_unique( TYPE_VIEW, &sinfo );
 	}
 
 	return( sliderview_type );
