@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -46,15 +46,19 @@ typedef struct _StatusviewBand {
 struct _Statusview {
 	GtkFrame parent_class;
 
-	Imagemodel *imagemodel;
-	guint changed_sid;
+	Conversion *conv;
+	Iimage *iimage;
+
+	guint conv_changed_sid;
+	guint conv_destroy_sid;
+	guint iimage_changed_sid;
+	guint iimage_destroy_sid;
 
 	GtkWidget *top;		/* Top label */
 	GtkWidget *pos;		/* Position */
 	GtkWidget *hb;		/* Band element hbox */
 	GtkWidget *mag;		/* Magnification display */
 	GSList *bands;		/* List of StatusviewBand */
-	int nb;			/* Last number of bands we saw */
 	int fmt;		/* The last bandfmt we set ... for spacing */
 };
 
@@ -66,6 +70,6 @@ typedef struct _StatusviewClass {
 } StatusviewClass;
 
 GtkType statusview_get_type( void );
-Statusview *statusview_new( Imagemodel *imagemodel );
+Statusview *statusview_new( Conversion *conv, Iimage *iimage );
 
 void statusview_mouse( Statusview *sv, int x, int y );
