@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -46,16 +46,13 @@ subcolumn_row_pred_none( Row *row )
 static gboolean
 subcolumn_row_pred_members( Row *row )
 {
-	if( row->sym &&
-		is_system( row->sym ) )
+	if( is_system( row->sym ) )
 		return( FALSE );
 
-	if( row->sym &&
-		is_super( row->sym ) )
+	if( is_super( row->sym ) )
 		return( FALSE );
 
-	if( row->sym &&
-		row->sym->type == SYM_PARAM )
+	if( row->sym->type == SYM_PARAM )
 		return( FALSE );
 
 	return( TRUE );
@@ -64,8 +61,7 @@ subcolumn_row_pred_members( Row *row )
 static gboolean
 subcolumn_row_pred_params( Row *row )
 {
-	return( row->sym && 
-		row->sym->type == SYM_PARAM );
+	return( row->sym->type == SYM_PARAM );
 }
 
 /* Everything but empty superclasses.
@@ -73,9 +69,7 @@ subcolumn_row_pred_params( Row *row )
 static gboolean
 subcolumn_row_pred_super( Row *row )
 {
-	if( row->sym &&
-		is_super( row->sym ) && 
-		PEISELIST( &row->expr->root ) )
+	if( is_super( row->sym ) && PEISELIST( &row->expr->root ) )
 		return( FALSE );
 
 	return( TRUE );
@@ -98,36 +92,34 @@ const int subcolumn_nvisibility = IM_NUMBER( subcolumn_visibility );
 void *
 subcolumn_map( Subcolumn *scol, row_map_fn fn, void *a, void *b )
 {
-	return( icontainer_map( ICONTAINER( scol ), 
-		(icontainer_map_fn) fn, a, b ) );
+	return( model_map( MODEL( scol ), (model_map_fn) fn, a, b ) );
 }
 
 static void
-subcolumn_dispose( GObject *gobject )
+subcolumn_destroy( GtkObject *object )
 {
 	Subcolumn *scol;
 
 #ifdef DEBUG
-	printf( "subcolumn_dispose\n" );
+	printf( "subcolumn_destroy\n" );
 #endif /*DEBUG*/
 
-	g_return_if_fail( gobject != NULL );
-	g_return_if_fail( IS_SUBCOLUMN( gobject ) );
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_SUBCOLUMN( object ) );
 
-	scol = SUBCOLUMN( gobject );
+	scol = SUBCOLUMN( object );
 
 	scol->col = NULL;
 	scol->scol = NULL;
 	scol->top_col = NULL;
 
-	heap_unregister_element( reduce_context->heap, &scol->base );
+	heap_unregister_element( reduce_context->hi, &scol->base );
 	scol->base.type = ELEMENT_NOVAL;
-	scol->base.ele = (void *) 13;
 
 	scol->this = NULL;
 	scol->super = NULL;
 
-	G_OBJECT_CLASS( parent_class )->dispose( gobject );
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
 
 /* Stuff we track during class instance display update.
@@ -153,8 +145,7 @@ subcolumn_test_sym( Row *row, Symbol *sym )
 static void *
 subcolumn_test_row_name( Row *row, Symbol *sym )
 {
-	if( !row->sym && 
-		strcmp( IOBJECT( row )->name, IOBJECT( sym )->name ) == 0 )
+	if( !row->sym && strcmp( MODEL( row )->name, MODEL( sym )->name ) == 0 )
 		return( row );
 
 	return( NULL );
@@ -169,11 +160,12 @@ subcolumn_class_new_heap_sub( ClassRefreshInfo *cri,
 	Row *row;
 
 #ifdef DEBUG
+	BufInfo buf;
 	char txt[200];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
 
+	buf_init_static( &buf, txt, 200 );
 	symbol_qualified_name( sym, &buf );
-	printf( "subcolumn_class_new_heap_sub: %s\n", vips_buf_all( &buf ) );
+	printf( "subcolumn_class_new_heap_sub: %s\n", buf_all( &buf ) );
 #endif /*DEBUG*/
 
 	/* Do we have a row for this symbol?
@@ -182,8 +174,7 @@ subcolumn_class_new_heap_sub( ClassRefreshInfo *cri,
 		(SListMapFn) subcolumn_test_sym, sym )) ) {
 		/* Update it.
 		 */
-		if( heapmodel_new_heap( HEAPMODEL( row ), value ) ) 
-			expr_error_set( row->expr );
+		heapmodel_new_heap( HEAPMODEL( row ), value );
 
 		cri->notused = g_slist_remove( cri->notused, row );
 	}
@@ -193,15 +184,13 @@ subcolumn_class_new_heap_sub( ClassRefreshInfo *cri,
 		 * load. Update the row with the correct symbol.
 		 */
 		row_link_symbol( row, sym, NULL );
-		if( heapmodel_new_heap( HEAPMODEL( row ), value ) ) 
-			expr_error_set( row->expr );
+		heapmodel_new_heap( HEAPMODEL( row ), value );
 
 		cri->notused = g_slist_remove( cri->notused, row );
 	}
 	else {
 		row = row_new( cri->scol, sym, value );
-		if( heapmodel_new_heap( HEAPMODEL( row ), value ) ) 
-			expr_error_set( row->expr );
+		heapmodel_new_heap( HEAPMODEL( row ), value );
 	}
 }
 
@@ -232,7 +221,7 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 
 	/* Must be a class display.
 	 */
-	g_assert( !scol->is_top );
+	assert( !scol->istop );
 	row = HEAPMODEL( scol )->row;
 	expr = row->expr;
 
@@ -243,17 +232,13 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 #endif /*DEBUG*/
 
 	/* Can loop here for some recursive classes.
-
-		FIXME
-
+	 */
 	if( mainw_countdown_animate( 99 ) )
 		return( FALSE );
-	 */
 
 	/* No displays for system rows.
 	 */
-	if( row->sym &&
-		is_system( row->sym ) )
+	if( is_system( row->sym ) )
 		return( TRUE );
 
 	/* If we are the top of a class instance display, get a new serial.
@@ -264,30 +249,30 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 		redisplay? more reliable, but even slower :-(
 
 	 */
-	if( scol->scol->is_top )
-		heap_serial_new( reduce_context->heap );
+	if( scol->scol->istop )
+		heap_serial_new( reduce_context->hi );
 
 	/* Is it a class with a typecheck member? Go through
 	 * that. Do an isclass first to force eval.
 	 */
-	if( !heap_is_class( &instance, &result ) )
+	if( !heap_isclass( &instance, &result ) )
 		return( FALSE );
 	if( result &&
-		class_get_member( &instance, MEMBER_CHECK, NULL, &member ) ) {
+		class_get_member( &instance, MEMBER_CHECK, &member ) ) {
 #ifdef DEBUG
 		printf( "subcolumn_class_new_heap: invoking arg checker\n" );
 #endif 
 
 		/* Force eval of the typecheck member.
 		 */
-		if( !heap_is_class( &member, &result ) || !result )
+		if( !heap_isclass( &member, &result ) || !result )
 			return( FALSE );
 	}
 
 	/* Have we already displayed this class?
 	 */
 	if( (PEGETVAL( &instance )->flgs & FLAG_SERIAL) == 
-		reduce_context->heap->serial ) {
+		reduce_context->hi->serial ) {
 		/* 
 		
 			FIXME ... display something here? "circular"?
@@ -295,7 +280,7 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 		 */
 		return( TRUE );
 	}
-	SETSERIAL( PEGETVAL( &instance )->flgs, reduce_context->heap->serial );
+	SETSERIAL( PEGETVAL( &instance )->flgs, reduce_context->hi->serial );
 
 	/* Note the heap root ... if this is the top of a row tree, then we
 	 * clone the class and use that private copy.
@@ -310,12 +295,12 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 	 * end, remove all unused rows.
 	 */
 	cri.scol = scol;
-	cri.notused = g_slist_copy( ICONTAINER( scol )->children );
+	cri.notused = g_slist_copy( MODEL( scol )->children );
 
 #ifdef DEBUG
 	printf( "subcolumn_class_new_heap: existing rows: " );
-	icontainer_map( ICONTAINER( scol ), 
-		(icontainer_map_fn) subcolumn_class_dump_tiny_row, NULL, NULL );
+	model_map( MODEL( scol ), 
+		(model_map_fn) subcolumn_class_dump_tiny_row, NULL, NULL );
 	printf( "\n" );
 #endif /*DEBUG*/
 
@@ -341,12 +326,10 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 			 * no point (and the default constructor has no text
 			 * equivalent anyway).
 			 */
-			if( strcmp( IOBJECT( sym )->name, MEMBER_NAME ) == 0 )
+			if( strcmp( MODEL( sym )->name, MEMBER_NAME ) == 0 )
 				continue;
-			if( is_member( sym ) && 
-				strcmp( IOBJECT( sym )->name, 
-				IOBJECT( symbol_get_parent( sym ) )->name ) == 
-				0 )
+			if( is_member( sym ) && strcmp( MODEL( sym )->name, 
+				MODEL( symbol_get_parent( sym ) )->name ) == 0 )
 				continue;
 
 			/* Display!
@@ -356,8 +339,8 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 
 	/* Remove all the rows we've not used. 
 	 */
-	slist_map( cri.notused, (SListMapFn) iobject_destroy, NULL );
-	IM_FREEF( g_slist_free, cri.notused );
+	slist_map( cri.notused, (SListMapFn) object_destroy, NULL );
+	FREEF( g_slist_free, cri.notused );
 
 	return( TRUE );
 }
@@ -375,14 +358,36 @@ subcolumn_new_heap( Heapmodel *heapmodel, PElement *root )
 
 	/* A bunch of locals? Update them all.
 	 */
-	if( !scol->is_top && !subcolumn_class_new_heap( scol, root ) )
+	if( !scol->istop && !subcolumn_class_new_heap( scol, root ) )
 		return( scol );
 
 	return( HEAPMODEL_CLASS( parent_class )->new_heap( heapmodel, root ) );
 }
 
 static void
-subcolumn_child_add( iContainer *parent, iContainer *child, int pos )
+subcolumn_child_remove( Model *parent, Model *child )
+{
+	Subcolumn *scol = SUBCOLUMN( parent );
+	Row *row = ROW( child );
+
+	/* top_col can be NULL during destroy.
+	 */
+	if( scol->istop && scol->top_col ) {
+		Workspace *ws = scol->top_col->ws;
+
+		filemodel_set_modified( FILEMODEL( ws ), TRUE );
+	}
+
+	MODEL_CLASS( parent_class )->child_remove( parent, child );
+
+	if( scol->this == row )
+		scol->this = NULL;
+	if( scol->super == row )
+		scol->super = NULL;
+}
+
+static void
+subcolumn_child_add( Model *parent, Model *child, int pos )
 {
 	Subcolumn *scol = SUBCOLUMN( parent );
 	Row *row = ROW( child );
@@ -393,7 +398,7 @@ subcolumn_child_add( iContainer *parent, iContainer *child, int pos )
 	 * We don't do this often, so strcmp() it.
 	 */
 	const char *name = 
-		row->sym ? IOBJECT( row->sym )->name : IOBJECT( row )->name;
+		row->sym ? MODEL( row->sym )->name : MODEL( row )->name;
 
 	if( strcmp( name, MEMBER_THIS ) == 0 ) 
 		scol->this = row;
@@ -401,21 +406,7 @@ subcolumn_child_add( iContainer *parent, iContainer *child, int pos )
 	if( strcmp( name, MEMBER_SUPER ) == 0 ) 
 		scol->super = row;
 
-	ICONTAINER_CLASS( parent_class )->child_add( parent, child, pos );
-}
-
-static void
-subcolumn_child_remove( iContainer *parent, iContainer *child )
-{
-	Subcolumn *scol = SUBCOLUMN( parent );
-	Row *row = ROW( child );
-
-	ICONTAINER_CLASS( parent_class )->child_remove( parent, child );
-
-	if( scol->this == row )
-		scol->this = NULL;
-	if( scol->super == row )
-		scol->super = NULL;
+	MODEL_CLASS( parent_class )->child_add( parent, child, pos );
 }
 
 /* If this is a top-level subcolumn, get the enclosing column.
@@ -423,9 +414,9 @@ subcolumn_child_remove( iContainer *parent, iContainer *child )
 static Column *
 subcolumn_get_column( Subcolumn *scol )
 {
-	g_assert( scol->is_top );
+	assert( scol->istop );
 
-	return( COLUMN( ICONTAINER( scol )->parent ) );
+	return( COLUMN( MODEL( scol )->parent ) );
 }
 
 /* If this is a nested subcolumn, get the enclosing subcolumn.
@@ -437,7 +428,7 @@ subcolumn_get_subcolumn( Subcolumn *scol )
 	Row *row;
 	Subcolumn *escol;
 
-	g_assert( !scol->is_top );
+	assert( !scol->istop );
 
 	rhs = HEAPMODEL( scol )->rhs;
 	row = HEAPMODEL( rhs )->row;
@@ -451,14 +442,14 @@ subcolumn_get_subcolumn( Subcolumn *scol )
 static Column *
 subcolumn_get_top_column( Subcolumn *scol )
 {
-	if( !scol->is_top ) 
+	if( !scol->istop ) 
 		return( subcolumn_get_top_column( 
 			subcolumn_get_subcolumn( scol ) ) );
 
 	return( subcolumn_get_column( scol ) );
 }
 
-/* Return the enclosing subcolumn ... but not the is_top one. Ie. the enclosing
+/* Return the enclosing subcolumn ... but not the istop one. Ie. the enclosing
  * subcolumn which has the base for this class tree.
  */
 static Subcolumn *
@@ -466,42 +457,42 @@ subcolumn_get_top_subcolumn( Subcolumn *scol )
 {
 	Subcolumn *enclosing;
 
-	if( scol->is_top )
+	if( scol->istop )
 		return( NULL );
 
 	enclosing = subcolumn_get_subcolumn( scol );
-	if( enclosing->is_top )
+	if( enclosing->istop )
 		return( scol );
 	else 
 		return( subcolumn_get_top_subcolumn( enclosing ) );
 }
 
 static void
-subcolumn_parent_add( iContainer *child )
+subcolumn_parent_add( Model *child, Model *parent )
 {
 	Subcolumn *scol = SUBCOLUMN( child );
 
-	ICONTAINER_CLASS( parent_class )->parent_add( child );
+	MODEL_CLASS( parent_class )->parent_add( child, parent );
 
-	g_assert( IS_COLUMN( child->parent ) || IS_RHS( child->parent ) );
-	g_assert( !IS_COLUMN( child->parent ) || 
-		g_slist_length( child->parent->children ) == 1 );
+	assert( IS_COLUMN( parent ) || IS_RHS( parent ) );
+	assert( !IS_COLUMN( parent ) || 
+		g_slist_length( parent->children ) == 1 );
 
-	scol->is_top = IS_COLUMN( child->parent );
+	scol->istop = IS_COLUMN( parent );
 
 	/* For sub-columns, default to nothing visible.
 	 */
-	if( !scol->is_top )
+	if( !scol->istop )
 		scol->vislevel = 0;
 
 	/* Update context pointers.
 	 */
-	if( scol->is_top )
+	if( scol->istop )
 		scol->col = subcolumn_get_column( scol );
 	else
 		scol->col = NULL;
 
-	if( !scol->is_top )
+	if( !scol->istop )
 		scol->scol = subcolumn_get_subcolumn( scol );
 	else
 		scol->scol = NULL;
@@ -511,25 +502,7 @@ subcolumn_parent_add( iContainer *child )
 
 	/* Top level subcolumns default to display on, others to display off.
 	 */
-	MODEL( scol )->display = scol->is_top;
-}
-
-static View *
-subcolumn_view_new( Model *model, View *parent )
-{
-	return( subcolumnview_new() );
-}
-
-static void
-subcolumn_display( Model *model, gboolean display )
-{
-	/*
-	printf( "subcolumn_display: " ); 
-	row_name_print( HEAPMODEL( model )->row );
-	printf( " %d\n", display ); 
-	 */
-
-	MODEL_CLASS( parent_class )->display( model, display );
+	MODEL( scol )->display = scol->istop;
 }
 
 static gboolean
@@ -538,7 +511,11 @@ subcolumn_load( Model *model,
 {
 	Subcolumn *scol = SUBCOLUMN( model );
 
-	g_assert( IS_COLUMN( parent ) || IS_RHS( parent ) );
+	if( !IS_COLUMN( parent ) && !IS_RHS( parent ) ) {
+		ierrors( "subcolumn_load: can only add a subcolumn to a "
+			"column or to a rhs" );
+		return( FALSE );
+	}
 
 	if( !get_iprop( xnode, "vislevel", &scol->vislevel ) )
 		return( FALSE );
@@ -559,35 +536,32 @@ subcolumn_save( Model *model, xmlNode *xnode )
 	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
 		return( NULL );
 
-	if( !set_iprop( xthis, "vislevel", scol->vislevel ) )
+	if( !set_prop( xthis, "vislevel", "%d", scol->vislevel ) )
 		return( NULL );
 
 	return( xthis );
 }
 
 static void
-subcolumn_class_init( SubcolumnClass *class )
+subcolumn_class_init( SubcolumnClass *klass )
 {
-	GObjectClass *gobject_class = (GObjectClass *) class;
-	iContainerClass *icontainer_class = (iContainerClass *) class;
-	ModelClass *model_class = (ModelClass *) class;
-	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	ModelClass *model_class = (ModelClass *) klass;
+	HeapmodelClass *heapmodel_class = (HeapmodelClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_HEAPMODEL );
+
+	object_class->destroy = subcolumn_destroy;
 
 	/* Create signals.
 	 */
 
 	/* Init methods.
 	 */
-	gobject_class->dispose = subcolumn_dispose;
-
-	icontainer_class->child_add = subcolumn_child_add;
-	icontainer_class->child_remove = subcolumn_child_remove;
-	icontainer_class->parent_add = subcolumn_parent_add;
-
-	model_class->view_new = subcolumn_view_new;
-	model_class->display = subcolumn_display;
+	model_class->view_new = subcolumnview_new;
+	model_class->child_remove = subcolumn_child_remove;
+	model_class->child_add = subcolumn_child_add;
+	model_class->parent_add = subcolumn_parent_add;
 	model_class->load = subcolumn_load;
 	model_class->save = subcolumn_save;
 
@@ -595,7 +569,7 @@ subcolumn_class_init( SubcolumnClass *class )
 
 	/* Static init.
 	 */
-	model_register_loadable( MODEL_CLASS( class ) );
+	model_register_loadable( MODEL_CLASS( klass ) );
 }
 
 static void
@@ -612,34 +586,31 @@ subcolumn_init( Subcolumn *scol )
         scol->vislevel = subcolumn_nvisibility - 1;
 
 	scol->base.type = ELEMENT_NOVAL;
-	scol->base.ele = (void *) 14;
-	heap_register_element( reduce_context->heap, &scol->base );
+	heap_register_element( reduce_context->hi, &scol->base );
 	scol->known_private = FALSE;
 
 	scol->this = NULL;
 	scol->super = NULL;
 }
 
-GType
+GtkType
 subcolumn_get_type( void )
 {
-	static GType subcolumn_type = 0;
+	static GtkType subcolumn_type = 0;
 
 	if( !subcolumn_type ) {
-		static const GTypeInfo info = {
-			sizeof( SubcolumnClass ),
-			NULL,           /* base_init */
-			NULL,           /* base_finalize */
-			(GClassInitFunc) subcolumn_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
+		static const GtkTypeInfo info = {
+			"Subcolumn",
 			sizeof( Subcolumn ),
-			32,             /* n_preallocs */
-			(GInstanceInitFunc) subcolumn_init,
+			sizeof( SubcolumnClass ),
+			(GtkClassInitFunc) subcolumn_class_init,
+			(GtkObjectInitFunc) subcolumn_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
 		};
 
-		subcolumn_type = g_type_register_static( TYPE_HEAPMODEL, 
-			"Subcolumn", &info, 0 );
+		subcolumn_type = gtk_type_unique( TYPE_HEAPMODEL, &info );
 	}
 
 	return( subcolumn_type );
@@ -648,24 +619,21 @@ subcolumn_get_type( void )
 static void
 subcolumn_link( Subcolumn *scol, Rhs *rhs, Column *col )
 {
-	g_assert( rhs == NULL || col == NULL );
+	assert( rhs == NULL || col == NULL );
 
-	/* parent_add() sets is_top for us.
+	/* parent_add() sets istop for us.
 	 */
 	if( rhs ) 
-		icontainer_child_add( ICONTAINER( rhs ), 
-			ICONTAINER( scol ), -1 );
+		model_child_add( MODEL( rhs ), MODEL( scol ), -1 );
 	else 
-		icontainer_child_add( ICONTAINER( col ), 
-			ICONTAINER( scol ), -1 );
+		model_child_add( MODEL( col ), MODEL( scol ), -1 );
 }
 
 Subcolumn *
 subcolumn_new( Rhs *rhs, Column *col )
 {
-	Subcolumn *scol;
+	Subcolumn *scol = gtk_type_new( TYPE_SUBCOLUMN );
 
-	scol = SUBCOLUMN( g_object_new( TYPE_SUBCOLUMN, NULL ) );
 	subcolumn_link( scol, rhs, col );
 
 	return( scol );
@@ -680,7 +648,7 @@ subcolumn_set_vislevel( Subcolumn *scol, int vislevel )
 	printf( "subcolumn_set_vislevel: %d\n", scol->vislevel );
 #endif /*DEBUG*/
 
-	iobject_changed( IOBJECT( scol ) );
+	model_changed( MODEL( scol ) );
 }
 
 /* Make sure we have a private copy of the graph for this tree of stuff.
@@ -707,7 +675,7 @@ subcolumn_make_private( Subcolumn *scol )
 	/* Clone from the class args and rebuild our tree.
 	 */
 	PEPOINTE( &base, &top_scol->base );
-	if( !class_clone_args( reduce_context->heap, &base, &base ) ||
+	if( !class_clone_args( reduce_context->hi, &base, &base ) ||
 		heapmodel_new_heap( HEAPMODEL( top_scol ), &base ) )
 		return( FALSE );
 
