@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -28,16 +28,12 @@
  */
 
 #define TYPE_SYMBOL (symbol_get_type())
-#define SYMBOL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_SYMBOL, Symbol ))
+#define SYMBOL( obj ) (GTK_CHECK_CAST( (obj), TYPE_SYMBOL, Symbol ))
 #define SYMBOL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_SYMBOL, SymbolClass))
-#define IS_SYMBOL( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_SYMBOL ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_SYMBOL, SymbolClass ))
+#define IS_SYMBOL( obj ) (GTK_CHECK_TYPE( (obj), TYPE_SYMBOL ))
 #define IS_SYMBOL_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_SYMBOL ))
-#define SYMBOL_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_SYMBOL, SymbolClass ))
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_SYMBOL ))
 
 /* The types of symbol we can have.
  */
@@ -46,56 +42,53 @@ typedef enum {
 	SYM_PARAM,		/* A parameter to a user function */
 	SYM_ZOMBIE,		/* A referred to but not defined */
 	SYM_WORKSPACE,		/* A loaded workspace */
-	SYM_WORKSPACEROOT,	/* Base of all workspaces */
+	SYM_WORKSPACEGROUP,	/* A collection of workspaces */
 	SYM_ROOT,		/* The root symbol */
 	SYM_EXTERNAL,		/* A reference to an external function */
 	SYM_BUILTIN		/* A reference to a built-in function */
-} SymbolType;
+} SymType;
 
 /* A symbol.
  */
 struct _Symbol {
 	Filemodel parent_class;
 
-	/* The type of this symbol.
+	/* Our name, and where we are.
 	 */
-	SymbolType type;
+	SymTable *tab;		/* Symbol table we are within */
+	SymType type; 		/* The type of this symbol */
+
+	int dest;		/* >0 if being deleted */
 
 	/* Track during parse. A list of pointers to pointers to this
 	 * symbol which we need to patch if we resolve to an outer scope.
 	 */
 	GSList *patch;
 
-	/* Main expression for this sym. All expressions are icontainer
-	 * children of us.
+	/* Main expression for this sym.
 	 */
 	Expr *expr;
 
+	/* Other expressions at this scope.
+	 */
+	GSList *ext_expr;
+
 	/* Base of graph for value of this symbol. Use sym->expr->root to get
 	 * value though .. we just hold pointer for GC here. Expressions on
-	 * ext_expr have their GC handled by their enclosing Subcolumn.
+	 * ext_expr have their GC handle in Subcolumn.
 	 */
 	Element base;		/* Value for this expr */
 
 	/* Recomputation links. Use these to work out what to build next.
 	 */
 	gboolean dirty;		/* True if this sym needs recalc */
-	GSList *parents;	/* Compiles which refer to this sym */
+	GSList *parents;	/* Compile which refer to this sym */
 
 	GSList *topchildren;	/* For top syms, all top-level children */
 	GSList *topparents;	/* For top syms, all top-level parents */
 	int ndirtychildren;	/* Number of dirty top syms we refer to */
 	gboolean leaf;		/* True for in recomp set */
 
-	/* This is a generated symbol, like $$result, $$fn1, whatever.
-	 */
-	gboolean generated;
-
-	/* A temporary intermediate symbol generated during parse to hold 
-	 * stuff until we need it. Don't generate code for these.
-	 */
-	gboolean placeholder;
-
 	/* X-tras for definitions.
 	 */
 	Tool *tool;		/* Tool and toolkit defined in */
@@ -108,73 +101,52 @@ struct _Symbol {
 	/* X-tras for SYM_BUILTIN ... our function.
 	 */
 	BuiltinInfo *builtin;
-
-	/* For WORKSPACEROOT ... the wsr we represent.
-	 */
-	Workspaceroot *wsr;
-
-	/* For WORKSPACE ... the ws we represent.
-	 */
-	Workspace *ws;
 };
 
 typedef struct _SymbolClass {
 	FilemodelClass parent_class;
 
-	/* 
-
-		new_value	sym->expr has a new value (this signal is
-				fwd'd from sym->expr)
-
+	/* Methods.
 	 */
-
-	void (*new_value)( Symbol *sym );
 } SymbolClass;
 
-GType symbol_get_type( void );
+GtkType symbol_get_type( void );
 
 /* All symbols come off this.
  */
 extern Symbol *symbol_root;
+extern Symbol *symbol_root_root;
 
 Symbol *symbol_map_all( Symbol *sym, symbol_map_fn fn, void *a, void *b );
 
 Symbol *symbol_get_parent( Symbol *sym );
 Workspace *symbol_get_workspace( Symbol *sym );
 Tool *symbol_get_tool( Symbol *sym );
-Symbol *symbol_get_scope( Symbol *sym );
 
-void symbol_qualified_name( Symbol *sym, VipsBuf *buf );
+void symbol_qualified_name( Symbol *sym, BufInfo *buf );
 void symbol_qualified_name_relative( Symbol *context, 
-	Symbol *sym, VipsBuf *buf );
-void *symbol_name_error( Symbol *sym, VipsBuf *buf );
+	Symbol *sym, BufInfo *buf );
 const char *symbol_name( Symbol *sym );
-void *symbol_name_print( Symbol *sym );
-const char *symbol_name_scope( Symbol *sym );
-void symbol_name_scope_print( Symbol *sym );
-
-void symbol_new_value( Symbol *sym );
+void symbol_name_print( Symbol *sym );
 
 void *symbol_patch_add( void **pnt, Symbol *sym );
+void symbol_patch_pointers( Symbol *nsym, Symbol *osym );
 
-Symbol *symbol_root_init( void );
+void symbol_root_init();
 
-Symbol *symbol_new( Compile *compile, const char *name );
-gboolean symbol_rename( Symbol *sym, const char *new_name );
-void symbol_error_redefine( Symbol *sym );
-Symbol *symbol_new_defining( Compile *compile, const char *name );
+void symbol_link( Symbol *sym, SymTable *tab, const char *name );
+Symbol *symbol_new( SymTable *tab, const char *name );
+Symbol *symbol_new_defining( Compile *Compile, const char *name );
 Symbol *symbol_new_reference( Compile *compile, const char *name );
 void symbol_made( Symbol *sym );
 
-void symbol_not_defined( Symbol *sym );
-
 void *symbol_link_break( Symbol *child, Compile *compile );
 
 gboolean symbol_user_init( Symbol *sym );
-gboolean symbol_parameter_init( Symbol *sym );
+gboolean symbol_parameter_init( Symbol *sym, Compile *compile );
 gboolean symbol_parameter_builtin_init( Symbol *sym );
 
-gboolean symbol_busy( void );
+Symbol *symbol_leaf_next( void );
 
 void *symbol_sanity( Symbol *sym );
 void symbol_leaf_set_sanity( void );
@@ -182,7 +154,9 @@ void *symbol_strip( Symbol *sym );
 
 void symbol_state_change( Symbol *sym );
 
-const char *symbol_get_last_calc( void );
-gboolean symbol_recalculate_check( Symbol *sym );
-void symbol_recalculate_all_force( gboolean now );
+void symbol_recalculate_all_force( void );
 void symbol_recalculate_all( void );
+void *symbol_recalculate( Symbol *sym );
+void *symbol_recalculate_check( Symbol *sym );
+
+void symbol_help( Symbol *sym, BufInfo *buf );
