@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -36,7 +36,7 @@
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_TOGGLEVIEW ))
 
 typedef struct _Toggleview {
-	Graphicview parent_object;
+	View view;
 
 	/* My instance vars.
 	 */
@@ -44,7 +44,7 @@ typedef struct _Toggleview {
 } Toggleview;
 
 typedef struct _ToggleviewClass {
-	GraphicviewClass parent_class;
+	ViewClass parent_class;
 
 	/* My methods.
 	 */
