@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -29,28 +29,21 @@
 
 #define TYPE_TOOLKITGROUP (toolkitgroup_get_type())
 #define TOOLKITGROUP( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_TOOLKITGROUP, Toolkitgroup ))
+	(GTK_CHECK_CAST( (obj), TYPE_TOOLKITGROUP, Toolkitgroup ))
 #define TOOLKITGROUP_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), TYPE_TOOLKITGROUP, \
-		ToolkitgroupClass))
-#define IS_TOOLKITGROUP( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), TYPE_TOOLKITGROUP ))
-#define IS_TOOLKITGROUP_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), TYPE_TOOLKITGROUP ))
-#define TOOLKITGROUP_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_TOOLKITGROUP, \
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_TOOLKITGROUP, \
 		ToolkitgroupClass ))
+#define IS_TOOLKITGROUP( obj ) (GTK_CHECK_TYPE( (obj), TYPE_TOOLKITGROUP ))
+#define IS_TOOLKITGROUP_CLASS( klass ) \
+	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_TOOLKITGROUP ))
 
 /* A toolkitgroup.
  */
 struct _Toolkitgroup {
 	Model parent_class;
 
-	/* Defs in toolkits in this group are created as locals of this
-	 * symbol. This is symbol_root for the main toolkitgroup, but can be
-	 * local to a workspace if we are loading a set of compatibility defs.
+	/* State.
 	 */
-	Symbol *root;
 };
 
 typedef struct _ToolkitgroupClass {
@@ -60,11 +53,8 @@ typedef struct _ToolkitgroupClass {
 	 */
 } ToolkitgroupClass;
 
-Toolkit *toolkitgroup_map( Toolkitgroup *kitg, 
-	toolkit_map_fn fn, void *a, void *b );
+GtkType toolkitgroup_get_type( void );
 
-GType toolkitgroup_get_type( void );
-
-Toolkitgroup *toolkitgroup_new( Symbol *root );
+Toolkitgroup *toolkitgroup_new( void );
 
 void toolkitgroup_sort( Toolkitgroup *kitg );
