@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -35,32 +35,28 @@
 
 static ViewClass *parent_class = NULL;
 
-static void *
-toolkitgroupview_dispose_sub( View *view, void *a, void *b )
-{
-	DESTROY_GTK( view );
-
-	return( NULL );
-}
-
 static void
-toolkitgroupview_dispose( GObject *gobject )
+toolkitgroupview_destroy( GtkObject *object )
 {
+	Toolkitgroupview *kitgview;
+
 #ifdef DEBUG
-	printf( "toolkitgroupview_dispose: %p\n", gobject );
+	printf( "toolkitgroupview_destroy\n" );
 #endif /*DEBUG*/
 
-	/* Toolkitviews are not child widgets of us, they are menu items pased
-	 * into the TK. Destroy them explicitly.
-	 */
-	view_map( VIEW( gobject ), 
-		toolkitgroupview_dispose_sub, NULL, NULL ); 
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_TOOLKITGROUPVIEW( object ) );
 
-	G_OBJECT_CLASS( parent_class )->dispose( gobject );
+	kitgview = TOOLKITGROUPVIEW( object );
+
+	/* Instance destroy.
+	 */
+
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
 
 static void 
-toolkitgroupview_refresh( vObject *vobject )
+toolkitgroupview_refresh( View *view )
 {
 	/* 
 	Toolkitgroupview *kitgview = TOOLKITGROUPVIEW( view );
@@ -74,30 +70,55 @@ toolkitgroupview_refresh( vObject *vobject )
 	printf( "toolkitgroup changed\n" );
 #endif /*DEBUG*/
 
-	VOBJECT_CLASS( parent_class )->refresh( vobject );
+	VIEW_CLASS( parent_class )->refresh( view );
 }
 
 static void
-toolkitgroupview_class_init( ToolkitgroupviewClass *class )
+toolkitgroupview_class_init( ToolkitgroupviewClass *klass )
 {
-	GObjectClass *gobject_class = (GObjectClass *) class;
-	vObjectClass *vobject_class = (vObjectClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	ViewClass *view_class = (ViewClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( TYPE_VIEW );
 
-	gobject_class->dispose = toolkitgroupview_dispose;
+	object_class->destroy = toolkitgroupview_destroy;
 
 	/* Create signals.
 	 */
 
 	/* Set methods.
 	 */
-	vobject_class->refresh = toolkitgroupview_refresh;
+	view_class->refresh = toolkitgroupview_refresh;
 }
 
 static void
 toolkitgroupview_init( Toolkitgroupview *kitgview )
 {
+	GtkWidget *swin;
+	GtkWidget *viewport;
+	GtkAdjustment *hadj, *vadj;
+
+	swin = gtk_scrolled_window_new( NULL, NULL );
+	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( swin ),
+		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC );
+        gtk_box_pack_start( GTK_BOX( kitgview ), swin, TRUE, TRUE, 0 );
+	gtk_widget_show( swin );
+
+	hadj = gtk_scrolled_window_get_hadjustment( 
+		GTK_SCROLLED_WINDOW( swin ) );
+	vadj = gtk_scrolled_window_get_vadjustment( 
+		GTK_SCROLLED_WINDOW( swin ) );
+	viewport = gtk_viewport_new( hadj, vadj );
+	gtk_viewport_set_shadow_type( GTK_VIEWPORT( viewport ),
+		GTK_SHADOW_NONE );
+	gtk_container_add( GTK_CONTAINER( swin ), viewport );
+        gtk_widget_show( viewport );
+
+        kitgview->menubar = gtk_menu_bar2_new();
+	gtk_menu_bar2_set_shadow_type( GTK_MENU_BAR2( kitgview->menubar ),
+		GTK_SHADOW_NONE );
+	gtk_container_add( GTK_CONTAINER( viewport ), kitgview->menubar );
+        gtk_widget_show( kitgview->menubar );
 }
 
 GtkType
@@ -131,9 +152,3 @@ toolkitgroupview_new( void )
 	return( VIEW( kitgview ) );
 }
 
-void
-toolkitgroupview_set_mainw( Toolkitgroupview *kitgview, Mainw *mainw )
-{
-	kitgview->mainw = mainw;
-        kitgview->menu = mainw->toolkit_menu;
-}
