@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -41,8 +41,7 @@
 struct _Toolkitgroupview {
 	View parent_class;
 
-	GtkWidget *menu;		/* Display the toolkits in this */
-	Mainw *mainw;			/* Mainw these menu items act on */
+	GtkWidget *menubar;		/* Display the toolkits in this */
 };
 
 typedef struct _ToolkitgroupviewClass {
@@ -54,5 +53,4 @@ typedef struct _ToolkitgroupviewClass {
 
 GtkType toolkitgroupview_get_type( void );
 View *toolkitgroupview_new( void );
-void toolkitgroupview_set_mainw( Toolkitgroupview *kitgview, Mainw *mainw );
 
