@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -41,9 +41,8 @@ struct _Toolkitview {
 
 	Toolkitgroupview *kitgview;
 
-	GtkWidget *menu;	/* Menu for this kit */
-	GtkWidget *item;	/* Menu item in enclosing menu */
-	guint destroy_sid;
+	GtkWidget *pane;	/* MenuPane for this kit */
+	GtkWidget *cascade;	/* Cascade for this kit */
 };
 
 typedef struct _ToolkitviewClass {
