@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -35,8 +35,6 @@
 #define IS_TOOLVIEW_CLASS( klass ) \
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_TOOLVIEW ))
 
-/* One of these for each top-level menu.
- */
 struct _Toolview {
 	View parent_class;
 
