@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -40,18 +40,20 @@
 typedef enum {
 	TRACE_BUILTIN = 1,		/* Calls to built in functions */
 	TRACE_OPERATOR = 2,		/* +, -, etc. */
-	TRACE_CLASS_NEW = 4,		/* Class construction */
-	TRACE_VIPS = 8			/* VIPS operations */
+	TRACE_CLASS_NEW = 4		/* Class construction */
 } TraceFlags;
 
 struct _Trace {
-	Log parent_class;
+	iWindow parent_class;
 
 	TraceFlags flags;
+
+	GtkWidget *text;
+	GtkItemFactory *ifac;
 };
 
 typedef struct _TraceClass {
-	LogClass parent_class;
+	iWindowClass parent_class;
 
 	/* My methods.
 	 */
@@ -64,23 +66,21 @@ void trace_unblock( void );
 
 void trace_reset( void );
 void trace_check( void );
-VipsBuf *trace_push( void );
+BufInfo *trace_push( void );
 void trace_pop( void );
-VipsBuf *trace_current( void );
+BufInfo *trace_current( void );
 void trace_pop_to( int n );
 int trace_get_mark( void );
 
 GtkType trace_get_type( void );
 Trace *trace_new( void );
 
-void trace_text( TraceFlags flags, const char *fmt, ... )
-	__attribute__((format(printf, 2, 3)));
+void trace_text( TraceFlags flags, const char *fmt, ... );
 
 void trace_pelement( PElement *pe );
 void trace_node( HeapNode *node );
 void trace_args( HeapNode **arg, int n );
-void trace_binop( Compile *compile, 
-	PElement *left, BinOp bop, PElement *right );
+void trace_binop( PElement *left, BinOp bop, PElement *right );
 void trace_uop( UnOp uop, PElement *arg );
 void trace_result( TraceFlags flags, PElement *out );
 
