@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,11 +27,11 @@
 
  */
 
-/* The kinds of nodes we can have in a parse tree. 
+/* The kinds of nodes we can have in a parse tree.
  */
 typedef enum {
 	NODE_NONE,		/* Empty */
-	NODE_APPLY,		/* A function application */
+	NODE_APPLY,		/* A function aplication */
 	NODE_BINOP,		/* A binary operator */
 	NODE_UOP,		/* A unary operator */
 	NODE_LEAF,		/* A leaf .. a symbol of some sort */
@@ -41,7 +41,6 @@ typedef enum {
 	NODE_GENERATOR,		/* A list generator */
 	NODE_COMPOSE,		/* Function composition */
 	NODE_SUPER,		/* Superclass constructor */
-	NODE_PATTERN_CLASS,	/* A class pattern match */
 	NODE_LISTCONST		/* A list expression "[1, 2, 3]" */
 } NodeTypes;
 
@@ -61,7 +60,7 @@ typedef enum {
 	BI_LSHIFT,		/* Shift left */
 	BI_RSHIFT,		/* Shift right */
 	BI_DIV,			/* Divide by a constant */
-	BI_JOIN,		/* Join of two objects */
+	BI_JOIN,		/* Join of two images */
 	BI_DOT,			/* Projection operator */
 	BI_COMMA,		/* Form complex number */
 	BI_MUL,			/* Mult by a constant */
@@ -76,10 +75,8 @@ typedef enum {
 	BI_PNOTEQ,
 	BI_LESS,		/* Relational ops */
 	BI_LESSEQ,
-	BI_MORE,		
-	BI_MOREEQ,
 	BI_IF,			/* if-then-else */
-	BI_CONS			/* List cons ... has to be last, see below */
+	BI_CONS			/* List cons */
 } BinOp;
 
 /* Unary operators.
@@ -101,20 +98,18 @@ typedef enum {
 	UN_CCOMPLEX,		/* Convert to complex */
 	UN_CDCOMPLEX,		/* Convert to double complex */
 	UN_MINUS,		/* Unary minus */
-	UN_NEG,			/* Logical negation, "!" */
-	UN_COMPLEMENT,		/* 1s complement, "~" */
-	UN_PLUS,		/* Unary plus */
-	UN_LAST			/* Sanity check with this */
+	UN_NEG,			/* Logical negation */
+	UN_COMPLEMENT,		/* 1s complement */
+	UN_PLUS			/* Unary plus */
 } UnOp;
 
-/* The sorts of constants we can have in expressions. 
+/* The sorts of constants we can have in expressions.
  */
 typedef enum {
-	PARSE_CONST_NONE,
+	PARSE_CONST_NONE = 0,
 	PARSE_CONST_STR,
 	PARSE_CONST_BOOL,
 	PARSE_CONST_NUM,
-	PARSE_CONST_COMPLEX,		/* Eg. 12j == (0, 12) */
 	PARSE_CONST_CHAR,
 	PARSE_CONST_ELIST		/* Empty list [] */
 } ParseConstTypes;
@@ -126,7 +121,7 @@ struct _ParseConst {
 	union {
 		double num;
 		char *str;
-		gboolean bol;
+		gboolean bool;
 		int ch;
 	} val;
 };
@@ -140,15 +135,13 @@ struct _ParseNode {
 
 	NodeTypes type;
 
-	/* Bundle for node types with up to two arguments.
+	/* Bundle for node types with up to three arguments.
+	 * Unary ops, binary ops, application and ifthenelse ops.
 	 */
 	BinOp biop;
 	UnOp uop;
 	ParseNode *arg1;
 	ParseNode *arg2;
-
-	/* Just for generators, eg. [a, b .. c]
-	 */
 	ParseNode *arg3;
 
 	/* A symbol reference.
@@ -192,16 +185,5 @@ ParseNode *tree_leafsym_new( Compile *compile, Symbol *sym );
 ParseNode *tree_const_new( Compile *compile, ParseConst n );
 ParseNode *tree_class_new( Compile *compile );
 ParseNode *tree_compose_new( Compile *compile, ParseNode *f, ParseNode *g );
-ParseNode *tree_pattern_new( Compile *compile );
-ParseNode *tree_pattern_class_new( Compile *compile, 
-	const char *class_name, ParseNode *l );
 
 void *tree_node_destroy( ParseNode *n );
-
-typedef ParseNode *(*tree_map_fn)( Compile *, ParseNode *, void *, void * );
-ParseNode *tree_map( Compile *compile, 
-	tree_map_fn fn, ParseNode *node, void *a, void *b );
-
-/* Copy a tree into a new context.
- */
-ParseNode *tree_copy( Compile *compile, ParseNode *node );
