@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -47,6 +47,52 @@ static GtkHBoxClass *parent_class = NULL;
 
 static guint tslider_signals[LAST_SIGNAL] = { 0 };
 
+static gint tslider_idle_running = 0;
+
+static GSList *tslider_to_unref = NULL;
+
+static void *
+tslider_idle_unref( GtkRange *range )
+{
+	/* 
+	
+		FIXME ... workaround for bug in gtk+-1.2.x :-( 
+
+	 */
+	FREEFI( gtk_timeout_remove, range->timer );
+
+	gtk_object_unref( GTK_OBJECT( range ) );
+
+	return( NULL );
+}
+
+/* Refresh everything on the dirty list.
+ */
+static gint
+tslider_idle_unref_cb( gpointer user_data )
+{
+	slist_map( tslider_to_unref, (SListMapFn) tslider_idle_unref, NULL );
+	FREEF( g_slist_free, tslider_to_unref );
+
+	tslider_idle_running = 0;
+
+	return( FALSE );
+}
+
+/* Add something to the pending-unref list. Delay unref until the idle loop.
+ */
+static void 
+tslider_pending_unref( GtkRange *range )
+{
+	gtk_object_ref( GTK_OBJECT( range ) );
+
+	tslider_to_unref = g_slist_prepend( tslider_to_unref, range );
+
+	if( !tslider_idle_running )
+		tslider_idle_running = 
+			gtk_idle_add( tslider_idle_unref_cb, NULL );
+}
+
 /* Are two doubles more or less equal. We need this when we check the sliders
  * for update to stop loops. The 0.0001 is a bit of a fudge :-(
  */
@@ -63,7 +109,7 @@ tslider_destroy( GtkObject *object )
 	tslider = TSLIDER( object );
 
 #ifdef DEBUG
-	printf( "tslider_destroy: %p\n", tslider );
+	printf( "tslider_destroy: 0x%x\n", (unsigned int) tslider );
 #endif /*DEBUG*/
 
 	/* My instance destroy stuff.
@@ -74,6 +120,11 @@ tslider_destroy( GtkObject *object )
 		tslider->adj = NULL;
 	}
 
+	/* Yuk!!! We can sometimes destroy a tslider from the adjustment
+	 * value_changed callback, causing extreme confusion. 
+	 */
+	tslider_pending_unref( GTK_RANGE( tslider->slider ) );
+
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
 
@@ -136,8 +187,8 @@ tslider_real_changed( Tslider *tslider )
 	GtkWidget *entry = tslider->entry;
 
 #ifdef DEBUG
-	printf( "tslider_real_changed: %p, val = %g\n", 
-		tslider, tslider->value );
+	printf( "tslider_real_changed: 0x%x, val = %g\n", 
+		(unsigned int) tslider, tslider->value );
 #endif /*DEBUG*/
 
 	if( tslider->auto_link ) 
@@ -182,49 +233,47 @@ tslider_real_changed( Tslider *tslider )
 }
 
 static void
-tslider_class_init( TsliderClass *class )
+tslider_class_init( TsliderClass *klass )
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
 
-	parent_class = g_type_class_peek_parent( class );
+	parent_class = gtk_type_class( GTK_TYPE_HBOX );
 
 	object_class->destroy = tslider_destroy;
 
-	class->changed = tslider_real_changed;
-	class->slider_changed = NULL;
-	class->activate = NULL;
+	klass->changed = tslider_real_changed;
+	klass->slider_changed = NULL;
+	klass->activate = NULL;
 
 	/* Create signals.
 	 */
-	tslider_signals[CHANGED] = g_signal_new( "changed",
-		G_OBJECT_CLASS_TYPE( gobject_class ),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET( TsliderClass, changed ),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE, 0 );
-	tslider_signals[ACTIVATE] = g_signal_new( "activate",
-		G_OBJECT_CLASS_TYPE( gobject_class ),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET( TsliderClass, activate ),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE, 0 );
-	tslider_signals[SLIDER_CHANGED] = g_signal_new( "slider_changed",
-		G_OBJECT_CLASS_TYPE( gobject_class ),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET( TsliderClass, slider_changed ),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE, 0 );
-	tslider_signals[TEXT_CHANGED] = g_signal_new( "text_changed",
-		G_OBJECT_CLASS_TYPE( gobject_class ),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET( TsliderClass, text_changed ),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE, 0 );
+	tslider_signals[CHANGED] = gtk_signal_new( "changed",
+		GTK_RUN_FIRST,
+		object_class->type,
+		GTK_SIGNAL_OFFSET( TsliderClass, changed ),
+		gtk_marshal_NONE__NONE,
+		GTK_TYPE_NONE, 0 );
+	tslider_signals[ACTIVATE] = gtk_signal_new( "activate",
+		GTK_RUN_FIRST,
+		object_class->type,
+		GTK_SIGNAL_OFFSET( TsliderClass, activate ),
+		gtk_marshal_NONE__NONE,
+		GTK_TYPE_NONE, 0 );
+	tslider_signals[SLIDER_CHANGED] = gtk_signal_new( "slider_changed",
+		GTK_RUN_FIRST,
+		object_class->type,
+		GTK_SIGNAL_OFFSET( TsliderClass, slider_changed ),
+		gtk_marshal_NONE__NONE,
+		GTK_TYPE_NONE, 0 );
+	tslider_signals[TEXT_CHANGED] = gtk_signal_new( "text_changed",
+		GTK_RUN_FIRST,
+		object_class->type,
+		GTK_SIGNAL_OFFSET( TsliderClass, text_changed ),
+		gtk_marshal_NONE__NONE,
+		GTK_TYPE_NONE, 0 );
+
+	gtk_object_class_add_signals( object_class, 
+		tslider_signals, LAST_SIGNAL );
 
 	/* Init methods.
 	 */
@@ -239,7 +288,7 @@ tslider_changed( Tslider *tslider )
 	printf( "tslider_changed\n" );
 #endif /*DEBUG*/
 
-	g_signal_emit( G_OBJECT( tslider ), tslider_signals[CHANGED], 0 );
+	gtk_signal_emit( GTK_OBJECT( tslider ), tslider_signals[CHANGED] );
 }
 
 /* Activated!
@@ -251,7 +300,7 @@ tslider_activate( Tslider *tslider )
 	printf( "tslider_activate\n" );
 #endif /*DEBUG*/
 
-	g_signal_emit( G_OBJECT( tslider ), tslider_signals[ACTIVATE], 0 );
+	gtk_signal_emit( GTK_OBJECT( tslider ), tslider_signals[ACTIVATE] );
 }
 
 /* Just the slider changed.
@@ -263,8 +312,8 @@ tslider_slider_changed( Tslider *tslider )
 	printf( "tslider_slider_changed\n" );
 #endif /*DEBUG*/
 
-	g_signal_emit( G_OBJECT( tslider ), 
-		tslider_signals[SLIDER_CHANGED], 0 );
+	gtk_signal_emit( GTK_OBJECT( tslider ), 
+		tslider_signals[SLIDER_CHANGED] );
 }
 
 /* Text has been touched.
@@ -276,7 +325,7 @@ tslider_text_changed( Tslider *tslider )
 	printf( "tslider_text_changed\n" );
 #endif /*DEBUG*/
 
-	g_signal_emit( G_OBJECT( tslider ), tslider_signals[TEXT_CHANGED], 0 );
+	gtk_signal_emit( GTK_OBJECT( tslider ), tslider_signals[TEXT_CHANGED] );
 }
 
 /* Enter in entry widget
@@ -287,7 +336,7 @@ tslider_value_activate_cb( GtkWidget *entry, Tslider *tslider )
 	double value;
 
 	if( !get_geditable_double( entry, &value ) ) {
-		iwindow_alert( entry, GTK_MESSAGE_ERROR );
+		box_alert( entry );
 		return;
 	}
 
@@ -344,27 +393,11 @@ tslider_conversion_id( double from, double to, double value )
 	return( value );
 }
 
-static gboolean
-tslider_scroll_cb( GtkWidget *wid, GdkEvent *event, Tslider *tslider )
-{
-	gboolean handled;
-
-	handled = FALSE;
-
-	/* Stop any other scroll handlers running. We don't want the scroll 
-	 * wheel to change widgets while we're moving.
-	 */
-	if( tslider->ignore_scroll )
-		handled = TRUE;
-
-	return( handled ); 
-}
-
 static void
 tslider_init( Tslider *tslider )
 {
 #ifdef DEBUG
-	printf( "tslider_init: %p\n", tslider );
+	printf( "tslider_init: 0x%x\n", (unsigned int) tslider );
 #endif /*DEBUG*/
 
 	/* Any old start values ... overridden later.
@@ -377,13 +410,12 @@ tslider_init( Tslider *tslider )
 	tslider->last_to = -1;
 	tslider->last_from = -1;
 	tslider->last_svalue = -1;
-	tslider->ignore_scroll = TRUE;
 
         gtk_box_set_spacing( GTK_BOX( tslider ), 2 );
 
-	tslider->entry = build_entry( 5 );
+	tslider->entry = build_entry( 4 );
 	gtk_entry_set_max_length( GTK_ENTRY( tslider->entry ), 10 );
-        set_tooltip( tslider->entry, _( "Slider value ... edit!" ) );
+        set_tooltip( tslider->entry, "Slider value ... edit!" );
         gtk_box_pack_start( GTK_BOX( tslider ), 
 		tslider->entry, FALSE, FALSE, 0 );
         gtk_signal_connect( GTK_OBJECT( tslider->entry ), "activate",
@@ -397,18 +429,17 @@ tslider_init( Tslider *tslider )
         gtk_range_set_update_policy( GTK_RANGE( tslider->slider ),
 		GTK_UPDATE_CONTINUOUS );
 #ifdef DEBUG
+	/*
         gtk_range_set_update_policy( GTK_RANGE( tslider->slider ),
 		GTK_UPDATE_DISCONTINUOUS );
+	 */
 #endif /*DEBUG*/
         gtk_scale_set_draw_value( GTK_SCALE( tslider->slider ), FALSE );
-	gtk_widget_set_size_request( GTK_WIDGET( tslider->slider ), 100, -1 );
         gtk_box_pack_start( GTK_BOX( tslider ), 
 		tslider->slider, TRUE, TRUE, 0 );
-        set_tooltip( tslider->slider, _( "Left-drag to set number" ) );
+        set_tooltip( tslider->slider, "Left-drag to set number" );
         gtk_signal_connect( GTK_OBJECT( tslider->adj ), "value_changed", 
 		GTK_SIGNAL_FUNC( tslider_value_changed_cb ), tslider );
-	g_signal_connect( tslider->slider, "scroll-event", 
-		G_CALLBACK( tslider_scroll_cb ), tslider );
 	gtk_widget_show( tslider->slider );
 
 	tslider->auto_link = TRUE;
@@ -488,9 +519,3 @@ tslider_log_slider_to_value( double from, double to, double value )
 
 	return( nvalue );
 }
-
-void
-tslider_set_ignore_scroll( Tslider *tslider, gboolean ignore_scroll )
-{
-	tslider->ignore_scroll = ignore_scroll;
-}
