@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  */
 
@@ -27,39 +27,29 @@
 
 */
 
+/* Quark links pages to symbols.
+ */
+extern GQuark workspaceview_quark;
+
 #define TYPE_WORKSPACEVIEW (workspaceview_get_type())
 #define WORKSPACEVIEW( obj ) \
 	(GTK_CHECK_CAST( (obj), TYPE_WORKSPACEVIEW, Workspaceview ))
 #define WORKSPACEVIEW_CLASS( klass ) \
-	(GTK_CHECK_CLASS_CAST( (klass), \
-		TYPE_WORKSPACEVIEW, WorkspaceviewClass ))
+	(GTK_CHECK_CLASS_CAST( (klass), TYPE_WORKSPACEVIEW, WorkspaceviewClass ))
 #define IS_WORKSPACEVIEW( obj ) (GTK_CHECK_TYPE( (obj), TYPE_WORKSPACEVIEW ))
 #define IS_WORKSPACEVIEW_CLASS( klass ) \
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_WORKSPACEVIEW ))
 
-/* Column margins.
- */
-#define WORKSPACEVIEW_MARGIN_LEFT (5)
-#define WORKSPACEVIEW_MARGIN_TOP (5)
-
 struct _Workspaceview {
 	View view;
 
+	Workspacegroupview *wgview;
+
+	GtkWidget *tab;			/* Notebook tab */
+	GtkWidget *label;		/* Label for our notebook page */
+	GtkWidget *frame;		/* Enclosing frame */
 	GtkWidget *fixed;		/* GtkFixed for tally */
 	GtkWidget *window;		/* ScrolledWindow holding fixed */
-	Toolkitbrowser *toolkitbrowser;
-	Workspacedefs *workspacedefs;
-	GtkWidget *label;		/* Tab label */
-	GtkWidget *padlock;		/* The padlock icon */
-	GtkWidget *alert;		/* The alert icon */
-
-	/* Left and right panes ... program window and toolkit browser.
-	 */
-	Pane *lpane;
-	Pane *rpane;
-
-	GtkWidget *popup;
-	GtkWidget *popup_jump;
 
 	/* Background window scroll.
 	 */
@@ -85,17 +75,13 @@ struct _Workspaceview {
 	int next_x;
 	int next_y;
 
+	/* If set, the columnview we know is on the front of the stack.
+	 */
+	Columnview *front;		
+
 	/* Context we use to change cursor shape.
 	 */
 	iWindowCursorContext *context;
-
-	/* Follow prefs changes.
-	 */
-	guint watch_changed_sid;
-
-	/* Only show the compat warning once.
-	 */
-	gboolean popped_compat;
 };
 
 typedef struct _WorkspaceviewClass {
@@ -105,13 +91,15 @@ typedef struct _WorkspaceviewClass {
 	 */
 } WorkspaceviewClass;
 
-void workspaceview_scroll( Workspaceview *wview, int x, int y, int w, int h );
-void workspaceview_scroll_background( Workspaceview *wview, int u, int v );
+void workspaceview_scroll( Workspaceview *wsv, int x, int y, int w, int h );
+void workspaceview_scroll_background( Workspaceview *wsv, int u, int v );
 
 void workspaceview_set_cursor( Workspaceview *wview, iWindowShape shape );
 
 GtkType workspaceview_get_type( void );
 View *workspaceview_new( void );
 
-void workspaceview_set_label( Workspaceview *wview, 
-	GtkWidget *label, GtkWidget *padlock, GtkWidget *alert );
+void workspaceview_select( Workspaceview *wsv );
+
+void workspaceview_pick_xy( Workspaceview *wsv, int *x, int *y );
+
